// +build !ignore_autogenerated

/*
Copyright 2018 The Kubernetes Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// This file was autogenerated by openapi-gen. Do not edit it manually!

package openapi

import (
	spec "github.com/go-openapi/spec"
	resource "k8s.io/apimachinery/pkg/api/resource"
	v1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	intstr "k8s.io/apimachinery/pkg/util/intstr"
	common "k8s.io/kube-openapi/pkg/common"
)

func GetOpenAPIDefinitions(ref common.ReferenceCallback) map[string]common.OpenAPIDefinition {
	return map[string]common.OpenAPIDefinition{
		"k8s.io/api/admissionregistration/v1alpha1.Initializer": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Initializer describes the name and the failure policy of an initializer, and what resources it applies to.",
					Properties: map[string]spec.Schema{
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "Name is the identifier of the initializer. It will be added to the object that needs to be initialized. Name should be fully qualified, e.g., alwayspullimages.kubernetes.io, where \"alwayspullimages\" is the name of the webhook, and kubernetes.io is the name of the organization. Required",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"rules": {
							SchemaProps: spec.SchemaProps{
								Description: "Rules describes what resources/subresources the initializer cares about. The initializer cares about an operation if it matches _any_ Rule. Rule.Resources must not include subresources.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/admissionregistration/v1alpha1.Rule"),
										},
									},
								},
							},
						},
					},
					Required: []string{"name"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/admissionregistration/v1alpha1.Rule"},
		},
		"k8s.io/api/admissionregistration/v1alpha1.InitializerConfiguration": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "InitializerConfiguration describes the configuration of initializers.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object metadata; More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"initializers": {
							VendorExtensible: spec.VendorExtensible{
								Extensions: spec.Extensions{
									"x-kubernetes-patch-merge-key": "name",
									"x-kubernetes-patch-strategy":  "merge",
								},
							},
							SchemaProps: spec.SchemaProps{
								Description: "Initializers is a list of resources and their default initializers Order-sensitive. When merging multiple InitializerConfigurations, we sort the initializers from different InitializerConfigurations by the name of the InitializerConfigurations; the order of the initializers from the same InitializerConfiguration is preserved.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/admissionregistration/v1alpha1.Initializer"),
										},
									},
								},
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/admissionregistration/v1alpha1.Initializer", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/admissionregistration/v1alpha1.InitializerConfigurationList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "InitializerConfigurationList is a list of InitializerConfiguration.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "List of InitializerConfiguration.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/admissionregistration/v1alpha1.InitializerConfiguration"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/admissionregistration/v1alpha1.InitializerConfiguration", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/admissionregistration/v1alpha1.Rule": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Rule is a tuple of APIGroups, APIVersion, and Resources.It is recommended to make sure that all the tuple expansions are valid.",
					Properties: map[string]spec.Schema{
						"apiGroups": {
							SchemaProps: spec.SchemaProps{
								Description: "APIGroups is the API groups the resources belong to. '*' is all groups. If '*' is present, the length of the slice must be one. Required.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"apiVersions": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersions is the API versions the resources belong to. '*' is all versions. If '*' is present, the length of the slice must be one. Required.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"resources": {
							SchemaProps: spec.SchemaProps{
								Description: "Resources is a list of resources this rule applies to.\n\nFor example: 'pods' means pods. 'pods/log' means the log subresource of pods. '*' means all resources, but not subresources. 'pods/*' means all subresources of pods. '*/scale' means all scale subresources. '*/*' means all resources and their subresources.\n\nIf wildcard is present, the validation rule will ensure resources do not overlap with each other.\n\nDepending on the enclosing object, subresources might not be allowed. Required.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/admissionregistration/v1beta1.MutatingWebhookConfiguration": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "MutatingWebhookConfiguration describes the configuration of and admission webhook that accept or reject and may change the object.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object metadata; More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"webhooks": {
							VendorExtensible: spec.VendorExtensible{
								Extensions: spec.Extensions{
									"x-kubernetes-patch-merge-key": "name",
									"x-kubernetes-patch-strategy":  "merge",
								},
							},
							SchemaProps: spec.SchemaProps{
								Description: "Webhooks is a list of webhooks and the affected resources and operations.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/admissionregistration/v1beta1.Webhook"),
										},
									},
								},
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/admissionregistration/v1beta1.Webhook", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/admissionregistration/v1beta1.MutatingWebhookConfigurationList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "MutatingWebhookConfigurationList is a list of MutatingWebhookConfiguration.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "List of MutatingWebhookConfiguration.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/admissionregistration/v1beta1.MutatingWebhookConfiguration"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/admissionregistration/v1beta1.MutatingWebhookConfiguration", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/admissionregistration/v1beta1.Rule": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Rule is a tuple of APIGroups, APIVersion, and Resources.It is recommended to make sure that all the tuple expansions are valid.",
					Properties: map[string]spec.Schema{
						"apiGroups": {
							SchemaProps: spec.SchemaProps{
								Description: "APIGroups is the API groups the resources belong to. '*' is all groups. If '*' is present, the length of the slice must be one. Required.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"apiVersions": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersions is the API versions the resources belong to. '*' is all versions. If '*' is present, the length of the slice must be one. Required.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"resources": {
							SchemaProps: spec.SchemaProps{
								Description: "Resources is a list of resources this rule applies to.\n\nFor example: 'pods' means pods. 'pods/log' means the log subresource of pods. '*' means all resources, but not subresources. 'pods/*' means all subresources of pods. '*/scale' means all scale subresources. '*/*' means all resources and their subresources.\n\nIf wildcard is present, the validation rule will ensure resources do not overlap with each other.\n\nDepending on the enclosing object, subresources might not be allowed. Required.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/admissionregistration/v1beta1.RuleWithOperations": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "RuleWithOperations is a tuple of Operations and Resources. It is recommended to make sure that all the tuple expansions are valid.",
					Properties: map[string]spec.Schema{
						"operations": {
							SchemaProps: spec.SchemaProps{
								Description: "Operations is the operations the admission hook cares about - CREATE, UPDATE, or * for all operations. If '*' is present, the length of the slice must be one. Required.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"apiGroups": {
							SchemaProps: spec.SchemaProps{
								Description: "APIGroups is the API groups the resources belong to. '*' is all groups. If '*' is present, the length of the slice must be one. Required.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"apiVersions": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersions is the API versions the resources belong to. '*' is all versions. If '*' is present, the length of the slice must be one. Required.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"resources": {
							SchemaProps: spec.SchemaProps{
								Description: "Resources is a list of resources this rule applies to.\n\nFor example: 'pods' means pods. 'pods/log' means the log subresource of pods. '*' means all resources, but not subresources. 'pods/*' means all subresources of pods. '*/scale' means all scale subresources. '*/*' means all resources and their subresources.\n\nIf wildcard is present, the validation rule will ensure resources do not overlap with each other.\n\nDepending on the enclosing object, subresources might not be allowed. Required.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/admissionregistration/v1beta1.ServiceReference": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ServiceReference holds a reference to Service.legacy.k8s.io",
					Properties: map[string]spec.Schema{
						"namespace": {
							SchemaProps: spec.SchemaProps{
								Description: "`namespace` is the namespace of the service. Required",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "`name` is the name of the service. Required",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"path": {
							SchemaProps: spec.SchemaProps{
								Description: "`path` is an optional URL path which will be sent in any request to this service.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"namespace", "name"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/admissionregistration/v1beta1.ValidatingWebhookConfiguration": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ValidatingWebhookConfiguration describes the configuration of and admission webhook that accept or reject and object without changing it.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object metadata; More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"webhooks": {
							VendorExtensible: spec.VendorExtensible{
								Extensions: spec.Extensions{
									"x-kubernetes-patch-merge-key": "name",
									"x-kubernetes-patch-strategy":  "merge",
								},
							},
							SchemaProps: spec.SchemaProps{
								Description: "Webhooks is a list of webhooks and the affected resources and operations.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/admissionregistration/v1beta1.Webhook"),
										},
									},
								},
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/admissionregistration/v1beta1.Webhook", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/admissionregistration/v1beta1.ValidatingWebhookConfigurationList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ValidatingWebhookConfigurationList is a list of ValidatingWebhookConfiguration.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "List of ValidatingWebhookConfiguration.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/admissionregistration/v1beta1.ValidatingWebhookConfiguration"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/admissionregistration/v1beta1.ValidatingWebhookConfiguration", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/admissionregistration/v1beta1.Webhook": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Webhook describes an admission webhook and the resources and operations it applies to.",
					Properties: map[string]spec.Schema{
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "The name of the admission webhook. Name should be fully qualified, e.g., imagepolicy.kubernetes.io, where \"imagepolicy\" is the name of the webhook, and kubernetes.io is the name of the organization. Required.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"clientConfig": {
							SchemaProps: spec.SchemaProps{
								Description: "ClientConfig defines how to communicate with the hook. Required",
								Ref:         ref("k8s.io/api/admissionregistration/v1beta1.WebhookClientConfig"),
							},
						},
						"rules": {
							SchemaProps: spec.SchemaProps{
								Description: "Rules describes what operations on what resources/subresources the webhook cares about. The webhook cares about an operation if it matches _any_ Rule.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/admissionregistration/v1beta1.RuleWithOperations"),
										},
									},
								},
							},
						},
						"failurePolicy": {
							SchemaProps: spec.SchemaProps{
								Description: "FailurePolicy defines how unrecognized errors from the admission endpoint are handled - allowed values are Ignore or Fail. Defaults to Ignore.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"namespaceSelector": {
							SchemaProps: spec.SchemaProps{
								Description: "NamespaceSelector decides whether to run the webhook on an object based on whether the namespace for that object matches the selector. If the object itself is a namespace, the matching is performed on object.metadata.labels. If the object is another cluster scoped resource, it never skips the webhook.\n\nFor example, to run the webhook on any objects whose namespace is not associated with \"runlevel\" of \"0\" or \"1\";  you will set the selector as follows: \"namespaceSelector\": {\n  \"matchExpressions\": [\n    {\n      \"key\": \"runlevel\",\n      \"operator\": \"NotIn\",\n      \"values\": [\n        \"0\",\n        \"1\"\n      ]\n    }\n  ]\n}\n\nIf instead you want to only run the webhook on any objects whose namespace is associated with the \"environment\" of \"prod\" or \"staging\"; you will set the selector as follows: \"namespaceSelector\": {\n  \"matchExpressions\": [\n    {\n      \"key\": \"environment\",\n      \"operator\": \"In\",\n      \"values\": [\n        \"prod\",\n        \"staging\"\n      ]\n    }\n  ]\n}\n\nSee https://kubernetes.io/docs/concepts/overview/working-with-objects/labels/ for more examples of label selectors.\n\nDefault to the empty LabelSelector, which matches everything.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"),
							},
						},
					},
					Required: []string{"name", "clientConfig"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/admissionregistration/v1beta1.RuleWithOperations", "k8s.io/api/admissionregistration/v1beta1.WebhookClientConfig", "k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"},
		},
		"k8s.io/api/admissionregistration/v1beta1.WebhookClientConfig": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "WebhookClientConfig contains the information to make a TLS connection with the webhook",
					Properties: map[string]spec.Schema{
						"url": {
							SchemaProps: spec.SchemaProps{
								Description: "`url` gives the location of the webhook, in standard URL form (`[scheme://]host:port/path`). Exactly one of `url` or `service` must be specified.\n\nThe `host` should not refer to a service running in the cluster; use the `service` field instead. The host might be resolved via external DNS in some apiservers (e.g., `kube-apiserver` cannot resolve in-cluster DNS as that would be a layering violation). `host` may also be an IP address.\n\nPlease note that using `localhost` or `127.0.0.1` as a `host` is risky unless you take great care to run this webhook on all hosts which run an apiserver which might need to make calls to this webhook. Such installs are likely to be non-portable, i.e., not easy to turn up in a new cluster.\n\nThe scheme must be \"https\"; the URL must begin with \"https://\".\n\nA path is optional, and if present may be any string permissible in a URL. You may use the path to pass an arbitrary string to the webhook, for example, a cluster identifier.\n\nAttempting to use a user or basic auth e.g. \"user:password@\" is not allowed. Fragments (\"#...\") and query parameters (\"?...\") are not allowed, either.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"service": {
							SchemaProps: spec.SchemaProps{
								Description: "`service` is a reference to the service for this webhook. Either `service` or `url` must be specified.\n\nIf the webhook is running within the cluster, then you should use `service`.\n\nIf there is only one port open for the service, that port will be used. If there are multiple ports open, port 443 will be used if it is open, otherwise it is an error.",
								Ref:         ref("k8s.io/api/admissionregistration/v1beta1.ServiceReference"),
							},
						},
						"caBundle": {
							SchemaProps: spec.SchemaProps{
								Description: "`caBundle` is a PEM encoded CA bundle which will be used to validate the webhook's server certificate. Required.",
								Type:        []string{"string"},
								Format:      "byte",
							},
						},
					},
					Required: []string{"caBundle"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/admissionregistration/v1beta1.ServiceReference"},
		},
		"k8s.io/api/apps/v1.ControllerRevision": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ControllerRevision implements an immutable snapshot of state data. Clients are responsible for serializing and deserializing the objects that contain their internal state. Once a ControllerRevision has been successfully created, it can not be updated. The API Server will fail validation of all requests that attempt to mutate the Data field. ControllerRevisions may, however, be deleted. Note that, due to its use by both the DaemonSet and StatefulSet controllers for update and rollback, this object is beta. However, it may be subject to name and representation changes in future releases, and clients should not depend on its stability. It is primarily for internal use by controllers.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"data": {
							SchemaProps: spec.SchemaProps{
								Description: "Data is the serialized representation of the state.",
								Ref:         ref("k8s.io/apimachinery/pkg/runtime.RawExtension"),
							},
						},
						"revision": {
							SchemaProps: spec.SchemaProps{
								Description: "Revision indicates the revision of the state represented by Data.",
								Type:        []string{"integer"},
								Format:      "int64",
							},
						},
					},
					Required: []string{"revision"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta", "k8s.io/apimachinery/pkg/runtime.RawExtension"},
		},
		"k8s.io/api/apps/v1.ControllerRevisionList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ControllerRevisionList is a resource containing a list of ControllerRevision objects.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "Items is the list of ControllerRevisions",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/apps/v1.ControllerRevision"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/apps/v1.ControllerRevision", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/apps/v1.DaemonSet": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DaemonSet represents the configuration of a daemon set.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "The desired behavior of this daemon set. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status",
								Ref:         ref("k8s.io/api/apps/v1.DaemonSetSpec"),
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "The current status of this daemon set. This data may be out of date by some window of time. Populated by the system. Read-only. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status",
								Ref:         ref("k8s.io/api/apps/v1.DaemonSetStatus"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/apps/v1.DaemonSetSpec", "k8s.io/api/apps/v1.DaemonSetStatus", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/apps/v1.DaemonSetCondition": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DaemonSetCondition describes the state of a DaemonSet at a certain point.",
					Properties: map[string]spec.Schema{
						"type": {
							SchemaProps: spec.SchemaProps{
								Description: "Type of DaemonSet condition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Status of the condition, one of True, False, Unknown.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"lastTransitionTime": {
							SchemaProps: spec.SchemaProps{
								Description: "Last time the condition transitioned from one status to another.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"reason": {
							SchemaProps: spec.SchemaProps{
								Description: "The reason for the condition's last transition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"message": {
							SchemaProps: spec.SchemaProps{
								Description: "A human readable message indicating details about the transition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"type", "status"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.Time"},
		},
		"k8s.io/api/apps/v1.DaemonSetList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DaemonSetList is a collection of daemon sets.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "A list of daemon sets.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/apps/v1.DaemonSet"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/apps/v1.DaemonSet", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/apps/v1.DaemonSetSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DaemonSetSpec is the specification of a daemon set.",
					Properties: map[string]spec.Schema{
						"selector": {
							SchemaProps: spec.SchemaProps{
								Description: "A label query over pods that are managed by the daemon set. Must match in order to be controlled. It must match the pod template's labels. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/labels/#label-selectors",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"),
							},
						},
						"template": {
							SchemaProps: spec.SchemaProps{
								Description: "An object that describes the pod that will be created. The DaemonSet will create exactly one copy of this pod on every node that matches the template's node selector (or on every node if no node selector is specified). More info: https://kubernetes.io/docs/concepts/workloads/controllers/replicationcontroller#pod-template",
								Ref:         ref("k8s.io/api/core/v1.PodTemplateSpec"),
							},
						},
						"updateStrategy": {
							SchemaProps: spec.SchemaProps{
								Description: "An update strategy to replace existing DaemonSet pods with new pods.",
								Ref:         ref("k8s.io/api/apps/v1.DaemonSetUpdateStrategy"),
							},
						},
						"minReadySeconds": {
							SchemaProps: spec.SchemaProps{
								Description: "The minimum number of seconds for which a newly created DaemonSet pod should be ready without any of its container crashing, for it to be considered available. Defaults to 0 (pod will be considered available as soon as it is ready).",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"revisionHistoryLimit": {
							SchemaProps: spec.SchemaProps{
								Description: "The number of old history to retain to allow rollback. This is a pointer to distinguish between explicit zero and not specified. Defaults to 10.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
					},
					Required: []string{"selector", "template"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/apps/v1.DaemonSetUpdateStrategy", "k8s.io/api/core/v1.PodTemplateSpec", "k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"},
		},
		"k8s.io/api/apps/v1.DaemonSetStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DaemonSetStatus represents the current status of a daemon set.",
					Properties: map[string]spec.Schema{
						"currentNumberScheduled": {
							SchemaProps: spec.SchemaProps{
								Description: "The number of nodes that are running at least 1 daemon pod and are supposed to run the daemon pod. More info: https://kubernetes.io/docs/concepts/workloads/controllers/daemonset/",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"numberMisscheduled": {
							SchemaProps: spec.SchemaProps{
								Description: "The number of nodes that are running the daemon pod, but are not supposed to run the daemon pod. More info: https://kubernetes.io/docs/concepts/workloads/controllers/daemonset/",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"desiredNumberScheduled": {
							SchemaProps: spec.SchemaProps{
								Description: "The total number of nodes that should be running the daemon pod (including nodes correctly running the daemon pod). More info: https://kubernetes.io/docs/concepts/workloads/controllers/daemonset/",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"numberReady": {
							SchemaProps: spec.SchemaProps{
								Description: "The number of nodes that should be running the daemon pod and have one or more of the daemon pod running and ready.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"observedGeneration": {
							SchemaProps: spec.SchemaProps{
								Description: "The most recent generation observed by the daemon set controller.",
								Type:        []string{"integer"},
								Format:      "int64",
							},
						},
						"updatedNumberScheduled": {
							SchemaProps: spec.SchemaProps{
								Description: "The total number of nodes that are running updated daemon pod",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"numberAvailable": {
							SchemaProps: spec.SchemaProps{
								Description: "The number of nodes that should be running the daemon pod and have one or more of the daemon pod running and available (ready for at least spec.minReadySeconds)",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"numberUnavailable": {
							SchemaProps: spec.SchemaProps{
								Description: "The number of nodes that should be running the daemon pod and have none of the daemon pod running and available (ready for at least spec.minReadySeconds)",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"collisionCount": {
							SchemaProps: spec.SchemaProps{
								Description: "Count of hash collisions for the DaemonSet. The DaemonSet controller uses this field as a collision avoidance mechanism when it needs to create the name for the newest ControllerRevision.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"conditions": {
							VendorExtensible: spec.VendorExtensible{
								Extensions: spec.Extensions{
									"x-kubernetes-patch-merge-key": "type",
									"x-kubernetes-patch-strategy":  "merge",
								},
							},
							SchemaProps: spec.SchemaProps{
								Description: "Represents the latest available observations of a DaemonSet's current state.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/apps/v1.DaemonSetCondition"),
										},
									},
								},
							},
						},
					},
					Required: []string{"currentNumberScheduled", "numberMisscheduled", "desiredNumberScheduled", "numberReady"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/apps/v1.DaemonSetCondition"},
		},
		"k8s.io/api/apps/v1.DaemonSetUpdateStrategy": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DaemonSetUpdateStrategy is a struct used to control the update strategy for a DaemonSet.",
					Properties: map[string]spec.Schema{
						"type": {
							SchemaProps: spec.SchemaProps{
								Description: "Type of daemon set update. Can be \"RollingUpdate\" or \"OnDelete\". Default is RollingUpdate.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"rollingUpdate": {
							SchemaProps: spec.SchemaProps{
								Description: "Rolling update config params. Present only if type = \"RollingUpdate\".",
								Ref:         ref("k8s.io/api/apps/v1.RollingUpdateDaemonSet"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/apps/v1.RollingUpdateDaemonSet"},
		},
		"k8s.io/api/apps/v1.Deployment": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Deployment enables declarative updates for Pods and ReplicaSets.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object metadata.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "Specification of the desired behavior of the Deployment.",
								Ref:         ref("k8s.io/api/apps/v1.DeploymentSpec"),
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Most recently observed status of the Deployment.",
								Ref:         ref("k8s.io/api/apps/v1.DeploymentStatus"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/apps/v1.DeploymentSpec", "k8s.io/api/apps/v1.DeploymentStatus", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/apps/v1.DeploymentCondition": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DeploymentCondition describes the state of a deployment at a certain point.",
					Properties: map[string]spec.Schema{
						"type": {
							SchemaProps: spec.SchemaProps{
								Description: "Type of deployment condition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Status of the condition, one of True, False, Unknown.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"lastUpdateTime": {
							SchemaProps: spec.SchemaProps{
								Description: "The last time this condition was updated.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"lastTransitionTime": {
							SchemaProps: spec.SchemaProps{
								Description: "Last time the condition transitioned from one status to another.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"reason": {
							SchemaProps: spec.SchemaProps{
								Description: "The reason for the condition's last transition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"message": {
							SchemaProps: spec.SchemaProps{
								Description: "A human readable message indicating details about the transition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"type", "status"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.Time"},
		},
		"k8s.io/api/apps/v1.DeploymentList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DeploymentList is a list of Deployments.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard list metadata.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "Items is the list of Deployments.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/apps/v1.Deployment"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/apps/v1.Deployment", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/apps/v1.DeploymentSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DeploymentSpec is the specification of the desired behavior of the Deployment.",
					Properties: map[string]spec.Schema{
						"replicas": {
							SchemaProps: spec.SchemaProps{
								Description: "Number of desired pods. This is a pointer to distinguish between explicit zero and not specified. Defaults to 1.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"selector": {
							SchemaProps: spec.SchemaProps{
								Description: "Label selector for pods. Existing ReplicaSets whose pods are selected by this will be the ones affected by this deployment. It must match the pod template's labels.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"),
							},
						},
						"template": {
							SchemaProps: spec.SchemaProps{
								Description: "Template describes the pods that will be created.",
								Ref:         ref("k8s.io/api/core/v1.PodTemplateSpec"),
							},
						},
						"strategy": {
							SchemaProps: spec.SchemaProps{
								Description: "The deployment strategy to use to replace existing pods with new ones.",
								Ref:         ref("k8s.io/api/apps/v1.DeploymentStrategy"),
							},
						},
						"minReadySeconds": {
							SchemaProps: spec.SchemaProps{
								Description: "Minimum number of seconds for which a newly created pod should be ready without any of its container crashing, for it to be considered available. Defaults to 0 (pod will be considered available as soon as it is ready)",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"revisionHistoryLimit": {
							SchemaProps: spec.SchemaProps{
								Description: "The number of old ReplicaSets to retain to allow rollback. This is a pointer to distinguish between explicit zero and not specified. Defaults to 10.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"paused": {
							SchemaProps: spec.SchemaProps{
								Description: "Indicates that the deployment is paused.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"progressDeadlineSeconds": {
							SchemaProps: spec.SchemaProps{
								Description: "The maximum time in seconds for a deployment to make progress before it is considered to be failed. The deployment controller will continue to process failed deployments and a condition with a ProgressDeadlineExceeded reason will be surfaced in the deployment status. Note that progress will not be estimated during the time a deployment is paused. Defaults to 600s.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
					},
					Required: []string{"selector", "template"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/apps/v1.DeploymentStrategy", "k8s.io/api/core/v1.PodTemplateSpec", "k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"},
		},
		"k8s.io/api/apps/v1.DeploymentStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DeploymentStatus is the most recently observed status of the Deployment.",
					Properties: map[string]spec.Schema{
						"observedGeneration": {
							SchemaProps: spec.SchemaProps{
								Description: "The generation observed by the deployment controller.",
								Type:        []string{"integer"},
								Format:      "int64",
							},
						},
						"replicas": {
							SchemaProps: spec.SchemaProps{
								Description: "Total number of non-terminated pods targeted by this deployment (their labels match the selector).",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"updatedReplicas": {
							SchemaProps: spec.SchemaProps{
								Description: "Total number of non-terminated pods targeted by this deployment that have the desired template spec.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"readyReplicas": {
							SchemaProps: spec.SchemaProps{
								Description: "Total number of ready pods targeted by this deployment.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"availableReplicas": {
							SchemaProps: spec.SchemaProps{
								Description: "Total number of available pods (ready for at least minReadySeconds) targeted by this deployment.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"unavailableReplicas": {
							SchemaProps: spec.SchemaProps{
								Description: "Total number of unavailable pods targeted by this deployment. This is the total number of pods that are still required for the deployment to have 100% available capacity. They may either be pods that are running but not yet available or pods that still have not been created.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"conditions": {
							VendorExtensible: spec.VendorExtensible{
								Extensions: spec.Extensions{
									"x-kubernetes-patch-merge-key": "type",
									"x-kubernetes-patch-strategy":  "merge",
								},
							},
							SchemaProps: spec.SchemaProps{
								Description: "Represents the latest available observations of a deployment's current state.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/apps/v1.DeploymentCondition"),
										},
									},
								},
							},
						},
						"collisionCount": {
							SchemaProps: spec.SchemaProps{
								Description: "Count of hash collisions for the Deployment. The Deployment controller uses this field as a collision avoidance mechanism when it needs to create the name for the newest ReplicaSet.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/apps/v1.DeploymentCondition"},
		},
		"k8s.io/api/apps/v1.DeploymentStrategy": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DeploymentStrategy describes how to replace existing pods with new ones.",
					Properties: map[string]spec.Schema{
						"type": {
							SchemaProps: spec.SchemaProps{
								Description: "Type of deployment. Can be \"Recreate\" or \"RollingUpdate\". Default is RollingUpdate.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"rollingUpdate": {
							SchemaProps: spec.SchemaProps{
								Description: "Rolling update config params. Present only if DeploymentStrategyType = RollingUpdate.",
								Ref:         ref("k8s.io/api/apps/v1.RollingUpdateDeployment"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/apps/v1.RollingUpdateDeployment"},
		},
		"k8s.io/api/apps/v1.ReplicaSet": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ReplicaSet ensures that a specified number of pod replicas are running at any given time.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "If the Labels of a ReplicaSet are empty, they are defaulted to be the same as the Pod(s) that the ReplicaSet manages. Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "Spec defines the specification of the desired behavior of the ReplicaSet. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status",
								Ref:         ref("k8s.io/api/apps/v1.ReplicaSetSpec"),
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Status is the most recently observed status of the ReplicaSet. This data may be out of date by some window of time. Populated by the system. Read-only. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status",
								Ref:         ref("k8s.io/api/apps/v1.ReplicaSetStatus"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/apps/v1.ReplicaSetSpec", "k8s.io/api/apps/v1.ReplicaSetStatus", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/apps/v1.ReplicaSetCondition": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ReplicaSetCondition describes the state of a replica set at a certain point.",
					Properties: map[string]spec.Schema{
						"type": {
							SchemaProps: spec.SchemaProps{
								Description: "Type of replica set condition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Status of the condition, one of True, False, Unknown.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"lastTransitionTime": {
							SchemaProps: spec.SchemaProps{
								Description: "The last time the condition transitioned from one status to another.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"reason": {
							SchemaProps: spec.SchemaProps{
								Description: "The reason for the condition's last transition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"message": {
							SchemaProps: spec.SchemaProps{
								Description: "A human readable message indicating details about the transition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"type", "status"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.Time"},
		},
		"k8s.io/api/apps/v1.ReplicaSetList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ReplicaSetList is a collection of ReplicaSets.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "List of ReplicaSets. More info: https://kubernetes.io/docs/concepts/workloads/controllers/replicationcontroller",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/apps/v1.ReplicaSet"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/apps/v1.ReplicaSet", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/apps/v1.ReplicaSetSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ReplicaSetSpec is the specification of a ReplicaSet.",
					Properties: map[string]spec.Schema{
						"replicas": {
							SchemaProps: spec.SchemaProps{
								Description: "Replicas is the number of desired replicas. This is a pointer to distinguish between explicit zero and unspecified. Defaults to 1. More info: https://kubernetes.io/docs/concepts/workloads/controllers/replicationcontroller/#what-is-a-replicationcontroller",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"minReadySeconds": {
							SchemaProps: spec.SchemaProps{
								Description: "Minimum number of seconds for which a newly created pod should be ready without any of its container crashing, for it to be considered available. Defaults to 0 (pod will be considered available as soon as it is ready)",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"selector": {
							SchemaProps: spec.SchemaProps{
								Description: "Selector is a label query over pods that should match the replica count. Label keys and values that must match in order to be controlled by this replica set. It must match the pod template's labels. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/labels/#label-selectors",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"),
							},
						},
						"template": {
							SchemaProps: spec.SchemaProps{
								Description: "Template is the object that describes the pod that will be created if insufficient replicas are detected. More info: https://kubernetes.io/docs/concepts/workloads/controllers/replicationcontroller#pod-template",
								Ref:         ref("k8s.io/api/core/v1.PodTemplateSpec"),
							},
						},
					},
					Required: []string{"selector"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.PodTemplateSpec", "k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"},
		},
		"k8s.io/api/apps/v1.ReplicaSetStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ReplicaSetStatus represents the current status of a ReplicaSet.",
					Properties: map[string]spec.Schema{
						"replicas": {
							SchemaProps: spec.SchemaProps{
								Description: "Replicas is the most recently oberved number of replicas. More info: https://kubernetes.io/docs/concepts/workloads/controllers/replicationcontroller/#what-is-a-replicationcontroller",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"fullyLabeledReplicas": {
							SchemaProps: spec.SchemaProps{
								Description: "The number of pods that have labels matching the labels of the pod template of the replicaset.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"readyReplicas": {
							SchemaProps: spec.SchemaProps{
								Description: "The number of ready replicas for this replica set.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"availableReplicas": {
							SchemaProps: spec.SchemaProps{
								Description: "The number of available replicas (ready for at least minReadySeconds) for this replica set.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"observedGeneration": {
							SchemaProps: spec.SchemaProps{
								Description: "ObservedGeneration reflects the generation of the most recently observed ReplicaSet.",
								Type:        []string{"integer"},
								Format:      "int64",
							},
						},
						"conditions": {
							VendorExtensible: spec.VendorExtensible{
								Extensions: spec.Extensions{
									"x-kubernetes-patch-merge-key": "type",
									"x-kubernetes-patch-strategy":  "merge",
								},
							},
							SchemaProps: spec.SchemaProps{
								Description: "Represents the latest available observations of a replica set's current state.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/apps/v1.ReplicaSetCondition"),
										},
									},
								},
							},
						},
					},
					Required: []string{"replicas"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/apps/v1.ReplicaSetCondition"},
		},
		"k8s.io/api/apps/v1.RollingUpdateDaemonSet": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Spec to control the desired behavior of daemon set rolling update.",
					Properties: map[string]spec.Schema{
						"maxUnavailable": {
							SchemaProps: spec.SchemaProps{
								Description: "The maximum number of DaemonSet pods that can be unavailable during the update. Value can be an absolute number (ex: 5) or a percentage of total number of DaemonSet pods at the start of the update (ex: 10%). Absolute number is calculated from percentage by rounding up. This cannot be 0. Default value is 1. Example: when this is set to 30%, at most 30% of the total number of nodes that should be running the daemon pod (i.e. status.desiredNumberScheduled) can have their pods stopped for an update at any given time. The update starts by stopping at most 30% of those DaemonSet pods and then brings up new DaemonSet pods in their place. Once the new pods are available, it then proceeds onto other DaemonSet pods, thus ensuring that at least 70% of original number of DaemonSet pods are available at all times during the update.",
								Ref:         ref("k8s.io/apimachinery/pkg/util/intstr.IntOrString"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/util/intstr.IntOrString"},
		},
		"k8s.io/api/apps/v1.RollingUpdateDeployment": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Spec to control the desired behavior of rolling update.",
					Properties: map[string]spec.Schema{
						"maxUnavailable": {
							SchemaProps: spec.SchemaProps{
								Description: "The maximum number of pods that can be unavailable during the update. Value can be an absolute number (ex: 5) or a percentage of desired pods (ex: 10%). Absolute number is calculated from percentage by rounding down. This can not be 0 if MaxSurge is 0. Defaults to 25%. Example: when this is set to 30%, the old RC can be scaled down to 70% of desired pods immediately when the rolling update starts. Once new pods are ready, old RC can be scaled down further, followed by scaling up the new RC, ensuring that the total number of pods available at all times during the update is at least 70% of desired pods.",
								Ref:         ref("k8s.io/apimachinery/pkg/util/intstr.IntOrString"),
							},
						},
						"maxSurge": {
							SchemaProps: spec.SchemaProps{
								Description: "The maximum number of pods that can be scheduled above the desired number of pods. Value can be an absolute number (ex: 5) or a percentage of desired pods (ex: 10%). This can not be 0 if MaxUnavailable is 0. Absolute number is calculated from percentage by rounding up. Defaults to 25%. Example: when this is set to 30%, the new RC can be scaled up immediately when the rolling update starts, such that the total number of old and new pods do not exceed 130% of desired pods. Once old pods have been killed, new RC can be scaled up further, ensuring that total number of pods running at any time during the update is atmost 130% of desired pods.",
								Ref:         ref("k8s.io/apimachinery/pkg/util/intstr.IntOrString"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/util/intstr.IntOrString"},
		},
		"k8s.io/api/apps/v1.RollingUpdateStatefulSetStrategy": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "RollingUpdateStatefulSetStrategy is used to communicate parameter for RollingUpdateStatefulSetStrategyType.",
					Properties: map[string]spec.Schema{
						"partition": {
							SchemaProps: spec.SchemaProps{
								Description: "Partition indicates the ordinal at which the StatefulSet should be partitioned. Default value is 0.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/apps/v1.StatefulSet": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "StatefulSet represents a set of pods with consistent identities. Identities are defined as:\n - Network: A single stable DNS and hostname.\n - Storage: As many VolumeClaims as requested.\nThe StatefulSet guarantees that a given network identity will always map to the same storage identity.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "Spec defines the desired identities of pods in this set.",
								Ref:         ref("k8s.io/api/apps/v1.StatefulSetSpec"),
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Status is the current status of Pods in this StatefulSet. This data may be out of date by some window of time.",
								Ref:         ref("k8s.io/api/apps/v1.StatefulSetStatus"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/apps/v1.StatefulSetSpec", "k8s.io/api/apps/v1.StatefulSetStatus", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/apps/v1.StatefulSetCondition": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "StatefulSetCondition describes the state of a statefulset at a certain point.",
					Properties: map[string]spec.Schema{
						"type": {
							SchemaProps: spec.SchemaProps{
								Description: "Type of statefulset condition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Status of the condition, one of True, False, Unknown.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"lastTransitionTime": {
							SchemaProps: spec.SchemaProps{
								Description: "Last time the condition transitioned from one status to another.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"reason": {
							SchemaProps: spec.SchemaProps{
								Description: "The reason for the condition's last transition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"message": {
							SchemaProps: spec.SchemaProps{
								Description: "A human readable message indicating details about the transition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"type", "status"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.Time"},
		},
		"k8s.io/api/apps/v1.StatefulSetList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "StatefulSetList is a collection of StatefulSets.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Type: []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/apps/v1.StatefulSet"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/apps/v1.StatefulSet", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/apps/v1.StatefulSetSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "A StatefulSetSpec is the specification of a StatefulSet.",
					Properties: map[string]spec.Schema{
						"replicas": {
							SchemaProps: spec.SchemaProps{
								Description: "replicas is the desired number of replicas of the given Template. These are replicas in the sense that they are instantiations of the same Template, but individual replicas also have a consistent identity. If unspecified, defaults to 1.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"selector": {
							SchemaProps: spec.SchemaProps{
								Description: "selector is a label query over pods that should match the replica count. It must match the pod template's labels. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/labels/#label-selectors",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"),
							},
						},
						"template": {
							SchemaProps: spec.SchemaProps{
								Description: "template is the object that describes the pod that will be created if insufficient replicas are detected. Each pod stamped out by the StatefulSet will fulfill this Template, but have a unique identity from the rest of the StatefulSet.",
								Ref:         ref("k8s.io/api/core/v1.PodTemplateSpec"),
							},
						},
						"volumeClaimTemplates": {
							SchemaProps: spec.SchemaProps{
								Description: "volumeClaimTemplates is a list of claims that pods are allowed to reference. The StatefulSet controller is responsible for mapping network identities to claims in a way that maintains the identity of a pod. Every claim in this list must have at least one matching (by name) volumeMount in one container in the template. A claim in this list takes precedence over any volumes in the template, with the same name.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.PersistentVolumeClaim"),
										},
									},
								},
							},
						},
						"serviceName": {
							SchemaProps: spec.SchemaProps{
								Description: "serviceName is the name of the service that governs this StatefulSet. This service must exist before the StatefulSet, and is responsible for the network identity of the set. Pods get DNS/hostnames that follow the pattern: pod-specific-string.serviceName.default.svc.cluster.local where \"pod-specific-string\" is managed by the StatefulSet controller.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"podManagementPolicy": {
							SchemaProps: spec.SchemaProps{
								Description: "podManagementPolicy controls how pods are created during initial scale up, when replacing pods on nodes, or when scaling down. The default policy is `OrderedReady`, where pods are created in increasing order (pod-0, then pod-1, etc) and the controller will wait until each pod is ready before continuing. When scaling down, the pods are removed in the opposite order. The alternative policy is `Parallel` which will create pods in parallel to match the desired scale without waiting, and on scale down will delete all pods at once.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"updateStrategy": {
							SchemaProps: spec.SchemaProps{
								Description: "updateStrategy indicates the StatefulSetUpdateStrategy that will be employed to update Pods in the StatefulSet when a revision is made to Template.",
								Ref:         ref("k8s.io/api/apps/v1.StatefulSetUpdateStrategy"),
							},
						},
						"revisionHistoryLimit": {
							SchemaProps: spec.SchemaProps{
								Description: "revisionHistoryLimit is the maximum number of revisions that will be maintained in the StatefulSet's revision history. The revision history consists of all revisions not represented by a currently applied StatefulSetSpec version. The default value is 10.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
					},
					Required: []string{"selector", "template", "serviceName"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/apps/v1.StatefulSetUpdateStrategy", "k8s.io/api/core/v1.PersistentVolumeClaim", "k8s.io/api/core/v1.PodTemplateSpec", "k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"},
		},
		"k8s.io/api/apps/v1.StatefulSetStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "StatefulSetStatus represents the current state of a StatefulSet.",
					Properties: map[string]spec.Schema{
						"observedGeneration": {
							SchemaProps: spec.SchemaProps{
								Description: "observedGeneration is the most recent generation observed for this StatefulSet. It corresponds to the StatefulSet's generation, which is updated on mutation by the API Server.",
								Type:        []string{"integer"},
								Format:      "int64",
							},
						},
						"replicas": {
							SchemaProps: spec.SchemaProps{
								Description: "replicas is the number of Pods created by the StatefulSet controller.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"readyReplicas": {
							SchemaProps: spec.SchemaProps{
								Description: "readyReplicas is the number of Pods created by the StatefulSet controller that have a Ready Condition.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"currentReplicas": {
							SchemaProps: spec.SchemaProps{
								Description: "currentReplicas is the number of Pods created by the StatefulSet controller from the StatefulSet version indicated by currentRevision.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"updatedReplicas": {
							SchemaProps: spec.SchemaProps{
								Description: "updatedReplicas is the number of Pods created by the StatefulSet controller from the StatefulSet version indicated by updateRevision.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"currentRevision": {
							SchemaProps: spec.SchemaProps{
								Description: "currentRevision, if not empty, indicates the version of the StatefulSet used to generate Pods in the sequence [0,currentReplicas).",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"updateRevision": {
							SchemaProps: spec.SchemaProps{
								Description: "updateRevision, if not empty, indicates the version of the StatefulSet used to generate Pods in the sequence [replicas-updatedReplicas,replicas)",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"collisionCount": {
							SchemaProps: spec.SchemaProps{
								Description: "collisionCount is the count of hash collisions for the StatefulSet. The StatefulSet controller uses this field as a collision avoidance mechanism when it needs to create the name for the newest ControllerRevision.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"conditions": {
							VendorExtensible: spec.VendorExtensible{
								Extensions: spec.Extensions{
									"x-kubernetes-patch-merge-key": "type",
									"x-kubernetes-patch-strategy":  "merge",
								},
							},
							SchemaProps: spec.SchemaProps{
								Description: "Represents the latest available observations of a statefulset's current state.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/apps/v1.StatefulSetCondition"),
										},
									},
								},
							},
						},
					},
					Required: []string{"replicas"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/apps/v1.StatefulSetCondition"},
		},
		"k8s.io/api/apps/v1.StatefulSetUpdateStrategy": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "StatefulSetUpdateStrategy indicates the strategy that the StatefulSet controller will use to perform updates. It includes any additional parameters necessary to perform the update for the indicated strategy.",
					Properties: map[string]spec.Schema{
						"type": {
							SchemaProps: spec.SchemaProps{
								Description: "Type indicates the type of the StatefulSetUpdateStrategy. Default is RollingUpdate.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"rollingUpdate": {
							SchemaProps: spec.SchemaProps{
								Description: "RollingUpdate is used to communicate parameters when Type is RollingUpdateStatefulSetStrategyType.",
								Ref:         ref("k8s.io/api/apps/v1.RollingUpdateStatefulSetStrategy"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/apps/v1.RollingUpdateStatefulSetStrategy"},
		},
		"k8s.io/api/apps/v1beta1.ControllerRevision": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DEPRECATED - This group version of ControllerRevision is deprecated by apps/v1beta2/ControllerRevision. See the release notes for more information. ControllerRevision implements an immutable snapshot of state data. Clients are responsible for serializing and deserializing the objects that contain their internal state. Once a ControllerRevision has been successfully created, it can not be updated. The API Server will fail validation of all requests that attempt to mutate the Data field. ControllerRevisions may, however, be deleted. Note that, due to its use by both the DaemonSet and StatefulSet controllers for update and rollback, this object is beta. However, it may be subject to name and representation changes in future releases, and clients should not depend on its stability. It is primarily for internal use by controllers.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"data": {
							SchemaProps: spec.SchemaProps{
								Description: "Data is the serialized representation of the state.",
								Ref:         ref("k8s.io/apimachinery/pkg/runtime.RawExtension"),
							},
						},
						"revision": {
							SchemaProps: spec.SchemaProps{
								Description: "Revision indicates the revision of the state represented by Data.",
								Type:        []string{"integer"},
								Format:      "int64",
							},
						},
					},
					Required: []string{"revision"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta", "k8s.io/apimachinery/pkg/runtime.RawExtension"},
		},
		"k8s.io/api/apps/v1beta1.ControllerRevisionList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ControllerRevisionList is a resource containing a list of ControllerRevision objects.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "Items is the list of ControllerRevisions",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/apps/v1beta1.ControllerRevision"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/apps/v1beta1.ControllerRevision", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/apps/v1beta1.Deployment": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DEPRECATED - This group version of Deployment is deprecated by apps/v1beta2/Deployment. See the release notes for more information. Deployment enables declarative updates for Pods and ReplicaSets.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object metadata.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "Specification of the desired behavior of the Deployment.",
								Ref:         ref("k8s.io/api/apps/v1beta1.DeploymentSpec"),
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Most recently observed status of the Deployment.",
								Ref:         ref("k8s.io/api/apps/v1beta1.DeploymentStatus"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/apps/v1beta1.DeploymentSpec", "k8s.io/api/apps/v1beta1.DeploymentStatus", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/apps/v1beta1.DeploymentCondition": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DeploymentCondition describes the state of a deployment at a certain point.",
					Properties: map[string]spec.Schema{
						"type": {
							SchemaProps: spec.SchemaProps{
								Description: "Type of deployment condition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Status of the condition, one of True, False, Unknown.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"lastUpdateTime": {
							SchemaProps: spec.SchemaProps{
								Description: "The last time this condition was updated.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"lastTransitionTime": {
							SchemaProps: spec.SchemaProps{
								Description: "Last time the condition transitioned from one status to another.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"reason": {
							SchemaProps: spec.SchemaProps{
								Description: "The reason for the condition's last transition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"message": {
							SchemaProps: spec.SchemaProps{
								Description: "A human readable message indicating details about the transition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"type", "status"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.Time"},
		},
		"k8s.io/api/apps/v1beta1.DeploymentList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DeploymentList is a list of Deployments.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard list metadata.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "Items is the list of Deployments.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/apps/v1beta1.Deployment"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/apps/v1beta1.Deployment", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/apps/v1beta1.DeploymentRollback": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DEPRECATED. DeploymentRollback stores the information required to rollback a deployment.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "Required: This must match the Name of a deployment.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"updatedAnnotations": {
							SchemaProps: spec.SchemaProps{
								Description: "The annotations to be updated to a deployment",
								Type:        []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"rollbackTo": {
							SchemaProps: spec.SchemaProps{
								Description: "The config of this deployment rollback.",
								Ref:         ref("k8s.io/api/apps/v1beta1.RollbackConfig"),
							},
						},
					},
					Required: []string{"name", "rollbackTo"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/apps/v1beta1.RollbackConfig"},
		},
		"k8s.io/api/apps/v1beta1.DeploymentSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DeploymentSpec is the specification of the desired behavior of the Deployment.",
					Properties: map[string]spec.Schema{
						"replicas": {
							SchemaProps: spec.SchemaProps{
								Description: "Number of desired pods. This is a pointer to distinguish between explicit zero and not specified. Defaults to 1.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"selector": {
							SchemaProps: spec.SchemaProps{
								Description: "Label selector for pods. Existing ReplicaSets whose pods are selected by this will be the ones affected by this deployment.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"),
							},
						},
						"template": {
							SchemaProps: spec.SchemaProps{
								Description: "Template describes the pods that will be created.",
								Ref:         ref("k8s.io/api/core/v1.PodTemplateSpec"),
							},
						},
						"strategy": {
							SchemaProps: spec.SchemaProps{
								Description: "The deployment strategy to use to replace existing pods with new ones.",
								Ref:         ref("k8s.io/api/apps/v1beta1.DeploymentStrategy"),
							},
						},
						"minReadySeconds": {
							SchemaProps: spec.SchemaProps{
								Description: "Minimum number of seconds for which a newly created pod should be ready without any of its container crashing, for it to be considered available. Defaults to 0 (pod will be considered available as soon as it is ready)",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"revisionHistoryLimit": {
							SchemaProps: spec.SchemaProps{
								Description: "The number of old ReplicaSets to retain to allow rollback. This is a pointer to distinguish between explicit zero and not specified. Defaults to 2.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"paused": {
							SchemaProps: spec.SchemaProps{
								Description: "Indicates that the deployment is paused.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"rollbackTo": {
							SchemaProps: spec.SchemaProps{
								Description: "DEPRECATED. The config this deployment is rolling back to. Will be cleared after rollback is done.",
								Ref:         ref("k8s.io/api/apps/v1beta1.RollbackConfig"),
							},
						},
						"progressDeadlineSeconds": {
							SchemaProps: spec.SchemaProps{
								Description: "The maximum time in seconds for a deployment to make progress before it is considered to be failed. The deployment controller will continue to process failed deployments and a condition with a ProgressDeadlineExceeded reason will be surfaced in the deployment status. Note that progress will not be estimated during the time a deployment is paused. Defaults to 600s.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
					},
					Required: []string{"template"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/apps/v1beta1.DeploymentStrategy", "k8s.io/api/apps/v1beta1.RollbackConfig", "k8s.io/api/core/v1.PodTemplateSpec", "k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"},
		},
		"k8s.io/api/apps/v1beta1.DeploymentStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DeploymentStatus is the most recently observed status of the Deployment.",
					Properties: map[string]spec.Schema{
						"observedGeneration": {
							SchemaProps: spec.SchemaProps{
								Description: "The generation observed by the deployment controller.",
								Type:        []string{"integer"},
								Format:      "int64",
							},
						},
						"replicas": {
							SchemaProps: spec.SchemaProps{
								Description: "Total number of non-terminated pods targeted by this deployment (their labels match the selector).",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"updatedReplicas": {
							SchemaProps: spec.SchemaProps{
								Description: "Total number of non-terminated pods targeted by this deployment that have the desired template spec.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"readyReplicas": {
							SchemaProps: spec.SchemaProps{
								Description: "Total number of ready pods targeted by this deployment.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"availableReplicas": {
							SchemaProps: spec.SchemaProps{
								Description: "Total number of available pods (ready for at least minReadySeconds) targeted by this deployment.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"unavailableReplicas": {
							SchemaProps: spec.SchemaProps{
								Description: "Total number of unavailable pods targeted by this deployment. This is the total number of pods that are still required for the deployment to have 100% available capacity. They may either be pods that are running but not yet available or pods that still have not been created.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"conditions": {
							VendorExtensible: spec.VendorExtensible{
								Extensions: spec.Extensions{
									"x-kubernetes-patch-merge-key": "type",
									"x-kubernetes-patch-strategy":  "merge",
								},
							},
							SchemaProps: spec.SchemaProps{
								Description: "Represents the latest available observations of a deployment's current state.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/apps/v1beta1.DeploymentCondition"),
										},
									},
								},
							},
						},
						"collisionCount": {
							SchemaProps: spec.SchemaProps{
								Description: "Count of hash collisions for the Deployment. The Deployment controller uses this field as a collision avoidance mechanism when it needs to create the name for the newest ReplicaSet.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/apps/v1beta1.DeploymentCondition"},
		},
		"k8s.io/api/apps/v1beta1.DeploymentStrategy": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DeploymentStrategy describes how to replace existing pods with new ones.",
					Properties: map[string]spec.Schema{
						"type": {
							SchemaProps: spec.SchemaProps{
								Description: "Type of deployment. Can be \"Recreate\" or \"RollingUpdate\". Default is RollingUpdate.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"rollingUpdate": {
							SchemaProps: spec.SchemaProps{
								Description: "Rolling update config params. Present only if DeploymentStrategyType = RollingUpdate.",
								Ref:         ref("k8s.io/api/apps/v1beta1.RollingUpdateDeployment"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/apps/v1beta1.RollingUpdateDeployment"},
		},
		"k8s.io/api/apps/v1beta1.RollbackConfig": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DEPRECATED.",
					Properties: map[string]spec.Schema{
						"revision": {
							SchemaProps: spec.SchemaProps{
								Description: "The revision to rollback to. If set to 0, rollback to the last revision.",
								Type:        []string{"integer"},
								Format:      "int64",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/apps/v1beta1.RollingUpdateDeployment": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Spec to control the desired behavior of rolling update.",
					Properties: map[string]spec.Schema{
						"maxUnavailable": {
							SchemaProps: spec.SchemaProps{
								Description: "The maximum number of pods that can be unavailable during the update. Value can be an absolute number (ex: 5) or a percentage of desired pods (ex: 10%). Absolute number is calculated from percentage by rounding down. This can not be 0 if MaxSurge is 0. Defaults to 25%. Example: when this is set to 30%, the old RC can be scaled down to 70% of desired pods immediately when the rolling update starts. Once new pods are ready, old RC can be scaled down further, followed by scaling up the new RC, ensuring that the total number of pods available at all times during the update is at least 70% of desired pods.",
								Ref:         ref("k8s.io/apimachinery/pkg/util/intstr.IntOrString"),
							},
						},
						"maxSurge": {
							SchemaProps: spec.SchemaProps{
								Description: "The maximum number of pods that can be scheduled above the desired number of pods. Value can be an absolute number (ex: 5) or a percentage of desired pods (ex: 10%). This can not be 0 if MaxUnavailable is 0. Absolute number is calculated from percentage by rounding up. Defaults to 25%. Example: when this is set to 30%, the new RC can be scaled up immediately when the rolling update starts, such that the total number of old and new pods do not exceed 130% of desired pods. Once old pods have been killed, new RC can be scaled up further, ensuring that total number of pods running at any time during the update is atmost 130% of desired pods.",
								Ref:         ref("k8s.io/apimachinery/pkg/util/intstr.IntOrString"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/util/intstr.IntOrString"},
		},
		"k8s.io/api/apps/v1beta1.RollingUpdateStatefulSetStrategy": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "RollingUpdateStatefulSetStrategy is used to communicate parameter for RollingUpdateStatefulSetStrategyType.",
					Properties: map[string]spec.Schema{
						"partition": {
							SchemaProps: spec.SchemaProps{
								Description: "Partition indicates the ordinal at which the StatefulSet should be partitioned.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/apps/v1beta1.Scale": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Scale represents a scaling request for a resource.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object metadata; More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "defines the behavior of the scale. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status.",
								Ref:         ref("k8s.io/api/apps/v1beta1.ScaleSpec"),
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "current status of the scale. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status. Read-only.",
								Ref:         ref("k8s.io/api/apps/v1beta1.ScaleStatus"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/apps/v1beta1.ScaleSpec", "k8s.io/api/apps/v1beta1.ScaleStatus", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/apps/v1beta1.ScaleSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ScaleSpec describes the attributes of a scale subresource",
					Properties: map[string]spec.Schema{
						"replicas": {
							SchemaProps: spec.SchemaProps{
								Description: "desired number of instances for the scaled object.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/apps/v1beta1.ScaleStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ScaleStatus represents the current status of a scale subresource.",
					Properties: map[string]spec.Schema{
						"replicas": {
							SchemaProps: spec.SchemaProps{
								Description: "actual number of observed instances of the scaled object.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"selector": {
							SchemaProps: spec.SchemaProps{
								Description: "label query over pods that should match the replicas count. More info: http://kubernetes.io/docs/user-guide/labels#label-selectors",
								Type:        []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"targetSelector": {
							SchemaProps: spec.SchemaProps{
								Description: "label selector for pods that should match the replicas count. This is a serializated version of both map-based and more expressive set-based selectors. This is done to avoid introspection in the clients. The string will be in the same format as the query-param syntax. If the target type only supports map-based selectors, both this field and map-based selector field are populated. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/labels/#label-selectors",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"replicas"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/apps/v1beta1.StatefulSet": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DEPRECATED - This group version of StatefulSet is deprecated by apps/v1beta2/StatefulSet. See the release notes for more information. StatefulSet represents a set of pods with consistent identities. Identities are defined as:\n - Network: A single stable DNS and hostname.\n - Storage: As many VolumeClaims as requested.\nThe StatefulSet guarantees that a given network identity will always map to the same storage identity.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "Spec defines the desired identities of pods in this set.",
								Ref:         ref("k8s.io/api/apps/v1beta1.StatefulSetSpec"),
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Status is the current status of Pods in this StatefulSet. This data may be out of date by some window of time.",
								Ref:         ref("k8s.io/api/apps/v1beta1.StatefulSetStatus"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/apps/v1beta1.StatefulSetSpec", "k8s.io/api/apps/v1beta1.StatefulSetStatus", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/apps/v1beta1.StatefulSetCondition": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "StatefulSetCondition describes the state of a statefulset at a certain point.",
					Properties: map[string]spec.Schema{
						"type": {
							SchemaProps: spec.SchemaProps{
								Description: "Type of statefulset condition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Status of the condition, one of True, False, Unknown.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"lastTransitionTime": {
							SchemaProps: spec.SchemaProps{
								Description: "Last time the condition transitioned from one status to another.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"reason": {
							SchemaProps: spec.SchemaProps{
								Description: "The reason for the condition's last transition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"message": {
							SchemaProps: spec.SchemaProps{
								Description: "A human readable message indicating details about the transition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"type", "status"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.Time"},
		},
		"k8s.io/api/apps/v1beta1.StatefulSetList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "StatefulSetList is a collection of StatefulSets.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Type: []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/apps/v1beta1.StatefulSet"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/apps/v1beta1.StatefulSet", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/apps/v1beta1.StatefulSetSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "A StatefulSetSpec is the specification of a StatefulSet.",
					Properties: map[string]spec.Schema{
						"replicas": {
							SchemaProps: spec.SchemaProps{
								Description: "replicas is the desired number of replicas of the given Template. These are replicas in the sense that they are instantiations of the same Template, but individual replicas also have a consistent identity. If unspecified, defaults to 1.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"selector": {
							SchemaProps: spec.SchemaProps{
								Description: "selector is a label query over pods that should match the replica count. If empty, defaulted to labels on the pod template. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/labels/#label-selectors",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"),
							},
						},
						"template": {
							SchemaProps: spec.SchemaProps{
								Description: "template is the object that describes the pod that will be created if insufficient replicas are detected. Each pod stamped out by the StatefulSet will fulfill this Template, but have a unique identity from the rest of the StatefulSet.",
								Ref:         ref("k8s.io/api/core/v1.PodTemplateSpec"),
							},
						},
						"volumeClaimTemplates": {
							SchemaProps: spec.SchemaProps{
								Description: "volumeClaimTemplates is a list of claims that pods are allowed to reference. The StatefulSet controller is responsible for mapping network identities to claims in a way that maintains the identity of a pod. Every claim in this list must have at least one matching (by name) volumeMount in one container in the template. A claim in this list takes precedence over any volumes in the template, with the same name.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.PersistentVolumeClaim"),
										},
									},
								},
							},
						},
						"serviceName": {
							SchemaProps: spec.SchemaProps{
								Description: "serviceName is the name of the service that governs this StatefulSet. This service must exist before the StatefulSet, and is responsible for the network identity of the set. Pods get DNS/hostnames that follow the pattern: pod-specific-string.serviceName.default.svc.cluster.local where \"pod-specific-string\" is managed by the StatefulSet controller.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"podManagementPolicy": {
							SchemaProps: spec.SchemaProps{
								Description: "podManagementPolicy controls how pods are created during initial scale up, when replacing pods on nodes, or when scaling down. The default policy is `OrderedReady`, where pods are created in increasing order (pod-0, then pod-1, etc) and the controller will wait until each pod is ready before continuing. When scaling down, the pods are removed in the opposite order. The alternative policy is `Parallel` which will create pods in parallel to match the desired scale without waiting, and on scale down will delete all pods at once.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"updateStrategy": {
							SchemaProps: spec.SchemaProps{
								Description: "updateStrategy indicates the StatefulSetUpdateStrategy that will be employed to update Pods in the StatefulSet when a revision is made to Template.",
								Ref:         ref("k8s.io/api/apps/v1beta1.StatefulSetUpdateStrategy"),
							},
						},
						"revisionHistoryLimit": {
							SchemaProps: spec.SchemaProps{
								Description: "revisionHistoryLimit is the maximum number of revisions that will be maintained in the StatefulSet's revision history. The revision history consists of all revisions not represented by a currently applied StatefulSetSpec version. The default value is 10.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
					},
					Required: []string{"template", "serviceName"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/apps/v1beta1.StatefulSetUpdateStrategy", "k8s.io/api/core/v1.PersistentVolumeClaim", "k8s.io/api/core/v1.PodTemplateSpec", "k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"},
		},
		"k8s.io/api/apps/v1beta1.StatefulSetStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "StatefulSetStatus represents the current state of a StatefulSet.",
					Properties: map[string]spec.Schema{
						"observedGeneration": {
							SchemaProps: spec.SchemaProps{
								Description: "observedGeneration is the most recent generation observed for this StatefulSet. It corresponds to the StatefulSet's generation, which is updated on mutation by the API Server.",
								Type:        []string{"integer"},
								Format:      "int64",
							},
						},
						"replicas": {
							SchemaProps: spec.SchemaProps{
								Description: "replicas is the number of Pods created by the StatefulSet controller.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"readyReplicas": {
							SchemaProps: spec.SchemaProps{
								Description: "readyReplicas is the number of Pods created by the StatefulSet controller that have a Ready Condition.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"currentReplicas": {
							SchemaProps: spec.SchemaProps{
								Description: "currentReplicas is the number of Pods created by the StatefulSet controller from the StatefulSet version indicated by currentRevision.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"updatedReplicas": {
							SchemaProps: spec.SchemaProps{
								Description: "updatedReplicas is the number of Pods created by the StatefulSet controller from the StatefulSet version indicated by updateRevision.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"currentRevision": {
							SchemaProps: spec.SchemaProps{
								Description: "currentRevision, if not empty, indicates the version of the StatefulSet used to generate Pods in the sequence [0,currentReplicas).",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"updateRevision": {
							SchemaProps: spec.SchemaProps{
								Description: "updateRevision, if not empty, indicates the version of the StatefulSet used to generate Pods in the sequence [replicas-updatedReplicas,replicas)",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"collisionCount": {
							SchemaProps: spec.SchemaProps{
								Description: "collisionCount is the count of hash collisions for the StatefulSet. The StatefulSet controller uses this field as a collision avoidance mechanism when it needs to create the name for the newest ControllerRevision.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"conditions": {
							VendorExtensible: spec.VendorExtensible{
								Extensions: spec.Extensions{
									"x-kubernetes-patch-merge-key": "type",
									"x-kubernetes-patch-strategy":  "merge",
								},
							},
							SchemaProps: spec.SchemaProps{
								Description: "Represents the latest available observations of a statefulset's current state.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/apps/v1beta1.StatefulSetCondition"),
										},
									},
								},
							},
						},
					},
					Required: []string{"replicas"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/apps/v1beta1.StatefulSetCondition"},
		},
		"k8s.io/api/apps/v1beta1.StatefulSetUpdateStrategy": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "StatefulSetUpdateStrategy indicates the strategy that the StatefulSet controller will use to perform updates. It includes any additional parameters necessary to perform the update for the indicated strategy.",
					Properties: map[string]spec.Schema{
						"type": {
							SchemaProps: spec.SchemaProps{
								Description: "Type indicates the type of the StatefulSetUpdateStrategy.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"rollingUpdate": {
							SchemaProps: spec.SchemaProps{
								Description: "RollingUpdate is used to communicate parameters when Type is RollingUpdateStatefulSetStrategyType.",
								Ref:         ref("k8s.io/api/apps/v1beta1.RollingUpdateStatefulSetStrategy"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/apps/v1beta1.RollingUpdateStatefulSetStrategy"},
		},
		"k8s.io/api/apps/v1beta2.ControllerRevision": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DEPRECATED - This group version of ControllerRevision is deprecated by apps/v1/ControllerRevision. See the release notes for more information. ControllerRevision implements an immutable snapshot of state data. Clients are responsible for serializing and deserializing the objects that contain their internal state. Once a ControllerRevision has been successfully created, it can not be updated. The API Server will fail validation of all requests that attempt to mutate the Data field. ControllerRevisions may, however, be deleted. Note that, due to its use by both the DaemonSet and StatefulSet controllers for update and rollback, this object is beta. However, it may be subject to name and representation changes in future releases, and clients should not depend on its stability. It is primarily for internal use by controllers.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"data": {
							SchemaProps: spec.SchemaProps{
								Description: "Data is the serialized representation of the state.",
								Ref:         ref("k8s.io/apimachinery/pkg/runtime.RawExtension"),
							},
						},
						"revision": {
							SchemaProps: spec.SchemaProps{
								Description: "Revision indicates the revision of the state represented by Data.",
								Type:        []string{"integer"},
								Format:      "int64",
							},
						},
					},
					Required: []string{"revision"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta", "k8s.io/apimachinery/pkg/runtime.RawExtension"},
		},
		"k8s.io/api/apps/v1beta2.ControllerRevisionList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ControllerRevisionList is a resource containing a list of ControllerRevision objects.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "Items is the list of ControllerRevisions",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/apps/v1beta2.ControllerRevision"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/apps/v1beta2.ControllerRevision", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/apps/v1beta2.DaemonSet": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DEPRECATED - This group version of DaemonSet is deprecated by apps/v1/DaemonSet. See the release notes for more information. DaemonSet represents the configuration of a daemon set.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "The desired behavior of this daemon set. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status",
								Ref:         ref("k8s.io/api/apps/v1beta2.DaemonSetSpec"),
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "The current status of this daemon set. This data may be out of date by some window of time. Populated by the system. Read-only. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status",
								Ref:         ref("k8s.io/api/apps/v1beta2.DaemonSetStatus"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/apps/v1beta2.DaemonSetSpec", "k8s.io/api/apps/v1beta2.DaemonSetStatus", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/apps/v1beta2.DaemonSetCondition": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DaemonSetCondition describes the state of a DaemonSet at a certain point.",
					Properties: map[string]spec.Schema{
						"type": {
							SchemaProps: spec.SchemaProps{
								Description: "Type of DaemonSet condition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Status of the condition, one of True, False, Unknown.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"lastTransitionTime": {
							SchemaProps: spec.SchemaProps{
								Description: "Last time the condition transitioned from one status to another.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"reason": {
							SchemaProps: spec.SchemaProps{
								Description: "The reason for the condition's last transition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"message": {
							SchemaProps: spec.SchemaProps{
								Description: "A human readable message indicating details about the transition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"type", "status"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.Time"},
		},
		"k8s.io/api/apps/v1beta2.DaemonSetList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DaemonSetList is a collection of daemon sets.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "A list of daemon sets.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/apps/v1beta2.DaemonSet"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/apps/v1beta2.DaemonSet", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/apps/v1beta2.DaemonSetSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DaemonSetSpec is the specification of a daemon set.",
					Properties: map[string]spec.Schema{
						"selector": {
							SchemaProps: spec.SchemaProps{
								Description: "A label query over pods that are managed by the daemon set. Must match in order to be controlled. It must match the pod template's labels. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/labels/#label-selectors",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"),
							},
						},
						"template": {
							SchemaProps: spec.SchemaProps{
								Description: "An object that describes the pod that will be created. The DaemonSet will create exactly one copy of this pod on every node that matches the template's node selector (or on every node if no node selector is specified). More info: https://kubernetes.io/docs/concepts/workloads/controllers/replicationcontroller#pod-template",
								Ref:         ref("k8s.io/api/core/v1.PodTemplateSpec"),
							},
						},
						"updateStrategy": {
							SchemaProps: spec.SchemaProps{
								Description: "An update strategy to replace existing DaemonSet pods with new pods.",
								Ref:         ref("k8s.io/api/apps/v1beta2.DaemonSetUpdateStrategy"),
							},
						},
						"minReadySeconds": {
							SchemaProps: spec.SchemaProps{
								Description: "The minimum number of seconds for which a newly created DaemonSet pod should be ready without any of its container crashing, for it to be considered available. Defaults to 0 (pod will be considered available as soon as it is ready).",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"revisionHistoryLimit": {
							SchemaProps: spec.SchemaProps{
								Description: "The number of old history to retain to allow rollback. This is a pointer to distinguish between explicit zero and not specified. Defaults to 10.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
					},
					Required: []string{"selector", "template"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/apps/v1beta2.DaemonSetUpdateStrategy", "k8s.io/api/core/v1.PodTemplateSpec", "k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"},
		},
		"k8s.io/api/apps/v1beta2.DaemonSetStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DaemonSetStatus represents the current status of a daemon set.",
					Properties: map[string]spec.Schema{
						"currentNumberScheduled": {
							SchemaProps: spec.SchemaProps{
								Description: "The number of nodes that are running at least 1 daemon pod and are supposed to run the daemon pod. More info: https://kubernetes.io/docs/concepts/workloads/controllers/daemonset/",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"numberMisscheduled": {
							SchemaProps: spec.SchemaProps{
								Description: "The number of nodes that are running the daemon pod, but are not supposed to run the daemon pod. More info: https://kubernetes.io/docs/concepts/workloads/controllers/daemonset/",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"desiredNumberScheduled": {
							SchemaProps: spec.SchemaProps{
								Description: "The total number of nodes that should be running the daemon pod (including nodes correctly running the daemon pod). More info: https://kubernetes.io/docs/concepts/workloads/controllers/daemonset/",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"numberReady": {
							SchemaProps: spec.SchemaProps{
								Description: "The number of nodes that should be running the daemon pod and have one or more of the daemon pod running and ready.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"observedGeneration": {
							SchemaProps: spec.SchemaProps{
								Description: "The most recent generation observed by the daemon set controller.",
								Type:        []string{"integer"},
								Format:      "int64",
							},
						},
						"updatedNumberScheduled": {
							SchemaProps: spec.SchemaProps{
								Description: "The total number of nodes that are running updated daemon pod",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"numberAvailable": {
							SchemaProps: spec.SchemaProps{
								Description: "The number of nodes that should be running the daemon pod and have one or more of the daemon pod running and available (ready for at least spec.minReadySeconds)",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"numberUnavailable": {
							SchemaProps: spec.SchemaProps{
								Description: "The number of nodes that should be running the daemon pod and have none of the daemon pod running and available (ready for at least spec.minReadySeconds)",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"collisionCount": {
							SchemaProps: spec.SchemaProps{
								Description: "Count of hash collisions for the DaemonSet. The DaemonSet controller uses this field as a collision avoidance mechanism when it needs to create the name for the newest ControllerRevision.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"conditions": {
							VendorExtensible: spec.VendorExtensible{
								Extensions: spec.Extensions{
									"x-kubernetes-patch-merge-key": "type",
									"x-kubernetes-patch-strategy":  "merge",
								},
							},
							SchemaProps: spec.SchemaProps{
								Description: "Represents the latest available observations of a DaemonSet's current state.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/apps/v1beta2.DaemonSetCondition"),
										},
									},
								},
							},
						},
					},
					Required: []string{"currentNumberScheduled", "numberMisscheduled", "desiredNumberScheduled", "numberReady"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/apps/v1beta2.DaemonSetCondition"},
		},
		"k8s.io/api/apps/v1beta2.DaemonSetUpdateStrategy": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DaemonSetUpdateStrategy is a struct used to control the update strategy for a DaemonSet.",
					Properties: map[string]spec.Schema{
						"type": {
							SchemaProps: spec.SchemaProps{
								Description: "Type of daemon set update. Can be \"RollingUpdate\" or \"OnDelete\". Default is RollingUpdate.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"rollingUpdate": {
							SchemaProps: spec.SchemaProps{
								Description: "Rolling update config params. Present only if type = \"RollingUpdate\".",
								Ref:         ref("k8s.io/api/apps/v1beta2.RollingUpdateDaemonSet"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/apps/v1beta2.RollingUpdateDaemonSet"},
		},
		"k8s.io/api/apps/v1beta2.Deployment": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DEPRECATED - This group version of Deployment is deprecated by apps/v1/Deployment. See the release notes for more information. Deployment enables declarative updates for Pods and ReplicaSets.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object metadata.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "Specification of the desired behavior of the Deployment.",
								Ref:         ref("k8s.io/api/apps/v1beta2.DeploymentSpec"),
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Most recently observed status of the Deployment.",
								Ref:         ref("k8s.io/api/apps/v1beta2.DeploymentStatus"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/apps/v1beta2.DeploymentSpec", "k8s.io/api/apps/v1beta2.DeploymentStatus", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/apps/v1beta2.DeploymentCondition": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DeploymentCondition describes the state of a deployment at a certain point.",
					Properties: map[string]spec.Schema{
						"type": {
							SchemaProps: spec.SchemaProps{
								Description: "Type of deployment condition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Status of the condition, one of True, False, Unknown.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"lastUpdateTime": {
							SchemaProps: spec.SchemaProps{
								Description: "The last time this condition was updated.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"lastTransitionTime": {
							SchemaProps: spec.SchemaProps{
								Description: "Last time the condition transitioned from one status to another.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"reason": {
							SchemaProps: spec.SchemaProps{
								Description: "The reason for the condition's last transition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"message": {
							SchemaProps: spec.SchemaProps{
								Description: "A human readable message indicating details about the transition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"type", "status"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.Time"},
		},
		"k8s.io/api/apps/v1beta2.DeploymentList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DeploymentList is a list of Deployments.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard list metadata.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "Items is the list of Deployments.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/apps/v1beta2.Deployment"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/apps/v1beta2.Deployment", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/apps/v1beta2.DeploymentSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DeploymentSpec is the specification of the desired behavior of the Deployment.",
					Properties: map[string]spec.Schema{
						"replicas": {
							SchemaProps: spec.SchemaProps{
								Description: "Number of desired pods. This is a pointer to distinguish between explicit zero and not specified. Defaults to 1.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"selector": {
							SchemaProps: spec.SchemaProps{
								Description: "Label selector for pods. Existing ReplicaSets whose pods are selected by this will be the ones affected by this deployment. It must match the pod template's labels.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"),
							},
						},
						"template": {
							SchemaProps: spec.SchemaProps{
								Description: "Template describes the pods that will be created.",
								Ref:         ref("k8s.io/api/core/v1.PodTemplateSpec"),
							},
						},
						"strategy": {
							SchemaProps: spec.SchemaProps{
								Description: "The deployment strategy to use to replace existing pods with new ones.",
								Ref:         ref("k8s.io/api/apps/v1beta2.DeploymentStrategy"),
							},
						},
						"minReadySeconds": {
							SchemaProps: spec.SchemaProps{
								Description: "Minimum number of seconds for which a newly created pod should be ready without any of its container crashing, for it to be considered available. Defaults to 0 (pod will be considered available as soon as it is ready)",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"revisionHistoryLimit": {
							SchemaProps: spec.SchemaProps{
								Description: "The number of old ReplicaSets to retain to allow rollback. This is a pointer to distinguish between explicit zero and not specified. Defaults to 10.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"paused": {
							SchemaProps: spec.SchemaProps{
								Description: "Indicates that the deployment is paused.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"progressDeadlineSeconds": {
							SchemaProps: spec.SchemaProps{
								Description: "The maximum time in seconds for a deployment to make progress before it is considered to be failed. The deployment controller will continue to process failed deployments and a condition with a ProgressDeadlineExceeded reason will be surfaced in the deployment status. Note that progress will not be estimated during the time a deployment is paused. Defaults to 600s.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
					},
					Required: []string{"selector", "template"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/apps/v1beta2.DeploymentStrategy", "k8s.io/api/core/v1.PodTemplateSpec", "k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"},
		},
		"k8s.io/api/apps/v1beta2.DeploymentStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DeploymentStatus is the most recently observed status of the Deployment.",
					Properties: map[string]spec.Schema{
						"observedGeneration": {
							SchemaProps: spec.SchemaProps{
								Description: "The generation observed by the deployment controller.",
								Type:        []string{"integer"},
								Format:      "int64",
							},
						},
						"replicas": {
							SchemaProps: spec.SchemaProps{
								Description: "Total number of non-terminated pods targeted by this deployment (their labels match the selector).",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"updatedReplicas": {
							SchemaProps: spec.SchemaProps{
								Description: "Total number of non-terminated pods targeted by this deployment that have the desired template spec.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"readyReplicas": {
							SchemaProps: spec.SchemaProps{
								Description: "Total number of ready pods targeted by this deployment.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"availableReplicas": {
							SchemaProps: spec.SchemaProps{
								Description: "Total number of available pods (ready for at least minReadySeconds) targeted by this deployment.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"unavailableReplicas": {
							SchemaProps: spec.SchemaProps{
								Description: "Total number of unavailable pods targeted by this deployment. This is the total number of pods that are still required for the deployment to have 100% available capacity. They may either be pods that are running but not yet available or pods that still have not been created.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"conditions": {
							VendorExtensible: spec.VendorExtensible{
								Extensions: spec.Extensions{
									"x-kubernetes-patch-merge-key": "type",
									"x-kubernetes-patch-strategy":  "merge",
								},
							},
							SchemaProps: spec.SchemaProps{
								Description: "Represents the latest available observations of a deployment's current state.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/apps/v1beta2.DeploymentCondition"),
										},
									},
								},
							},
						},
						"collisionCount": {
							SchemaProps: spec.SchemaProps{
								Description: "Count of hash collisions for the Deployment. The Deployment controller uses this field as a collision avoidance mechanism when it needs to create the name for the newest ReplicaSet.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/apps/v1beta2.DeploymentCondition"},
		},
		"k8s.io/api/apps/v1beta2.DeploymentStrategy": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DeploymentStrategy describes how to replace existing pods with new ones.",
					Properties: map[string]spec.Schema{
						"type": {
							SchemaProps: spec.SchemaProps{
								Description: "Type of deployment. Can be \"Recreate\" or \"RollingUpdate\". Default is RollingUpdate.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"rollingUpdate": {
							SchemaProps: spec.SchemaProps{
								Description: "Rolling update config params. Present only if DeploymentStrategyType = RollingUpdate.",
								Ref:         ref("k8s.io/api/apps/v1beta2.RollingUpdateDeployment"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/apps/v1beta2.RollingUpdateDeployment"},
		},
		"k8s.io/api/apps/v1beta2.ReplicaSet": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DEPRECATED - This group version of ReplicaSet is deprecated by apps/v1/ReplicaSet. See the release notes for more information. ReplicaSet ensures that a specified number of pod replicas are running at any given time.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "If the Labels of a ReplicaSet are empty, they are defaulted to be the same as the Pod(s) that the ReplicaSet manages. Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "Spec defines the specification of the desired behavior of the ReplicaSet. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status",
								Ref:         ref("k8s.io/api/apps/v1beta2.ReplicaSetSpec"),
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Status is the most recently observed status of the ReplicaSet. This data may be out of date by some window of time. Populated by the system. Read-only. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status",
								Ref:         ref("k8s.io/api/apps/v1beta2.ReplicaSetStatus"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/apps/v1beta2.ReplicaSetSpec", "k8s.io/api/apps/v1beta2.ReplicaSetStatus", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/apps/v1beta2.ReplicaSetCondition": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ReplicaSetCondition describes the state of a replica set at a certain point.",
					Properties: map[string]spec.Schema{
						"type": {
							SchemaProps: spec.SchemaProps{
								Description: "Type of replica set condition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Status of the condition, one of True, False, Unknown.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"lastTransitionTime": {
							SchemaProps: spec.SchemaProps{
								Description: "The last time the condition transitioned from one status to another.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"reason": {
							SchemaProps: spec.SchemaProps{
								Description: "The reason for the condition's last transition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"message": {
							SchemaProps: spec.SchemaProps{
								Description: "A human readable message indicating details about the transition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"type", "status"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.Time"},
		},
		"k8s.io/api/apps/v1beta2.ReplicaSetList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ReplicaSetList is a collection of ReplicaSets.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "List of ReplicaSets. More info: https://kubernetes.io/docs/concepts/workloads/controllers/replicationcontroller",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/apps/v1beta2.ReplicaSet"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/apps/v1beta2.ReplicaSet", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/apps/v1beta2.ReplicaSetSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ReplicaSetSpec is the specification of a ReplicaSet.",
					Properties: map[string]spec.Schema{
						"replicas": {
							SchemaProps: spec.SchemaProps{
								Description: "Replicas is the number of desired replicas. This is a pointer to distinguish between explicit zero and unspecified. Defaults to 1. More info: https://kubernetes.io/docs/concepts/workloads/controllers/replicationcontroller/#what-is-a-replicationcontroller",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"minReadySeconds": {
							SchemaProps: spec.SchemaProps{
								Description: "Minimum number of seconds for which a newly created pod should be ready without any of its container crashing, for it to be considered available. Defaults to 0 (pod will be considered available as soon as it is ready)",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"selector": {
							SchemaProps: spec.SchemaProps{
								Description: "Selector is a label query over pods that should match the replica count. Label keys and values that must match in order to be controlled by this replica set. It must match the pod template's labels. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/labels/#label-selectors",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"),
							},
						},
						"template": {
							SchemaProps: spec.SchemaProps{
								Description: "Template is the object that describes the pod that will be created if insufficient replicas are detected. More info: https://kubernetes.io/docs/concepts/workloads/controllers/replicationcontroller#pod-template",
								Ref:         ref("k8s.io/api/core/v1.PodTemplateSpec"),
							},
						},
					},
					Required: []string{"selector"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.PodTemplateSpec", "k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"},
		},
		"k8s.io/api/apps/v1beta2.ReplicaSetStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ReplicaSetStatus represents the current status of a ReplicaSet.",
					Properties: map[string]spec.Schema{
						"replicas": {
							SchemaProps: spec.SchemaProps{
								Description: "Replicas is the most recently oberved number of replicas. More info: https://kubernetes.io/docs/concepts/workloads/controllers/replicationcontroller/#what-is-a-replicationcontroller",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"fullyLabeledReplicas": {
							SchemaProps: spec.SchemaProps{
								Description: "The number of pods that have labels matching the labels of the pod template of the replicaset.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"readyReplicas": {
							SchemaProps: spec.SchemaProps{
								Description: "The number of ready replicas for this replica set.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"availableReplicas": {
							SchemaProps: spec.SchemaProps{
								Description: "The number of available replicas (ready for at least minReadySeconds) for this replica set.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"observedGeneration": {
							SchemaProps: spec.SchemaProps{
								Description: "ObservedGeneration reflects the generation of the most recently observed ReplicaSet.",
								Type:        []string{"integer"},
								Format:      "int64",
							},
						},
						"conditions": {
							VendorExtensible: spec.VendorExtensible{
								Extensions: spec.Extensions{
									"x-kubernetes-patch-merge-key": "type",
									"x-kubernetes-patch-strategy":  "merge",
								},
							},
							SchemaProps: spec.SchemaProps{
								Description: "Represents the latest available observations of a replica set's current state.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/apps/v1beta2.ReplicaSetCondition"),
										},
									},
								},
							},
						},
					},
					Required: []string{"replicas"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/apps/v1beta2.ReplicaSetCondition"},
		},
		"k8s.io/api/apps/v1beta2.RollingUpdateDaemonSet": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Spec to control the desired behavior of daemon set rolling update.",
					Properties: map[string]spec.Schema{
						"maxUnavailable": {
							SchemaProps: spec.SchemaProps{
								Description: "The maximum number of DaemonSet pods that can be unavailable during the update. Value can be an absolute number (ex: 5) or a percentage of total number of DaemonSet pods at the start of the update (ex: 10%). Absolute number is calculated from percentage by rounding up. This cannot be 0. Default value is 1. Example: when this is set to 30%, at most 30% of the total number of nodes that should be running the daemon pod (i.e. status.desiredNumberScheduled) can have their pods stopped for an update at any given time. The update starts by stopping at most 30% of those DaemonSet pods and then brings up new DaemonSet pods in their place. Once the new pods are available, it then proceeds onto other DaemonSet pods, thus ensuring that at least 70% of original number of DaemonSet pods are available at all times during the update.",
								Ref:         ref("k8s.io/apimachinery/pkg/util/intstr.IntOrString"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/util/intstr.IntOrString"},
		},
		"k8s.io/api/apps/v1beta2.RollingUpdateDeployment": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Spec to control the desired behavior of rolling update.",
					Properties: map[string]spec.Schema{
						"maxUnavailable": {
							SchemaProps: spec.SchemaProps{
								Description: "The maximum number of pods that can be unavailable during the update. Value can be an absolute number (ex: 5) or a percentage of desired pods (ex: 10%). Absolute number is calculated from percentage by rounding down. This can not be 0 if MaxSurge is 0. Defaults to 25%. Example: when this is set to 30%, the old RC can be scaled down to 70% of desired pods immediately when the rolling update starts. Once new pods are ready, old RC can be scaled down further, followed by scaling up the new RC, ensuring that the total number of pods available at all times during the update is at least 70% of desired pods.",
								Ref:         ref("k8s.io/apimachinery/pkg/util/intstr.IntOrString"),
							},
						},
						"maxSurge": {
							SchemaProps: spec.SchemaProps{
								Description: "The maximum number of pods that can be scheduled above the desired number of pods. Value can be an absolute number (ex: 5) or a percentage of desired pods (ex: 10%). This can not be 0 if MaxUnavailable is 0. Absolute number is calculated from percentage by rounding up. Defaults to 25%. Example: when this is set to 30%, the new RC can be scaled up immediately when the rolling update starts, such that the total number of old and new pods do not exceed 130% of desired pods. Once old pods have been killed, new RC can be scaled up further, ensuring that total number of pods running at any time during the update is atmost 130% of desired pods.",
								Ref:         ref("k8s.io/apimachinery/pkg/util/intstr.IntOrString"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/util/intstr.IntOrString"},
		},
		"k8s.io/api/apps/v1beta2.RollingUpdateStatefulSetStrategy": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "RollingUpdateStatefulSetStrategy is used to communicate parameter for RollingUpdateStatefulSetStrategyType.",
					Properties: map[string]spec.Schema{
						"partition": {
							SchemaProps: spec.SchemaProps{
								Description: "Partition indicates the ordinal at which the StatefulSet should be partitioned. Default value is 0.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/apps/v1beta2.Scale": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Scale represents a scaling request for a resource.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object metadata; More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "defines the behavior of the scale. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status.",
								Ref:         ref("k8s.io/api/apps/v1beta2.ScaleSpec"),
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "current status of the scale. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status. Read-only.",
								Ref:         ref("k8s.io/api/apps/v1beta2.ScaleStatus"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/apps/v1beta2.ScaleSpec", "k8s.io/api/apps/v1beta2.ScaleStatus", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/apps/v1beta2.ScaleSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ScaleSpec describes the attributes of a scale subresource",
					Properties: map[string]spec.Schema{
						"replicas": {
							SchemaProps: spec.SchemaProps{
								Description: "desired number of instances for the scaled object.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/apps/v1beta2.ScaleStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ScaleStatus represents the current status of a scale subresource.",
					Properties: map[string]spec.Schema{
						"replicas": {
							SchemaProps: spec.SchemaProps{
								Description: "actual number of observed instances of the scaled object.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"selector": {
							SchemaProps: spec.SchemaProps{
								Description: "label query over pods that should match the replicas count. More info: http://kubernetes.io/docs/user-guide/labels#label-selectors",
								Type:        []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"targetSelector": {
							SchemaProps: spec.SchemaProps{
								Description: "label selector for pods that should match the replicas count. This is a serializated version of both map-based and more expressive set-based selectors. This is done to avoid introspection in the clients. The string will be in the same format as the query-param syntax. If the target type only supports map-based selectors, both this field and map-based selector field are populated. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/labels/#label-selectors",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"replicas"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/apps/v1beta2.StatefulSet": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DEPRECATED - This group version of StatefulSet is deprecated by apps/v1/StatefulSet. See the release notes for more information. StatefulSet represents a set of pods with consistent identities. Identities are defined as:\n - Network: A single stable DNS and hostname.\n - Storage: As many VolumeClaims as requested.\nThe StatefulSet guarantees that a given network identity will always map to the same storage identity.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "Spec defines the desired identities of pods in this set.",
								Ref:         ref("k8s.io/api/apps/v1beta2.StatefulSetSpec"),
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Status is the current status of Pods in this StatefulSet. This data may be out of date by some window of time.",
								Ref:         ref("k8s.io/api/apps/v1beta2.StatefulSetStatus"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/apps/v1beta2.StatefulSetSpec", "k8s.io/api/apps/v1beta2.StatefulSetStatus", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/apps/v1beta2.StatefulSetCondition": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "StatefulSetCondition describes the state of a statefulset at a certain point.",
					Properties: map[string]spec.Schema{
						"type": {
							SchemaProps: spec.SchemaProps{
								Description: "Type of statefulset condition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Status of the condition, one of True, False, Unknown.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"lastTransitionTime": {
							SchemaProps: spec.SchemaProps{
								Description: "Last time the condition transitioned from one status to another.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"reason": {
							SchemaProps: spec.SchemaProps{
								Description: "The reason for the condition's last transition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"message": {
							SchemaProps: spec.SchemaProps{
								Description: "A human readable message indicating details about the transition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"type", "status"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.Time"},
		},
		"k8s.io/api/apps/v1beta2.StatefulSetList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "StatefulSetList is a collection of StatefulSets.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Type: []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/apps/v1beta2.StatefulSet"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/apps/v1beta2.StatefulSet", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/apps/v1beta2.StatefulSetSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "A StatefulSetSpec is the specification of a StatefulSet.",
					Properties: map[string]spec.Schema{
						"replicas": {
							SchemaProps: spec.SchemaProps{
								Description: "replicas is the desired number of replicas of the given Template. These are replicas in the sense that they are instantiations of the same Template, but individual replicas also have a consistent identity. If unspecified, defaults to 1.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"selector": {
							SchemaProps: spec.SchemaProps{
								Description: "selector is a label query over pods that should match the replica count. It must match the pod template's labels. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/labels/#label-selectors",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"),
							},
						},
						"template": {
							SchemaProps: spec.SchemaProps{
								Description: "template is the object that describes the pod that will be created if insufficient replicas are detected. Each pod stamped out by the StatefulSet will fulfill this Template, but have a unique identity from the rest of the StatefulSet.",
								Ref:         ref("k8s.io/api/core/v1.PodTemplateSpec"),
							},
						},
						"volumeClaimTemplates": {
							SchemaProps: spec.SchemaProps{
								Description: "volumeClaimTemplates is a list of claims that pods are allowed to reference. The StatefulSet controller is responsible for mapping network identities to claims in a way that maintains the identity of a pod. Every claim in this list must have at least one matching (by name) volumeMount in one container in the template. A claim in this list takes precedence over any volumes in the template, with the same name.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.PersistentVolumeClaim"),
										},
									},
								},
							},
						},
						"serviceName": {
							SchemaProps: spec.SchemaProps{
								Description: "serviceName is the name of the service that governs this StatefulSet. This service must exist before the StatefulSet, and is responsible for the network identity of the set. Pods get DNS/hostnames that follow the pattern: pod-specific-string.serviceName.default.svc.cluster.local where \"pod-specific-string\" is managed by the StatefulSet controller.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"podManagementPolicy": {
							SchemaProps: spec.SchemaProps{
								Description: "podManagementPolicy controls how pods are created during initial scale up, when replacing pods on nodes, or when scaling down. The default policy is `OrderedReady`, where pods are created in increasing order (pod-0, then pod-1, etc) and the controller will wait until each pod is ready before continuing. When scaling down, the pods are removed in the opposite order. The alternative policy is `Parallel` which will create pods in parallel to match the desired scale without waiting, and on scale down will delete all pods at once.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"updateStrategy": {
							SchemaProps: spec.SchemaProps{
								Description: "updateStrategy indicates the StatefulSetUpdateStrategy that will be employed to update Pods in the StatefulSet when a revision is made to Template.",
								Ref:         ref("k8s.io/api/apps/v1beta2.StatefulSetUpdateStrategy"),
							},
						},
						"revisionHistoryLimit": {
							SchemaProps: spec.SchemaProps{
								Description: "revisionHistoryLimit is the maximum number of revisions that will be maintained in the StatefulSet's revision history. The revision history consists of all revisions not represented by a currently applied StatefulSetSpec version. The default value is 10.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
					},
					Required: []string{"selector", "template", "serviceName"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/apps/v1beta2.StatefulSetUpdateStrategy", "k8s.io/api/core/v1.PersistentVolumeClaim", "k8s.io/api/core/v1.PodTemplateSpec", "k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"},
		},
		"k8s.io/api/apps/v1beta2.StatefulSetStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "StatefulSetStatus represents the current state of a StatefulSet.",
					Properties: map[string]spec.Schema{
						"observedGeneration": {
							SchemaProps: spec.SchemaProps{
								Description: "observedGeneration is the most recent generation observed for this StatefulSet. It corresponds to the StatefulSet's generation, which is updated on mutation by the API Server.",
								Type:        []string{"integer"},
								Format:      "int64",
							},
						},
						"replicas": {
							SchemaProps: spec.SchemaProps{
								Description: "replicas is the number of Pods created by the StatefulSet controller.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"readyReplicas": {
							SchemaProps: spec.SchemaProps{
								Description: "readyReplicas is the number of Pods created by the StatefulSet controller that have a Ready Condition.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"currentReplicas": {
							SchemaProps: spec.SchemaProps{
								Description: "currentReplicas is the number of Pods created by the StatefulSet controller from the StatefulSet version indicated by currentRevision.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"updatedReplicas": {
							SchemaProps: spec.SchemaProps{
								Description: "updatedReplicas is the number of Pods created by the StatefulSet controller from the StatefulSet version indicated by updateRevision.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"currentRevision": {
							SchemaProps: spec.SchemaProps{
								Description: "currentRevision, if not empty, indicates the version of the StatefulSet used to generate Pods in the sequence [0,currentReplicas).",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"updateRevision": {
							SchemaProps: spec.SchemaProps{
								Description: "updateRevision, if not empty, indicates the version of the StatefulSet used to generate Pods in the sequence [replicas-updatedReplicas,replicas)",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"collisionCount": {
							SchemaProps: spec.SchemaProps{
								Description: "collisionCount is the count of hash collisions for the StatefulSet. The StatefulSet controller uses this field as a collision avoidance mechanism when it needs to create the name for the newest ControllerRevision.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"conditions": {
							VendorExtensible: spec.VendorExtensible{
								Extensions: spec.Extensions{
									"x-kubernetes-patch-merge-key": "type",
									"x-kubernetes-patch-strategy":  "merge",
								},
							},
							SchemaProps: spec.SchemaProps{
								Description: "Represents the latest available observations of a statefulset's current state.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/apps/v1beta2.StatefulSetCondition"),
										},
									},
								},
							},
						},
					},
					Required: []string{"replicas"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/apps/v1beta2.StatefulSetCondition"},
		},
		"k8s.io/api/apps/v1beta2.StatefulSetUpdateStrategy": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "StatefulSetUpdateStrategy indicates the strategy that the StatefulSet controller will use to perform updates. It includes any additional parameters necessary to perform the update for the indicated strategy.",
					Properties: map[string]spec.Schema{
						"type": {
							SchemaProps: spec.SchemaProps{
								Description: "Type indicates the type of the StatefulSetUpdateStrategy. Default is RollingUpdate.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"rollingUpdate": {
							SchemaProps: spec.SchemaProps{
								Description: "RollingUpdate is used to communicate parameters when Type is RollingUpdateStatefulSetStrategyType.",
								Ref:         ref("k8s.io/api/apps/v1beta2.RollingUpdateStatefulSetStrategy"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/apps/v1beta2.RollingUpdateStatefulSetStrategy"},
		},
		"k8s.io/api/authentication/v1.TokenReview": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "TokenReview attempts to authenticate a token to a known user. Note: TokenReview requests may be cached by the webhook token authenticator plugin in the kube-apiserver.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "Spec holds information about the request being evaluated",
								Ref:         ref("k8s.io/api/authentication/v1.TokenReviewSpec"),
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Status is filled in by the server and indicates whether the request can be authenticated.",
								Ref:         ref("k8s.io/api/authentication/v1.TokenReviewStatus"),
							},
						},
					},
					Required: []string{"spec"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/authentication/v1.TokenReviewSpec", "k8s.io/api/authentication/v1.TokenReviewStatus", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/authentication/v1.TokenReviewSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "TokenReviewSpec is a description of the token authentication request.",
					Properties: map[string]spec.Schema{
						"token": {
							SchemaProps: spec.SchemaProps{
								Description: "Token is the opaque bearer token.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/authentication/v1.TokenReviewStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "TokenReviewStatus is the result of the token authentication request.",
					Properties: map[string]spec.Schema{
						"authenticated": {
							SchemaProps: spec.SchemaProps{
								Description: "Authenticated indicates that the token was associated with a known user.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"user": {
							SchemaProps: spec.SchemaProps{
								Description: "User is the UserInfo associated with the provided token.",
								Ref:         ref("k8s.io/api/authentication/v1.UserInfo"),
							},
						},
						"error": {
							SchemaProps: spec.SchemaProps{
								Description: "Error indicates that the token couldn't be checked",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/authentication/v1.UserInfo"},
		},
		"k8s.io/api/authentication/v1.UserInfo": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "UserInfo holds the information about the user needed to implement the user.Info interface.",
					Properties: map[string]spec.Schema{
						"username": {
							SchemaProps: spec.SchemaProps{
								Description: "The name that uniquely identifies this user among all active users.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"uid": {
							SchemaProps: spec.SchemaProps{
								Description: "A unique value that identifies this user across time. If this user is deleted and another user by the same name is added, they will have different UIDs.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"groups": {
							SchemaProps: spec.SchemaProps{
								Description: "The names of groups this user is a part of.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"extra": {
							SchemaProps: spec.SchemaProps{
								Description: "Any additional information provided by the authenticator.",
								Type:        []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type: []string{"array"},
											Items: &spec.SchemaOrArray{
												Schema: &spec.Schema{
													SchemaProps: spec.SchemaProps{
														Type:   []string{"string"},
														Format: "",
													},
												},
											},
										},
									},
								},
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/authentication/v1beta1.TokenReview": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "TokenReview attempts to authenticate a token to a known user. Note: TokenReview requests may be cached by the webhook token authenticator plugin in the kube-apiserver.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "Spec holds information about the request being evaluated",
								Ref:         ref("k8s.io/api/authentication/v1beta1.TokenReviewSpec"),
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Status is filled in by the server and indicates whether the request can be authenticated.",
								Ref:         ref("k8s.io/api/authentication/v1beta1.TokenReviewStatus"),
							},
						},
					},
					Required: []string{"spec"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/authentication/v1beta1.TokenReviewSpec", "k8s.io/api/authentication/v1beta1.TokenReviewStatus", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/authentication/v1beta1.TokenReviewSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "TokenReviewSpec is a description of the token authentication request.",
					Properties: map[string]spec.Schema{
						"token": {
							SchemaProps: spec.SchemaProps{
								Description: "Token is the opaque bearer token.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/authentication/v1beta1.TokenReviewStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "TokenReviewStatus is the result of the token authentication request.",
					Properties: map[string]spec.Schema{
						"authenticated": {
							SchemaProps: spec.SchemaProps{
								Description: "Authenticated indicates that the token was associated with a known user.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"user": {
							SchemaProps: spec.SchemaProps{
								Description: "User is the UserInfo associated with the provided token.",
								Ref:         ref("k8s.io/api/authentication/v1beta1.UserInfo"),
							},
						},
						"error": {
							SchemaProps: spec.SchemaProps{
								Description: "Error indicates that the token couldn't be checked",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/authentication/v1beta1.UserInfo"},
		},
		"k8s.io/api/authentication/v1beta1.UserInfo": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "UserInfo holds the information about the user needed to implement the user.Info interface.",
					Properties: map[string]spec.Schema{
						"username": {
							SchemaProps: spec.SchemaProps{
								Description: "The name that uniquely identifies this user among all active users.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"uid": {
							SchemaProps: spec.SchemaProps{
								Description: "A unique value that identifies this user across time. If this user is deleted and another user by the same name is added, they will have different UIDs.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"groups": {
							SchemaProps: spec.SchemaProps{
								Description: "The names of groups this user is a part of.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"extra": {
							SchemaProps: spec.SchemaProps{
								Description: "Any additional information provided by the authenticator.",
								Type:        []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type: []string{"array"},
											Items: &spec.SchemaOrArray{
												Schema: &spec.Schema{
													SchemaProps: spec.SchemaProps{
														Type:   []string{"string"},
														Format: "",
													},
												},
											},
										},
									},
								},
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/authorization/v1.LocalSubjectAccessReview": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "LocalSubjectAccessReview checks whether or not a user or group can perform an action in a given namespace. Having a namespace scoped resource makes it much easier to grant namespace scoped policy that includes permissions checking.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "Spec holds information about the request being evaluated.  spec.namespace must be equal to the namespace you made the request against.  If empty, it is defaulted.",
								Ref:         ref("k8s.io/api/authorization/v1.SubjectAccessReviewSpec"),
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Status is filled in by the server and indicates whether the request is allowed or not",
								Ref:         ref("k8s.io/api/authorization/v1.SubjectAccessReviewStatus"),
							},
						},
					},
					Required: []string{"spec"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/authorization/v1.SubjectAccessReviewSpec", "k8s.io/api/authorization/v1.SubjectAccessReviewStatus", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/authorization/v1.NonResourceAttributes": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "NonResourceAttributes includes the authorization attributes available for non-resource requests to the Authorizer interface",
					Properties: map[string]spec.Schema{
						"path": {
							SchemaProps: spec.SchemaProps{
								Description: "Path is the URL path of the request",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"verb": {
							SchemaProps: spec.SchemaProps{
								Description: "Verb is the standard HTTP verb",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/authorization/v1.NonResourceRule": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "NonResourceRule holds information that describes a rule for the non-resource",
					Properties: map[string]spec.Schema{
						"verbs": {
							SchemaProps: spec.SchemaProps{
								Description: "Verb is a list of kubernetes non-resource API verbs, like: get, post, put, delete, patch, head, options.  \"*\" means all.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"nonResourceURLs": {
							SchemaProps: spec.SchemaProps{
								Description: "NonResourceURLs is a set of partial urls that a user should have access to.  *s are allowed, but only as the full, final step in the path.  \"*\" means all.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
					},
					Required: []string{"verbs"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/authorization/v1.ResourceAttributes": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ResourceAttributes includes the authorization attributes available for resource requests to the Authorizer interface",
					Properties: map[string]spec.Schema{
						"namespace": {
							SchemaProps: spec.SchemaProps{
								Description: "Namespace is the namespace of the action being requested.  Currently, there is no distinction between no namespace and all namespaces \"\" (empty) is defaulted for LocalSubjectAccessReviews \"\" (empty) is empty for cluster-scoped resources \"\" (empty) means \"all\" for namespace scoped resources from a SubjectAccessReview or SelfSubjectAccessReview",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"verb": {
							SchemaProps: spec.SchemaProps{
								Description: "Verb is a kubernetes resource API verb, like: get, list, watch, create, update, delete, proxy.  \"*\" means all.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"group": {
							SchemaProps: spec.SchemaProps{
								Description: "Group is the API Group of the Resource.  \"*\" means all.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"version": {
							SchemaProps: spec.SchemaProps{
								Description: "Version is the API Version of the Resource.  \"*\" means all.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"resource": {
							SchemaProps: spec.SchemaProps{
								Description: "Resource is one of the existing resource types.  \"*\" means all.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"subresource": {
							SchemaProps: spec.SchemaProps{
								Description: "Subresource is one of the existing resource types.  \"\" means none.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "Name is the name of the resource being requested for a \"get\" or deleted for a \"delete\". \"\" (empty) means all.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/authorization/v1.ResourceRule": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ResourceRule is the list of actions the subject is allowed to perform on resources. The list ordering isn't significant, may contain duplicates, and possibly be incomplete.",
					Properties: map[string]spec.Schema{
						"verbs": {
							SchemaProps: spec.SchemaProps{
								Description: "Verb is a list of kubernetes resource API verbs, like: get, list, watch, create, update, delete, proxy.  \"*\" means all.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"apiGroups": {
							SchemaProps: spec.SchemaProps{
								Description: "APIGroups is the name of the APIGroup that contains the resources.  If multiple API groups are specified, any action requested against one of the enumerated resources in any API group will be allowed.  \"*\" means all.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"resources": {
							SchemaProps: spec.SchemaProps{
								Description: "Resources is a list of resources this rule applies to.  \"*\" means all in the specified apiGroups.\n \"*/foo\" represents the subresource 'foo' for all resources in the specified apiGroups.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"resourceNames": {
							SchemaProps: spec.SchemaProps{
								Description: "ResourceNames is an optional white list of names that the rule applies to.  An empty set means that everything is allowed.  \"*\" means all.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
					},
					Required: []string{"verbs"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/authorization/v1.SelfSubjectAccessReview": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "SelfSubjectAccessReview checks whether or the current user can perform an action.  Not filling in a spec.namespace means \"in all namespaces\".  Self is a special case, because users should always be able to check whether they can perform an action",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "Spec holds information about the request being evaluated.  user and groups must be empty",
								Ref:         ref("k8s.io/api/authorization/v1.SelfSubjectAccessReviewSpec"),
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Status is filled in by the server and indicates whether the request is allowed or not",
								Ref:         ref("k8s.io/api/authorization/v1.SubjectAccessReviewStatus"),
							},
						},
					},
					Required: []string{"spec"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/authorization/v1.SelfSubjectAccessReviewSpec", "k8s.io/api/authorization/v1.SubjectAccessReviewStatus", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/authorization/v1.SelfSubjectAccessReviewSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "SelfSubjectAccessReviewSpec is a description of the access request.  Exactly one of ResourceAuthorizationAttributes and NonResourceAuthorizationAttributes must be set",
					Properties: map[string]spec.Schema{
						"resourceAttributes": {
							SchemaProps: spec.SchemaProps{
								Description: "ResourceAuthorizationAttributes describes information for a resource access request",
								Ref:         ref("k8s.io/api/authorization/v1.ResourceAttributes"),
							},
						},
						"nonResourceAttributes": {
							SchemaProps: spec.SchemaProps{
								Description: "NonResourceAttributes describes information for a non-resource access request",
								Ref:         ref("k8s.io/api/authorization/v1.NonResourceAttributes"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/authorization/v1.NonResourceAttributes", "k8s.io/api/authorization/v1.ResourceAttributes"},
		},
		"k8s.io/api/authorization/v1.SelfSubjectRulesReview": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "SelfSubjectRulesReview enumerates the set of actions the current user can perform within a namespace. The returned list of actions may be incomplete depending on the server's authorization mode, and any errors experienced during the evaluation. SelfSubjectRulesReview should be used by UIs to show/hide actions, or to quickly let an end user reason about their permissions. It should NOT Be used by external systems to drive authorization decisions as this raises confused deputy, cache lifetime/revocation, and correctness concerns. SubjectAccessReview, and LocalAccessReview are the correct way to defer authorization decisions to the API server.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "Spec holds information about the request being evaluated.",
								Ref:         ref("k8s.io/api/authorization/v1.SelfSubjectRulesReviewSpec"),
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Status is filled in by the server and indicates the set of actions a user can perform.",
								Ref:         ref("k8s.io/api/authorization/v1.SubjectRulesReviewStatus"),
							},
						},
					},
					Required: []string{"spec"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/authorization/v1.SelfSubjectRulesReviewSpec", "k8s.io/api/authorization/v1.SubjectRulesReviewStatus", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/authorization/v1.SelfSubjectRulesReviewSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Properties: map[string]spec.Schema{
						"namespace": {
							SchemaProps: spec.SchemaProps{
								Description: "Namespace to evaluate rules for. Required.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/authorization/v1.SubjectAccessReview": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "SubjectAccessReview checks whether or not a user or group can perform an action.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "Spec holds information about the request being evaluated",
								Ref:         ref("k8s.io/api/authorization/v1.SubjectAccessReviewSpec"),
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Status is filled in by the server and indicates whether the request is allowed or not",
								Ref:         ref("k8s.io/api/authorization/v1.SubjectAccessReviewStatus"),
							},
						},
					},
					Required: []string{"spec"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/authorization/v1.SubjectAccessReviewSpec", "k8s.io/api/authorization/v1.SubjectAccessReviewStatus", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/authorization/v1.SubjectAccessReviewSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "SubjectAccessReviewSpec is a description of the access request.  Exactly one of ResourceAuthorizationAttributes and NonResourceAuthorizationAttributes must be set",
					Properties: map[string]spec.Schema{
						"resourceAttributes": {
							SchemaProps: spec.SchemaProps{
								Description: "ResourceAuthorizationAttributes describes information for a resource access request",
								Ref:         ref("k8s.io/api/authorization/v1.ResourceAttributes"),
							},
						},
						"nonResourceAttributes": {
							SchemaProps: spec.SchemaProps{
								Description: "NonResourceAttributes describes information for a non-resource access request",
								Ref:         ref("k8s.io/api/authorization/v1.NonResourceAttributes"),
							},
						},
						"user": {
							SchemaProps: spec.SchemaProps{
								Description: "User is the user you're testing for. If you specify \"User\" but not \"Groups\", then is it interpreted as \"What if User were not a member of any groups",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"groups": {
							SchemaProps: spec.SchemaProps{
								Description: "Groups is the groups you're testing for.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"extra": {
							SchemaProps: spec.SchemaProps{
								Description: "Extra corresponds to the user.Info.GetExtra() method from the authenticator.  Since that is input to the authorizer it needs a reflection here.",
								Type:        []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type: []string{"array"},
											Items: &spec.SchemaOrArray{
												Schema: &spec.Schema{
													SchemaProps: spec.SchemaProps{
														Type:   []string{"string"},
														Format: "",
													},
												},
											},
										},
									},
								},
							},
						},
						"uid": {
							SchemaProps: spec.SchemaProps{
								Description: "UID information about the requesting user.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/authorization/v1.NonResourceAttributes", "k8s.io/api/authorization/v1.ResourceAttributes"},
		},
		"k8s.io/api/authorization/v1.SubjectAccessReviewStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "SubjectAccessReviewStatus",
					Properties: map[string]spec.Schema{
						"allowed": {
							SchemaProps: spec.SchemaProps{
								Description: "Allowed is required. True if the action would be allowed, false otherwise.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"denied": {
							SchemaProps: spec.SchemaProps{
								Description: "Denied is optional. True if the action would be denied, otherwise false. If both allowed is false and denied is false, then the authorizer has no opinion on whether to authorize the action. Denied may not be true if Allowed is true.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"reason": {
							SchemaProps: spec.SchemaProps{
								Description: "Reason is optional.  It indicates why a request was allowed or denied.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"evaluationError": {
							SchemaProps: spec.SchemaProps{
								Description: "EvaluationError is an indication that some error occurred during the authorization check. It is entirely possible to get an error and be able to continue determine authorization status in spite of it. For instance, RBAC can be missing a role, but enough roles are still present and bound to reason about the request.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"allowed"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/authorization/v1.SubjectRulesReviewStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "SubjectRulesReviewStatus contains the result of a rules check. This check can be incomplete depending on the set of authorizers the server is configured with and any errors experienced during evaluation. Because authorization rules are additive, if a rule appears in a list it's safe to assume the subject has that permission, even if that list is incomplete.",
					Properties: map[string]spec.Schema{
						"resourceRules": {
							SchemaProps: spec.SchemaProps{
								Description: "ResourceRules is the list of actions the subject is allowed to perform on resources. The list ordering isn't significant, may contain duplicates, and possibly be incomplete.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/authorization/v1.ResourceRule"),
										},
									},
								},
							},
						},
						"nonResourceRules": {
							SchemaProps: spec.SchemaProps{
								Description: "NonResourceRules is the list of actions the subject is allowed to perform on non-resources. The list ordering isn't significant, may contain duplicates, and possibly be incomplete.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/authorization/v1.NonResourceRule"),
										},
									},
								},
							},
						},
						"incomplete": {
							SchemaProps: spec.SchemaProps{
								Description: "Incomplete is true when the rules returned by this call are incomplete. This is most commonly encountered when an authorizer, such as an external authorizer, doesn't support rules evaluation.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"evaluationError": {
							SchemaProps: spec.SchemaProps{
								Description: "EvaluationError can appear in combination with Rules. It indicates an error occurred during rule evaluation, such as an authorizer that doesn't support rule evaluation, and that ResourceRules and/or NonResourceRules may be incomplete.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"resourceRules", "nonResourceRules", "incomplete"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/authorization/v1.NonResourceRule", "k8s.io/api/authorization/v1.ResourceRule"},
		},
		"k8s.io/api/authorization/v1beta1.LocalSubjectAccessReview": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "LocalSubjectAccessReview checks whether or not a user or group can perform an action in a given namespace. Having a namespace scoped resource makes it much easier to grant namespace scoped policy that includes permissions checking.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "Spec holds information about the request being evaluated.  spec.namespace must be equal to the namespace you made the request against.  If empty, it is defaulted.",
								Ref:         ref("k8s.io/api/authorization/v1beta1.SubjectAccessReviewSpec"),
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Status is filled in by the server and indicates whether the request is allowed or not",
								Ref:         ref("k8s.io/api/authorization/v1beta1.SubjectAccessReviewStatus"),
							},
						},
					},
					Required: []string{"spec"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/authorization/v1beta1.SubjectAccessReviewSpec", "k8s.io/api/authorization/v1beta1.SubjectAccessReviewStatus", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/authorization/v1beta1.NonResourceAttributes": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "NonResourceAttributes includes the authorization attributes available for non-resource requests to the Authorizer interface",
					Properties: map[string]spec.Schema{
						"path": {
							SchemaProps: spec.SchemaProps{
								Description: "Path is the URL path of the request",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"verb": {
							SchemaProps: spec.SchemaProps{
								Description: "Verb is the standard HTTP verb",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/authorization/v1beta1.NonResourceRule": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "NonResourceRule holds information that describes a rule for the non-resource",
					Properties: map[string]spec.Schema{
						"verbs": {
							SchemaProps: spec.SchemaProps{
								Description: "Verb is a list of kubernetes non-resource API verbs, like: get, post, put, delete, patch, head, options.  \"*\" means all.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"nonResourceURLs": {
							SchemaProps: spec.SchemaProps{
								Description: "NonResourceURLs is a set of partial urls that a user should have access to.  *s are allowed, but only as the full, final step in the path.  \"*\" means all.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
					},
					Required: []string{"verbs"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/authorization/v1beta1.ResourceAttributes": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ResourceAttributes includes the authorization attributes available for resource requests to the Authorizer interface",
					Properties: map[string]spec.Schema{
						"namespace": {
							SchemaProps: spec.SchemaProps{
								Description: "Namespace is the namespace of the action being requested.  Currently, there is no distinction between no namespace and all namespaces \"\" (empty) is defaulted for LocalSubjectAccessReviews \"\" (empty) is empty for cluster-scoped resources \"\" (empty) means \"all\" for namespace scoped resources from a SubjectAccessReview or SelfSubjectAccessReview",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"verb": {
							SchemaProps: spec.SchemaProps{
								Description: "Verb is a kubernetes resource API verb, like: get, list, watch, create, update, delete, proxy.  \"*\" means all.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"group": {
							SchemaProps: spec.SchemaProps{
								Description: "Group is the API Group of the Resource.  \"*\" means all.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"version": {
							SchemaProps: spec.SchemaProps{
								Description: "Version is the API Version of the Resource.  \"*\" means all.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"resource": {
							SchemaProps: spec.SchemaProps{
								Description: "Resource is one of the existing resource types.  \"*\" means all.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"subresource": {
							SchemaProps: spec.SchemaProps{
								Description: "Subresource is one of the existing resource types.  \"\" means none.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "Name is the name of the resource being requested for a \"get\" or deleted for a \"delete\". \"\" (empty) means all.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/authorization/v1beta1.ResourceRule": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ResourceRule is the list of actions the subject is allowed to perform on resources. The list ordering isn't significant, may contain duplicates, and possibly be incomplete.",
					Properties: map[string]spec.Schema{
						"verbs": {
							SchemaProps: spec.SchemaProps{
								Description: "Verb is a list of kubernetes resource API verbs, like: get, list, watch, create, update, delete, proxy.  \"*\" means all.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"apiGroups": {
							SchemaProps: spec.SchemaProps{
								Description: "APIGroups is the name of the APIGroup that contains the resources.  If multiple API groups are specified, any action requested against one of the enumerated resources in any API group will be allowed.  \"*\" means all.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"resources": {
							SchemaProps: spec.SchemaProps{
								Description: "Resources is a list of resources this rule applies to.  \"*\" means all in the specified apiGroups.\n \"*/foo\" represents the subresource 'foo' for all resources in the specified apiGroups.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"resourceNames": {
							SchemaProps: spec.SchemaProps{
								Description: "ResourceNames is an optional white list of names that the rule applies to.  An empty set means that everything is allowed.  \"*\" means all.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
					},
					Required: []string{"verbs"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/authorization/v1beta1.SelfSubjectAccessReview": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "SelfSubjectAccessReview checks whether or the current user can perform an action.  Not filling in a spec.namespace means \"in all namespaces\".  Self is a special case, because users should always be able to check whether they can perform an action",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "Spec holds information about the request being evaluated.  user and groups must be empty",
								Ref:         ref("k8s.io/api/authorization/v1beta1.SelfSubjectAccessReviewSpec"),
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Status is filled in by the server and indicates whether the request is allowed or not",
								Ref:         ref("k8s.io/api/authorization/v1beta1.SubjectAccessReviewStatus"),
							},
						},
					},
					Required: []string{"spec"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/authorization/v1beta1.SelfSubjectAccessReviewSpec", "k8s.io/api/authorization/v1beta1.SubjectAccessReviewStatus", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/authorization/v1beta1.SelfSubjectAccessReviewSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "SelfSubjectAccessReviewSpec is a description of the access request.  Exactly one of ResourceAuthorizationAttributes and NonResourceAuthorizationAttributes must be set",
					Properties: map[string]spec.Schema{
						"resourceAttributes": {
							SchemaProps: spec.SchemaProps{
								Description: "ResourceAuthorizationAttributes describes information for a resource access request",
								Ref:         ref("k8s.io/api/authorization/v1beta1.ResourceAttributes"),
							},
						},
						"nonResourceAttributes": {
							SchemaProps: spec.SchemaProps{
								Description: "NonResourceAttributes describes information for a non-resource access request",
								Ref:         ref("k8s.io/api/authorization/v1beta1.NonResourceAttributes"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/authorization/v1beta1.NonResourceAttributes", "k8s.io/api/authorization/v1beta1.ResourceAttributes"},
		},
		"k8s.io/api/authorization/v1beta1.SelfSubjectRulesReview": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "SelfSubjectRulesReview enumerates the set of actions the current user can perform within a namespace. The returned list of actions may be incomplete depending on the server's authorization mode, and any errors experienced during the evaluation. SelfSubjectRulesReview should be used by UIs to show/hide actions, or to quickly let an end user reason about their permissions. It should NOT Be used by external systems to drive authorization decisions as this raises confused deputy, cache lifetime/revocation, and correctness concerns. SubjectAccessReview, and LocalAccessReview are the correct way to defer authorization decisions to the API server.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "Spec holds information about the request being evaluated.",
								Ref:         ref("k8s.io/api/authorization/v1beta1.SelfSubjectRulesReviewSpec"),
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Status is filled in by the server and indicates the set of actions a user can perform.",
								Ref:         ref("k8s.io/api/authorization/v1beta1.SubjectRulesReviewStatus"),
							},
						},
					},
					Required: []string{"spec"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/authorization/v1beta1.SelfSubjectRulesReviewSpec", "k8s.io/api/authorization/v1beta1.SubjectRulesReviewStatus", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/authorization/v1beta1.SelfSubjectRulesReviewSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Properties: map[string]spec.Schema{
						"namespace": {
							SchemaProps: spec.SchemaProps{
								Description: "Namespace to evaluate rules for. Required.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/authorization/v1beta1.SubjectAccessReview": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "SubjectAccessReview checks whether or not a user or group can perform an action.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "Spec holds information about the request being evaluated",
								Ref:         ref("k8s.io/api/authorization/v1beta1.SubjectAccessReviewSpec"),
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Status is filled in by the server and indicates whether the request is allowed or not",
								Ref:         ref("k8s.io/api/authorization/v1beta1.SubjectAccessReviewStatus"),
							},
						},
					},
					Required: []string{"spec"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/authorization/v1beta1.SubjectAccessReviewSpec", "k8s.io/api/authorization/v1beta1.SubjectAccessReviewStatus", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/authorization/v1beta1.SubjectAccessReviewSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "SubjectAccessReviewSpec is a description of the access request.  Exactly one of ResourceAuthorizationAttributes and NonResourceAuthorizationAttributes must be set",
					Properties: map[string]spec.Schema{
						"resourceAttributes": {
							SchemaProps: spec.SchemaProps{
								Description: "ResourceAuthorizationAttributes describes information for a resource access request",
								Ref:         ref("k8s.io/api/authorization/v1beta1.ResourceAttributes"),
							},
						},
						"nonResourceAttributes": {
							SchemaProps: spec.SchemaProps{
								Description: "NonResourceAttributes describes information for a non-resource access request",
								Ref:         ref("k8s.io/api/authorization/v1beta1.NonResourceAttributes"),
							},
						},
						"user": {
							SchemaProps: spec.SchemaProps{
								Description: "User is the user you're testing for. If you specify \"User\" but not \"Group\", then is it interpreted as \"What if User were not a member of any groups",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"group": {
							SchemaProps: spec.SchemaProps{
								Description: "Groups is the groups you're testing for.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"extra": {
							SchemaProps: spec.SchemaProps{
								Description: "Extra corresponds to the user.Info.GetExtra() method from the authenticator.  Since that is input to the authorizer it needs a reflection here.",
								Type:        []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type: []string{"array"},
											Items: &spec.SchemaOrArray{
												Schema: &spec.Schema{
													SchemaProps: spec.SchemaProps{
														Type:   []string{"string"},
														Format: "",
													},
												},
											},
										},
									},
								},
							},
						},
						"uid": {
							SchemaProps: spec.SchemaProps{
								Description: "UID information about the requesting user.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/authorization/v1beta1.NonResourceAttributes", "k8s.io/api/authorization/v1beta1.ResourceAttributes"},
		},
		"k8s.io/api/authorization/v1beta1.SubjectAccessReviewStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "SubjectAccessReviewStatus",
					Properties: map[string]spec.Schema{
						"allowed": {
							SchemaProps: spec.SchemaProps{
								Description: "Allowed is required. True if the action would be allowed, false otherwise.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"denied": {
							SchemaProps: spec.SchemaProps{
								Description: "Denied is optional. True if the action would be denied, otherwise false. If both allowed is false and denied is false, then the authorizer has no opinion on whether to authorize the action. Denied may not be true if Allowed is true.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"reason": {
							SchemaProps: spec.SchemaProps{
								Description: "Reason is optional.  It indicates why a request was allowed or denied.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"evaluationError": {
							SchemaProps: spec.SchemaProps{
								Description: "EvaluationError is an indication that some error occurred during the authorization check. It is entirely possible to get an error and be able to continue determine authorization status in spite of it. For instance, RBAC can be missing a role, but enough roles are still present and bound to reason about the request.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"allowed"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/authorization/v1beta1.SubjectRulesReviewStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "SubjectRulesReviewStatus contains the result of a rules check. This check can be incomplete depending on the set of authorizers the server is configured with and any errors experienced during evaluation. Because authorization rules are additive, if a rule appears in a list it's safe to assume the subject has that permission, even if that list is incomplete.",
					Properties: map[string]spec.Schema{
						"resourceRules": {
							SchemaProps: spec.SchemaProps{
								Description: "ResourceRules is the list of actions the subject is allowed to perform on resources. The list ordering isn't significant, may contain duplicates, and possibly be incomplete.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/authorization/v1beta1.ResourceRule"),
										},
									},
								},
							},
						},
						"nonResourceRules": {
							SchemaProps: spec.SchemaProps{
								Description: "NonResourceRules is the list of actions the subject is allowed to perform on non-resources. The list ordering isn't significant, may contain duplicates, and possibly be incomplete.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/authorization/v1beta1.NonResourceRule"),
										},
									},
								},
							},
						},
						"incomplete": {
							SchemaProps: spec.SchemaProps{
								Description: "Incomplete is true when the rules returned by this call are incomplete. This is most commonly encountered when an authorizer, such as an external authorizer, doesn't support rules evaluation.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"evaluationError": {
							SchemaProps: spec.SchemaProps{
								Description: "EvaluationError can appear in combination with Rules. It indicates an error occurred during rule evaluation, such as an authorizer that doesn't support rule evaluation, and that ResourceRules and/or NonResourceRules may be incomplete.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"resourceRules", "nonResourceRules", "incomplete"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/authorization/v1beta1.NonResourceRule", "k8s.io/api/authorization/v1beta1.ResourceRule"},
		},
		"k8s.io/api/autoscaling/v1.CrossVersionObjectReference": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "CrossVersionObjectReference contains enough information to let you identify the referred resource.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind of the referent; More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds\"",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "Name of the referent; More info: http://kubernetes.io/docs/user-guide/identifiers#names",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "API version of the referent",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"kind", "name"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/autoscaling/v1.HorizontalPodAutoscaler": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "configuration of a horizontal pod autoscaler.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "behaviour of autoscaler. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status.",
								Ref:         ref("k8s.io/api/autoscaling/v1.HorizontalPodAutoscalerSpec"),
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "current information about the autoscaler.",
								Ref:         ref("k8s.io/api/autoscaling/v1.HorizontalPodAutoscalerStatus"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/autoscaling/v1.HorizontalPodAutoscalerSpec", "k8s.io/api/autoscaling/v1.HorizontalPodAutoscalerStatus", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/autoscaling/v1.HorizontalPodAutoscalerCondition": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "HorizontalPodAutoscalerCondition describes the state of a HorizontalPodAutoscaler at a certain point.",
					Properties: map[string]spec.Schema{
						"type": {
							SchemaProps: spec.SchemaProps{
								Description: "type describes the current condition",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "status is the status of the condition (True, False, Unknown)",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"lastTransitionTime": {
							SchemaProps: spec.SchemaProps{
								Description: "lastTransitionTime is the last time the condition transitioned from one status to another",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"reason": {
							SchemaProps: spec.SchemaProps{
								Description: "reason is the reason for the condition's last transition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"message": {
							SchemaProps: spec.SchemaProps{
								Description: "message is a human-readable explanation containing details about the transition",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"type", "status"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.Time"},
		},
		"k8s.io/api/autoscaling/v1.HorizontalPodAutoscalerList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "list of horizontal pod autoscaler objects.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard list metadata.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "list of horizontal pod autoscaler objects.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/autoscaling/v1.HorizontalPodAutoscaler"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/autoscaling/v1.HorizontalPodAutoscaler", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/autoscaling/v1.HorizontalPodAutoscalerSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "specification of a horizontal pod autoscaler.",
					Properties: map[string]spec.Schema{
						"scaleTargetRef": {
							SchemaProps: spec.SchemaProps{
								Description: "reference to scaled resource; horizontal pod autoscaler will learn the current resource consumption and will set the desired number of pods by using its Scale subresource.",
								Ref:         ref("k8s.io/api/autoscaling/v1.CrossVersionObjectReference"),
							},
						},
						"minReplicas": {
							SchemaProps: spec.SchemaProps{
								Description: "lower limit for the number of pods that can be set by the autoscaler, default 1.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"maxReplicas": {
							SchemaProps: spec.SchemaProps{
								Description: "upper limit for the number of pods that can be set by the autoscaler; cannot be smaller than MinReplicas.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"targetCPUUtilizationPercentage": {
							SchemaProps: spec.SchemaProps{
								Description: "target average CPU utilization (represented as a percentage of requested CPU) over all the pods; if not specified the default autoscaling policy will be used.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
					},
					Required: []string{"scaleTargetRef", "maxReplicas"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/autoscaling/v1.CrossVersionObjectReference"},
		},
		"k8s.io/api/autoscaling/v1.HorizontalPodAutoscalerStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "current status of a horizontal pod autoscaler",
					Properties: map[string]spec.Schema{
						"observedGeneration": {
							SchemaProps: spec.SchemaProps{
								Description: "most recent generation observed by this autoscaler.",
								Type:        []string{"integer"},
								Format:      "int64",
							},
						},
						"lastScaleTime": {
							SchemaProps: spec.SchemaProps{
								Description: "last time the HorizontalPodAutoscaler scaled the number of pods; used by the autoscaler to control how often the number of pods is changed.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"currentReplicas": {
							SchemaProps: spec.SchemaProps{
								Description: "current number of replicas of pods managed by this autoscaler.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"desiredReplicas": {
							SchemaProps: spec.SchemaProps{
								Description: "desired number of replicas of pods managed by this autoscaler.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"currentCPUUtilizationPercentage": {
							SchemaProps: spec.SchemaProps{
								Description: "current average CPU utilization over all pods, represented as a percentage of requested CPU, e.g. 70 means that an average pod is using now 70% of its requested CPU.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
					},
					Required: []string{"currentReplicas", "desiredReplicas"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.Time"},
		},
		"k8s.io/api/autoscaling/v1.MetricSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "MetricSpec specifies how to scale based on a single metric (only `type` and one other matching field should be set at once).",
					Properties: map[string]spec.Schema{
						"type": {
							SchemaProps: spec.SchemaProps{
								Description: "type is the type of metric source.  It should match one of the fields below.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"object": {
							SchemaProps: spec.SchemaProps{
								Description: "object refers to a metric describing a single kubernetes object (for example, hits-per-second on an Ingress object).",
								Ref:         ref("k8s.io/api/autoscaling/v1.ObjectMetricSource"),
							},
						},
						"pods": {
							SchemaProps: spec.SchemaProps{
								Description: "pods refers to a metric describing each pod in the current scale target (for example, transactions-processed-per-second).  The values will be averaged together before being compared to the target value.",
								Ref:         ref("k8s.io/api/autoscaling/v1.PodsMetricSource"),
							},
						},
						"resource": {
							SchemaProps: spec.SchemaProps{
								Description: "resource refers to a resource metric (such as those specified in requests and limits) known to Kubernetes describing each pod in the current scale target (e.g. CPU or memory). Such metrics are built in to Kubernetes, and have special scaling options on top of those available to normal per-pod metrics using the \"pods\" source.",
								Ref:         ref("k8s.io/api/autoscaling/v1.ResourceMetricSource"),
							},
						},
					},
					Required: []string{"type"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/autoscaling/v1.ObjectMetricSource", "k8s.io/api/autoscaling/v1.PodsMetricSource", "k8s.io/api/autoscaling/v1.ResourceMetricSource"},
		},
		"k8s.io/api/autoscaling/v1.MetricStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "MetricStatus describes the last-read state of a single metric.",
					Properties: map[string]spec.Schema{
						"type": {
							SchemaProps: spec.SchemaProps{
								Description: "type is the type of metric source.  It will match one of the fields below.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"object": {
							SchemaProps: spec.SchemaProps{
								Description: "object refers to a metric describing a single kubernetes object (for example, hits-per-second on an Ingress object).",
								Ref:         ref("k8s.io/api/autoscaling/v1.ObjectMetricStatus"),
							},
						},
						"pods": {
							SchemaProps: spec.SchemaProps{
								Description: "pods refers to a metric describing each pod in the current scale target (for example, transactions-processed-per-second).  The values will be averaged together before being compared to the target value.",
								Ref:         ref("k8s.io/api/autoscaling/v1.PodsMetricStatus"),
							},
						},
						"resource": {
							SchemaProps: spec.SchemaProps{
								Description: "resource refers to a resource metric (such as those specified in requests and limits) known to Kubernetes describing each pod in the current scale target (e.g. CPU or memory). Such metrics are built in to Kubernetes, and have special scaling options on top of those available to normal per-pod metrics using the \"pods\" source.",
								Ref:         ref("k8s.io/api/autoscaling/v1.ResourceMetricStatus"),
							},
						},
					},
					Required: []string{"type"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/autoscaling/v1.ObjectMetricStatus", "k8s.io/api/autoscaling/v1.PodsMetricStatus", "k8s.io/api/autoscaling/v1.ResourceMetricStatus"},
		},
		"k8s.io/api/autoscaling/v1.ObjectMetricSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ObjectMetricSource indicates how to scale on a metric describing a kubernetes object (for example, hits-per-second on an Ingress object).",
					Properties: map[string]spec.Schema{
						"target": {
							SchemaProps: spec.SchemaProps{
								Description: "target is the described Kubernetes object.",
								Ref:         ref("k8s.io/api/autoscaling/v1.CrossVersionObjectReference"),
							},
						},
						"metricName": {
							SchemaProps: spec.SchemaProps{
								Description: "metricName is the name of the metric in question.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"targetValue": {
							SchemaProps: spec.SchemaProps{
								Description: "targetValue is the target value of the metric (as a quantity).",
								Ref:         ref("k8s.io/apimachinery/pkg/api/resource.Quantity"),
							},
						},
					},
					Required: []string{"target", "metricName", "targetValue"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/autoscaling/v1.CrossVersionObjectReference", "k8s.io/apimachinery/pkg/api/resource.Quantity"},
		},
		"k8s.io/api/autoscaling/v1.ObjectMetricStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ObjectMetricStatus indicates the current value of a metric describing a kubernetes object (for example, hits-per-second on an Ingress object).",
					Properties: map[string]spec.Schema{
						"target": {
							SchemaProps: spec.SchemaProps{
								Description: "target is the described Kubernetes object.",
								Ref:         ref("k8s.io/api/autoscaling/v1.CrossVersionObjectReference"),
							},
						},
						"metricName": {
							SchemaProps: spec.SchemaProps{
								Description: "metricName is the name of the metric in question.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"currentValue": {
							SchemaProps: spec.SchemaProps{
								Description: "currentValue is the current value of the metric (as a quantity).",
								Ref:         ref("k8s.io/apimachinery/pkg/api/resource.Quantity"),
							},
						},
					},
					Required: []string{"target", "metricName", "currentValue"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/autoscaling/v1.CrossVersionObjectReference", "k8s.io/apimachinery/pkg/api/resource.Quantity"},
		},
		"k8s.io/api/autoscaling/v1.PodsMetricSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PodsMetricSource indicates how to scale on a metric describing each pod in the current scale target (for example, transactions-processed-per-second). The values will be averaged together before being compared to the target value.",
					Properties: map[string]spec.Schema{
						"metricName": {
							SchemaProps: spec.SchemaProps{
								Description: "metricName is the name of the metric in question",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"targetAverageValue": {
							SchemaProps: spec.SchemaProps{
								Description: "targetAverageValue is the target value of the average of the metric across all relevant pods (as a quantity)",
								Ref:         ref("k8s.io/apimachinery/pkg/api/resource.Quantity"),
							},
						},
					},
					Required: []string{"metricName", "targetAverageValue"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/api/resource.Quantity"},
		},
		"k8s.io/api/autoscaling/v1.PodsMetricStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PodsMetricStatus indicates the current value of a metric describing each pod in the current scale target (for example, transactions-processed-per-second).",
					Properties: map[string]spec.Schema{
						"metricName": {
							SchemaProps: spec.SchemaProps{
								Description: "metricName is the name of the metric in question",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"currentAverageValue": {
							SchemaProps: spec.SchemaProps{
								Description: "currentAverageValue is the current value of the average of the metric across all relevant pods (as a quantity)",
								Ref:         ref("k8s.io/apimachinery/pkg/api/resource.Quantity"),
							},
						},
					},
					Required: []string{"metricName", "currentAverageValue"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/api/resource.Quantity"},
		},
		"k8s.io/api/autoscaling/v1.ResourceMetricSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ResourceMetricSource indicates how to scale on a resource metric known to Kubernetes, as specified in requests and limits, describing each pod in the current scale target (e.g. CPU or memory).  The values will be averaged together before being compared to the target.  Such metrics are built in to Kubernetes, and have special scaling options on top of those available to normal per-pod metrics using the \"pods\" source.  Only one \"target\" type should be set.",
					Properties: map[string]spec.Schema{
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "name is the name of the resource in question.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"targetAverageUtilization": {
							SchemaProps: spec.SchemaProps{
								Description: "targetAverageUtilization is the target value of the average of the resource metric across all relevant pods, represented as a percentage of the requested value of the resource for the pods.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"targetAverageValue": {
							SchemaProps: spec.SchemaProps{
								Description: "targetAverageValue is the target value of the average of the resource metric across all relevant pods, as a raw value (instead of as a percentage of the request), similar to the \"pods\" metric source type.",
								Ref:         ref("k8s.io/apimachinery/pkg/api/resource.Quantity"),
							},
						},
					},
					Required: []string{"name"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/api/resource.Quantity"},
		},
		"k8s.io/api/autoscaling/v1.ResourceMetricStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ResourceMetricStatus indicates the current value of a resource metric known to Kubernetes, as specified in requests and limits, describing each pod in the current scale target (e.g. CPU or memory).  Such metrics are built in to Kubernetes, and have special scaling options on top of those available to normal per-pod metrics using the \"pods\" source.",
					Properties: map[string]spec.Schema{
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "name is the name of the resource in question.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"currentAverageUtilization": {
							SchemaProps: spec.SchemaProps{
								Description: "currentAverageUtilization is the current value of the average of the resource metric across all relevant pods, represented as a percentage of the requested value of the resource for the pods.  It will only be present if `targetAverageValue` was set in the corresponding metric specification.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"currentAverageValue": {
							SchemaProps: spec.SchemaProps{
								Description: "currentAverageValue is the current value of the average of the resource metric across all relevant pods, as a raw value (instead of as a percentage of the request), similar to the \"pods\" metric source type. It will always be set, regardless of the corresponding metric specification.",
								Ref:         ref("k8s.io/apimachinery/pkg/api/resource.Quantity"),
							},
						},
					},
					Required: []string{"name", "currentAverageValue"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/api/resource.Quantity"},
		},
		"k8s.io/api/autoscaling/v1.Scale": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Scale represents a scaling request for a resource.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object metadata; More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "defines the behavior of the scale. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status.",
								Ref:         ref("k8s.io/api/autoscaling/v1.ScaleSpec"),
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "current status of the scale. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status. Read-only.",
								Ref:         ref("k8s.io/api/autoscaling/v1.ScaleStatus"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/autoscaling/v1.ScaleSpec", "k8s.io/api/autoscaling/v1.ScaleStatus", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/autoscaling/v1.ScaleSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ScaleSpec describes the attributes of a scale subresource.",
					Properties: map[string]spec.Schema{
						"replicas": {
							SchemaProps: spec.SchemaProps{
								Description: "desired number of instances for the scaled object.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/autoscaling/v1.ScaleStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ScaleStatus represents the current status of a scale subresource.",
					Properties: map[string]spec.Schema{
						"replicas": {
							SchemaProps: spec.SchemaProps{
								Description: "actual number of observed instances of the scaled object.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"selector": {
							SchemaProps: spec.SchemaProps{
								Description: "label query over pods that should match the replicas count. This is same as the label selector but in the string format to avoid introspection by clients. The string will be in the same format as the query-param syntax. More info about label selectors: http://kubernetes.io/docs/user-guide/labels#label-selectors",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"replicas"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/autoscaling/v2beta1.CrossVersionObjectReference": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "CrossVersionObjectReference contains enough information to let you identify the referred resource.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind of the referent; More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds\"",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "Name of the referent; More info: http://kubernetes.io/docs/user-guide/identifiers#names",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "API version of the referent",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"kind", "name"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/autoscaling/v2beta1.HorizontalPodAutoscaler": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "HorizontalPodAutoscaler is the configuration for a horizontal pod autoscaler, which automatically manages the replica count of any resource implementing the scale subresource based on the metrics specified.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "metadata is the standard object metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "spec is the specification for the behaviour of the autoscaler. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status.",
								Ref:         ref("k8s.io/api/autoscaling/v2beta1.HorizontalPodAutoscalerSpec"),
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "status is the current information about the autoscaler.",
								Ref:         ref("k8s.io/api/autoscaling/v2beta1.HorizontalPodAutoscalerStatus"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/autoscaling/v2beta1.HorizontalPodAutoscalerSpec", "k8s.io/api/autoscaling/v2beta1.HorizontalPodAutoscalerStatus", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/autoscaling/v2beta1.HorizontalPodAutoscalerCondition": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "HorizontalPodAutoscalerCondition describes the state of a HorizontalPodAutoscaler at a certain point.",
					Properties: map[string]spec.Schema{
						"type": {
							SchemaProps: spec.SchemaProps{
								Description: "type describes the current condition",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "status is the status of the condition (True, False, Unknown)",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"lastTransitionTime": {
							SchemaProps: spec.SchemaProps{
								Description: "lastTransitionTime is the last time the condition transitioned from one status to another",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"reason": {
							SchemaProps: spec.SchemaProps{
								Description: "reason is the reason for the condition's last transition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"message": {
							SchemaProps: spec.SchemaProps{
								Description: "message is a human-readable explanation containing details about the transition",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"type", "status"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.Time"},
		},
		"k8s.io/api/autoscaling/v2beta1.HorizontalPodAutoscalerList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "HorizontalPodAutoscaler is a list of horizontal pod autoscaler objects.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "metadata is the standard list metadata.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "items is the list of horizontal pod autoscaler objects.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/autoscaling/v2beta1.HorizontalPodAutoscaler"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/autoscaling/v2beta1.HorizontalPodAutoscaler", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/autoscaling/v2beta1.HorizontalPodAutoscalerSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "HorizontalPodAutoscalerSpec describes the desired functionality of the HorizontalPodAutoscaler.",
					Properties: map[string]spec.Schema{
						"scaleTargetRef": {
							SchemaProps: spec.SchemaProps{
								Description: "scaleTargetRef points to the target resource to scale, and is used to the pods for which metrics should be collected, as well as to actually change the replica count.",
								Ref:         ref("k8s.io/api/autoscaling/v2beta1.CrossVersionObjectReference"),
							},
						},
						"minReplicas": {
							SchemaProps: spec.SchemaProps{
								Description: "minReplicas is the lower limit for the number of replicas to which the autoscaler can scale down. It defaults to 1 pod.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"maxReplicas": {
							SchemaProps: spec.SchemaProps{
								Description: "maxReplicas is the upper limit for the number of replicas to which the autoscaler can scale up. It cannot be less that minReplicas.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"metrics": {
							SchemaProps: spec.SchemaProps{
								Description: "metrics contains the specifications for which to use to calculate the desired replica count (the maximum replica count across all metrics will be used).  The desired replica count is calculated multiplying the ratio between the target value and the current value by the current number of pods.  Ergo, metrics used must decrease as the pod count is increased, and vice-versa.  See the individual metric source types for more information about how each type of metric must respond.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/autoscaling/v2beta1.MetricSpec"),
										},
									},
								},
							},
						},
					},
					Required: []string{"scaleTargetRef", "maxReplicas"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/autoscaling/v2beta1.CrossVersionObjectReference", "k8s.io/api/autoscaling/v2beta1.MetricSpec"},
		},
		"k8s.io/api/autoscaling/v2beta1.HorizontalPodAutoscalerStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "HorizontalPodAutoscalerStatus describes the current status of a horizontal pod autoscaler.",
					Properties: map[string]spec.Schema{
						"observedGeneration": {
							SchemaProps: spec.SchemaProps{
								Description: "observedGeneration is the most recent generation observed by this autoscaler.",
								Type:        []string{"integer"},
								Format:      "int64",
							},
						},
						"lastScaleTime": {
							SchemaProps: spec.SchemaProps{
								Description: "lastScaleTime is the last time the HorizontalPodAutoscaler scaled the number of pods, used by the autoscaler to control how often the number of pods is changed.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"currentReplicas": {
							SchemaProps: spec.SchemaProps{
								Description: "currentReplicas is current number of replicas of pods managed by this autoscaler, as last seen by the autoscaler.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"desiredReplicas": {
							SchemaProps: spec.SchemaProps{
								Description: "desiredReplicas is the desired number of replicas of pods managed by this autoscaler, as last calculated by the autoscaler.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"currentMetrics": {
							SchemaProps: spec.SchemaProps{
								Description: "currentMetrics is the last read state of the metrics used by this autoscaler.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/autoscaling/v2beta1.MetricStatus"),
										},
									},
								},
							},
						},
						"conditions": {
							SchemaProps: spec.SchemaProps{
								Description: "conditions is the set of conditions required for this autoscaler to scale its target, and indicates whether or not those conditions are met.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/autoscaling/v2beta1.HorizontalPodAutoscalerCondition"),
										},
									},
								},
							},
						},
					},
					Required: []string{"currentReplicas", "desiredReplicas", "currentMetrics", "conditions"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/autoscaling/v2beta1.HorizontalPodAutoscalerCondition", "k8s.io/api/autoscaling/v2beta1.MetricStatus", "k8s.io/apimachinery/pkg/apis/meta/v1.Time"},
		},
		"k8s.io/api/autoscaling/v2beta1.MetricSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "MetricSpec specifies how to scale based on a single metric (only `type` and one other matching field should be set at once).",
					Properties: map[string]spec.Schema{
						"type": {
							SchemaProps: spec.SchemaProps{
								Description: "type is the type of metric source.  It should match one of the fields below.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"object": {
							SchemaProps: spec.SchemaProps{
								Description: "object refers to a metric describing a single kubernetes object (for example, hits-per-second on an Ingress object).",
								Ref:         ref("k8s.io/api/autoscaling/v2beta1.ObjectMetricSource"),
							},
						},
						"pods": {
							SchemaProps: spec.SchemaProps{
								Description: "pods refers to a metric describing each pod in the current scale target (for example, transactions-processed-per-second).  The values will be averaged together before being compared to the target value.",
								Ref:         ref("k8s.io/api/autoscaling/v2beta1.PodsMetricSource"),
							},
						},
						"resource": {
							SchemaProps: spec.SchemaProps{
								Description: "resource refers to a resource metric (such as those specified in requests and limits) known to Kubernetes describing each pod in the current scale target (e.g. CPU or memory). Such metrics are built in to Kubernetes, and have special scaling options on top of those available to normal per-pod metrics using the \"pods\" source.",
								Ref:         ref("k8s.io/api/autoscaling/v2beta1.ResourceMetricSource"),
							},
						},
					},
					Required: []string{"type"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/autoscaling/v2beta1.ObjectMetricSource", "k8s.io/api/autoscaling/v2beta1.PodsMetricSource", "k8s.io/api/autoscaling/v2beta1.ResourceMetricSource"},
		},
		"k8s.io/api/autoscaling/v2beta1.MetricStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "MetricStatus describes the last-read state of a single metric.",
					Properties: map[string]spec.Schema{
						"type": {
							SchemaProps: spec.SchemaProps{
								Description: "type is the type of metric source.  It will match one of the fields below.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"object": {
							SchemaProps: spec.SchemaProps{
								Description: "object refers to a metric describing a single kubernetes object (for example, hits-per-second on an Ingress object).",
								Ref:         ref("k8s.io/api/autoscaling/v2beta1.ObjectMetricStatus"),
							},
						},
						"pods": {
							SchemaProps: spec.SchemaProps{
								Description: "pods refers to a metric describing each pod in the current scale target (for example, transactions-processed-per-second).  The values will be averaged together before being compared to the target value.",
								Ref:         ref("k8s.io/api/autoscaling/v2beta1.PodsMetricStatus"),
							},
						},
						"resource": {
							SchemaProps: spec.SchemaProps{
								Description: "resource refers to a resource metric (such as those specified in requests and limits) known to Kubernetes describing each pod in the current scale target (e.g. CPU or memory). Such metrics are built in to Kubernetes, and have special scaling options on top of those available to normal per-pod metrics using the \"pods\" source.",
								Ref:         ref("k8s.io/api/autoscaling/v2beta1.ResourceMetricStatus"),
							},
						},
					},
					Required: []string{"type"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/autoscaling/v2beta1.ObjectMetricStatus", "k8s.io/api/autoscaling/v2beta1.PodsMetricStatus", "k8s.io/api/autoscaling/v2beta1.ResourceMetricStatus"},
		},
		"k8s.io/api/autoscaling/v2beta1.ObjectMetricSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ObjectMetricSource indicates how to scale on a metric describing a kubernetes object (for example, hits-per-second on an Ingress object).",
					Properties: map[string]spec.Schema{
						"target": {
							SchemaProps: spec.SchemaProps{
								Description: "target is the described Kubernetes object.",
								Ref:         ref("k8s.io/api/autoscaling/v2beta1.CrossVersionObjectReference"),
							},
						},
						"metricName": {
							SchemaProps: spec.SchemaProps{
								Description: "metricName is the name of the metric in question.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"targetValue": {
							SchemaProps: spec.SchemaProps{
								Description: "targetValue is the target value of the metric (as a quantity).",
								Ref:         ref("k8s.io/apimachinery/pkg/api/resource.Quantity"),
							},
						},
					},
					Required: []string{"target", "metricName", "targetValue"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/autoscaling/v2beta1.CrossVersionObjectReference", "k8s.io/apimachinery/pkg/api/resource.Quantity"},
		},
		"k8s.io/api/autoscaling/v2beta1.ObjectMetricStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ObjectMetricStatus indicates the current value of a metric describing a kubernetes object (for example, hits-per-second on an Ingress object).",
					Properties: map[string]spec.Schema{
						"target": {
							SchemaProps: spec.SchemaProps{
								Description: "target is the described Kubernetes object.",
								Ref:         ref("k8s.io/api/autoscaling/v2beta1.CrossVersionObjectReference"),
							},
						},
						"metricName": {
							SchemaProps: spec.SchemaProps{
								Description: "metricName is the name of the metric in question.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"currentValue": {
							SchemaProps: spec.SchemaProps{
								Description: "currentValue is the current value of the metric (as a quantity).",
								Ref:         ref("k8s.io/apimachinery/pkg/api/resource.Quantity"),
							},
						},
					},
					Required: []string{"target", "metricName", "currentValue"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/autoscaling/v2beta1.CrossVersionObjectReference", "k8s.io/apimachinery/pkg/api/resource.Quantity"},
		},
		"k8s.io/api/autoscaling/v2beta1.PodsMetricSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PodsMetricSource indicates how to scale on a metric describing each pod in the current scale target (for example, transactions-processed-per-second). The values will be averaged together before being compared to the target value.",
					Properties: map[string]spec.Schema{
						"metricName": {
							SchemaProps: spec.SchemaProps{
								Description: "metricName is the name of the metric in question",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"targetAverageValue": {
							SchemaProps: spec.SchemaProps{
								Description: "targetAverageValue is the target value of the average of the metric across all relevant pods (as a quantity)",
								Ref:         ref("k8s.io/apimachinery/pkg/api/resource.Quantity"),
							},
						},
					},
					Required: []string{"metricName", "targetAverageValue"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/api/resource.Quantity"},
		},
		"k8s.io/api/autoscaling/v2beta1.PodsMetricStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PodsMetricStatus indicates the current value of a metric describing each pod in the current scale target (for example, transactions-processed-per-second).",
					Properties: map[string]spec.Schema{
						"metricName": {
							SchemaProps: spec.SchemaProps{
								Description: "metricName is the name of the metric in question",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"currentAverageValue": {
							SchemaProps: spec.SchemaProps{
								Description: "currentAverageValue is the current value of the average of the metric across all relevant pods (as a quantity)",
								Ref:         ref("k8s.io/apimachinery/pkg/api/resource.Quantity"),
							},
						},
					},
					Required: []string{"metricName", "currentAverageValue"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/api/resource.Quantity"},
		},
		"k8s.io/api/autoscaling/v2beta1.ResourceMetricSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ResourceMetricSource indicates how to scale on a resource metric known to Kubernetes, as specified in requests and limits, describing each pod in the current scale target (e.g. CPU or memory).  The values will be averaged together before being compared to the target.  Such metrics are built in to Kubernetes, and have special scaling options on top of those available to normal per-pod metrics using the \"pods\" source.  Only one \"target\" type should be set.",
					Properties: map[string]spec.Schema{
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "name is the name of the resource in question.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"targetAverageUtilization": {
							SchemaProps: spec.SchemaProps{
								Description: "targetAverageUtilization is the target value of the average of the resource metric across all relevant pods, represented as a percentage of the requested value of the resource for the pods.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"targetAverageValue": {
							SchemaProps: spec.SchemaProps{
								Description: "targetAverageValue is the target value of the average of the resource metric across all relevant pods, as a raw value (instead of as a percentage of the request), similar to the \"pods\" metric source type.",
								Ref:         ref("k8s.io/apimachinery/pkg/api/resource.Quantity"),
							},
						},
					},
					Required: []string{"name"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/api/resource.Quantity"},
		},
		"k8s.io/api/autoscaling/v2beta1.ResourceMetricStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ResourceMetricStatus indicates the current value of a resource metric known to Kubernetes, as specified in requests and limits, describing each pod in the current scale target (e.g. CPU or memory).  Such metrics are built in to Kubernetes, and have special scaling options on top of those available to normal per-pod metrics using the \"pods\" source.",
					Properties: map[string]spec.Schema{
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "name is the name of the resource in question.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"currentAverageUtilization": {
							SchemaProps: spec.SchemaProps{
								Description: "currentAverageUtilization is the current value of the average of the resource metric across all relevant pods, represented as a percentage of the requested value of the resource for the pods.  It will only be present if `targetAverageValue` was set in the corresponding metric specification.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"currentAverageValue": {
							SchemaProps: spec.SchemaProps{
								Description: "currentAverageValue is the current value of the average of the resource metric across all relevant pods, as a raw value (instead of as a percentage of the request), similar to the \"pods\" metric source type. It will always be set, regardless of the corresponding metric specification.",
								Ref:         ref("k8s.io/apimachinery/pkg/api/resource.Quantity"),
							},
						},
					},
					Required: []string{"name", "currentAverageValue"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/api/resource.Quantity"},
		},
		"k8s.io/api/batch/v1.Job": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Job represents the configuration of a single job.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "Specification of the desired behavior of a job. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status",
								Ref:         ref("k8s.io/api/batch/v1.JobSpec"),
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Current status of a job. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status",
								Ref:         ref("k8s.io/api/batch/v1.JobStatus"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/batch/v1.JobSpec", "k8s.io/api/batch/v1.JobStatus", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/batch/v1.JobCondition": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "JobCondition describes current state of a job.",
					Properties: map[string]spec.Schema{
						"type": {
							SchemaProps: spec.SchemaProps{
								Description: "Type of job condition, Complete or Failed.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Status of the condition, one of True, False, Unknown.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"lastProbeTime": {
							SchemaProps: spec.SchemaProps{
								Description: "Last time the condition was checked.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"lastTransitionTime": {
							SchemaProps: spec.SchemaProps{
								Description: "Last time the condition transit from one status to another.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"reason": {
							SchemaProps: spec.SchemaProps{
								Description: "(brief) reason for the condition's last transition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"message": {
							SchemaProps: spec.SchemaProps{
								Description: "Human readable message indicating details about last transition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"type", "status"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.Time"},
		},
		"k8s.io/api/batch/v1.JobList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "JobList is a collection of jobs.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "items is the list of Jobs.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/batch/v1.Job"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/batch/v1.Job", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/batch/v1.JobSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "JobSpec describes how the job execution will look like.",
					Properties: map[string]spec.Schema{
						"parallelism": {
							SchemaProps: spec.SchemaProps{
								Description: "Specifies the maximum desired number of pods the job should run at any given time. The actual number of pods running in steady state will be less than this number when ((.spec.completions - .status.successful) < .spec.parallelism), i.e. when the work left to do is less than max parallelism. More info: https://kubernetes.io/docs/concepts/workloads/controllers/jobs-run-to-completion/",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"completions": {
							SchemaProps: spec.SchemaProps{
								Description: "Specifies the desired number of successfully finished pods the job should be run with.  Setting to nil means that the success of any pod signals the success of all pods, and allows parallelism to have any positive value.  Setting to 1 means that parallelism is limited to 1 and the success of that pod signals the success of the job. More info: https://kubernetes.io/docs/concepts/workloads/controllers/jobs-run-to-completion/",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"activeDeadlineSeconds": {
							SchemaProps: spec.SchemaProps{
								Description: "Specifies the duration in seconds relative to the startTime that the job may be active before the system tries to terminate it; value must be positive integer",
								Type:        []string{"integer"},
								Format:      "int64",
							},
						},
						"backoffLimit": {
							SchemaProps: spec.SchemaProps{
								Description: "Specifies the number of retries before marking this job failed. Defaults to 6",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"selector": {
							SchemaProps: spec.SchemaProps{
								Description: "A label query over pods that should match the pod count. Normally, the system sets this field for you. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/labels/#label-selectors",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"),
							},
						},
						"manualSelector": {
							SchemaProps: spec.SchemaProps{
								Description: "manualSelector controls generation of pod labels and pod selectors. Leave `manualSelector` unset unless you are certain what you are doing. When false or unset, the system pick labels unique to this job and appends those labels to the pod template.  When true, the user is responsible for picking unique labels and specifying the selector.  Failure to pick a unique label may cause this and other jobs to not function correctly.  However, You may see `manualSelector=true` in jobs that were created with the old `extensions/v1beta1` API. More info: https://kubernetes.io/docs/concepts/workloads/controllers/jobs-run-to-completion/#specifying-your-own-pod-selector",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"template": {
							SchemaProps: spec.SchemaProps{
								Description: "Describes the pod that will be created when executing a job. More info: https://kubernetes.io/docs/concepts/workloads/controllers/jobs-run-to-completion/",
								Ref:         ref("k8s.io/api/core/v1.PodTemplateSpec"),
							},
						},
					},
					Required: []string{"template"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.PodTemplateSpec", "k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"},
		},
		"k8s.io/api/batch/v1.JobStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "JobStatus represents the current state of a Job.",
					Properties: map[string]spec.Schema{
						"conditions": {
							VendorExtensible: spec.VendorExtensible{
								Extensions: spec.Extensions{
									"x-kubernetes-patch-merge-key": "type",
									"x-kubernetes-patch-strategy":  "merge",
								},
							},
							SchemaProps: spec.SchemaProps{
								Description: "The latest available observations of an object's current state. More info: https://kubernetes.io/docs/concepts/workloads/controllers/jobs-run-to-completion/",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/batch/v1.JobCondition"),
										},
									},
								},
							},
						},
						"startTime": {
							SchemaProps: spec.SchemaProps{
								Description: "Represents time when the job was acknowledged by the job controller. It is not guaranteed to be set in happens-before order across separate operations. It is represented in RFC3339 form and is in UTC.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"completionTime": {
							SchemaProps: spec.SchemaProps{
								Description: "Represents time when the job was completed. It is not guaranteed to be set in happens-before order across separate operations. It is represented in RFC3339 form and is in UTC.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"active": {
							SchemaProps: spec.SchemaProps{
								Description: "The number of actively running pods.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"succeeded": {
							SchemaProps: spec.SchemaProps{
								Description: "The number of pods which reached phase Succeeded.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"failed": {
							SchemaProps: spec.SchemaProps{
								Description: "The number of pods which reached phase Failed.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/batch/v1.JobCondition", "k8s.io/apimachinery/pkg/apis/meta/v1.Time"},
		},
		"k8s.io/api/batch/v1beta1.CronJob": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "CronJob represents the configuration of a single cron job.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "Specification of the desired behavior of a cron job, including the schedule. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status",
								Ref:         ref("k8s.io/api/batch/v1beta1.CronJobSpec"),
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Current status of a cron job. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status",
								Ref:         ref("k8s.io/api/batch/v1beta1.CronJobStatus"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/batch/v1beta1.CronJobSpec", "k8s.io/api/batch/v1beta1.CronJobStatus", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/batch/v1beta1.CronJobList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "CronJobList is a collection of cron jobs.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "items is the list of CronJobs.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/batch/v1beta1.CronJob"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/batch/v1beta1.CronJob", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/batch/v1beta1.CronJobSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "CronJobSpec describes how the job execution will look like and when it will actually run.",
					Properties: map[string]spec.Schema{
						"schedule": {
							SchemaProps: spec.SchemaProps{
								Description: "The schedule in Cron format, see https://en.wikipedia.org/wiki/Cron.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"startingDeadlineSeconds": {
							SchemaProps: spec.SchemaProps{
								Description: "Optional deadline in seconds for starting the job if it misses scheduled time for any reason.  Missed jobs executions will be counted as failed ones.",
								Type:        []string{"integer"},
								Format:      "int64",
							},
						},
						"concurrencyPolicy": {
							SchemaProps: spec.SchemaProps{
								Description: "Specifies how to treat concurrent executions of a Job. Valid values are: - \"Allow\" (default): allows CronJobs to run concurrently; - \"Forbid\": forbids concurrent runs, skipping next run if previous run hasn't finished yet; - \"Replace\": cancels currently running job and replaces it with a new one",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"suspend": {
							SchemaProps: spec.SchemaProps{
								Description: "This flag tells the controller to suspend subsequent executions, it does not apply to already started executions.  Defaults to false.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"jobTemplate": {
							SchemaProps: spec.SchemaProps{
								Description: "Specifies the job that will be created when executing a CronJob.",
								Ref:         ref("k8s.io/api/batch/v1beta1.JobTemplateSpec"),
							},
						},
						"successfulJobsHistoryLimit": {
							SchemaProps: spec.SchemaProps{
								Description: "The number of successful finished jobs to retain. This is a pointer to distinguish between explicit zero and not specified. Defaults to 3.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"failedJobsHistoryLimit": {
							SchemaProps: spec.SchemaProps{
								Description: "The number of failed finished jobs to retain. This is a pointer to distinguish between explicit zero and not specified. Defaults to 1.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
					},
					Required: []string{"schedule", "jobTemplate"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/batch/v1beta1.JobTemplateSpec"},
		},
		"k8s.io/api/batch/v1beta1.CronJobStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "CronJobStatus represents the current state of a cron job.",
					Properties: map[string]spec.Schema{
						"active": {
							SchemaProps: spec.SchemaProps{
								Description: "A list of pointers to currently running jobs.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.ObjectReference"),
										},
									},
								},
							},
						},
						"lastScheduleTime": {
							SchemaProps: spec.SchemaProps{
								Description: "Information when was the last time the job was successfully scheduled.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.ObjectReference", "k8s.io/apimachinery/pkg/apis/meta/v1.Time"},
		},
		"k8s.io/api/batch/v1beta1.JobTemplate": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "JobTemplate describes a template for creating copies of a predefined pod.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"template": {
							SchemaProps: spec.SchemaProps{
								Description: "Defines jobs that will be created from this template. https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status",
								Ref:         ref("k8s.io/api/batch/v1beta1.JobTemplateSpec"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/batch/v1beta1.JobTemplateSpec", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/batch/v1beta1.JobTemplateSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "JobTemplateSpec describes the data a Job should have when created from a template",
					Properties: map[string]spec.Schema{
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata of the jobs created from this template. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "Specification of the desired behavior of the job. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status",
								Ref:         ref("k8s.io/api/batch/v1.JobSpec"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/batch/v1.JobSpec", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/batch/v2alpha1.CronJob": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "CronJob represents the configuration of a single cron job.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "Specification of the desired behavior of a cron job, including the schedule. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status",
								Ref:         ref("k8s.io/api/batch/v2alpha1.CronJobSpec"),
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Current status of a cron job. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status",
								Ref:         ref("k8s.io/api/batch/v2alpha1.CronJobStatus"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/batch/v2alpha1.CronJobSpec", "k8s.io/api/batch/v2alpha1.CronJobStatus", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/batch/v2alpha1.CronJobList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "CronJobList is a collection of cron jobs.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "items is the list of CronJobs.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/batch/v2alpha1.CronJob"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/batch/v2alpha1.CronJob", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/batch/v2alpha1.CronJobSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "CronJobSpec describes how the job execution will look like and when it will actually run.",
					Properties: map[string]spec.Schema{
						"schedule": {
							SchemaProps: spec.SchemaProps{
								Description: "The schedule in Cron format, see https://en.wikipedia.org/wiki/Cron.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"startingDeadlineSeconds": {
							SchemaProps: spec.SchemaProps{
								Description: "Optional deadline in seconds for starting the job if it misses scheduled time for any reason.  Missed jobs executions will be counted as failed ones.",
								Type:        []string{"integer"},
								Format:      "int64",
							},
						},
						"concurrencyPolicy": {
							SchemaProps: spec.SchemaProps{
								Description: "Specifies how to treat concurrent executions of a Job. Valid values are: - \"Allow\" (default): allows CronJobs to run concurrently; - \"Forbid\": forbids concurrent runs, skipping next run if previous run hasn't finished yet; - \"Replace\": cancels currently running job and replaces it with a new one",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"suspend": {
							SchemaProps: spec.SchemaProps{
								Description: "This flag tells the controller to suspend subsequent executions, it does not apply to already started executions.  Defaults to false.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"jobTemplate": {
							SchemaProps: spec.SchemaProps{
								Description: "Specifies the job that will be created when executing a CronJob.",
								Ref:         ref("k8s.io/api/batch/v2alpha1.JobTemplateSpec"),
							},
						},
						"successfulJobsHistoryLimit": {
							SchemaProps: spec.SchemaProps{
								Description: "The number of successful finished jobs to retain. This is a pointer to distinguish between explicit zero and not specified.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"failedJobsHistoryLimit": {
							SchemaProps: spec.SchemaProps{
								Description: "The number of failed finished jobs to retain. This is a pointer to distinguish between explicit zero and not specified.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
					},
					Required: []string{"schedule", "jobTemplate"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/batch/v2alpha1.JobTemplateSpec"},
		},
		"k8s.io/api/batch/v2alpha1.CronJobStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "CronJobStatus represents the current state of a cron job.",
					Properties: map[string]spec.Schema{
						"active": {
							SchemaProps: spec.SchemaProps{
								Description: "A list of pointers to currently running jobs.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.ObjectReference"),
										},
									},
								},
							},
						},
						"lastScheduleTime": {
							SchemaProps: spec.SchemaProps{
								Description: "Information when was the last time the job was successfully scheduled.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.ObjectReference", "k8s.io/apimachinery/pkg/apis/meta/v1.Time"},
		},
		"k8s.io/api/batch/v2alpha1.JobTemplate": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "JobTemplate describes a template for creating copies of a predefined pod.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"template": {
							SchemaProps: spec.SchemaProps{
								Description: "Defines jobs that will be created from this template. https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status",
								Ref:         ref("k8s.io/api/batch/v2alpha1.JobTemplateSpec"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/batch/v2alpha1.JobTemplateSpec", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/batch/v2alpha1.JobTemplateSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "JobTemplateSpec describes the data a Job should have when created from a template",
					Properties: map[string]spec.Schema{
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata of the jobs created from this template. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "Specification of the desired behavior of the job. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status",
								Ref:         ref("k8s.io/api/batch/v1.JobSpec"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/batch/v1.JobSpec", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/certificates/v1beta1.CertificateSigningRequest": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Describes a certificate signing request",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "The certificate request itself and any additional information.",
								Ref:         ref("k8s.io/api/certificates/v1beta1.CertificateSigningRequestSpec"),
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Derived information about the request.",
								Ref:         ref("k8s.io/api/certificates/v1beta1.CertificateSigningRequestStatus"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/certificates/v1beta1.CertificateSigningRequestSpec", "k8s.io/api/certificates/v1beta1.CertificateSigningRequestStatus", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/certificates/v1beta1.CertificateSigningRequestCondition": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Properties: map[string]spec.Schema{
						"type": {
							SchemaProps: spec.SchemaProps{
								Description: "request approval state, currently Approved or Denied.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"reason": {
							SchemaProps: spec.SchemaProps{
								Description: "brief reason for the request state",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"message": {
							SchemaProps: spec.SchemaProps{
								Description: "human readable message with details about the request state",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"lastUpdateTime": {
							SchemaProps: spec.SchemaProps{
								Description: "timestamp for the last update to this condition",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
					},
					Required: []string{"type"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.Time"},
		},
		"k8s.io/api/certificates/v1beta1.CertificateSigningRequestList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Type: []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/certificates/v1beta1.CertificateSigningRequest"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/certificates/v1beta1.CertificateSigningRequest", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/certificates/v1beta1.CertificateSigningRequestSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "This information is immutable after the request is created. Only the Request and Usages fields can be set on creation, other fields are derived by Kubernetes and cannot be modified by users.",
					Properties: map[string]spec.Schema{
						"request": {
							SchemaProps: spec.SchemaProps{
								Description: "Base64-encoded PKCS#10 CSR data",
								Type:        []string{"string"},
								Format:      "byte",
							},
						},
						"usages": {
							SchemaProps: spec.SchemaProps{
								Description: "allowedUsages specifies a set of usage contexts the key will be valid for. See: https://tools.ietf.org/html/rfc5280#section-4.2.1.3\n     https://tools.ietf.org/html/rfc5280#section-4.2.1.12",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"username": {
							SchemaProps: spec.SchemaProps{
								Description: "Information about the requesting user. See user.Info interface for details.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"uid": {
							SchemaProps: spec.SchemaProps{
								Description: "UID information about the requesting user. See user.Info interface for details.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"groups": {
							SchemaProps: spec.SchemaProps{
								Description: "Group information about the requesting user. See user.Info interface for details.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"extra": {
							SchemaProps: spec.SchemaProps{
								Description: "Extra information about the requesting user. See user.Info interface for details.",
								Type:        []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type: []string{"array"},
											Items: &spec.SchemaOrArray{
												Schema: &spec.Schema{
													SchemaProps: spec.SchemaProps{
														Type:   []string{"string"},
														Format: "",
													},
												},
											},
										},
									},
								},
							},
						},
					},
					Required: []string{"request"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/certificates/v1beta1.CertificateSigningRequestStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Properties: map[string]spec.Schema{
						"conditions": {
							SchemaProps: spec.SchemaProps{
								Description: "Conditions applied to the request, such as approval or denial.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/certificates/v1beta1.CertificateSigningRequestCondition"),
										},
									},
								},
							},
						},
						"certificate": {
							SchemaProps: spec.SchemaProps{
								Description: "If request was approved, the controller will place the issued certificate here.",
								Type:        []string{"string"},
								Format:      "byte",
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/certificates/v1beta1.CertificateSigningRequestCondition"},
		},
		"k8s.io/api/core/v1.AWSElasticBlockStoreVolumeSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Represents a Persistent Disk resource in AWS.\n\nAn AWS EBS disk must exist before mounting to a container. The disk must also be in the same AWS zone as the kubelet. An AWS EBS disk can only be mounted as read/write once. AWS EBS volumes support ownership management and SELinux relabeling.",
					Properties: map[string]spec.Schema{
						"volumeID": {
							SchemaProps: spec.SchemaProps{
								Description: "Unique ID of the persistent disk resource in AWS (Amazon EBS volume). More info: https://kubernetes.io/docs/concepts/storage/volumes#awselasticblockstore",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"fsType": {
							SchemaProps: spec.SchemaProps{
								Description: "Filesystem type of the volume that you want to mount. Tip: Ensure that the filesystem type is supported by the host operating system. Examples: \"ext4\", \"xfs\", \"ntfs\". Implicitly inferred to be \"ext4\" if unspecified. More info: https://kubernetes.io/docs/concepts/storage/volumes#awselasticblockstore",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"partition": {
							SchemaProps: spec.SchemaProps{
								Description: "The partition in the volume that you want to mount. If omitted, the default is to mount by volume name. Examples: For volume /dev/sda1, you specify the partition as \"1\". Similarly, the volume partition for /dev/sda is \"0\" (or you can leave the property empty).",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"readOnly": {
							SchemaProps: spec.SchemaProps{
								Description: "Specify \"true\" to force and set the ReadOnly property in VolumeMounts to \"true\". If omitted, the default is \"false\". More info: https://kubernetes.io/docs/concepts/storage/volumes#awselasticblockstore",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
					},
					Required: []string{"volumeID"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.Affinity": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Affinity is a group of affinity scheduling rules.",
					Properties: map[string]spec.Schema{
						"nodeAffinity": {
							SchemaProps: spec.SchemaProps{
								Description: "Describes node affinity scheduling rules for the pod.",
								Ref:         ref("k8s.io/api/core/v1.NodeAffinity"),
							},
						},
						"podAffinity": {
							SchemaProps: spec.SchemaProps{
								Description: "Describes pod affinity scheduling rules (e.g. co-locate this pod in the same node, zone, etc. as some other pod(s)).",
								Ref:         ref("k8s.io/api/core/v1.PodAffinity"),
							},
						},
						"podAntiAffinity": {
							SchemaProps: spec.SchemaProps{
								Description: "Describes pod anti-affinity scheduling rules (e.g. avoid putting this pod in the same node, zone, etc. as some other pod(s)).",
								Ref:         ref("k8s.io/api/core/v1.PodAntiAffinity"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.NodeAffinity", "k8s.io/api/core/v1.PodAffinity", "k8s.io/api/core/v1.PodAntiAffinity"},
		},
		"k8s.io/api/core/v1.AttachedVolume": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "AttachedVolume describes a volume attached to a node",
					Properties: map[string]spec.Schema{
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "Name of the attached volume",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"devicePath": {
							SchemaProps: spec.SchemaProps{
								Description: "DevicePath represents the device path where the volume should be available",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"name", "devicePath"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.AvoidPods": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "AvoidPods describes pods that should avoid this node. This is the value for a Node annotation with key scheduler.alpha.kubernetes.io/preferAvoidPods and will eventually become a field of NodeStatus.",
					Properties: map[string]spec.Schema{
						"preferAvoidPods": {
							SchemaProps: spec.SchemaProps{
								Description: "Bounded-sized list of signatures of pods that should avoid this node, sorted in timestamp order from oldest to newest. Size of the slice is unspecified.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.PreferAvoidPodsEntry"),
										},
									},
								},
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.PreferAvoidPodsEntry"},
		},
		"k8s.io/api/core/v1.AzureDiskVolumeSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "AzureDisk represents an Azure Data Disk mount on the host and bind mount to the pod.",
					Properties: map[string]spec.Schema{
						"diskName": {
							SchemaProps: spec.SchemaProps{
								Description: "The Name of the data disk in the blob storage",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"diskURI": {
							SchemaProps: spec.SchemaProps{
								Description: "The URI the data disk in the blob storage",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"cachingMode": {
							SchemaProps: spec.SchemaProps{
								Description: "Host Caching mode: None, Read Only, Read Write.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"fsType": {
							SchemaProps: spec.SchemaProps{
								Description: "Filesystem type to mount. Must be a filesystem type supported by the host operating system. Ex. \"ext4\", \"xfs\", \"ntfs\". Implicitly inferred to be \"ext4\" if unspecified.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"readOnly": {
							SchemaProps: spec.SchemaProps{
								Description: "Defaults to false (read/write). ReadOnly here will force the ReadOnly setting in VolumeMounts.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Expected values Shared: multiple blob disks per storage account  Dedicated: single blob disk per storage account  Managed: azure managed data disk (only in managed availability set). defaults to shared",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"diskName", "diskURI"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.AzureFilePersistentVolumeSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "AzureFile represents an Azure File Service mount on the host and bind mount to the pod.",
					Properties: map[string]spec.Schema{
						"secretName": {
							SchemaProps: spec.SchemaProps{
								Description: "the name of secret that contains Azure Storage Account Name and Key",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"shareName": {
							SchemaProps: spec.SchemaProps{
								Description: "Share Name",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"readOnly": {
							SchemaProps: spec.SchemaProps{
								Description: "Defaults to false (read/write). ReadOnly here will force the ReadOnly setting in VolumeMounts.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"secretNamespace": {
							SchemaProps: spec.SchemaProps{
								Description: "the namespace of the secret that contains Azure Storage Account Name and Key default is the same as the Pod",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"secretName", "shareName"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.AzureFileVolumeSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "AzureFile represents an Azure File Service mount on the host and bind mount to the pod.",
					Properties: map[string]spec.Schema{
						"secretName": {
							SchemaProps: spec.SchemaProps{
								Description: "the name of secret that contains Azure Storage Account Name and Key",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"shareName": {
							SchemaProps: spec.SchemaProps{
								Description: "Share Name",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"readOnly": {
							SchemaProps: spec.SchemaProps{
								Description: "Defaults to false (read/write). ReadOnly here will force the ReadOnly setting in VolumeMounts.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
					},
					Required: []string{"secretName", "shareName"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.Binding": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Binding ties one object to another; for example, a pod is bound to a node by a scheduler. Deprecated in 1.7, please use the bindings subresource of pods instead.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"target": {
							SchemaProps: spec.SchemaProps{
								Description: "The target object that you want to bind to the standard object.",
								Ref:         ref("k8s.io/api/core/v1.ObjectReference"),
							},
						},
					},
					Required: []string{"target"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.ObjectReference", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/core/v1.CSIPersistentVolumeSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Represents storage that is managed by an external CSI volume driver",
					Properties: map[string]spec.Schema{
						"driver": {
							SchemaProps: spec.SchemaProps{
								Description: "Driver is the name of the driver to use for this volume. Required.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"volumeHandle": {
							SchemaProps: spec.SchemaProps{
								Description: "VolumeHandle is the unique volume name returned by the CSI volume plugin’s CreateVolume to refer to the volume on all subsequent calls. Required.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"readOnly": {
							SchemaProps: spec.SchemaProps{
								Description: "Optional: The value to pass to ControllerPublishVolumeRequest. Defaults to false (read/write).",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
					},
					Required: []string{"driver", "volumeHandle"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.Capabilities": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Adds and removes POSIX capabilities from running containers.",
					Properties: map[string]spec.Schema{
						"add": {
							SchemaProps: spec.SchemaProps{
								Description: "Added capabilities",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"drop": {
							SchemaProps: spec.SchemaProps{
								Description: "Removed capabilities",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.CephFSPersistentVolumeSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Represents a Ceph Filesystem mount that lasts the lifetime of a pod Cephfs volumes do not support ownership management or SELinux relabeling.",
					Properties: map[string]spec.Schema{
						"monitors": {
							SchemaProps: spec.SchemaProps{
								Description: "Required: Monitors is a collection of Ceph monitors More info: https://releases.k8s.io/HEAD/examples/volumes/cephfs/README.md#how-to-use-it",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"path": {
							SchemaProps: spec.SchemaProps{
								Description: "Optional: Used as the mounted root, rather than the full Ceph tree, default is /",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"user": {
							SchemaProps: spec.SchemaProps{
								Description: "Optional: User is the rados user name, default is admin More info: https://releases.k8s.io/HEAD/examples/volumes/cephfs/README.md#how-to-use-it",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"secretFile": {
							SchemaProps: spec.SchemaProps{
								Description: "Optional: SecretFile is the path to key ring for User, default is /etc/ceph/user.secret More info: https://releases.k8s.io/HEAD/examples/volumes/cephfs/README.md#how-to-use-it",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"secretRef": {
							SchemaProps: spec.SchemaProps{
								Description: "Optional: SecretRef is reference to the authentication secret for User, default is empty. More info: https://releases.k8s.io/HEAD/examples/volumes/cephfs/README.md#how-to-use-it",
								Ref:         ref("k8s.io/api/core/v1.SecretReference"),
							},
						},
						"readOnly": {
							SchemaProps: spec.SchemaProps{
								Description: "Optional: Defaults to false (read/write). ReadOnly here will force the ReadOnly setting in VolumeMounts. More info: https://releases.k8s.io/HEAD/examples/volumes/cephfs/README.md#how-to-use-it",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
					},
					Required: []string{"monitors"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.SecretReference"},
		},
		"k8s.io/api/core/v1.CephFSVolumeSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Represents a Ceph Filesystem mount that lasts the lifetime of a pod Cephfs volumes do not support ownership management or SELinux relabeling.",
					Properties: map[string]spec.Schema{
						"monitors": {
							SchemaProps: spec.SchemaProps{
								Description: "Required: Monitors is a collection of Ceph monitors More info: https://releases.k8s.io/HEAD/examples/volumes/cephfs/README.md#how-to-use-it",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"path": {
							SchemaProps: spec.SchemaProps{
								Description: "Optional: Used as the mounted root, rather than the full Ceph tree, default is /",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"user": {
							SchemaProps: spec.SchemaProps{
								Description: "Optional: User is the rados user name, default is admin More info: https://releases.k8s.io/HEAD/examples/volumes/cephfs/README.md#how-to-use-it",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"secretFile": {
							SchemaProps: spec.SchemaProps{
								Description: "Optional: SecretFile is the path to key ring for User, default is /etc/ceph/user.secret More info: https://releases.k8s.io/HEAD/examples/volumes/cephfs/README.md#how-to-use-it",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"secretRef": {
							SchemaProps: spec.SchemaProps{
								Description: "Optional: SecretRef is reference to the authentication secret for User, default is empty. More info: https://releases.k8s.io/HEAD/examples/volumes/cephfs/README.md#how-to-use-it",
								Ref:         ref("k8s.io/api/core/v1.LocalObjectReference"),
							},
						},
						"readOnly": {
							SchemaProps: spec.SchemaProps{
								Description: "Optional: Defaults to false (read/write). ReadOnly here will force the ReadOnly setting in VolumeMounts. More info: https://releases.k8s.io/HEAD/examples/volumes/cephfs/README.md#how-to-use-it",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
					},
					Required: []string{"monitors"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.LocalObjectReference"},
		},
		"k8s.io/api/core/v1.CinderVolumeSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Represents a cinder volume resource in Openstack. A Cinder volume must exist before mounting to a container. The volume must also be in the same region as the kubelet. Cinder volumes support ownership management and SELinux relabeling.",
					Properties: map[string]spec.Schema{
						"volumeID": {
							SchemaProps: spec.SchemaProps{
								Description: "volume id used to identify the volume in cinder More info: https://releases.k8s.io/HEAD/examples/mysql-cinder-pd/README.md",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"fsType": {
							SchemaProps: spec.SchemaProps{
								Description: "Filesystem type to mount. Must be a filesystem type supported by the host operating system. Examples: \"ext4\", \"xfs\", \"ntfs\". Implicitly inferred to be \"ext4\" if unspecified. More info: https://releases.k8s.io/HEAD/examples/mysql-cinder-pd/README.md",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"readOnly": {
							SchemaProps: spec.SchemaProps{
								Description: "Optional: Defaults to false (read/write). ReadOnly here will force the ReadOnly setting in VolumeMounts. More info: https://releases.k8s.io/HEAD/examples/mysql-cinder-pd/README.md",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
					},
					Required: []string{"volumeID"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.ClientIPConfig": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ClientIPConfig represents the configurations of Client IP based session affinity.",
					Properties: map[string]spec.Schema{
						"timeoutSeconds": {
							SchemaProps: spec.SchemaProps{
								Description: "timeoutSeconds specifies the seconds of ClientIP type session sticky time. The value must be >0 && <=86400(for 1 day) if ServiceAffinity == \"ClientIP\". Default value is 10800(for 3 hours).",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.ComponentCondition": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Information about the condition of a component.",
					Properties: map[string]spec.Schema{
						"type": {
							SchemaProps: spec.SchemaProps{
								Description: "Type of condition for a component. Valid value: \"Healthy\"",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Status of the condition for a component. Valid values for \"Healthy\": \"True\", \"False\", or \"Unknown\".",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"message": {
							SchemaProps: spec.SchemaProps{
								Description: "Message about the condition for a component. For example, information about a health check.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"error": {
							SchemaProps: spec.SchemaProps{
								Description: "Condition error code for a component. For example, a health check error code.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"type", "status"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.ComponentStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ComponentStatus (and ComponentStatusList) holds the cluster validation info.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"conditions": {
							VendorExtensible: spec.VendorExtensible{
								Extensions: spec.Extensions{
									"x-kubernetes-patch-merge-key": "type",
									"x-kubernetes-patch-strategy":  "merge",
								},
							},
							SchemaProps: spec.SchemaProps{
								Description: "List of component conditions observed",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.ComponentCondition"),
										},
									},
								},
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.ComponentCondition", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/core/v1.ComponentStatusList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Status of all the conditions for the component as a list of ComponentStatus objects.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "List of ComponentStatus objects.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.ComponentStatus"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.ComponentStatus", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/core/v1.ConfigMap": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ConfigMap holds configuration data for pods to consume.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"data": {
							SchemaProps: spec.SchemaProps{
								Description: "Data contains the configuration data. Each key must consist of alphanumeric characters, '-', '_' or '.'.",
								Type:        []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/core/v1.ConfigMapEnvSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ConfigMapEnvSource selects a ConfigMap to populate the environment variables with.\n\nThe contents of the target ConfigMap's Data field will represent the key-value pairs as environment variables.",
					Properties: map[string]spec.Schema{
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "Name of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/names/#names",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"optional": {
							SchemaProps: spec.SchemaProps{
								Description: "Specify whether the ConfigMap must be defined",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.ConfigMapKeySelector": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Selects a key from a ConfigMap.",
					Properties: map[string]spec.Schema{
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "Name of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/names/#names",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"key": {
							SchemaProps: spec.SchemaProps{
								Description: "The key to select.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"optional": {
							SchemaProps: spec.SchemaProps{
								Description: "Specify whether the ConfigMap or it's key must be defined",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
					},
					Required: []string{"key"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.ConfigMapList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ConfigMapList is a resource containing a list of ConfigMap objects.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "Items is the list of ConfigMaps.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.ConfigMap"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.ConfigMap", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/core/v1.ConfigMapProjection": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Adapts a ConfigMap into a projected volume.\n\nThe contents of the target ConfigMap's Data field will be presented in a projected volume as files using the keys in the Data field as the file names, unless the items element is populated with specific mappings of keys to paths. Note that this is identical to a configmap volume source without the default mode.",
					Properties: map[string]spec.Schema{
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "Name of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/names/#names",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "If unspecified, each key-value pair in the Data field of the referenced ConfigMap will be projected into the volume as a file whose name is the key and content is the value. If specified, the listed keys will be projected into the specified paths, and unlisted keys will not be present. If a key is specified which is not present in the ConfigMap, the volume setup will error unless it is marked optional. Paths must be relative and may not contain the '..' path or start with '..'.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.KeyToPath"),
										},
									},
								},
							},
						},
						"optional": {
							SchemaProps: spec.SchemaProps{
								Description: "Specify whether the ConfigMap or it's keys must be defined",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.KeyToPath"},
		},
		"k8s.io/api/core/v1.ConfigMapVolumeSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Adapts a ConfigMap into a volume.\n\nThe contents of the target ConfigMap's Data field will be presented in a volume as files using the keys in the Data field as the file names, unless the items element is populated with specific mappings of keys to paths. ConfigMap volumes support ownership management and SELinux relabeling.",
					Properties: map[string]spec.Schema{
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "Name of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/names/#names",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "If unspecified, each key-value pair in the Data field of the referenced ConfigMap will be projected into the volume as a file whose name is the key and content is the value. If specified, the listed keys will be projected into the specified paths, and unlisted keys will not be present. If a key is specified which is not present in the ConfigMap, the volume setup will error unless it is marked optional. Paths must be relative and may not contain the '..' path or start with '..'.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.KeyToPath"),
										},
									},
								},
							},
						},
						"defaultMode": {
							SchemaProps: spec.SchemaProps{
								Description: "Optional: mode bits to use on created files by default. Must be a value between 0 and 0777. Defaults to 0644. Directories within the path are not affected by this setting. This might be in conflict with other options that affect the file mode, like fsGroup, and the result can be other mode bits set.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"optional": {
							SchemaProps: spec.SchemaProps{
								Description: "Specify whether the ConfigMap or it's keys must be defined",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.KeyToPath"},
		},
		"k8s.io/api/core/v1.Container": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "A single application container that you want to run within a pod.",
					Properties: map[string]spec.Schema{
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "Name of the container specified as a DNS_LABEL. Each container in a pod must have a unique name (DNS_LABEL). Cannot be updated.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"image": {
							SchemaProps: spec.SchemaProps{
								Description: "Docker image name. More info: https://kubernetes.io/docs/concepts/containers/images This field is optional to allow higher level config management to default or override container images in workload controllers like Deployments and StatefulSets.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"command": {
							SchemaProps: spec.SchemaProps{
								Description: "Entrypoint array. Not executed within a shell. The docker image's ENTRYPOINT is used if this is not provided. Variable references $(VAR_NAME) are expanded using the container's environment. If a variable cannot be resolved, the reference in the input string will be unchanged. The $(VAR_NAME) syntax can be escaped with a double $$, ie: $$(VAR_NAME). Escaped references will never be expanded, regardless of whether the variable exists or not. Cannot be updated. More info: https://kubernetes.io/docs/tasks/inject-data-application/define-command-argument-container/#running-a-command-in-a-shell",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"args": {
							SchemaProps: spec.SchemaProps{
								Description: "Arguments to the entrypoint. The docker image's CMD is used if this is not provided. Variable references $(VAR_NAME) are expanded using the container's environment. If a variable cannot be resolved, the reference in the input string will be unchanged. The $(VAR_NAME) syntax can be escaped with a double $$, ie: $$(VAR_NAME). Escaped references will never be expanded, regardless of whether the variable exists or not. Cannot be updated. More info: https://kubernetes.io/docs/tasks/inject-data-application/define-command-argument-container/#running-a-command-in-a-shell",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"workingDir": {
							SchemaProps: spec.SchemaProps{
								Description: "Container's working directory. If not specified, the container runtime's default will be used, which might be configured in the container image. Cannot be updated.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"ports": {
							VendorExtensible: spec.VendorExtensible{
								Extensions: spec.Extensions{
									"x-kubernetes-patch-merge-key": "containerPort",
									"x-kubernetes-patch-strategy":  "merge",
								},
							},
							SchemaProps: spec.SchemaProps{
								Description: "List of ports to expose from the container. Exposing a port here gives the system additional information about the network connections a container uses, but is primarily informational. Not specifying a port here DOES NOT prevent that port from being exposed. Any port which is listening on the default \"0.0.0.0\" address inside a container will be accessible from the network. Cannot be updated.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.ContainerPort"),
										},
									},
								},
							},
						},
						"envFrom": {
							SchemaProps: spec.SchemaProps{
								Description: "List of sources to populate environment variables in the container. The keys defined within a source must be a C_IDENTIFIER. All invalid keys will be reported as an event when the container is starting. When a key exists in multiple sources, the value associated with the last source will take precedence. Values defined by an Env with a duplicate key will take precedence. Cannot be updated.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.EnvFromSource"),
										},
									},
								},
							},
						},
						"env": {
							VendorExtensible: spec.VendorExtensible{
								Extensions: spec.Extensions{
									"x-kubernetes-patch-merge-key": "name",
									"x-kubernetes-patch-strategy":  "merge",
								},
							},
							SchemaProps: spec.SchemaProps{
								Description: "List of environment variables to set in the container. Cannot be updated.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.EnvVar"),
										},
									},
								},
							},
						},
						"resources": {
							SchemaProps: spec.SchemaProps{
								Description: "Compute Resources required by this container. Cannot be updated. More info: https://kubernetes.io/docs/concepts/storage/persistent-volumes#resources",
								Ref:         ref("k8s.io/api/core/v1.ResourceRequirements"),
							},
						},
						"volumeMounts": {
							VendorExtensible: spec.VendorExtensible{
								Extensions: spec.Extensions{
									"x-kubernetes-patch-merge-key": "mountPath",
									"x-kubernetes-patch-strategy":  "merge",
								},
							},
							SchemaProps: spec.SchemaProps{
								Description: "Pod volumes to mount into the container's filesystem. Cannot be updated.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.VolumeMount"),
										},
									},
								},
							},
						},
						"volumeDevices": {
							VendorExtensible: spec.VendorExtensible{
								Extensions: spec.Extensions{
									"x-kubernetes-patch-merge-key": "devicePath",
									"x-kubernetes-patch-strategy":  "merge",
								},
							},
							SchemaProps: spec.SchemaProps{
								Description: "volumeDevices is the list of block devices to be used by the container. This is an alpha feature and may change in the future.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.VolumeDevice"),
										},
									},
								},
							},
						},
						"livenessProbe": {
							SchemaProps: spec.SchemaProps{
								Description: "Periodic probe of container liveness. Container will be restarted if the probe fails. Cannot be updated. More info: https://kubernetes.io/docs/concepts/workloads/pods/pod-lifecycle#container-probes",
								Ref:         ref("k8s.io/api/core/v1.Probe"),
							},
						},
						"readinessProbe": {
							SchemaProps: spec.SchemaProps{
								Description: "Periodic probe of container service readiness. Container will be removed from service endpoints if the probe fails. Cannot be updated. More info: https://kubernetes.io/docs/concepts/workloads/pods/pod-lifecycle#container-probes",
								Ref:         ref("k8s.io/api/core/v1.Probe"),
							},
						},
						"lifecycle": {
							SchemaProps: spec.SchemaProps{
								Description: "Actions that the management system should take in response to container lifecycle events. Cannot be updated.",
								Ref:         ref("k8s.io/api/core/v1.Lifecycle"),
							},
						},
						"terminationMessagePath": {
							SchemaProps: spec.SchemaProps{
								Description: "Optional: Path at which the file to which the container's termination message will be written is mounted into the container's filesystem. Message written is intended to be brief final status, such as an assertion failure message. Will be truncated by the node if greater than 4096 bytes. The total message length across all containers will be limited to 12kb. Defaults to /dev/termination-log. Cannot be updated.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"terminationMessagePolicy": {
							SchemaProps: spec.SchemaProps{
								Description: "Indicate how the termination message should be populated. File will use the contents of terminationMessagePath to populate the container status message on both success and failure. FallbackToLogsOnError will use the last chunk of container log output if the termination message file is empty and the container exited with an error. The log output is limited to 2048 bytes or 80 lines, whichever is smaller. Defaults to File. Cannot be updated.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"imagePullPolicy": {
							SchemaProps: spec.SchemaProps{
								Description: "Image pull policy. One of Always, Never, IfNotPresent. Defaults to Always if :latest tag is specified, or IfNotPresent otherwise. Cannot be updated. More info: https://kubernetes.io/docs/concepts/containers/images#updating-images",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"securityContext": {
							SchemaProps: spec.SchemaProps{
								Description: "Security options the pod should run with. More info: https://kubernetes.io/docs/concepts/policy/security-context/ More info: https://kubernetes.io/docs/tasks/configure-pod-container/security-context/",
								Ref:         ref("k8s.io/api/core/v1.SecurityContext"),
							},
						},
						"stdin": {
							SchemaProps: spec.SchemaProps{
								Description: "Whether this container should allocate a buffer for stdin in the container runtime. If this is not set, reads from stdin in the container will always result in EOF. Default is false.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"stdinOnce": {
							SchemaProps: spec.SchemaProps{
								Description: "Whether the container runtime should close the stdin channel after it has been opened by a single attach. When stdin is true the stdin stream will remain open across multiple attach sessions. If stdinOnce is set to true, stdin is opened on container start, is empty until the first client attaches to stdin, and then remains open and accepts data until the client disconnects, at which time stdin is closed and remains closed until the container is restarted. If this flag is false, a container processes that reads from stdin will never receive an EOF. Default is false",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"tty": {
							SchemaProps: spec.SchemaProps{
								Description: "Whether this container should allocate a TTY for itself, also requires 'stdin' to be true. Default is false.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
					},
					Required: []string{"name"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.ContainerPort", "k8s.io/api/core/v1.EnvFromSource", "k8s.io/api/core/v1.EnvVar", "k8s.io/api/core/v1.Lifecycle", "k8s.io/api/core/v1.Probe", "k8s.io/api/core/v1.ResourceRequirements", "k8s.io/api/core/v1.SecurityContext", "k8s.io/api/core/v1.VolumeDevice", "k8s.io/api/core/v1.VolumeMount"},
		},
		"k8s.io/api/core/v1.ContainerImage": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Describe a container image",
					Properties: map[string]spec.Schema{
						"names": {
							SchemaProps: spec.SchemaProps{
								Description: "Names by which this image is known. e.g. [\"gcr.io/google_containers/hyperkube:v1.0.7\", \"dockerhub.io/google_containers/hyperkube:v1.0.7\"]",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"sizeBytes": {
							SchemaProps: spec.SchemaProps{
								Description: "The size of the image in bytes.",
								Type:        []string{"integer"},
								Format:      "int64",
							},
						},
					},
					Required: []string{"names"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.ContainerPort": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ContainerPort represents a network port in a single container.",
					Properties: map[string]spec.Schema{
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "If specified, this must be an IANA_SVC_NAME and unique within the pod. Each named port in a pod must have a unique name. Name for the port that can be referred to by services.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"hostPort": {
							SchemaProps: spec.SchemaProps{
								Description: "Number of port to expose on the host. If specified, this must be a valid port number, 0 < x < 65536. If HostNetwork is specified, this must match ContainerPort. Most containers do not need this.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"containerPort": {
							SchemaProps: spec.SchemaProps{
								Description: "Number of port to expose on the pod's IP address. This must be a valid port number, 0 < x < 65536.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"protocol": {
							SchemaProps: spec.SchemaProps{
								Description: "Protocol for port. Must be UDP or TCP. Defaults to \"TCP\".",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"hostIP": {
							SchemaProps: spec.SchemaProps{
								Description: "What host IP to bind the external port to.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"containerPort"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.ContainerState": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ContainerState holds a possible state of container. Only one of its members may be specified. If none of them is specified, the default one is ContainerStateWaiting.",
					Properties: map[string]spec.Schema{
						"waiting": {
							SchemaProps: spec.SchemaProps{
								Description: "Details about a waiting container",
								Ref:         ref("k8s.io/api/core/v1.ContainerStateWaiting"),
							},
						},
						"running": {
							SchemaProps: spec.SchemaProps{
								Description: "Details about a running container",
								Ref:         ref("k8s.io/api/core/v1.ContainerStateRunning"),
							},
						},
						"terminated": {
							SchemaProps: spec.SchemaProps{
								Description: "Details about a terminated container",
								Ref:         ref("k8s.io/api/core/v1.ContainerStateTerminated"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.ContainerStateRunning", "k8s.io/api/core/v1.ContainerStateTerminated", "k8s.io/api/core/v1.ContainerStateWaiting"},
		},
		"k8s.io/api/core/v1.ContainerStateRunning": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ContainerStateRunning is a running state of a container.",
					Properties: map[string]spec.Schema{
						"startedAt": {
							SchemaProps: spec.SchemaProps{
								Description: "Time at which the container was last (re-)started",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.Time"},
		},
		"k8s.io/api/core/v1.ContainerStateTerminated": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ContainerStateTerminated is a terminated state of a container.",
					Properties: map[string]spec.Schema{
						"exitCode": {
							SchemaProps: spec.SchemaProps{
								Description: "Exit status from the last termination of the container",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"signal": {
							SchemaProps: spec.SchemaProps{
								Description: "Signal from the last termination of the container",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"reason": {
							SchemaProps: spec.SchemaProps{
								Description: "(brief) reason from the last termination of the container",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"message": {
							SchemaProps: spec.SchemaProps{
								Description: "Message regarding the last termination of the container",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"startedAt": {
							SchemaProps: spec.SchemaProps{
								Description: "Time at which previous execution of the container started",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"finishedAt": {
							SchemaProps: spec.SchemaProps{
								Description: "Time at which the container last terminated",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"containerID": {
							SchemaProps: spec.SchemaProps{
								Description: "Container's ID in the format 'docker://<container_id>'",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"exitCode"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.Time"},
		},
		"k8s.io/api/core/v1.ContainerStateWaiting": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ContainerStateWaiting is a waiting state of a container.",
					Properties: map[string]spec.Schema{
						"reason": {
							SchemaProps: spec.SchemaProps{
								Description: "(brief) reason the container is not yet running.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"message": {
							SchemaProps: spec.SchemaProps{
								Description: "Message regarding why the container is not yet running.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.ContainerStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ContainerStatus contains details for the current status of this container.",
					Properties: map[string]spec.Schema{
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "This must be a DNS_LABEL. Each container in a pod must have a unique name. Cannot be updated.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"state": {
							SchemaProps: spec.SchemaProps{
								Description: "Details about the container's current condition.",
								Ref:         ref("k8s.io/api/core/v1.ContainerState"),
							},
						},
						"lastState": {
							SchemaProps: spec.SchemaProps{
								Description: "Details about the container's last termination condition.",
								Ref:         ref("k8s.io/api/core/v1.ContainerState"),
							},
						},
						"ready": {
							SchemaProps: spec.SchemaProps{
								Description: "Specifies whether the container has passed its readiness probe.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"restartCount": {
							SchemaProps: spec.SchemaProps{
								Description: "The number of times the container has been restarted, currently based on the number of dead containers that have not yet been removed. Note that this is calculated from dead containers. But those containers are subject to garbage collection. This value will get capped at 5 by GC.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"image": {
							SchemaProps: spec.SchemaProps{
								Description: "The image the container is running. More info: https://kubernetes.io/docs/concepts/containers/images",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"imageID": {
							SchemaProps: spec.SchemaProps{
								Description: "ImageID of the container's image.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"containerID": {
							SchemaProps: spec.SchemaProps{
								Description: "Container's ID in the format 'docker://<container_id>'.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"name", "ready", "restartCount", "image", "imageID"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.ContainerState"},
		},
		"k8s.io/api/core/v1.DaemonEndpoint": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DaemonEndpoint contains information about a single Daemon endpoint.",
					Properties: map[string]spec.Schema{
						"Port": {
							SchemaProps: spec.SchemaProps{
								Description: "Port number of the given endpoint.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
					},
					Required: []string{"Port"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.DownwardAPIProjection": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Represents downward API info for projecting into a projected volume. Note that this is identical to a downwardAPI volume source without the default mode.",
					Properties: map[string]spec.Schema{
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "Items is a list of DownwardAPIVolume file",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.DownwardAPIVolumeFile"),
										},
									},
								},
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.DownwardAPIVolumeFile"},
		},
		"k8s.io/api/core/v1.DownwardAPIVolumeFile": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DownwardAPIVolumeFile represents information to create the file containing the pod field",
					Properties: map[string]spec.Schema{
						"path": {
							SchemaProps: spec.SchemaProps{
								Description: "Required: Path is  the relative path name of the file to be created. Must not be absolute or contain the '..' path. Must be utf-8 encoded. The first item of the relative path must not start with '..'",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"fieldRef": {
							SchemaProps: spec.SchemaProps{
								Description: "Required: Selects a field of the pod: only annotations, labels, name and namespace are supported.",
								Ref:         ref("k8s.io/api/core/v1.ObjectFieldSelector"),
							},
						},
						"resourceFieldRef": {
							SchemaProps: spec.SchemaProps{
								Description: "Selects a resource of the container: only resources limits and requests (limits.cpu, limits.memory, requests.cpu and requests.memory) are currently supported.",
								Ref:         ref("k8s.io/api/core/v1.ResourceFieldSelector"),
							},
						},
						"mode": {
							SchemaProps: spec.SchemaProps{
								Description: "Optional: mode bits to use on this file, must be a value between 0 and 0777. If not specified, the volume defaultMode will be used. This might be in conflict with other options that affect the file mode, like fsGroup, and the result can be other mode bits set.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
					},
					Required: []string{"path"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.ObjectFieldSelector", "k8s.io/api/core/v1.ResourceFieldSelector"},
		},
		"k8s.io/api/core/v1.DownwardAPIVolumeSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DownwardAPIVolumeSource represents a volume containing downward API info. Downward API volumes support ownership management and SELinux relabeling.",
					Properties: map[string]spec.Schema{
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "Items is a list of downward API volume file",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.DownwardAPIVolumeFile"),
										},
									},
								},
							},
						},
						"defaultMode": {
							SchemaProps: spec.SchemaProps{
								Description: "Optional: mode bits to use on created files by default. Must be a value between 0 and 0777. Defaults to 0644. Directories within the path are not affected by this setting. This might be in conflict with other options that affect the file mode, like fsGroup, and the result can be other mode bits set.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.DownwardAPIVolumeFile"},
		},
		"k8s.io/api/core/v1.EmptyDirVolumeSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Represents an empty directory for a pod. Empty directory volumes support ownership management and SELinux relabeling.",
					Properties: map[string]spec.Schema{
						"medium": {
							SchemaProps: spec.SchemaProps{
								Description: "What type of storage medium should back this directory. The default is \"\" which means to use the node's default medium. Must be an empty string (default) or Memory. More info: https://kubernetes.io/docs/concepts/storage/volumes#emptydir",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"sizeLimit": {
							SchemaProps: spec.SchemaProps{
								Description: "Total amount of local storage required for this EmptyDir volume. The size limit is also applicable for memory medium. The maximum usage on memory medium EmptyDir would be the minimum value between the SizeLimit specified here and the sum of memory limits of all containers in a pod. The default is nil which means that the limit is undefined. More info: http://kubernetes.io/docs/user-guide/volumes#emptydir",
								Ref:         ref("k8s.io/apimachinery/pkg/api/resource.Quantity"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/api/resource.Quantity"},
		},
		"k8s.io/api/core/v1.EndpointAddress": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "EndpointAddress is a tuple that describes single IP address.",
					Properties: map[string]spec.Schema{
						"ip": {
							SchemaProps: spec.SchemaProps{
								Description: "The IP of this endpoint. May not be loopback (127.0.0.0/8), link-local (169.254.0.0/16), or link-local multicast ((224.0.0.0/24). IPv6 is also accepted but not fully supported on all platforms. Also, certain kubernetes components, like kube-proxy, are not IPv6 ready.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"hostname": {
							SchemaProps: spec.SchemaProps{
								Description: "The Hostname of this endpoint",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"nodeName": {
							SchemaProps: spec.SchemaProps{
								Description: "Optional: Node hosting this endpoint. This can be used to determine endpoints local to a node.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"targetRef": {
							SchemaProps: spec.SchemaProps{
								Description: "Reference to object providing the endpoint.",
								Ref:         ref("k8s.io/api/core/v1.ObjectReference"),
							},
						},
					},
					Required: []string{"ip"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.ObjectReference"},
		},
		"k8s.io/api/core/v1.EndpointPort": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "EndpointPort is a tuple that describes a single port.",
					Properties: map[string]spec.Schema{
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "The name of this port (corresponds to ServicePort.Name). Must be a DNS_LABEL. Optional only if one port is defined.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"port": {
							SchemaProps: spec.SchemaProps{
								Description: "The port number of the endpoint.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"protocol": {
							SchemaProps: spec.SchemaProps{
								Description: "The IP protocol for this port. Must be UDP or TCP. Default is TCP.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"port"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.EndpointSubset": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "EndpointSubset is a group of addresses with a common set of ports. The expanded set of endpoints is the Cartesian product of Addresses x Ports. For example, given:\n  {\n    Addresses: [{\"ip\": \"10.10.1.1\"}, {\"ip\": \"10.10.2.2\"}],\n    Ports:     [{\"name\": \"a\", \"port\": 8675}, {\"name\": \"b\", \"port\": 309}]\n  }\nThe resulting set of endpoints can be viewed as:\n    a: [ 10.10.1.1:8675, 10.10.2.2:8675 ],\n    b: [ 10.10.1.1:309, 10.10.2.2:309 ]",
					Properties: map[string]spec.Schema{
						"addresses": {
							SchemaProps: spec.SchemaProps{
								Description: "IP addresses which offer the related ports that are marked as ready. These endpoints should be considered safe for load balancers and clients to utilize.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.EndpointAddress"),
										},
									},
								},
							},
						},
						"notReadyAddresses": {
							SchemaProps: spec.SchemaProps{
								Description: "IP addresses which offer the related ports but are not currently marked as ready because they have not yet finished starting, have recently failed a readiness check, or have recently failed a liveness check.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.EndpointAddress"),
										},
									},
								},
							},
						},
						"ports": {
							SchemaProps: spec.SchemaProps{
								Description: "Port numbers available on the related IP addresses.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.EndpointPort"),
										},
									},
								},
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.EndpointAddress", "k8s.io/api/core/v1.EndpointPort"},
		},
		"k8s.io/api/core/v1.Endpoints": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Endpoints is a collection of endpoints that implement the actual service. Example:\n  Name: \"mysvc\",\n  Subsets: [\n    {\n      Addresses: [{\"ip\": \"10.10.1.1\"}, {\"ip\": \"10.10.2.2\"}],\n      Ports: [{\"name\": \"a\", \"port\": 8675}, {\"name\": \"b\", \"port\": 309}]\n    },\n    {\n      Addresses: [{\"ip\": \"10.10.3.3\"}],\n      Ports: [{\"name\": \"a\", \"port\": 93}, {\"name\": \"b\", \"port\": 76}]\n    },\n ]",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"subsets": {
							SchemaProps: spec.SchemaProps{
								Description: "The set of all endpoints is the union of all subsets. Addresses are placed into subsets according to the IPs they share. A single address with multiple ports, some of which are ready and some of which are not (because they come from different containers) will result in the address being displayed in different subsets for the different ports. No address will appear in both Addresses and NotReadyAddresses in the same subset. Sets of addresses and ports that comprise a service.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.EndpointSubset"),
										},
									},
								},
							},
						},
					},
					Required: []string{"subsets"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.EndpointSubset", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/core/v1.EndpointsList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "EndpointsList is a list of endpoints.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "List of endpoints.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.Endpoints"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.Endpoints", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/core/v1.EnvFromSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "EnvFromSource represents the source of a set of ConfigMaps",
					Properties: map[string]spec.Schema{
						"prefix": {
							SchemaProps: spec.SchemaProps{
								Description: "An optional identifer to prepend to each key in the ConfigMap. Must be a C_IDENTIFIER.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"configMapRef": {
							SchemaProps: spec.SchemaProps{
								Description: "The ConfigMap to select from",
								Ref:         ref("k8s.io/api/core/v1.ConfigMapEnvSource"),
							},
						},
						"secretRef": {
							SchemaProps: spec.SchemaProps{
								Description: "The Secret to select from",
								Ref:         ref("k8s.io/api/core/v1.SecretEnvSource"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.ConfigMapEnvSource", "k8s.io/api/core/v1.SecretEnvSource"},
		},
		"k8s.io/api/core/v1.EnvVar": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "EnvVar represents an environment variable present in a Container.",
					Properties: map[string]spec.Schema{
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "Name of the environment variable. Must be a C_IDENTIFIER.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"value": {
							SchemaProps: spec.SchemaProps{
								Description: "Variable references $(VAR_NAME) are expanded using the previous defined environment variables in the container and any service environment variables. If a variable cannot be resolved, the reference in the input string will be unchanged. The $(VAR_NAME) syntax can be escaped with a double $$, ie: $$(VAR_NAME). Escaped references will never be expanded, regardless of whether the variable exists or not. Defaults to \"\".",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"valueFrom": {
							SchemaProps: spec.SchemaProps{
								Description: "Source for the environment variable's value. Cannot be used if value is not empty.",
								Ref:         ref("k8s.io/api/core/v1.EnvVarSource"),
							},
						},
					},
					Required: []string{"name"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.EnvVarSource"},
		},
		"k8s.io/api/core/v1.EnvVarSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "EnvVarSource represents a source for the value of an EnvVar.",
					Properties: map[string]spec.Schema{
						"fieldRef": {
							SchemaProps: spec.SchemaProps{
								Description: "Selects a field of the pod: supports metadata.name, metadata.namespace, metadata.labels, metadata.annotations, spec.nodeName, spec.serviceAccountName, status.hostIP, status.podIP.",
								Ref:         ref("k8s.io/api/core/v1.ObjectFieldSelector"),
							},
						},
						"resourceFieldRef": {
							SchemaProps: spec.SchemaProps{
								Description: "Selects a resource of the container: only resources limits and requests (limits.cpu, limits.memory, limits.ephemeral-storage, requests.cpu, requests.memory and requests.ephemeral-storage) are currently supported.",
								Ref:         ref("k8s.io/api/core/v1.ResourceFieldSelector"),
							},
						},
						"configMapKeyRef": {
							SchemaProps: spec.SchemaProps{
								Description: "Selects a key of a ConfigMap.",
								Ref:         ref("k8s.io/api/core/v1.ConfigMapKeySelector"),
							},
						},
						"secretKeyRef": {
							SchemaProps: spec.SchemaProps{
								Description: "Selects a key of a secret in the pod's namespace",
								Ref:         ref("k8s.io/api/core/v1.SecretKeySelector"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.ConfigMapKeySelector", "k8s.io/api/core/v1.ObjectFieldSelector", "k8s.io/api/core/v1.ResourceFieldSelector", "k8s.io/api/core/v1.SecretKeySelector"},
		},
		"k8s.io/api/core/v1.Event": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Event is a report of an event somewhere in the cluster.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"involvedObject": {
							SchemaProps: spec.SchemaProps{
								Description: "The object that this event is about.",
								Ref:         ref("k8s.io/api/core/v1.ObjectReference"),
							},
						},
						"reason": {
							SchemaProps: spec.SchemaProps{
								Description: "This should be a short, machine understandable string that gives the reason for the transition into the object's current status.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"message": {
							SchemaProps: spec.SchemaProps{
								Description: "A human-readable description of the status of this operation.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"source": {
							SchemaProps: spec.SchemaProps{
								Description: "The component reporting this event. Should be a short machine understandable string.",
								Ref:         ref("k8s.io/api/core/v1.EventSource"),
							},
						},
						"firstTimestamp": {
							SchemaProps: spec.SchemaProps{
								Description: "The time at which the event was first recorded. (Time of server receipt is in TypeMeta.)",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"lastTimestamp": {
							SchemaProps: spec.SchemaProps{
								Description: "The time at which the most recent occurrence of this event was recorded.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"count": {
							SchemaProps: spec.SchemaProps{
								Description: "The number of times this event has occurred.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"type": {
							SchemaProps: spec.SchemaProps{
								Description: "Type of this event (Normal, Warning), new types could be added in the future",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"eventTime": {
							SchemaProps: spec.SchemaProps{
								Description: "Time when this Event was first observed.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.MicroTime"),
							},
						},
						"series": {
							SchemaProps: spec.SchemaProps{
								Description: "Data about the Event series this event represents or nil if it's a singleton Event.",
								Ref:         ref("k8s.io/api/core/v1.EventSeries"),
							},
						},
						"action": {
							SchemaProps: spec.SchemaProps{
								Description: "What action was taken/failed regarding to the Regarding object.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"related": {
							SchemaProps: spec.SchemaProps{
								Description: "Optional secondary object for more complex actions.",
								Ref:         ref("k8s.io/api/core/v1.ObjectReference"),
							},
						},
						"reportingComponent": {
							SchemaProps: spec.SchemaProps{
								Description: "Name of the controller that emitted this Event, e.g. `kubernetes.io/kubelet`.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"reportingInstance": {
							SchemaProps: spec.SchemaProps{
								Description: "ID of the controller instance, e.g. `kubelet-xyzf`.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"metadata", "involvedObject"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.EventSeries", "k8s.io/api/core/v1.EventSource", "k8s.io/api/core/v1.ObjectReference", "k8s.io/apimachinery/pkg/apis/meta/v1.MicroTime", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta", "k8s.io/apimachinery/pkg/apis/meta/v1.Time"},
		},
		"k8s.io/api/core/v1.EventList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "EventList is a list of events.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "List of events",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.Event"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.Event", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/core/v1.EventSeries": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "EventSeries contain information on series of events, i.e. thing that was/is happening continously for some time.",
					Properties: map[string]spec.Schema{
						"count": {
							SchemaProps: spec.SchemaProps{
								Description: "Number of occurrences in this series up to the last heartbeat time",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"lastObservedTime": {
							SchemaProps: spec.SchemaProps{
								Description: "Time of the last occurence observed",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.MicroTime"),
							},
						},
						"state": {
							SchemaProps: spec.SchemaProps{
								Description: "State of this Series: Ongoing or Finished",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.MicroTime"},
		},
		"k8s.io/api/core/v1.EventSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "EventSource contains information for an event.",
					Properties: map[string]spec.Schema{
						"component": {
							SchemaProps: spec.SchemaProps{
								Description: "Component from which the event is generated.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"host": {
							SchemaProps: spec.SchemaProps{
								Description: "Node name on which the event is generated.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.ExecAction": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ExecAction describes a \"run in container\" action.",
					Properties: map[string]spec.Schema{
						"command": {
							SchemaProps: spec.SchemaProps{
								Description: "Command is the command line to execute inside the container, the working directory for the command  is root ('/') in the container's filesystem. The command is simply exec'd, it is not run inside a shell, so traditional shell instructions ('|', etc) won't work. To use a shell, you need to explicitly call out to that shell. Exit status of 0 is treated as live/healthy and non-zero is unhealthy.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.FCVolumeSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Represents a Fibre Channel volume. Fibre Channel volumes can only be mounted as read/write once. Fibre Channel volumes support ownership management and SELinux relabeling.",
					Properties: map[string]spec.Schema{
						"targetWWNs": {
							SchemaProps: spec.SchemaProps{
								Description: "Optional: FC target worldwide names (WWNs)",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"lun": {
							SchemaProps: spec.SchemaProps{
								Description: "Optional: FC target lun number",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"fsType": {
							SchemaProps: spec.SchemaProps{
								Description: "Filesystem type to mount. Must be a filesystem type supported by the host operating system. Ex. \"ext4\", \"xfs\", \"ntfs\". Implicitly inferred to be \"ext4\" if unspecified.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"readOnly": {
							SchemaProps: spec.SchemaProps{
								Description: "Optional: Defaults to false (read/write). ReadOnly here will force the ReadOnly setting in VolumeMounts.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"wwids": {
							SchemaProps: spec.SchemaProps{
								Description: "Optional: FC volume world wide identifiers (wwids) Either wwids or combination of targetWWNs and lun must be set, but not both simultaneously.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.FlexVolumeSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "FlexVolume represents a generic volume resource that is provisioned/attached using an exec based plugin.",
					Properties: map[string]spec.Schema{
						"driver": {
							SchemaProps: spec.SchemaProps{
								Description: "Driver is the name of the driver to use for this volume.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"fsType": {
							SchemaProps: spec.SchemaProps{
								Description: "Filesystem type to mount. Must be a filesystem type supported by the host operating system. Ex. \"ext4\", \"xfs\", \"ntfs\". The default filesystem depends on FlexVolume script.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"secretRef": {
							SchemaProps: spec.SchemaProps{
								Description: "Optional: SecretRef is reference to the secret object containing sensitive information to pass to the plugin scripts. This may be empty if no secret object is specified. If the secret object contains more than one secret, all secrets are passed to the plugin scripts.",
								Ref:         ref("k8s.io/api/core/v1.LocalObjectReference"),
							},
						},
						"readOnly": {
							SchemaProps: spec.SchemaProps{
								Description: "Optional: Defaults to false (read/write). ReadOnly here will force the ReadOnly setting in VolumeMounts.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"options": {
							SchemaProps: spec.SchemaProps{
								Description: "Optional: Extra command options if any.",
								Type:        []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
					},
					Required: []string{"driver"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.LocalObjectReference"},
		},
		"k8s.io/api/core/v1.FlockerVolumeSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Represents a Flocker volume mounted by the Flocker agent. One and only one of datasetName and datasetUUID should be set. Flocker volumes do not support ownership management or SELinux relabeling.",
					Properties: map[string]spec.Schema{
						"datasetName": {
							SchemaProps: spec.SchemaProps{
								Description: "Name of the dataset stored as metadata -> name on the dataset for Flocker should be considered as deprecated",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"datasetUUID": {
							SchemaProps: spec.SchemaProps{
								Description: "UUID of the dataset. This is unique identifier of a Flocker dataset",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.GCEPersistentDiskVolumeSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Represents a Persistent Disk resource in Google Compute Engine.\n\nA GCE PD must exist before mounting to a container. The disk must also be in the same GCE project and zone as the kubelet. A GCE PD can only be mounted as read/write once or read-only many times. GCE PDs support ownership management and SELinux relabeling.",
					Properties: map[string]spec.Schema{
						"pdName": {
							SchemaProps: spec.SchemaProps{
								Description: "Unique name of the PD resource in GCE. Used to identify the disk in GCE. More info: https://kubernetes.io/docs/concepts/storage/volumes#gcepersistentdisk",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"fsType": {
							SchemaProps: spec.SchemaProps{
								Description: "Filesystem type of the volume that you want to mount. Tip: Ensure that the filesystem type is supported by the host operating system. Examples: \"ext4\", \"xfs\", \"ntfs\". Implicitly inferred to be \"ext4\" if unspecified. More info: https://kubernetes.io/docs/concepts/storage/volumes#gcepersistentdisk",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"partition": {
							SchemaProps: spec.SchemaProps{
								Description: "The partition in the volume that you want to mount. If omitted, the default is to mount by volume name. Examples: For volume /dev/sda1, you specify the partition as \"1\". Similarly, the volume partition for /dev/sda is \"0\" (or you can leave the property empty). More info: https://kubernetes.io/docs/concepts/storage/volumes#gcepersistentdisk",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"readOnly": {
							SchemaProps: spec.SchemaProps{
								Description: "ReadOnly here will force the ReadOnly setting in VolumeMounts. Defaults to false. More info: https://kubernetes.io/docs/concepts/storage/volumes#gcepersistentdisk",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
					},
					Required: []string{"pdName"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.GitRepoVolumeSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Represents a volume that is populated with the contents of a git repository. Git repo volumes do not support ownership management. Git repo volumes support SELinux relabeling.",
					Properties: map[string]spec.Schema{
						"repository": {
							SchemaProps: spec.SchemaProps{
								Description: "Repository URL",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"revision": {
							SchemaProps: spec.SchemaProps{
								Description: "Commit hash for the specified revision.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"directory": {
							SchemaProps: spec.SchemaProps{
								Description: "Target directory name. Must not contain or start with '..'.  If '.' is supplied, the volume directory will be the git repository.  Otherwise, if specified, the volume will contain the git repository in the subdirectory with the given name.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"repository"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.GlusterfsVolumeSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Represents a Glusterfs mount that lasts the lifetime of a pod. Glusterfs volumes do not support ownership management or SELinux relabeling.",
					Properties: map[string]spec.Schema{
						"endpoints": {
							SchemaProps: spec.SchemaProps{
								Description: "EndpointsName is the endpoint name that details Glusterfs topology. More info: https://releases.k8s.io/HEAD/examples/volumes/glusterfs/README.md#create-a-pod",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"path": {
							SchemaProps: spec.SchemaProps{
								Description: "Path is the Glusterfs volume path. More info: https://releases.k8s.io/HEAD/examples/volumes/glusterfs/README.md#create-a-pod",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"readOnly": {
							SchemaProps: spec.SchemaProps{
								Description: "ReadOnly here will force the Glusterfs volume to be mounted with read-only permissions. Defaults to false. More info: https://releases.k8s.io/HEAD/examples/volumes/glusterfs/README.md#create-a-pod",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
					},
					Required: []string{"endpoints", "path"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.HTTPGetAction": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "HTTPGetAction describes an action based on HTTP Get requests.",
					Properties: map[string]spec.Schema{
						"path": {
							SchemaProps: spec.SchemaProps{
								Description: "Path to access on the HTTP server.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"port": {
							SchemaProps: spec.SchemaProps{
								Description: "Name or number of the port to access on the container. Number must be in the range 1 to 65535. Name must be an IANA_SVC_NAME.",
								Ref:         ref("k8s.io/apimachinery/pkg/util/intstr.IntOrString"),
							},
						},
						"host": {
							SchemaProps: spec.SchemaProps{
								Description: "Host name to connect to, defaults to the pod IP. You probably want to set \"Host\" in httpHeaders instead.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"scheme": {
							SchemaProps: spec.SchemaProps{
								Description: "Scheme to use for connecting to the host. Defaults to HTTP.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"httpHeaders": {
							SchemaProps: spec.SchemaProps{
								Description: "Custom headers to set in the request. HTTP allows repeated headers.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.HTTPHeader"),
										},
									},
								},
							},
						},
					},
					Required: []string{"port"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.HTTPHeader", "k8s.io/apimachinery/pkg/util/intstr.IntOrString"},
		},
		"k8s.io/api/core/v1.HTTPHeader": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "HTTPHeader describes a custom header to be used in HTTP probes",
					Properties: map[string]spec.Schema{
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "The header field name",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"value": {
							SchemaProps: spec.SchemaProps{
								Description: "The header field value",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"name", "value"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.Handler": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Handler defines a specific action that should be taken",
					Properties: map[string]spec.Schema{
						"exec": {
							SchemaProps: spec.SchemaProps{
								Description: "One and only one of the following should be specified. Exec specifies the action to take.",
								Ref:         ref("k8s.io/api/core/v1.ExecAction"),
							},
						},
						"httpGet": {
							SchemaProps: spec.SchemaProps{
								Description: "HTTPGet specifies the http request to perform.",
								Ref:         ref("k8s.io/api/core/v1.HTTPGetAction"),
							},
						},
						"tcpSocket": {
							SchemaProps: spec.SchemaProps{
								Description: "TCPSocket specifies an action involving a TCP port. TCP hooks not yet supported",
								Ref:         ref("k8s.io/api/core/v1.TCPSocketAction"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.ExecAction", "k8s.io/api/core/v1.HTTPGetAction", "k8s.io/api/core/v1.TCPSocketAction"},
		},
		"k8s.io/api/core/v1.HostAlias": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "HostAlias holds the mapping between IP and hostnames that will be injected as an entry in the pod's hosts file.",
					Properties: map[string]spec.Schema{
						"ip": {
							SchemaProps: spec.SchemaProps{
								Description: "IP address of the host file entry.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"hostnames": {
							SchemaProps: spec.SchemaProps{
								Description: "Hostnames for the above IP address.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.HostPathVolumeSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Represents a host path mapped into a pod. Host path volumes do not support ownership management or SELinux relabeling.",
					Properties: map[string]spec.Schema{
						"path": {
							SchemaProps: spec.SchemaProps{
								Description: "Path of the directory on the host. If the path is a symlink, it will follow the link to the real path. More info: https://kubernetes.io/docs/concepts/storage/volumes#hostpath",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"type": {
							SchemaProps: spec.SchemaProps{
								Description: "Type for HostPath Volume Defaults to \"\" More info: https://kubernetes.io/docs/concepts/storage/volumes#hostpath",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"path"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.ISCSIPersistentVolumeSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ISCSIPersistentVolumeSource represents an ISCSI disk. ISCSI volumes can only be mounted as read/write once. ISCSI volumes support ownership management and SELinux relabeling.",
					Properties: map[string]spec.Schema{
						"targetPortal": {
							SchemaProps: spec.SchemaProps{
								Description: "iSCSI Target Portal. The Portal is either an IP or ip_addr:port if the port is other than default (typically TCP ports 860 and 3260).",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"iqn": {
							SchemaProps: spec.SchemaProps{
								Description: "Target iSCSI Qualified Name.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"lun": {
							SchemaProps: spec.SchemaProps{
								Description: "iSCSI Target Lun number.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"iscsiInterface": {
							SchemaProps: spec.SchemaProps{
								Description: "iSCSI Interface Name that uses an iSCSI transport. Defaults to 'default' (tcp).",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"fsType": {
							SchemaProps: spec.SchemaProps{
								Description: "Filesystem type of the volume that you want to mount. Tip: Ensure that the filesystem type is supported by the host operating system. Examples: \"ext4\", \"xfs\", \"ntfs\". Implicitly inferred to be \"ext4\" if unspecified. More info: https://kubernetes.io/docs/concepts/storage/volumes#iscsi",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"readOnly": {
							SchemaProps: spec.SchemaProps{
								Description: "ReadOnly here will force the ReadOnly setting in VolumeMounts. Defaults to false.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"portals": {
							SchemaProps: spec.SchemaProps{
								Description: "iSCSI Target Portal List. The Portal is either an IP or ip_addr:port if the port is other than default (typically TCP ports 860 and 3260).",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"chapAuthDiscovery": {
							SchemaProps: spec.SchemaProps{
								Description: "whether support iSCSI Discovery CHAP authentication",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"chapAuthSession": {
							SchemaProps: spec.SchemaProps{
								Description: "whether support iSCSI Session CHAP authentication",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"secretRef": {
							SchemaProps: spec.SchemaProps{
								Description: "CHAP Secret for iSCSI target and initiator authentication",
								Ref:         ref("k8s.io/api/core/v1.SecretReference"),
							},
						},
						"initiatorName": {
							SchemaProps: spec.SchemaProps{
								Description: "Custom iSCSI Initiator Name. If initiatorName is specified with iscsiInterface simultaneously, new iSCSI interface <target portal>:<volume name> will be created for the connection.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"targetPortal", "iqn", "lun"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.SecretReference"},
		},
		"k8s.io/api/core/v1.ISCSIVolumeSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Represents an ISCSI disk. ISCSI volumes can only be mounted as read/write once. ISCSI volumes support ownership management and SELinux relabeling.",
					Properties: map[string]spec.Schema{
						"targetPortal": {
							SchemaProps: spec.SchemaProps{
								Description: "iSCSI Target Portal. The Portal is either an IP or ip_addr:port if the port is other than default (typically TCP ports 860 and 3260).",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"iqn": {
							SchemaProps: spec.SchemaProps{
								Description: "Target iSCSI Qualified Name.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"lun": {
							SchemaProps: spec.SchemaProps{
								Description: "iSCSI Target Lun number.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"iscsiInterface": {
							SchemaProps: spec.SchemaProps{
								Description: "iSCSI Interface Name that uses an iSCSI transport. Defaults to 'default' (tcp).",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"fsType": {
							SchemaProps: spec.SchemaProps{
								Description: "Filesystem type of the volume that you want to mount. Tip: Ensure that the filesystem type is supported by the host operating system. Examples: \"ext4\", \"xfs\", \"ntfs\". Implicitly inferred to be \"ext4\" if unspecified. More info: https://kubernetes.io/docs/concepts/storage/volumes#iscsi",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"readOnly": {
							SchemaProps: spec.SchemaProps{
								Description: "ReadOnly here will force the ReadOnly setting in VolumeMounts. Defaults to false.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"portals": {
							SchemaProps: spec.SchemaProps{
								Description: "iSCSI Target Portal List. The portal is either an IP or ip_addr:port if the port is other than default (typically TCP ports 860 and 3260).",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"chapAuthDiscovery": {
							SchemaProps: spec.SchemaProps{
								Description: "whether support iSCSI Discovery CHAP authentication",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"chapAuthSession": {
							SchemaProps: spec.SchemaProps{
								Description: "whether support iSCSI Session CHAP authentication",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"secretRef": {
							SchemaProps: spec.SchemaProps{
								Description: "CHAP Secret for iSCSI target and initiator authentication",
								Ref:         ref("k8s.io/api/core/v1.LocalObjectReference"),
							},
						},
						"initiatorName": {
							SchemaProps: spec.SchemaProps{
								Description: "Custom iSCSI Initiator Name. If initiatorName is specified with iscsiInterface simultaneously, new iSCSI interface <target portal>:<volume name> will be created for the connection.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"targetPortal", "iqn", "lun"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.LocalObjectReference"},
		},
		"k8s.io/api/core/v1.KeyToPath": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Maps a string key to a path within a volume.",
					Properties: map[string]spec.Schema{
						"key": {
							SchemaProps: spec.SchemaProps{
								Description: "The key to project.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"path": {
							SchemaProps: spec.SchemaProps{
								Description: "The relative path of the file to map the key to. May not be an absolute path. May not contain the path element '..'. May not start with the string '..'.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"mode": {
							SchemaProps: spec.SchemaProps{
								Description: "Optional: mode bits to use on this file, must be a value between 0 and 0777. If not specified, the volume defaultMode will be used. This might be in conflict with other options that affect the file mode, like fsGroup, and the result can be other mode bits set.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
					},
					Required: []string{"key", "path"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.Lifecycle": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Lifecycle describes actions that the management system should take in response to container lifecycle events. For the PostStart and PreStop lifecycle handlers, management of the container blocks until the action is complete, unless the container process fails, in which case the handler is aborted.",
					Properties: map[string]spec.Schema{
						"postStart": {
							SchemaProps: spec.SchemaProps{
								Description: "PostStart is called immediately after a container is created. If the handler fails, the container is terminated and restarted according to its restart policy. Other management of the container blocks until the hook completes. More info: https://kubernetes.io/docs/concepts/containers/container-lifecycle-hooks/#container-hooks",
								Ref:         ref("k8s.io/api/core/v1.Handler"),
							},
						},
						"preStop": {
							SchemaProps: spec.SchemaProps{
								Description: "PreStop is called immediately before a container is terminated. The container is terminated after the handler completes. The reason for termination is passed to the handler. Regardless of the outcome of the handler, the container is eventually terminated. Other management of the container blocks until the hook completes. More info: https://kubernetes.io/docs/concepts/containers/container-lifecycle-hooks/#container-hooks",
								Ref:         ref("k8s.io/api/core/v1.Handler"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.Handler"},
		},
		"k8s.io/api/core/v1.LimitRange": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "LimitRange sets resource usage limits for each kind of resource in a Namespace.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "Spec defines the limits enforced. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status",
								Ref:         ref("k8s.io/api/core/v1.LimitRangeSpec"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.LimitRangeSpec", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/core/v1.LimitRangeItem": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "LimitRangeItem defines a min/max usage limit for any resource that matches on kind.",
					Properties: map[string]spec.Schema{
						"type": {
							SchemaProps: spec.SchemaProps{
								Description: "Type of resource that this limit applies to.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"max": {
							SchemaProps: spec.SchemaProps{
								Description: "Max usage constraints on this kind by resource name.",
								Type:        []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apimachinery/pkg/api/resource.Quantity"),
										},
									},
								},
							},
						},
						"min": {
							SchemaProps: spec.SchemaProps{
								Description: "Min usage constraints on this kind by resource name.",
								Type:        []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apimachinery/pkg/api/resource.Quantity"),
										},
									},
								},
							},
						},
						"default": {
							SchemaProps: spec.SchemaProps{
								Description: "Default resource requirement limit value by resource name if resource limit is omitted.",
								Type:        []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apimachinery/pkg/api/resource.Quantity"),
										},
									},
								},
							},
						},
						"defaultRequest": {
							SchemaProps: spec.SchemaProps{
								Description: "DefaultRequest is the default resource requirement request value by resource name if resource request is omitted.",
								Type:        []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apimachinery/pkg/api/resource.Quantity"),
										},
									},
								},
							},
						},
						"maxLimitRequestRatio": {
							SchemaProps: spec.SchemaProps{
								Description: "MaxLimitRequestRatio if specified, the named resource must have a request and limit that are both non-zero where limit divided by request is less than or equal to the enumerated value; this represents the max burst for the named resource.",
								Type:        []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apimachinery/pkg/api/resource.Quantity"),
										},
									},
								},
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/api/resource.Quantity"},
		},
		"k8s.io/api/core/v1.LimitRangeList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "LimitRangeList is a list of LimitRange items.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "Items is a list of LimitRange objects. More info: https://kubernetes.io/docs/concepts/configuration/manage-compute-resources-container/",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.LimitRange"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.LimitRange", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/core/v1.LimitRangeSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "LimitRangeSpec defines a min/max usage limit for resources that match on kind.",
					Properties: map[string]spec.Schema{
						"limits": {
							SchemaProps: spec.SchemaProps{
								Description: "Limits is the list of LimitRangeItem objects that are enforced.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.LimitRangeItem"),
										},
									},
								},
							},
						},
					},
					Required: []string{"limits"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.LimitRangeItem"},
		},
		"k8s.io/api/core/v1.List": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "List holds a list of objects, which may not be known by the server.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "List of objects",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apimachinery/pkg/runtime.RawExtension"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta", "k8s.io/apimachinery/pkg/runtime.RawExtension"},
		},
		"k8s.io/api/core/v1.LoadBalancerIngress": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "LoadBalancerIngress represents the status of a load-balancer ingress point: traffic intended for the service should be sent to an ingress point.",
					Properties: map[string]spec.Schema{
						"ip": {
							SchemaProps: spec.SchemaProps{
								Description: "IP is set for load-balancer ingress points that are IP based (typically GCE or OpenStack load-balancers)",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"hostname": {
							SchemaProps: spec.SchemaProps{
								Description: "Hostname is set for load-balancer ingress points that are DNS based (typically AWS load-balancers)",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.LoadBalancerStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "LoadBalancerStatus represents the status of a load-balancer.",
					Properties: map[string]spec.Schema{
						"ingress": {
							SchemaProps: spec.SchemaProps{
								Description: "Ingress is a list containing ingress points for the load-balancer. Traffic intended for the service should be sent to these ingress points.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.LoadBalancerIngress"),
										},
									},
								},
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.LoadBalancerIngress"},
		},
		"k8s.io/api/core/v1.LocalObjectReference": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "LocalObjectReference contains enough information to let you locate the referenced object inside the same namespace.",
					Properties: map[string]spec.Schema{
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "Name of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/names/#names",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.LocalVolumeSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Local represents directly-attached storage with node affinity",
					Properties: map[string]spec.Schema{
						"path": {
							SchemaProps: spec.SchemaProps{
								Description: "The full path to the volume on the node For alpha, this path must be a directory Once block as a source is supported, then this path can point to a block device",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"path"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.NFSVolumeSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Represents an NFS mount that lasts the lifetime of a pod. NFS volumes do not support ownership management or SELinux relabeling.",
					Properties: map[string]spec.Schema{
						"server": {
							SchemaProps: spec.SchemaProps{
								Description: "Server is the hostname or IP address of the NFS server. More info: https://kubernetes.io/docs/concepts/storage/volumes#nfs",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"path": {
							SchemaProps: spec.SchemaProps{
								Description: "Path that is exported by the NFS server. More info: https://kubernetes.io/docs/concepts/storage/volumes#nfs",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"readOnly": {
							SchemaProps: spec.SchemaProps{
								Description: "ReadOnly here will force the NFS export to be mounted with read-only permissions. Defaults to false. More info: https://kubernetes.io/docs/concepts/storage/volumes#nfs",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
					},
					Required: []string{"server", "path"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.Namespace": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Namespace provides a scope for Names. Use of multiple namespaces is optional.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "Spec defines the behavior of the Namespace. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status",
								Ref:         ref("k8s.io/api/core/v1.NamespaceSpec"),
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Status describes the current status of a Namespace. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status",
								Ref:         ref("k8s.io/api/core/v1.NamespaceStatus"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.NamespaceSpec", "k8s.io/api/core/v1.NamespaceStatus", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/core/v1.NamespaceList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "NamespaceList is a list of Namespaces.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "Items is the list of Namespace objects in the list. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/namespaces/",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.Namespace"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.Namespace", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/core/v1.NamespaceSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "NamespaceSpec describes the attributes on a Namespace.",
					Properties: map[string]spec.Schema{
						"finalizers": {
							SchemaProps: spec.SchemaProps{
								Description: "Finalizers is an opaque list of values that must be empty to permanently remove object from storage. More info: https://kubernetes.io/docs/tasks/administer-cluster/namespaces/",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.NamespaceStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "NamespaceStatus is information about the current status of a Namespace.",
					Properties: map[string]spec.Schema{
						"phase": {
							SchemaProps: spec.SchemaProps{
								Description: "Phase is the current lifecycle phase of the namespace. More info: https://kubernetes.io/docs/tasks/administer-cluster/namespaces/",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.Node": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Node is a worker node in Kubernetes. Each node will have a unique identifier in the cache (i.e. in etcd).",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "Spec defines the behavior of a node. https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status",
								Ref:         ref("k8s.io/api/core/v1.NodeSpec"),
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Most recently observed status of the node. Populated by the system. Read-only. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status",
								Ref:         ref("k8s.io/api/core/v1.NodeStatus"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.NodeSpec", "k8s.io/api/core/v1.NodeStatus", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/core/v1.NodeAddress": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "NodeAddress contains information for the node's address.",
					Properties: map[string]spec.Schema{
						"type": {
							SchemaProps: spec.SchemaProps{
								Description: "Node address type, one of Hostname, ExternalIP or InternalIP.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"address": {
							SchemaProps: spec.SchemaProps{
								Description: "The node address.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"type", "address"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.NodeAffinity": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Node affinity is a group of node affinity scheduling rules.",
					Properties: map[string]spec.Schema{
						"requiredDuringSchedulingIgnoredDuringExecution": {
							SchemaProps: spec.SchemaProps{
								Description: "If the affinity requirements specified by this field are not met at scheduling time, the pod will not be scheduled onto the node. If the affinity requirements specified by this field cease to be met at some point during pod execution (e.g. due to an update), the system may or may not try to eventually evict the pod from its node.",
								Ref:         ref("k8s.io/api/core/v1.NodeSelector"),
							},
						},
						"preferredDuringSchedulingIgnoredDuringExecution": {
							SchemaProps: spec.SchemaProps{
								Description: "The scheduler will prefer to schedule pods to nodes that satisfy the affinity expressions specified by this field, but it may choose a node that violates one or more of the expressions. The node that is most preferred is the one with the greatest sum of weights, i.e. for each node that meets all of the scheduling requirements (resource request, requiredDuringScheduling affinity expressions, etc.), compute a sum by iterating through the elements of this field and adding \"weight\" to the sum if the node matches the corresponding matchExpressions; the node(s) with the highest sum are the most preferred.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.PreferredSchedulingTerm"),
										},
									},
								},
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.NodeSelector", "k8s.io/api/core/v1.PreferredSchedulingTerm"},
		},
		"k8s.io/api/core/v1.NodeCondition": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "NodeCondition contains condition information for a node.",
					Properties: map[string]spec.Schema{
						"type": {
							SchemaProps: spec.SchemaProps{
								Description: "Type of node condition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Status of the condition, one of True, False, Unknown.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"lastHeartbeatTime": {
							SchemaProps: spec.SchemaProps{
								Description: "Last time we got an update on a given condition.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"lastTransitionTime": {
							SchemaProps: spec.SchemaProps{
								Description: "Last time the condition transit from one status to another.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"reason": {
							SchemaProps: spec.SchemaProps{
								Description: "(brief) reason for the condition's last transition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"message": {
							SchemaProps: spec.SchemaProps{
								Description: "Human readable message indicating details about last transition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"type", "status"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.Time"},
		},
		"k8s.io/api/core/v1.NodeConfigSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "NodeConfigSource specifies a source of node configuration. Exactly one subfield (excluding metadata) must be non-nil.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"configMapRef": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/api/core/v1.ObjectReference"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.ObjectReference"},
		},
		"k8s.io/api/core/v1.NodeDaemonEndpoints": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "NodeDaemonEndpoints lists ports opened by daemons running on the Node.",
					Properties: map[string]spec.Schema{
						"kubeletEndpoint": {
							SchemaProps: spec.SchemaProps{
								Description: "Endpoint on which Kubelet is listening.",
								Ref:         ref("k8s.io/api/core/v1.DaemonEndpoint"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.DaemonEndpoint"},
		},
		"k8s.io/api/core/v1.NodeList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "NodeList is the whole list of all Nodes which have been registered with master.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "List of nodes",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.Node"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.Node", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/core/v1.NodeProxyOptions": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "NodeProxyOptions is the query options to a Node's proxy call.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"path": {
							SchemaProps: spec.SchemaProps{
								Description: "Path is the URL path to use for the current proxy request to node.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.NodeResources": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "NodeResources is an object for conveying resource information about a node. see http://releases.k8s.io/HEAD/docs/design/resources.md for more details.",
					Properties: map[string]spec.Schema{
						"Capacity": {
							SchemaProps: spec.SchemaProps{
								Description: "Capacity represents the available resources of a node",
								Type:        []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apimachinery/pkg/api/resource.Quantity"),
										},
									},
								},
							},
						},
					},
					Required: []string{"Capacity"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/api/resource.Quantity"},
		},
		"k8s.io/api/core/v1.NodeSelector": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "A node selector represents the union of the results of one or more label queries over a set of nodes; that is, it represents the OR of the selectors represented by the node selector terms.",
					Properties: map[string]spec.Schema{
						"nodeSelectorTerms": {
							SchemaProps: spec.SchemaProps{
								Description: "Required. A list of node selector terms. The terms are ORed.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.NodeSelectorTerm"),
										},
									},
								},
							},
						},
					},
					Required: []string{"nodeSelectorTerms"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.NodeSelectorTerm"},
		},
		"k8s.io/api/core/v1.NodeSelectorRequirement": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "A node selector requirement is a selector that contains values, a key, and an operator that relates the key and values.",
					Properties: map[string]spec.Schema{
						"key": {
							SchemaProps: spec.SchemaProps{
								Description: "The label key that the selector applies to.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"operator": {
							SchemaProps: spec.SchemaProps{
								Description: "Represents a key's relationship to a set of values. Valid operators are In, NotIn, Exists, DoesNotExist. Gt, and Lt.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"values": {
							SchemaProps: spec.SchemaProps{
								Description: "An array of string values. If the operator is In or NotIn, the values array must be non-empty. If the operator is Exists or DoesNotExist, the values array must be empty. If the operator is Gt or Lt, the values array must have a single element, which will be interpreted as an integer. This array is replaced during a strategic merge patch.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
					},
					Required: []string{"key", "operator"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.NodeSelectorTerm": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "A null or empty node selector term matches no objects.",
					Properties: map[string]spec.Schema{
						"matchExpressions": {
							SchemaProps: spec.SchemaProps{
								Description: "Required. A list of node selector requirements. The requirements are ANDed.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.NodeSelectorRequirement"),
										},
									},
								},
							},
						},
					},
					Required: []string{"matchExpressions"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.NodeSelectorRequirement"},
		},
		"k8s.io/api/core/v1.NodeSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "NodeSpec describes the attributes that a node is created with.",
					Properties: map[string]spec.Schema{
						"podCIDR": {
							SchemaProps: spec.SchemaProps{
								Description: "PodCIDR represents the pod IP range assigned to the node.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"externalID": {
							SchemaProps: spec.SchemaProps{
								Description: "External ID of the node assigned by some machine database (e.g. a cloud provider). Deprecated.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"providerID": {
							SchemaProps: spec.SchemaProps{
								Description: "ID of the node assigned by the cloud provider in the format: <ProviderName>://<ProviderSpecificNodeID>",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"unschedulable": {
							SchemaProps: spec.SchemaProps{
								Description: "Unschedulable controls node schedulability of new pods. By default, node is schedulable. More info: https://kubernetes.io/docs/concepts/nodes/node/#manual-node-administration",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"taints": {
							SchemaProps: spec.SchemaProps{
								Description: "If specified, the node's taints.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.Taint"),
										},
									},
								},
							},
						},
						"configSource": {
							SchemaProps: spec.SchemaProps{
								Description: "If specified, the source to get node configuration from The DynamicKubeletConfig feature gate must be enabled for the Kubelet to use this field",
								Ref:         ref("k8s.io/api/core/v1.NodeConfigSource"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.NodeConfigSource", "k8s.io/api/core/v1.Taint"},
		},
		"k8s.io/api/core/v1.NodeStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "NodeStatus is information about the current status of a node.",
					Properties: map[string]spec.Schema{
						"capacity": {
							SchemaProps: spec.SchemaProps{
								Description: "Capacity represents the total resources of a node. More info: https://kubernetes.io/docs/concepts/storage/persistent-volumes#capacity",
								Type:        []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apimachinery/pkg/api/resource.Quantity"),
										},
									},
								},
							},
						},
						"allocatable": {
							SchemaProps: spec.SchemaProps{
								Description: "Allocatable represents the resources of a node that are available for scheduling. Defaults to Capacity.",
								Type:        []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apimachinery/pkg/api/resource.Quantity"),
										},
									},
								},
							},
						},
						"phase": {
							SchemaProps: spec.SchemaProps{
								Description: "NodePhase is the recently observed lifecycle phase of the node. More info: https://kubernetes.io/docs/concepts/nodes/node/#phase The field is never populated, and now is deprecated.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"conditions": {
							VendorExtensible: spec.VendorExtensible{
								Extensions: spec.Extensions{
									"x-kubernetes-patch-merge-key": "type",
									"x-kubernetes-patch-strategy":  "merge",
								},
							},
							SchemaProps: spec.SchemaProps{
								Description: "Conditions is an array of current observed node conditions. More info: https://kubernetes.io/docs/concepts/nodes/node/#condition",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.NodeCondition"),
										},
									},
								},
							},
						},
						"addresses": {
							VendorExtensible: spec.VendorExtensible{
								Extensions: spec.Extensions{
									"x-kubernetes-patch-merge-key": "type",
									"x-kubernetes-patch-strategy":  "merge",
								},
							},
							SchemaProps: spec.SchemaProps{
								Description: "List of addresses reachable to the node. Queried from cloud provider, if available. More info: https://kubernetes.io/docs/concepts/nodes/node/#addresses",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.NodeAddress"),
										},
									},
								},
							},
						},
						"daemonEndpoints": {
							SchemaProps: spec.SchemaProps{
								Description: "Endpoints of daemons running on the Node.",
								Ref:         ref("k8s.io/api/core/v1.NodeDaemonEndpoints"),
							},
						},
						"nodeInfo": {
							SchemaProps: spec.SchemaProps{
								Description: "Set of ids/uuids to uniquely identify the node. More info: https://kubernetes.io/docs/concepts/nodes/node/#info",
								Ref:         ref("k8s.io/api/core/v1.NodeSystemInfo"),
							},
						},
						"images": {
							SchemaProps: spec.SchemaProps{
								Description: "List of container images on this node",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.ContainerImage"),
										},
									},
								},
							},
						},
						"volumesInUse": {
							SchemaProps: spec.SchemaProps{
								Description: "List of attachable volumes in use (mounted) by the node.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"volumesAttached": {
							SchemaProps: spec.SchemaProps{
								Description: "List of volumes that are attached to the node.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.AttachedVolume"),
										},
									},
								},
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.AttachedVolume", "k8s.io/api/core/v1.ContainerImage", "k8s.io/api/core/v1.NodeAddress", "k8s.io/api/core/v1.NodeCondition", "k8s.io/api/core/v1.NodeDaemonEndpoints", "k8s.io/api/core/v1.NodeSystemInfo", "k8s.io/apimachinery/pkg/api/resource.Quantity"},
		},
		"k8s.io/api/core/v1.NodeSystemInfo": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "NodeSystemInfo is a set of ids/uuids to uniquely identify the node.",
					Properties: map[string]spec.Schema{
						"machineID": {
							SchemaProps: spec.SchemaProps{
								Description: "MachineID reported by the node. For unique machine identification in the cluster this field is preferred. Learn more from man(5) machine-id: http://man7.org/linux/man-pages/man5/machine-id.5.html",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"systemUUID": {
							SchemaProps: spec.SchemaProps{
								Description: "SystemUUID reported by the node. For unique machine identification MachineID is preferred. This field is specific to Red Hat hosts https://access.redhat.com/documentation/en-US/Red_Hat_Subscription_Management/1/html/RHSM/getting-system-uuid.html",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"bootID": {
							SchemaProps: spec.SchemaProps{
								Description: "Boot ID reported by the node.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"kernelVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "Kernel Version reported by the node from 'uname -r' (e.g. 3.16.0-0.bpo.4-amd64).",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"osImage": {
							SchemaProps: spec.SchemaProps{
								Description: "OS Image reported by the node from /etc/os-release (e.g. Debian GNU/Linux 7 (wheezy)).",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"containerRuntimeVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "ContainerRuntime Version reported by the node through runtime remote API (e.g. docker://1.5.0).",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"kubeletVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "Kubelet Version reported by the node.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"kubeProxyVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "KubeProxy Version reported by the node.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"operatingSystem": {
							SchemaProps: spec.SchemaProps{
								Description: "The Operating System reported by the node",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"architecture": {
							SchemaProps: spec.SchemaProps{
								Description: "The Architecture reported by the node",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"machineID", "systemUUID", "bootID", "kernelVersion", "osImage", "containerRuntimeVersion", "kubeletVersion", "kubeProxyVersion", "operatingSystem", "architecture"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.ObjectFieldSelector": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ObjectFieldSelector selects an APIVersioned field of an object.",
					Properties: map[string]spec.Schema{
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "Version of the schema the FieldPath is written in terms of, defaults to \"v1\".",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"fieldPath": {
							SchemaProps: spec.SchemaProps{
								Description: "Path of the field to select in the specified API version.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"fieldPath"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.ObjectReference": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ObjectReference contains enough information to let you inspect or modify the referred object.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind of the referent. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"namespace": {
							SchemaProps: spec.SchemaProps{
								Description: "Namespace of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/namespaces/",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "Name of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/names/#names",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"uid": {
							SchemaProps: spec.SchemaProps{
								Description: "UID of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/names/#uids",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "API version of the referent.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"resourceVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "Specific resourceVersion to which this reference is made, if any. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#concurrency-control-and-consistency",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"fieldPath": {
							SchemaProps: spec.SchemaProps{
								Description: "If referring to a piece of an object instead of an entire object, this string should contain a valid JSON/Go field access statement, such as desiredState.manifest.containers[2]. For example, if the object reference is to a container within a pod, this would take on a value like: \"spec.containers{name}\" (where \"name\" refers to the name of the container that triggered the event) or if no container name is specified \"spec.containers[2]\" (container with index 2 in this pod). This syntax is chosen only to have some well-defined way of referencing a part of an object.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.PersistentVolume": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PersistentVolume (PV) is a storage resource provisioned by an administrator. It is analogous to a node. More info: https://kubernetes.io/docs/concepts/storage/persistent-volumes",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "Spec defines a specification of a persistent volume owned by the cluster. Provisioned by an administrator. More info: https://kubernetes.io/docs/concepts/storage/persistent-volumes#persistent-volumes",
								Ref:         ref("k8s.io/api/core/v1.PersistentVolumeSpec"),
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Status represents the current information/status for the persistent volume. Populated by the system. Read-only. More info: https://kubernetes.io/docs/concepts/storage/persistent-volumes#persistent-volumes",
								Ref:         ref("k8s.io/api/core/v1.PersistentVolumeStatus"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.PersistentVolumeSpec", "k8s.io/api/core/v1.PersistentVolumeStatus", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/core/v1.PersistentVolumeClaim": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PersistentVolumeClaim is a user's request for and claim to a persistent volume",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "Spec defines the desired characteristics of a volume requested by a pod author. More info: https://kubernetes.io/docs/concepts/storage/persistent-volumes#persistentvolumeclaims",
								Ref:         ref("k8s.io/api/core/v1.PersistentVolumeClaimSpec"),
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Status represents the current information/status of a persistent volume claim. Read-only. More info: https://kubernetes.io/docs/concepts/storage/persistent-volumes#persistentvolumeclaims",
								Ref:         ref("k8s.io/api/core/v1.PersistentVolumeClaimStatus"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.PersistentVolumeClaimSpec", "k8s.io/api/core/v1.PersistentVolumeClaimStatus", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/core/v1.PersistentVolumeClaimCondition": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PersistentVolumeClaimCondition contails details about state of pvc",
					Properties: map[string]spec.Schema{
						"type": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
						"lastProbeTime": {
							SchemaProps: spec.SchemaProps{
								Description: "Last time we probed the condition.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"lastTransitionTime": {
							SchemaProps: spec.SchemaProps{
								Description: "Last time the condition transitioned from one status to another.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"reason": {
							SchemaProps: spec.SchemaProps{
								Description: "Unique, this should be a short, machine understandable string that gives the reason for condition's last transition. If it reports \"ResizeStarted\" that means the underlying persistent volume is being resized.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"message": {
							SchemaProps: spec.SchemaProps{
								Description: "Human-readable message indicating details about last transition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"type", "status"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.Time"},
		},
		"k8s.io/api/core/v1.PersistentVolumeClaimList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PersistentVolumeClaimList is a list of PersistentVolumeClaim items.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "A list of persistent volume claims. More info: https://kubernetes.io/docs/concepts/storage/persistent-volumes#persistentvolumeclaims",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.PersistentVolumeClaim"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.PersistentVolumeClaim", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/core/v1.PersistentVolumeClaimSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PersistentVolumeClaimSpec describes the common attributes of storage devices and allows a Source for provider-specific attributes",
					Properties: map[string]spec.Schema{
						"accessModes": {
							SchemaProps: spec.SchemaProps{
								Description: "AccessModes contains the desired access modes the volume should have. More info: https://kubernetes.io/docs/concepts/storage/persistent-volumes#access-modes-1",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"selector": {
							SchemaProps: spec.SchemaProps{
								Description: "A label query over volumes to consider for binding.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"),
							},
						},
						"resources": {
							SchemaProps: spec.SchemaProps{
								Description: "Resources represents the minimum resources the volume should have. More info: https://kubernetes.io/docs/concepts/storage/persistent-volumes#resources",
								Ref:         ref("k8s.io/api/core/v1.ResourceRequirements"),
							},
						},
						"volumeName": {
							SchemaProps: spec.SchemaProps{
								Description: "VolumeName is the binding reference to the PersistentVolume backing this claim.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"storageClassName": {
							SchemaProps: spec.SchemaProps{
								Description: "Name of the StorageClass required by the claim. More info: https://kubernetes.io/docs/concepts/storage/persistent-volumes#class-1",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"volumeMode": {
							SchemaProps: spec.SchemaProps{
								Description: "volumeMode defines what type of volume is required by the claim. Value of Filesystem is implied when not included in claim spec. This is an alpha feature and may change in the future.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.ResourceRequirements", "k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"},
		},
		"k8s.io/api/core/v1.PersistentVolumeClaimStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PersistentVolumeClaimStatus is the current status of a persistent volume claim.",
					Properties: map[string]spec.Schema{
						"phase": {
							SchemaProps: spec.SchemaProps{
								Description: "Phase represents the current phase of PersistentVolumeClaim.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"accessModes": {
							SchemaProps: spec.SchemaProps{
								Description: "AccessModes contains the actual access modes the volume backing the PVC has. More info: https://kubernetes.io/docs/concepts/storage/persistent-volumes#access-modes-1",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"capacity": {
							SchemaProps: spec.SchemaProps{
								Description: "Represents the actual resources of the underlying volume.",
								Type:        []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apimachinery/pkg/api/resource.Quantity"),
										},
									},
								},
							},
						},
						"conditions": {
							VendorExtensible: spec.VendorExtensible{
								Extensions: spec.Extensions{
									"x-kubernetes-patch-merge-key": "type",
									"x-kubernetes-patch-strategy":  "merge",
								},
							},
							SchemaProps: spec.SchemaProps{
								Description: "Current Condition of persistent volume claim. If underlying persistent volume is being resized then the Condition will be set to 'ResizeStarted'.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.PersistentVolumeClaimCondition"),
										},
									},
								},
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.PersistentVolumeClaimCondition", "k8s.io/apimachinery/pkg/api/resource.Quantity"},
		},
		"k8s.io/api/core/v1.PersistentVolumeClaimVolumeSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PersistentVolumeClaimVolumeSource references the user's PVC in the same namespace. This volume finds the bound PV and mounts that volume for the pod. A PersistentVolumeClaimVolumeSource is, essentially, a wrapper around another type of volume that is owned by someone else (the system).",
					Properties: map[string]spec.Schema{
						"claimName": {
							SchemaProps: spec.SchemaProps{
								Description: "ClaimName is the name of a PersistentVolumeClaim in the same namespace as the pod using this volume. More info: https://kubernetes.io/docs/concepts/storage/persistent-volumes#persistentvolumeclaims",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"readOnly": {
							SchemaProps: spec.SchemaProps{
								Description: "Will force the ReadOnly setting in VolumeMounts. Default false.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
					},
					Required: []string{"claimName"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.PersistentVolumeList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PersistentVolumeList is a list of PersistentVolume items.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "List of persistent volumes. More info: https://kubernetes.io/docs/concepts/storage/persistent-volumes",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.PersistentVolume"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.PersistentVolume", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/core/v1.PersistentVolumeSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PersistentVolumeSource is similar to VolumeSource but meant for the administrator who creates PVs. Exactly one of its members must be set.",
					Properties: map[string]spec.Schema{
						"gcePersistentDisk": {
							SchemaProps: spec.SchemaProps{
								Description: "GCEPersistentDisk represents a GCE Disk resource that is attached to a kubelet's host machine and then exposed to the pod. Provisioned by an admin. More info: https://kubernetes.io/docs/concepts/storage/volumes#gcepersistentdisk",
								Ref:         ref("k8s.io/api/core/v1.GCEPersistentDiskVolumeSource"),
							},
						},
						"awsElasticBlockStore": {
							SchemaProps: spec.SchemaProps{
								Description: "AWSElasticBlockStore represents an AWS Disk resource that is attached to a kubelet's host machine and then exposed to the pod. More info: https://kubernetes.io/docs/concepts/storage/volumes#awselasticblockstore",
								Ref:         ref("k8s.io/api/core/v1.AWSElasticBlockStoreVolumeSource"),
							},
						},
						"hostPath": {
							SchemaProps: spec.SchemaProps{
								Description: "HostPath represents a directory on the host. Provisioned by a developer or tester. This is useful for single-node development and testing only! On-host storage is not supported in any way and WILL NOT WORK in a multi-node cluster. More info: https://kubernetes.io/docs/concepts/storage/volumes#hostpath",
								Ref:         ref("k8s.io/api/core/v1.HostPathVolumeSource"),
							},
						},
						"glusterfs": {
							SchemaProps: spec.SchemaProps{
								Description: "Glusterfs represents a Glusterfs volume that is attached to a host and exposed to the pod. Provisioned by an admin. More info: https://releases.k8s.io/HEAD/examples/volumes/glusterfs/README.md",
								Ref:         ref("k8s.io/api/core/v1.GlusterfsVolumeSource"),
							},
						},
						"nfs": {
							SchemaProps: spec.SchemaProps{
								Description: "NFS represents an NFS mount on the host. Provisioned by an admin. More info: https://kubernetes.io/docs/concepts/storage/volumes#nfs",
								Ref:         ref("k8s.io/api/core/v1.NFSVolumeSource"),
							},
						},
						"rbd": {
							SchemaProps: spec.SchemaProps{
								Description: "RBD represents a Rados Block Device mount on the host that shares a pod's lifetime. More info: https://releases.k8s.io/HEAD/examples/volumes/rbd/README.md",
								Ref:         ref("k8s.io/api/core/v1.RBDPersistentVolumeSource"),
							},
						},
						"iscsi": {
							SchemaProps: spec.SchemaProps{
								Description: "ISCSI represents an ISCSI Disk resource that is attached to a kubelet's host machine and then exposed to the pod. Provisioned by an admin.",
								Ref:         ref("k8s.io/api/core/v1.ISCSIPersistentVolumeSource"),
							},
						},
						"cinder": {
							SchemaProps: spec.SchemaProps{
								Description: "Cinder represents a cinder volume attached and mounted on kubelets host machine More info: https://releases.k8s.io/HEAD/examples/mysql-cinder-pd/README.md",
								Ref:         ref("k8s.io/api/core/v1.CinderVolumeSource"),
							},
						},
						"cephfs": {
							SchemaProps: spec.SchemaProps{
								Description: "CephFS represents a Ceph FS mount on the host that shares a pod's lifetime",
								Ref:         ref("k8s.io/api/core/v1.CephFSPersistentVolumeSource"),
							},
						},
						"fc": {
							SchemaProps: spec.SchemaProps{
								Description: "FC represents a Fibre Channel resource that is attached to a kubelet's host machine and then exposed to the pod.",
								Ref:         ref("k8s.io/api/core/v1.FCVolumeSource"),
							},
						},
						"flocker": {
							SchemaProps: spec.SchemaProps{
								Description: "Flocker represents a Flocker volume attached to a kubelet's host machine and exposed to the pod for its usage. This depends on the Flocker control service being running",
								Ref:         ref("k8s.io/api/core/v1.FlockerVolumeSource"),
							},
						},
						"flexVolume": {
							SchemaProps: spec.SchemaProps{
								Description: "FlexVolume represents a generic volume resource that is provisioned/attached using an exec based plugin.",
								Ref:         ref("k8s.io/api/core/v1.FlexVolumeSource"),
							},
						},
						"azureFile": {
							SchemaProps: spec.SchemaProps{
								Description: "AzureFile represents an Azure File Service mount on the host and bind mount to the pod.",
								Ref:         ref("k8s.io/api/core/v1.AzureFilePersistentVolumeSource"),
							},
						},
						"vsphereVolume": {
							SchemaProps: spec.SchemaProps{
								Description: "VsphereVolume represents a vSphere volume attached and mounted on kubelets host machine",
								Ref:         ref("k8s.io/api/core/v1.VsphereVirtualDiskVolumeSource"),
							},
						},
						"quobyte": {
							SchemaProps: spec.SchemaProps{
								Description: "Quobyte represents a Quobyte mount on the host that shares a pod's lifetime",
								Ref:         ref("k8s.io/api/core/v1.QuobyteVolumeSource"),
							},
						},
						"azureDisk": {
							SchemaProps: spec.SchemaProps{
								Description: "AzureDisk represents an Azure Data Disk mount on the host and bind mount to the pod.",
								Ref:         ref("k8s.io/api/core/v1.AzureDiskVolumeSource"),
							},
						},
						"photonPersistentDisk": {
							SchemaProps: spec.SchemaProps{
								Description: "PhotonPersistentDisk represents a PhotonController persistent disk attached and mounted on kubelets host machine",
								Ref:         ref("k8s.io/api/core/v1.PhotonPersistentDiskVolumeSource"),
							},
						},
						"portworxVolume": {
							SchemaProps: spec.SchemaProps{
								Description: "PortworxVolume represents a portworx volume attached and mounted on kubelets host machine",
								Ref:         ref("k8s.io/api/core/v1.PortworxVolumeSource"),
							},
						},
						"scaleIO": {
							SchemaProps: spec.SchemaProps{
								Description: "ScaleIO represents a ScaleIO persistent volume attached and mounted on Kubernetes nodes.",
								Ref:         ref("k8s.io/api/core/v1.ScaleIOPersistentVolumeSource"),
							},
						},
						"local": {
							SchemaProps: spec.SchemaProps{
								Description: "Local represents directly-attached storage with node affinity",
								Ref:         ref("k8s.io/api/core/v1.LocalVolumeSource"),
							},
						},
						"storageos": {
							SchemaProps: spec.SchemaProps{
								Description: "StorageOS represents a StorageOS volume that is attached to the kubelet's host machine and mounted into the pod More info: https://releases.k8s.io/HEAD/examples/volumes/storageos/README.md",
								Ref:         ref("k8s.io/api/core/v1.StorageOSPersistentVolumeSource"),
							},
						},
						"csi": {
							SchemaProps: spec.SchemaProps{
								Description: "CSI represents storage that handled by an external CSI driver",
								Ref:         ref("k8s.io/api/core/v1.CSIPersistentVolumeSource"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.AWSElasticBlockStoreVolumeSource", "k8s.io/api/core/v1.AzureDiskVolumeSource", "k8s.io/api/core/v1.AzureFilePersistentVolumeSource", "k8s.io/api/core/v1.CSIPersistentVolumeSource", "k8s.io/api/core/v1.CephFSPersistentVolumeSource", "k8s.io/api/core/v1.CinderVolumeSource", "k8s.io/api/core/v1.FCVolumeSource", "k8s.io/api/core/v1.FlexVolumeSource", "k8s.io/api/core/v1.FlockerVolumeSource", "k8s.io/api/core/v1.GCEPersistentDiskVolumeSource", "k8s.io/api/core/v1.GlusterfsVolumeSource", "k8s.io/api/core/v1.HostPathVolumeSource", "k8s.io/api/core/v1.ISCSIPersistentVolumeSource", "k8s.io/api/core/v1.LocalVolumeSource", "k8s.io/api/core/v1.NFSVolumeSource", "k8s.io/api/core/v1.PhotonPersistentDiskVolumeSource", "k8s.io/api/core/v1.PortworxVolumeSource", "k8s.io/api/core/v1.QuobyteVolumeSource", "k8s.io/api/core/v1.RBDPersistentVolumeSource", "k8s.io/api/core/v1.ScaleIOPersistentVolumeSource", "k8s.io/api/core/v1.StorageOSPersistentVolumeSource", "k8s.io/api/core/v1.VsphereVirtualDiskVolumeSource"},
		},
		"k8s.io/api/core/v1.PersistentVolumeSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PersistentVolumeSpec is the specification of a persistent volume.",
					Properties: map[string]spec.Schema{
						"capacity": {
							SchemaProps: spec.SchemaProps{
								Description: "A description of the persistent volume's resources and capacity. More info: https://kubernetes.io/docs/concepts/storage/persistent-volumes#capacity",
								Type:        []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apimachinery/pkg/api/resource.Quantity"),
										},
									},
								},
							},
						},
						"gcePersistentDisk": {
							SchemaProps: spec.SchemaProps{
								Description: "GCEPersistentDisk represents a GCE Disk resource that is attached to a kubelet's host machine and then exposed to the pod. Provisioned by an admin. More info: https://kubernetes.io/docs/concepts/storage/volumes#gcepersistentdisk",
								Ref:         ref("k8s.io/api/core/v1.GCEPersistentDiskVolumeSource"),
							},
						},
						"awsElasticBlockStore": {
							SchemaProps: spec.SchemaProps{
								Description: "AWSElasticBlockStore represents an AWS Disk resource that is attached to a kubelet's host machine and then exposed to the pod. More info: https://kubernetes.io/docs/concepts/storage/volumes#awselasticblockstore",
								Ref:         ref("k8s.io/api/core/v1.AWSElasticBlockStoreVolumeSource"),
							},
						},
						"hostPath": {
							SchemaProps: spec.SchemaProps{
								Description: "HostPath represents a directory on the host. Provisioned by a developer or tester. This is useful for single-node development and testing only! On-host storage is not supported in any way and WILL NOT WORK in a multi-node cluster. More info: https://kubernetes.io/docs/concepts/storage/volumes#hostpath",
								Ref:         ref("k8s.io/api/core/v1.HostPathVolumeSource"),
							},
						},
						"glusterfs": {
							SchemaProps: spec.SchemaProps{
								Description: "Glusterfs represents a Glusterfs volume that is attached to a host and exposed to the pod. Provisioned by an admin. More info: https://releases.k8s.io/HEAD/examples/volumes/glusterfs/README.md",
								Ref:         ref("k8s.io/api/core/v1.GlusterfsVolumeSource"),
							},
						},
						"nfs": {
							SchemaProps: spec.SchemaProps{
								Description: "NFS represents an NFS mount on the host. Provisioned by an admin. More info: https://kubernetes.io/docs/concepts/storage/volumes#nfs",
								Ref:         ref("k8s.io/api/core/v1.NFSVolumeSource"),
							},
						},
						"rbd": {
							SchemaProps: spec.SchemaProps{
								Description: "RBD represents a Rados Block Device mount on the host that shares a pod's lifetime. More info: https://releases.k8s.io/HEAD/examples/volumes/rbd/README.md",
								Ref:         ref("k8s.io/api/core/v1.RBDPersistentVolumeSource"),
							},
						},
						"iscsi": {
							SchemaProps: spec.SchemaProps{
								Description: "ISCSI represents an ISCSI Disk resource that is attached to a kubelet's host machine and then exposed to the pod. Provisioned by an admin.",
								Ref:         ref("k8s.io/api/core/v1.ISCSIPersistentVolumeSource"),
							},
						},
						"cinder": {
							SchemaProps: spec.SchemaProps{
								Description: "Cinder represents a cinder volume attached and mounted on kubelets host machine More info: https://releases.k8s.io/HEAD/examples/mysql-cinder-pd/README.md",
								Ref:         ref("k8s.io/api/core/v1.CinderVolumeSource"),
							},
						},
						"cephfs": {
							SchemaProps: spec.SchemaProps{
								Description: "CephFS represents a Ceph FS mount on the host that shares a pod's lifetime",
								Ref:         ref("k8s.io/api/core/v1.CephFSPersistentVolumeSource"),
							},
						},
						"fc": {
							SchemaProps: spec.SchemaProps{
								Description: "FC represents a Fibre Channel resource that is attached to a kubelet's host machine and then exposed to the pod.",
								Ref:         ref("k8s.io/api/core/v1.FCVolumeSource"),
							},
						},
						"flocker": {
							SchemaProps: spec.SchemaProps{
								Description: "Flocker represents a Flocker volume attached to a kubelet's host machine and exposed to the pod for its usage. This depends on the Flocker control service being running",
								Ref:         ref("k8s.io/api/core/v1.FlockerVolumeSource"),
							},
						},
						"flexVolume": {
							SchemaProps: spec.SchemaProps{
								Description: "FlexVolume represents a generic volume resource that is provisioned/attached using an exec based plugin.",
								Ref:         ref("k8s.io/api/core/v1.FlexVolumeSource"),
							},
						},
						"azureFile": {
							SchemaProps: spec.SchemaProps{
								Description: "AzureFile represents an Azure File Service mount on the host and bind mount to the pod.",
								Ref:         ref("k8s.io/api/core/v1.AzureFilePersistentVolumeSource"),
							},
						},
						"vsphereVolume": {
							SchemaProps: spec.SchemaProps{
								Description: "VsphereVolume represents a vSphere volume attached and mounted on kubelets host machine",
								Ref:         ref("k8s.io/api/core/v1.VsphereVirtualDiskVolumeSource"),
							},
						},
						"quobyte": {
							SchemaProps: spec.SchemaProps{
								Description: "Quobyte represents a Quobyte mount on the host that shares a pod's lifetime",
								Ref:         ref("k8s.io/api/core/v1.QuobyteVolumeSource"),
							},
						},
						"azureDisk": {
							SchemaProps: spec.SchemaProps{
								Description: "AzureDisk represents an Azure Data Disk mount on the host and bind mount to the pod.",
								Ref:         ref("k8s.io/api/core/v1.AzureDiskVolumeSource"),
							},
						},
						"photonPersistentDisk": {
							SchemaProps: spec.SchemaProps{
								Description: "PhotonPersistentDisk represents a PhotonController persistent disk attached and mounted on kubelets host machine",
								Ref:         ref("k8s.io/api/core/v1.PhotonPersistentDiskVolumeSource"),
							},
						},
						"portworxVolume": {
							SchemaProps: spec.SchemaProps{
								Description: "PortworxVolume represents a portworx volume attached and mounted on kubelets host machine",
								Ref:         ref("k8s.io/api/core/v1.PortworxVolumeSource"),
							},
						},
						"scaleIO": {
							SchemaProps: spec.SchemaProps{
								Description: "ScaleIO represents a ScaleIO persistent volume attached and mounted on Kubernetes nodes.",
								Ref:         ref("k8s.io/api/core/v1.ScaleIOPersistentVolumeSource"),
							},
						},
						"local": {
							SchemaProps: spec.SchemaProps{
								Description: "Local represents directly-attached storage with node affinity",
								Ref:         ref("k8s.io/api/core/v1.LocalVolumeSource"),
							},
						},
						"storageos": {
							SchemaProps: spec.SchemaProps{
								Description: "StorageOS represents a StorageOS volume that is attached to the kubelet's host machine and mounted into the pod More info: https://releases.k8s.io/HEAD/examples/volumes/storageos/README.md",
								Ref:         ref("k8s.io/api/core/v1.StorageOSPersistentVolumeSource"),
							},
						},
						"csi": {
							SchemaProps: spec.SchemaProps{
								Description: "CSI represents storage that handled by an external CSI driver",
								Ref:         ref("k8s.io/api/core/v1.CSIPersistentVolumeSource"),
							},
						},
						"accessModes": {
							SchemaProps: spec.SchemaProps{
								Description: "AccessModes contains all ways the volume can be mounted. More info: https://kubernetes.io/docs/concepts/storage/persistent-volumes#access-modes",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"claimRef": {
							SchemaProps: spec.SchemaProps{
								Description: "ClaimRef is part of a bi-directional binding between PersistentVolume and PersistentVolumeClaim. Expected to be non-nil when bound. claim.VolumeName is the authoritative bind between PV and PVC. More info: https://kubernetes.io/docs/concepts/storage/persistent-volumes#binding",
								Ref:         ref("k8s.io/api/core/v1.ObjectReference"),
							},
						},
						"persistentVolumeReclaimPolicy": {
							SchemaProps: spec.SchemaProps{
								Description: "What happens to a persistent volume when released from its claim. Valid options are Retain (default) and Recycle. Recycling must be supported by the volume plugin underlying this persistent volume. More info: https://kubernetes.io/docs/concepts/storage/persistent-volumes#reclaiming",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"storageClassName": {
							SchemaProps: spec.SchemaProps{
								Description: "Name of StorageClass to which this persistent volume belongs. Empty value means that this volume does not belong to any StorageClass.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"mountOptions": {
							SchemaProps: spec.SchemaProps{
								Description: "A list of mount options, e.g. [\"ro\", \"soft\"]. Not validated - mount will simply fail if one is invalid. More info: https://kubernetes.io/docs/concepts/storage/persistent-volumes/#mount-options",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"volumeMode": {
							SchemaProps: spec.SchemaProps{
								Description: "volumeMode defines if a volume is intended to be used with a formatted filesystem or to remain in raw block state. Value of Filesystem is implied when not included in spec. This is an alpha feature and may change in the future.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.AWSElasticBlockStoreVolumeSource", "k8s.io/api/core/v1.AzureDiskVolumeSource", "k8s.io/api/core/v1.AzureFilePersistentVolumeSource", "k8s.io/api/core/v1.CSIPersistentVolumeSource", "k8s.io/api/core/v1.CephFSPersistentVolumeSource", "k8s.io/api/core/v1.CinderVolumeSource", "k8s.io/api/core/v1.FCVolumeSource", "k8s.io/api/core/v1.FlexVolumeSource", "k8s.io/api/core/v1.FlockerVolumeSource", "k8s.io/api/core/v1.GCEPersistentDiskVolumeSource", "k8s.io/api/core/v1.GlusterfsVolumeSource", "k8s.io/api/core/v1.HostPathVolumeSource", "k8s.io/api/core/v1.ISCSIPersistentVolumeSource", "k8s.io/api/core/v1.LocalVolumeSource", "k8s.io/api/core/v1.NFSVolumeSource", "k8s.io/api/core/v1.ObjectReference", "k8s.io/api/core/v1.PhotonPersistentDiskVolumeSource", "k8s.io/api/core/v1.PortworxVolumeSource", "k8s.io/api/core/v1.QuobyteVolumeSource", "k8s.io/api/core/v1.RBDPersistentVolumeSource", "k8s.io/api/core/v1.ScaleIOPersistentVolumeSource", "k8s.io/api/core/v1.StorageOSPersistentVolumeSource", "k8s.io/api/core/v1.VsphereVirtualDiskVolumeSource", "k8s.io/apimachinery/pkg/api/resource.Quantity"},
		},
		"k8s.io/api/core/v1.PersistentVolumeStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PersistentVolumeStatus is the current status of a persistent volume.",
					Properties: map[string]spec.Schema{
						"phase": {
							SchemaProps: spec.SchemaProps{
								Description: "Phase indicates if a volume is available, bound to a claim, or released by a claim. More info: https://kubernetes.io/docs/concepts/storage/persistent-volumes#phase",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"message": {
							SchemaProps: spec.SchemaProps{
								Description: "A human-readable message indicating details about why the volume is in this state.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"reason": {
							SchemaProps: spec.SchemaProps{
								Description: "Reason is a brief CamelCase string that describes any failure and is meant for machine parsing and tidy display in the CLI.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.PhotonPersistentDiskVolumeSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Represents a Photon Controller persistent disk resource.",
					Properties: map[string]spec.Schema{
						"pdID": {
							SchemaProps: spec.SchemaProps{
								Description: "ID that identifies Photon Controller persistent disk",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"fsType": {
							SchemaProps: spec.SchemaProps{
								Description: "Filesystem type to mount. Must be a filesystem type supported by the host operating system. Ex. \"ext4\", \"xfs\", \"ntfs\". Implicitly inferred to be \"ext4\" if unspecified.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"pdID"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.Pod": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Pod is a collection of containers that can run on a host. This resource is created by clients and scheduled onto hosts.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "Specification of the desired behavior of the pod. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status",
								Ref:         ref("k8s.io/api/core/v1.PodSpec"),
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Most recently observed status of the pod. This data may not be up to date. Populated by the system. Read-only. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status",
								Ref:         ref("k8s.io/api/core/v1.PodStatus"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.PodSpec", "k8s.io/api/core/v1.PodStatus", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/core/v1.PodAffinity": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Pod affinity is a group of inter pod affinity scheduling rules.",
					Properties: map[string]spec.Schema{
						"requiredDuringSchedulingIgnoredDuringExecution": {
							SchemaProps: spec.SchemaProps{
								Description: "If the affinity requirements specified by this field are not met at scheduling time, the pod will not be scheduled onto the node. If the affinity requirements specified by this field cease to be met at some point during pod execution (e.g. due to a pod label update), the system may or may not try to eventually evict the pod from its node. When there are multiple elements, the lists of nodes corresponding to each podAffinityTerm are intersected, i.e. all terms must be satisfied.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.PodAffinityTerm"),
										},
									},
								},
							},
						},
						"preferredDuringSchedulingIgnoredDuringExecution": {
							SchemaProps: spec.SchemaProps{
								Description: "The scheduler will prefer to schedule pods to nodes that satisfy the affinity expressions specified by this field, but it may choose a node that violates one or more of the expressions. The node that is most preferred is the one with the greatest sum of weights, i.e. for each node that meets all of the scheduling requirements (resource request, requiredDuringScheduling affinity expressions, etc.), compute a sum by iterating through the elements of this field and adding \"weight\" to the sum if the node has pods which matches the corresponding podAffinityTerm; the node(s) with the highest sum are the most preferred.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.WeightedPodAffinityTerm"),
										},
									},
								},
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.PodAffinityTerm", "k8s.io/api/core/v1.WeightedPodAffinityTerm"},
		},
		"k8s.io/api/core/v1.PodAffinityTerm": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Defines a set of pods (namely those matching the labelSelector relative to the given namespace(s)) that this pod should be co-located (affinity) or not co-located (anti-affinity) with, where co-located is defined as running on a node whose value of the label with key <topologyKey> matches that of any node on which a pod of the set of pods is running",
					Properties: map[string]spec.Schema{
						"labelSelector": {
							SchemaProps: spec.SchemaProps{
								Description: "A label query over a set of resources, in this case pods.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"),
							},
						},
						"namespaces": {
							SchemaProps: spec.SchemaProps{
								Description: "namespaces specifies which namespaces the labelSelector applies to (matches against); null or empty list means \"this pod's namespace\"",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"topologyKey": {
							SchemaProps: spec.SchemaProps{
								Description: "This pod should be co-located (affinity) or not co-located (anti-affinity) with the pods matching the labelSelector in the specified namespaces, where co-located is defined as running on a node whose value of the label with key topologyKey matches that of any node on which any of the selected pods is running. Empty topologyKey is not allowed.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"topologyKey"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"},
		},
		"k8s.io/api/core/v1.PodAntiAffinity": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Pod anti affinity is a group of inter pod anti affinity scheduling rules.",
					Properties: map[string]spec.Schema{
						"requiredDuringSchedulingIgnoredDuringExecution": {
							SchemaProps: spec.SchemaProps{
								Description: "If the anti-affinity requirements specified by this field are not met at scheduling time, the pod will not be scheduled onto the node. If the anti-affinity requirements specified by this field cease to be met at some point during pod execution (e.g. due to a pod label update), the system may or may not try to eventually evict the pod from its node. When there are multiple elements, the lists of nodes corresponding to each podAffinityTerm are intersected, i.e. all terms must be satisfied.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.PodAffinityTerm"),
										},
									},
								},
							},
						},
						"preferredDuringSchedulingIgnoredDuringExecution": {
							SchemaProps: spec.SchemaProps{
								Description: "The scheduler will prefer to schedule pods to nodes that satisfy the anti-affinity expressions specified by this field, but it may choose a node that violates one or more of the expressions. The node that is most preferred is the one with the greatest sum of weights, i.e. for each node that meets all of the scheduling requirements (resource request, requiredDuringScheduling anti-affinity expressions, etc.), compute a sum by iterating through the elements of this field and adding \"weight\" to the sum if the node has pods which matches the corresponding podAffinityTerm; the node(s) with the highest sum are the most preferred.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.WeightedPodAffinityTerm"),
										},
									},
								},
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.PodAffinityTerm", "k8s.io/api/core/v1.WeightedPodAffinityTerm"},
		},
		"k8s.io/api/core/v1.PodAttachOptions": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PodAttachOptions is the query options to a Pod's remote attach call.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"stdin": {
							SchemaProps: spec.SchemaProps{
								Description: "Stdin if true, redirects the standard input stream of the pod for this call. Defaults to false.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"stdout": {
							SchemaProps: spec.SchemaProps{
								Description: "Stdout if true indicates that stdout is to be redirected for the attach call. Defaults to true.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"stderr": {
							SchemaProps: spec.SchemaProps{
								Description: "Stderr if true indicates that stderr is to be redirected for the attach call. Defaults to true.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"tty": {
							SchemaProps: spec.SchemaProps{
								Description: "TTY if true indicates that a tty will be allocated for the attach call. This is passed through the container runtime so the tty is allocated on the worker node by the container runtime. Defaults to false.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"container": {
							SchemaProps: spec.SchemaProps{
								Description: "The container in which to execute the command. Defaults to only container if there is only one container in the pod.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.PodCondition": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PodCondition contains details for the current condition of this pod.",
					Properties: map[string]spec.Schema{
						"type": {
							SchemaProps: spec.SchemaProps{
								Description: "Type is the type of the condition. Currently only Ready. More info: https://kubernetes.io/docs/concepts/workloads/pods/pod-lifecycle#pod-conditions",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Status is the status of the condition. Can be True, False, Unknown. More info: https://kubernetes.io/docs/concepts/workloads/pods/pod-lifecycle#pod-conditions",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"lastProbeTime": {
							SchemaProps: spec.SchemaProps{
								Description: "Last time we probed the condition.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"lastTransitionTime": {
							SchemaProps: spec.SchemaProps{
								Description: "Last time the condition transitioned from one status to another.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"reason": {
							SchemaProps: spec.SchemaProps{
								Description: "Unique, one-word, CamelCase reason for the condition's last transition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"message": {
							SchemaProps: spec.SchemaProps{
								Description: "Human-readable message indicating details about last transition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"type", "status"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.Time"},
		},
		"k8s.io/api/core/v1.PodDNSConfig": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PodDNSConfig defines the DNS parameters of a pod in addition to those generated from DNSPolicy.",
					Properties: map[string]spec.Schema{
						"nameservers": {
							SchemaProps: spec.SchemaProps{
								Description: "A list of DNS name server IP addresses. This will be appended to the base nameservers generated from DNSPolicy. Duplicated nameservers will be removed.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"searches": {
							SchemaProps: spec.SchemaProps{
								Description: "A list of DNS search domains for host-name lookup. This will be appended to the base search paths generated from DNSPolicy. Duplicated search paths will be removed.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"options": {
							SchemaProps: spec.SchemaProps{
								Description: "A list of DNS resolver options. This will be merged with the base options generated from DNSPolicy. Duplicated entries will be removed. Resolution options given in Options will override those that appear in the base DNSPolicy.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.PodDNSConfigOption"),
										},
									},
								},
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.PodDNSConfigOption"},
		},
		"k8s.io/api/core/v1.PodDNSConfigOption": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PodDNSConfigOption defines DNS resolver options of a pod.",
					Properties: map[string]spec.Schema{
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "Required.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"value": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.PodExecOptions": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PodExecOptions is the query options to a Pod's remote exec call.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"stdin": {
							SchemaProps: spec.SchemaProps{
								Description: "Redirect the standard input stream of the pod for this call. Defaults to false.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"stdout": {
							SchemaProps: spec.SchemaProps{
								Description: "Redirect the standard output stream of the pod for this call. Defaults to true.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"stderr": {
							SchemaProps: spec.SchemaProps{
								Description: "Redirect the standard error stream of the pod for this call. Defaults to true.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"tty": {
							SchemaProps: spec.SchemaProps{
								Description: "TTY if true indicates that a tty will be allocated for the exec call. Defaults to false.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"container": {
							SchemaProps: spec.SchemaProps{
								Description: "Container in which to execute the command. Defaults to only container if there is only one container in the pod.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"command": {
							SchemaProps: spec.SchemaProps{
								Description: "Command is the remote command to execute. argv array. Not executed within a shell.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
					},
					Required: []string{"command"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.PodList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PodList is a list of Pods.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "List of pods. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.Pod"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.Pod", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/core/v1.PodLogOptions": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PodLogOptions is the query options for a Pod's logs REST call.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"container": {
							SchemaProps: spec.SchemaProps{
								Description: "The container for which to stream logs. Defaults to only container if there is one container in the pod.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"follow": {
							SchemaProps: spec.SchemaProps{
								Description: "Follow the log stream of the pod. Defaults to false.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"previous": {
							SchemaProps: spec.SchemaProps{
								Description: "Return previous terminated container logs. Defaults to false.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"sinceSeconds": {
							SchemaProps: spec.SchemaProps{
								Description: "A relative time in seconds before the current time from which to show logs. If this value precedes the time a pod was started, only logs since the pod start will be returned. If this value is in the future, no logs will be returned. Only one of sinceSeconds or sinceTime may be specified.",
								Type:        []string{"integer"},
								Format:      "int64",
							},
						},
						"sinceTime": {
							SchemaProps: spec.SchemaProps{
								Description: "An RFC3339 timestamp from which to show logs. If this value precedes the time a pod was started, only logs since the pod start will be returned. If this value is in the future, no logs will be returned. Only one of sinceSeconds or sinceTime may be specified.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"timestamps": {
							SchemaProps: spec.SchemaProps{
								Description: "If true, add an RFC3339 or RFC3339Nano timestamp at the beginning of every line of log output. Defaults to false.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"tailLines": {
							SchemaProps: spec.SchemaProps{
								Description: "If set, the number of lines from the end of the logs to show. If not specified, logs are shown from the creation of the container or sinceSeconds or sinceTime",
								Type:        []string{"integer"},
								Format:      "int64",
							},
						},
						"limitBytes": {
							SchemaProps: spec.SchemaProps{
								Description: "If set, the number of bytes to read from the server before terminating the log output. This may not display a complete final line of logging, and may return slightly more or slightly less than the specified limit.",
								Type:        []string{"integer"},
								Format:      "int64",
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.Time"},
		},
		"k8s.io/api/core/v1.PodPortForwardOptions": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PodPortForwardOptions is the query options to a Pod's port forward call when using WebSockets. The `port` query parameter must specify the port or ports (comma separated) to forward over. Port forwarding over SPDY does not use these options. It requires the port to be passed in the `port` header as part of request.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"ports": {
							SchemaProps: spec.SchemaProps{
								Description: "List of ports to forward Required when using WebSockets",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"integer"},
											Format: "int32",
										},
									},
								},
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.PodProxyOptions": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PodProxyOptions is the query options to a Pod's proxy call.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"path": {
							SchemaProps: spec.SchemaProps{
								Description: "Path is the URL path to use for the current proxy request to pod.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.PodSecurityContext": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PodSecurityContext holds pod-level security attributes and common container settings. Some fields are also present in container.securityContext.  Field values of container.securityContext take precedence over field values of PodSecurityContext.",
					Properties: map[string]spec.Schema{
						"seLinuxOptions": {
							SchemaProps: spec.SchemaProps{
								Description: "The SELinux context to be applied to all containers. If unspecified, the container runtime will allocate a random SELinux context for each container.  May also be set in SecurityContext.  If set in both SecurityContext and PodSecurityContext, the value specified in SecurityContext takes precedence for that container.",
								Ref:         ref("k8s.io/api/core/v1.SELinuxOptions"),
							},
						},
						"runAsUser": {
							SchemaProps: spec.SchemaProps{
								Description: "The UID to run the entrypoint of the container process. Defaults to user specified in image metadata if unspecified. May also be set in SecurityContext.  If set in both SecurityContext and PodSecurityContext, the value specified in SecurityContext takes precedence for that container.",
								Type:        []string{"integer"},
								Format:      "int64",
							},
						},
						"runAsNonRoot": {
							SchemaProps: spec.SchemaProps{
								Description: "Indicates that the container must run as a non-root user. If true, the Kubelet will validate the image at runtime to ensure that it does not run as UID 0 (root) and fail to start the container if it does. If unset or false, no such validation will be performed. May also be set in SecurityContext.  If set in both SecurityContext and PodSecurityContext, the value specified in SecurityContext takes precedence.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"supplementalGroups": {
							SchemaProps: spec.SchemaProps{
								Description: "A list of groups applied to the first process run in each container, in addition to the container's primary GID.  If unspecified, no groups will be added to any container.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"integer"},
											Format: "int64",
										},
									},
								},
							},
						},
						"fsGroup": {
							SchemaProps: spec.SchemaProps{
								Description: "A special supplemental group that applies to all containers in a pod. Some volume types allow the Kubelet to change the ownership of that volume to be owned by the pod:\n\n1. The owning GID will be the FSGroup 2. The setgid bit is set (new files created in the volume will be owned by FSGroup) 3. The permission bits are OR'd with rw-rw----\n\nIf unset, the Kubelet will not modify the ownership and permissions of any volume.",
								Type:        []string{"integer"},
								Format:      "int64",
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.SELinuxOptions"},
		},
		"k8s.io/api/core/v1.PodSignature": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Describes the class of pods that should avoid this node. Exactly one field should be set.",
					Properties: map[string]spec.Schema{
						"podController": {
							SchemaProps: spec.SchemaProps{
								Description: "Reference to controller whose pods should avoid this node.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.OwnerReference"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.OwnerReference"},
		},
		"k8s.io/api/core/v1.PodSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PodSpec is a description of a pod.",
					Properties: map[string]spec.Schema{
						"volumes": {
							VendorExtensible: spec.VendorExtensible{
								Extensions: spec.Extensions{
									"x-kubernetes-patch-merge-key": "name",
									"x-kubernetes-patch-strategy":  "merge,retainKeys",
								},
							},
							SchemaProps: spec.SchemaProps{
								Description: "List of volumes that can be mounted by containers belonging to the pod. More info: https://kubernetes.io/docs/concepts/storage/volumes",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.Volume"),
										},
									},
								},
							},
						},
						"initContainers": {
							VendorExtensible: spec.VendorExtensible{
								Extensions: spec.Extensions{
									"x-kubernetes-patch-merge-key": "name",
									"x-kubernetes-patch-strategy":  "merge",
								},
							},
							SchemaProps: spec.SchemaProps{
								Description: "List of initialization containers belonging to the pod. Init containers are executed in order prior to containers being started. If any init container fails, the pod is considered to have failed and is handled according to its restartPolicy. The name for an init container or normal container must be unique among all containers. Init containers may not have Lifecycle actions, Readiness probes, or Liveness probes. The resourceRequirements of an init container are taken into account during scheduling by finding the highest request/limit for each resource type, and then using the max of of that value or the sum of the normal containers. Limits are applied to init containers in a similar fashion. Init containers cannot currently be added or removed. Cannot be updated. More info: https://kubernetes.io/docs/concepts/workloads/pods/init-containers/",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.Container"),
										},
									},
								},
							},
						},
						"containers": {
							VendorExtensible: spec.VendorExtensible{
								Extensions: spec.Extensions{
									"x-kubernetes-patch-merge-key": "name",
									"x-kubernetes-patch-strategy":  "merge",
								},
							},
							SchemaProps: spec.SchemaProps{
								Description: "List of containers belonging to the pod. Containers cannot currently be added or removed. There must be at least one container in a Pod. Cannot be updated.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.Container"),
										},
									},
								},
							},
						},
						"restartPolicy": {
							SchemaProps: spec.SchemaProps{
								Description: "Restart policy for all containers within the pod. One of Always, OnFailure, Never. Default to Always. More info: https://kubernetes.io/docs/concepts/workloads/pods/pod-lifecycle/#restart-policy",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"terminationGracePeriodSeconds": {
							SchemaProps: spec.SchemaProps{
								Description: "Optional duration in seconds the pod needs to terminate gracefully. May be decreased in delete request. Value must be non-negative integer. The value zero indicates delete immediately. If this value is nil, the default grace period will be used instead. The grace period is the duration in seconds after the processes running in the pod are sent a termination signal and the time when the processes are forcibly halted with a kill signal. Set this value longer than the expected cleanup time for your process. Defaults to 30 seconds.",
								Type:        []string{"integer"},
								Format:      "int64",
							},
						},
						"activeDeadlineSeconds": {
							SchemaProps: spec.SchemaProps{
								Description: "Optional duration in seconds the pod may be active on the node relative to StartTime before the system will actively try to mark it failed and kill associated containers. Value must be a positive integer.",
								Type:        []string{"integer"},
								Format:      "int64",
							},
						},
						"dnsPolicy": {
							SchemaProps: spec.SchemaProps{
								Description: "Set DNS policy for the pod. Defaults to \"ClusterFirst\". Valid values are 'ClusterFirstWithHostNet', 'ClusterFirst', 'Default' or 'None'. DNS parameters given in DNSConfig will be merged with the policy selected with DNSPolicy. To have DNS options set along with hostNetwork, you have to specify DNS policy explicitly to 'ClusterFirstWithHostNet'. Note that 'None' policy is an alpha feature introduced in v1.9 and CustomPodDNS feature gate must be enabled to use it.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"nodeSelector": {
							SchemaProps: spec.SchemaProps{
								Description: "NodeSelector is a selector which must be true for the pod to fit on a node. Selector which must match a node's labels for the pod to be scheduled on that node. More info: https://kubernetes.io/docs/concepts/configuration/assign-pod-node/",
								Type:        []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"serviceAccountName": {
							SchemaProps: spec.SchemaProps{
								Description: "ServiceAccountName is the name of the ServiceAccount to use to run this pod. More info: https://kubernetes.io/docs/tasks/configure-pod-container/configure-service-account/",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"serviceAccount": {
							SchemaProps: spec.SchemaProps{
								Description: "DeprecatedServiceAccount is a depreciated alias for ServiceAccountName. Deprecated: Use serviceAccountName instead.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"automountServiceAccountToken": {
							SchemaProps: spec.SchemaProps{
								Description: "AutomountServiceAccountToken indicates whether a service account token should be automatically mounted.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"nodeName": {
							SchemaProps: spec.SchemaProps{
								Description: "NodeName is a request to schedule this pod onto a specific node. If it is non-empty, the scheduler simply schedules this pod onto that node, assuming that it fits resource requirements.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"hostNetwork": {
							SchemaProps: spec.SchemaProps{
								Description: "Host networking requested for this pod. Use the host's network namespace. If this option is set, the ports that will be used must be specified. Default to false.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"hostPID": {
							SchemaProps: spec.SchemaProps{
								Description: "Use the host's pid namespace. Optional: Default to false.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"hostIPC": {
							SchemaProps: spec.SchemaProps{
								Description: "Use the host's ipc namespace. Optional: Default to false.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"securityContext": {
							SchemaProps: spec.SchemaProps{
								Description: "SecurityContext holds pod-level security attributes and common container settings. Optional: Defaults to empty.  See type description for default values of each field.",
								Ref:         ref("k8s.io/api/core/v1.PodSecurityContext"),
							},
						},
						"imagePullSecrets": {
							VendorExtensible: spec.VendorExtensible{
								Extensions: spec.Extensions{
									"x-kubernetes-patch-merge-key": "name",
									"x-kubernetes-patch-strategy":  "merge",
								},
							},
							SchemaProps: spec.SchemaProps{
								Description: "ImagePullSecrets is an optional list of references to secrets in the same namespace to use for pulling any of the images used by this PodSpec. If specified, these secrets will be passed to individual puller implementations for them to use. For example, in the case of docker, only DockerConfig type secrets are honored. More info: https://kubernetes.io/docs/concepts/containers/images#specifying-imagepullsecrets-on-a-pod",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.LocalObjectReference"),
										},
									},
								},
							},
						},
						"hostname": {
							SchemaProps: spec.SchemaProps{
								Description: "Specifies the hostname of the Pod If not specified, the pod's hostname will be set to a system-defined value.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"subdomain": {
							SchemaProps: spec.SchemaProps{
								Description: "If specified, the fully qualified Pod hostname will be \"<hostname>.<subdomain>.<pod namespace>.svc.<cluster domain>\". If not specified, the pod will not have a domainname at all.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"affinity": {
							SchemaProps: spec.SchemaProps{
								Description: "If specified, the pod's scheduling constraints",
								Ref:         ref("k8s.io/api/core/v1.Affinity"),
							},
						},
						"schedulerName": {
							SchemaProps: spec.SchemaProps{
								Description: "If specified, the pod will be dispatched by specified scheduler. If not specified, the pod will be dispatched by default scheduler.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"tolerations": {
							SchemaProps: spec.SchemaProps{
								Description: "If specified, the pod's tolerations.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.Toleration"),
										},
									},
								},
							},
						},
						"hostAliases": {
							VendorExtensible: spec.VendorExtensible{
								Extensions: spec.Extensions{
									"x-kubernetes-patch-merge-key": "ip",
									"x-kubernetes-patch-strategy":  "merge",
								},
							},
							SchemaProps: spec.SchemaProps{
								Description: "HostAliases is an optional list of hosts and IPs that will be injected into the pod's hosts file if specified. This is only valid for non-hostNetwork pods.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.HostAlias"),
										},
									},
								},
							},
						},
						"priorityClassName": {
							SchemaProps: spec.SchemaProps{
								Description: "If specified, indicates the pod's priority. \"SYSTEM\" is a special keyword which indicates the highest priority. Any other name must be defined by creating a PriorityClass object with that name. If not specified, the pod priority will be default or zero if there is no default.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"priority": {
							SchemaProps: spec.SchemaProps{
								Description: "The priority value. Various system components use this field to find the priority of the pod. When Priority Admission Controller is enabled, it prevents users from setting this field. The admission controller populates this field from PriorityClassName. The higher the value, the higher the priority.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"dnsConfig": {
							SchemaProps: spec.SchemaProps{
								Description: "Specifies the DNS parameters of a pod. Parameters specified here will be merged to the generated DNS configuration based on DNSPolicy. This is an alpha feature introduced in v1.9 and CustomPodDNS feature gate must be enabled to use it.",
								Ref:         ref("k8s.io/api/core/v1.PodDNSConfig"),
							},
						},
					},
					Required: []string{"containers"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.Affinity", "k8s.io/api/core/v1.Container", "k8s.io/api/core/v1.HostAlias", "k8s.io/api/core/v1.LocalObjectReference", "k8s.io/api/core/v1.PodDNSConfig", "k8s.io/api/core/v1.PodSecurityContext", "k8s.io/api/core/v1.Toleration", "k8s.io/api/core/v1.Volume"},
		},
		"k8s.io/api/core/v1.PodStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PodStatus represents information about the status of a pod. Status may trail the actual state of a system.",
					Properties: map[string]spec.Schema{
						"phase": {
							SchemaProps: spec.SchemaProps{
								Description: "Current condition of the pod. More info: https://kubernetes.io/docs/concepts/workloads/pods/pod-lifecycle#pod-phase",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"conditions": {
							VendorExtensible: spec.VendorExtensible{
								Extensions: spec.Extensions{
									"x-kubernetes-patch-merge-key": "type",
									"x-kubernetes-patch-strategy":  "merge",
								},
							},
							SchemaProps: spec.SchemaProps{
								Description: "Current service state of pod. More info: https://kubernetes.io/docs/concepts/workloads/pods/pod-lifecycle#pod-conditions",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.PodCondition"),
										},
									},
								},
							},
						},
						"message": {
							SchemaProps: spec.SchemaProps{
								Description: "A human readable message indicating details about why the pod is in this condition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"reason": {
							SchemaProps: spec.SchemaProps{
								Description: "A brief CamelCase message indicating details about why the pod is in this state. e.g. 'Evicted'",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"hostIP": {
							SchemaProps: spec.SchemaProps{
								Description: "IP address of the host to which the pod is assigned. Empty if not yet scheduled.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"podIP": {
							SchemaProps: spec.SchemaProps{
								Description: "IP address allocated to the pod. Routable at least within the cluster. Empty if not yet allocated.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"startTime": {
							SchemaProps: spec.SchemaProps{
								Description: "RFC 3339 date and time at which the object was acknowledged by the Kubelet. This is before the Kubelet pulled the container image(s) for the pod.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"initContainerStatuses": {
							SchemaProps: spec.SchemaProps{
								Description: "The list has one entry per init container in the manifest. The most recent successful init container will have ready = true, the most recently started container will have startTime set. More info: https://kubernetes.io/docs/concepts/workloads/pods/pod-lifecycle#pod-and-container-status",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.ContainerStatus"),
										},
									},
								},
							},
						},
						"containerStatuses": {
							SchemaProps: spec.SchemaProps{
								Description: "The list has one entry per container in the manifest. Each entry is currently the output of `docker inspect`. More info: https://kubernetes.io/docs/concepts/workloads/pods/pod-lifecycle#pod-and-container-status",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.ContainerStatus"),
										},
									},
								},
							},
						},
						"qosClass": {
							SchemaProps: spec.SchemaProps{
								Description: "The Quality of Service (QOS) classification assigned to the pod based on resource requirements See PodQOSClass type for available QOS classes More info: https://github.com/kubernetes/kubernetes/blob/master/docs/design/resource-qos.md",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.ContainerStatus", "k8s.io/api/core/v1.PodCondition", "k8s.io/apimachinery/pkg/apis/meta/v1.Time"},
		},
		"k8s.io/api/core/v1.PodStatusResult": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PodStatusResult is a wrapper for PodStatus returned by kubelet that can be encode/decoded",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Most recently observed status of the pod. This data may not be up to date. Populated by the system. Read-only. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status",
								Ref:         ref("k8s.io/api/core/v1.PodStatus"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.PodStatus", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/core/v1.PodTemplate": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PodTemplate describes a template for creating copies of a predefined pod.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"template": {
							SchemaProps: spec.SchemaProps{
								Description: "Template defines the pods that will be created from this pod template. https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status",
								Ref:         ref("k8s.io/api/core/v1.PodTemplateSpec"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.PodTemplateSpec", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/core/v1.PodTemplateList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PodTemplateList is a list of PodTemplates.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "List of pod templates",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.PodTemplate"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.PodTemplate", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/core/v1.PodTemplateSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PodTemplateSpec describes the data a pod should have when created from a template",
					Properties: map[string]spec.Schema{
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "Specification of the desired behavior of the pod. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status",
								Ref:         ref("k8s.io/api/core/v1.PodSpec"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.PodSpec", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/core/v1.PortworxVolumeSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PortworxVolumeSource represents a Portworx volume resource.",
					Properties: map[string]spec.Schema{
						"volumeID": {
							SchemaProps: spec.SchemaProps{
								Description: "VolumeID uniquely identifies a Portworx volume",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"fsType": {
							SchemaProps: spec.SchemaProps{
								Description: "FSType represents the filesystem type to mount Must be a filesystem type supported by the host operating system. Ex. \"ext4\", \"xfs\". Implicitly inferred to be \"ext4\" if unspecified.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"readOnly": {
							SchemaProps: spec.SchemaProps{
								Description: "Defaults to false (read/write). ReadOnly here will force the ReadOnly setting in VolumeMounts.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
					},
					Required: []string{"volumeID"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.PreferAvoidPodsEntry": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Describes a class of pods that should avoid this node.",
					Properties: map[string]spec.Schema{
						"podSignature": {
							SchemaProps: spec.SchemaProps{
								Description: "The class of pods.",
								Ref:         ref("k8s.io/api/core/v1.PodSignature"),
							},
						},
						"evictionTime": {
							SchemaProps: spec.SchemaProps{
								Description: "Time at which this entry was added to the list.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"reason": {
							SchemaProps: spec.SchemaProps{
								Description: "(brief) reason why this entry was added to the list.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"message": {
							SchemaProps: spec.SchemaProps{
								Description: "Human readable message indicating why this entry was added to the list.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"podSignature"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.PodSignature", "k8s.io/apimachinery/pkg/apis/meta/v1.Time"},
		},
		"k8s.io/api/core/v1.PreferredSchedulingTerm": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "An empty preferred scheduling term matches all objects with implicit weight 0 (i.e. it's a no-op). A null preferred scheduling term matches no objects (i.e. is also a no-op).",
					Properties: map[string]spec.Schema{
						"weight": {
							SchemaProps: spec.SchemaProps{
								Description: "Weight associated with matching the corresponding nodeSelectorTerm, in the range 1-100.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"preference": {
							SchemaProps: spec.SchemaProps{
								Description: "A node selector term, associated with the corresponding weight.",
								Ref:         ref("k8s.io/api/core/v1.NodeSelectorTerm"),
							},
						},
					},
					Required: []string{"weight", "preference"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.NodeSelectorTerm"},
		},
		"k8s.io/api/core/v1.Probe": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Probe describes a health check to be performed against a container to determine whether it is alive or ready to receive traffic.",
					Properties: map[string]spec.Schema{
						"exec": {
							SchemaProps: spec.SchemaProps{
								Description: "One and only one of the following should be specified. Exec specifies the action to take.",
								Ref:         ref("k8s.io/api/core/v1.ExecAction"),
							},
						},
						"httpGet": {
							SchemaProps: spec.SchemaProps{
								Description: "HTTPGet specifies the http request to perform.",
								Ref:         ref("k8s.io/api/core/v1.HTTPGetAction"),
							},
						},
						"tcpSocket": {
							SchemaProps: spec.SchemaProps{
								Description: "TCPSocket specifies an action involving a TCP port. TCP hooks not yet supported",
								Ref:         ref("k8s.io/api/core/v1.TCPSocketAction"),
							},
						},
						"initialDelaySeconds": {
							SchemaProps: spec.SchemaProps{
								Description: "Number of seconds after the container has started before liveness probes are initiated. More info: https://kubernetes.io/docs/concepts/workloads/pods/pod-lifecycle#container-probes",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"timeoutSeconds": {
							SchemaProps: spec.SchemaProps{
								Description: "Number of seconds after which the probe times out. Defaults to 1 second. Minimum value is 1. More info: https://kubernetes.io/docs/concepts/workloads/pods/pod-lifecycle#container-probes",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"periodSeconds": {
							SchemaProps: spec.SchemaProps{
								Description: "How often (in seconds) to perform the probe. Default to 10 seconds. Minimum value is 1.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"successThreshold": {
							SchemaProps: spec.SchemaProps{
								Description: "Minimum consecutive successes for the probe to be considered successful after having failed. Defaults to 1. Must be 1 for liveness. Minimum value is 1.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"failureThreshold": {
							SchemaProps: spec.SchemaProps{
								Description: "Minimum consecutive failures for the probe to be considered failed after having succeeded. Defaults to 3. Minimum value is 1.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.ExecAction", "k8s.io/api/core/v1.HTTPGetAction", "k8s.io/api/core/v1.TCPSocketAction"},
		},
		"k8s.io/api/core/v1.ProjectedVolumeSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Represents a projected volume source",
					Properties: map[string]spec.Schema{
						"sources": {
							SchemaProps: spec.SchemaProps{
								Description: "list of volume projections",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.VolumeProjection"),
										},
									},
								},
							},
						},
						"defaultMode": {
							SchemaProps: spec.SchemaProps{
								Description: "Mode bits to use on created files by default. Must be a value between 0 and 0777. Directories within the path are not affected by this setting. This might be in conflict with other options that affect the file mode, like fsGroup, and the result can be other mode bits set.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
					},
					Required: []string{"sources"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.VolumeProjection"},
		},
		"k8s.io/api/core/v1.QuobyteVolumeSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Represents a Quobyte mount that lasts the lifetime of a pod. Quobyte volumes do not support ownership management or SELinux relabeling.",
					Properties: map[string]spec.Schema{
						"registry": {
							SchemaProps: spec.SchemaProps{
								Description: "Registry represents a single or multiple Quobyte Registry services specified as a string as host:port pair (multiple entries are separated with commas) which acts as the central registry for volumes",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"volume": {
							SchemaProps: spec.SchemaProps{
								Description: "Volume is a string that references an already created Quobyte volume by name.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"readOnly": {
							SchemaProps: spec.SchemaProps{
								Description: "ReadOnly here will force the Quobyte volume to be mounted with read-only permissions. Defaults to false.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"user": {
							SchemaProps: spec.SchemaProps{
								Description: "User to map volume access to Defaults to serivceaccount user",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"group": {
							SchemaProps: spec.SchemaProps{
								Description: "Group to map volume access to Default is no group",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"registry", "volume"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.RBDPersistentVolumeSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Represents a Rados Block Device mount that lasts the lifetime of a pod. RBD volumes support ownership management and SELinux relabeling.",
					Properties: map[string]spec.Schema{
						"monitors": {
							SchemaProps: spec.SchemaProps{
								Description: "A collection of Ceph monitors. More info: https://releases.k8s.io/HEAD/examples/volumes/rbd/README.md#how-to-use-it",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"image": {
							SchemaProps: spec.SchemaProps{
								Description: "The rados image name. More info: https://releases.k8s.io/HEAD/examples/volumes/rbd/README.md#how-to-use-it",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"fsType": {
							SchemaProps: spec.SchemaProps{
								Description: "Filesystem type of the volume that you want to mount. Tip: Ensure that the filesystem type is supported by the host operating system. Examples: \"ext4\", \"xfs\", \"ntfs\". Implicitly inferred to be \"ext4\" if unspecified. More info: https://kubernetes.io/docs/concepts/storage/volumes#rbd",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"pool": {
							SchemaProps: spec.SchemaProps{
								Description: "The rados pool name. Default is rbd. More info: https://releases.k8s.io/HEAD/examples/volumes/rbd/README.md#how-to-use-it",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"user": {
							SchemaProps: spec.SchemaProps{
								Description: "The rados user name. Default is admin. More info: https://releases.k8s.io/HEAD/examples/volumes/rbd/README.md#how-to-use-it",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"keyring": {
							SchemaProps: spec.SchemaProps{
								Description: "Keyring is the path to key ring for RBDUser. Default is /etc/ceph/keyring. More info: https://releases.k8s.io/HEAD/examples/volumes/rbd/README.md#how-to-use-it",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"secretRef": {
							SchemaProps: spec.SchemaProps{
								Description: "SecretRef is name of the authentication secret for RBDUser. If provided overrides keyring. Default is nil. More info: https://releases.k8s.io/HEAD/examples/volumes/rbd/README.md#how-to-use-it",
								Ref:         ref("k8s.io/api/core/v1.SecretReference"),
							},
						},
						"readOnly": {
							SchemaProps: spec.SchemaProps{
								Description: "ReadOnly here will force the ReadOnly setting in VolumeMounts. Defaults to false. More info: https://releases.k8s.io/HEAD/examples/volumes/rbd/README.md#how-to-use-it",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
					},
					Required: []string{"monitors", "image"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.SecretReference"},
		},
		"k8s.io/api/core/v1.RBDVolumeSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Represents a Rados Block Device mount that lasts the lifetime of a pod. RBD volumes support ownership management and SELinux relabeling.",
					Properties: map[string]spec.Schema{
						"monitors": {
							SchemaProps: spec.SchemaProps{
								Description: "A collection of Ceph monitors. More info: https://releases.k8s.io/HEAD/examples/volumes/rbd/README.md#how-to-use-it",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"image": {
							SchemaProps: spec.SchemaProps{
								Description: "The rados image name. More info: https://releases.k8s.io/HEAD/examples/volumes/rbd/README.md#how-to-use-it",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"fsType": {
							SchemaProps: spec.SchemaProps{
								Description: "Filesystem type of the volume that you want to mount. Tip: Ensure that the filesystem type is supported by the host operating system. Examples: \"ext4\", \"xfs\", \"ntfs\". Implicitly inferred to be \"ext4\" if unspecified. More info: https://kubernetes.io/docs/concepts/storage/volumes#rbd",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"pool": {
							SchemaProps: spec.SchemaProps{
								Description: "The rados pool name. Default is rbd. More info: https://releases.k8s.io/HEAD/examples/volumes/rbd/README.md#how-to-use-it",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"user": {
							SchemaProps: spec.SchemaProps{
								Description: "The rados user name. Default is admin. More info: https://releases.k8s.io/HEAD/examples/volumes/rbd/README.md#how-to-use-it",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"keyring": {
							SchemaProps: spec.SchemaProps{
								Description: "Keyring is the path to key ring for RBDUser. Default is /etc/ceph/keyring. More info: https://releases.k8s.io/HEAD/examples/volumes/rbd/README.md#how-to-use-it",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"secretRef": {
							SchemaProps: spec.SchemaProps{
								Description: "SecretRef is name of the authentication secret for RBDUser. If provided overrides keyring. Default is nil. More info: https://releases.k8s.io/HEAD/examples/volumes/rbd/README.md#how-to-use-it",
								Ref:         ref("k8s.io/api/core/v1.LocalObjectReference"),
							},
						},
						"readOnly": {
							SchemaProps: spec.SchemaProps{
								Description: "ReadOnly here will force the ReadOnly setting in VolumeMounts. Defaults to false. More info: https://releases.k8s.io/HEAD/examples/volumes/rbd/README.md#how-to-use-it",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
					},
					Required: []string{"monitors", "image"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.LocalObjectReference"},
		},
		"k8s.io/api/core/v1.RangeAllocation": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "RangeAllocation is not a public type.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"range": {
							SchemaProps: spec.SchemaProps{
								Description: "Range is string that identifies the range represented by 'data'.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"data": {
							SchemaProps: spec.SchemaProps{
								Description: "Data is a bit array containing all allocated addresses in the previous segment.",
								Type:        []string{"string"},
								Format:      "byte",
							},
						},
					},
					Required: []string{"range", "data"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/core/v1.ReplicationController": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ReplicationController represents the configuration of a replication controller.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "If the Labels of a ReplicationController are empty, they are defaulted to be the same as the Pod(s) that the replication controller manages. Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "Spec defines the specification of the desired behavior of the replication controller. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status",
								Ref:         ref("k8s.io/api/core/v1.ReplicationControllerSpec"),
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Status is the most recently observed status of the replication controller. This data may be out of date by some window of time. Populated by the system. Read-only. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status",
								Ref:         ref("k8s.io/api/core/v1.ReplicationControllerStatus"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.ReplicationControllerSpec", "k8s.io/api/core/v1.ReplicationControllerStatus", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/core/v1.ReplicationControllerCondition": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ReplicationControllerCondition describes the state of a replication controller at a certain point.",
					Properties: map[string]spec.Schema{
						"type": {
							SchemaProps: spec.SchemaProps{
								Description: "Type of replication controller condition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Status of the condition, one of True, False, Unknown.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"lastTransitionTime": {
							SchemaProps: spec.SchemaProps{
								Description: "The last time the condition transitioned from one status to another.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"reason": {
							SchemaProps: spec.SchemaProps{
								Description: "The reason for the condition's last transition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"message": {
							SchemaProps: spec.SchemaProps{
								Description: "A human readable message indicating details about the transition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"type", "status"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.Time"},
		},
		"k8s.io/api/core/v1.ReplicationControllerList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ReplicationControllerList is a collection of replication controllers.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "List of replication controllers. More info: https://kubernetes.io/docs/concepts/workloads/controllers/replicationcontroller",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.ReplicationController"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.ReplicationController", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/core/v1.ReplicationControllerSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ReplicationControllerSpec is the specification of a replication controller.",
					Properties: map[string]spec.Schema{
						"replicas": {
							SchemaProps: spec.SchemaProps{
								Description: "Replicas is the number of desired replicas. This is a pointer to distinguish between explicit zero and unspecified. Defaults to 1. More info: https://kubernetes.io/docs/concepts/workloads/controllers/replicationcontroller#what-is-a-replicationcontroller",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"minReadySeconds": {
							SchemaProps: spec.SchemaProps{
								Description: "Minimum number of seconds for which a newly created pod should be ready without any of its container crashing, for it to be considered available. Defaults to 0 (pod will be considered available as soon as it is ready)",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"selector": {
							SchemaProps: spec.SchemaProps{
								Description: "Selector is a label query over pods that should match the Replicas count. If Selector is empty, it is defaulted to the labels present on the Pod template. Label keys and values that must match in order to be controlled by this replication controller, if empty defaulted to labels on Pod template. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/labels/#label-selectors",
								Type:        []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"template": {
							SchemaProps: spec.SchemaProps{
								Description: "Template is the object that describes the pod that will be created if insufficient replicas are detected. This takes precedence over a TemplateRef. More info: https://kubernetes.io/docs/concepts/workloads/controllers/replicationcontroller#pod-template",
								Ref:         ref("k8s.io/api/core/v1.PodTemplateSpec"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.PodTemplateSpec"},
		},
		"k8s.io/api/core/v1.ReplicationControllerStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ReplicationControllerStatus represents the current status of a replication controller.",
					Properties: map[string]spec.Schema{
						"replicas": {
							SchemaProps: spec.SchemaProps{
								Description: "Replicas is the most recently oberved number of replicas. More info: https://kubernetes.io/docs/concepts/workloads/controllers/replicationcontroller#what-is-a-replicationcontroller",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"fullyLabeledReplicas": {
							SchemaProps: spec.SchemaProps{
								Description: "The number of pods that have labels matching the labels of the pod template of the replication controller.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"readyReplicas": {
							SchemaProps: spec.SchemaProps{
								Description: "The number of ready replicas for this replication controller.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"availableReplicas": {
							SchemaProps: spec.SchemaProps{
								Description: "The number of available replicas (ready for at least minReadySeconds) for this replication controller.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"observedGeneration": {
							SchemaProps: spec.SchemaProps{
								Description: "ObservedGeneration reflects the generation of the most recently observed replication controller.",
								Type:        []string{"integer"},
								Format:      "int64",
							},
						},
						"conditions": {
							VendorExtensible: spec.VendorExtensible{
								Extensions: spec.Extensions{
									"x-kubernetes-patch-merge-key": "type",
									"x-kubernetes-patch-strategy":  "merge",
								},
							},
							SchemaProps: spec.SchemaProps{
								Description: "Represents the latest available observations of a replication controller's current state.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.ReplicationControllerCondition"),
										},
									},
								},
							},
						},
					},
					Required: []string{"replicas"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.ReplicationControllerCondition"},
		},
		"k8s.io/api/core/v1.ResourceFieldSelector": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ResourceFieldSelector represents container resources (cpu, memory) and their output format",
					Properties: map[string]spec.Schema{
						"containerName": {
							SchemaProps: spec.SchemaProps{
								Description: "Container name: required for volumes, optional for env vars",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"resource": {
							SchemaProps: spec.SchemaProps{
								Description: "Required: resource to select",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"divisor": {
							SchemaProps: spec.SchemaProps{
								Description: "Specifies the output format of the exposed resources, defaults to \"1\"",
								Ref:         ref("k8s.io/apimachinery/pkg/api/resource.Quantity"),
							},
						},
					},
					Required: []string{"resource"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/api/resource.Quantity"},
		},
		"k8s.io/api/core/v1.ResourceQuota": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ResourceQuota sets aggregate quota restrictions enforced per namespace",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "Spec defines the desired quota. https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status",
								Ref:         ref("k8s.io/api/core/v1.ResourceQuotaSpec"),
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Status defines the actual enforced quota and its current usage. https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status",
								Ref:         ref("k8s.io/api/core/v1.ResourceQuotaStatus"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.ResourceQuotaSpec", "k8s.io/api/core/v1.ResourceQuotaStatus", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/core/v1.ResourceQuotaList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ResourceQuotaList is a list of ResourceQuota items.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "Items is a list of ResourceQuota objects. More info: https://kubernetes.io/docs/concepts/policy/resource-quotas/",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.ResourceQuota"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.ResourceQuota", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/core/v1.ResourceQuotaSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ResourceQuotaSpec defines the desired hard limits to enforce for Quota.",
					Properties: map[string]spec.Schema{
						"hard": {
							SchemaProps: spec.SchemaProps{
								Description: "Hard is the set of desired hard limits for each named resource. More info: https://kubernetes.io/docs/concepts/policy/resource-quotas/",
								Type:        []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apimachinery/pkg/api/resource.Quantity"),
										},
									},
								},
							},
						},
						"scopes": {
							SchemaProps: spec.SchemaProps{
								Description: "A collection of filters that must match each object tracked by a quota. If not specified, the quota matches all objects.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/api/resource.Quantity"},
		},
		"k8s.io/api/core/v1.ResourceQuotaStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ResourceQuotaStatus defines the enforced hard limits and observed use.",
					Properties: map[string]spec.Schema{
						"hard": {
							SchemaProps: spec.SchemaProps{
								Description: "Hard is the set of enforced hard limits for each named resource. More info: https://kubernetes.io/docs/concepts/policy/resource-quotas/",
								Type:        []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apimachinery/pkg/api/resource.Quantity"),
										},
									},
								},
							},
						},
						"used": {
							SchemaProps: spec.SchemaProps{
								Description: "Used is the current observed total usage of the resource in the namespace.",
								Type:        []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apimachinery/pkg/api/resource.Quantity"),
										},
									},
								},
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/api/resource.Quantity"},
		},
		"k8s.io/api/core/v1.ResourceRequirements": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ResourceRequirements describes the compute resource requirements.",
					Properties: map[string]spec.Schema{
						"limits": {
							SchemaProps: spec.SchemaProps{
								Description: "Limits describes the maximum amount of compute resources allowed. More info: https://kubernetes.io/docs/concepts/configuration/manage-compute-resources-container/",
								Type:        []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apimachinery/pkg/api/resource.Quantity"),
										},
									},
								},
							},
						},
						"requests": {
							SchemaProps: spec.SchemaProps{
								Description: "Requests describes the minimum amount of compute resources required. If Requests is omitted for a container, it defaults to Limits if that is explicitly specified, otherwise to an implementation-defined value. More info: https://kubernetes.io/docs/concepts/configuration/manage-compute-resources-container/",
								Type:        []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apimachinery/pkg/api/resource.Quantity"),
										},
									},
								},
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/api/resource.Quantity"},
		},
		"k8s.io/api/core/v1.SELinuxOptions": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "SELinuxOptions are the labels to be applied to the container",
					Properties: map[string]spec.Schema{
						"user": {
							SchemaProps: spec.SchemaProps{
								Description: "User is a SELinux user label that applies to the container.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"role": {
							SchemaProps: spec.SchemaProps{
								Description: "Role is a SELinux role label that applies to the container.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"type": {
							SchemaProps: spec.SchemaProps{
								Description: "Type is a SELinux type label that applies to the container.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"level": {
							SchemaProps: spec.SchemaProps{
								Description: "Level is SELinux level label that applies to the container.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.ScaleIOPersistentVolumeSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ScaleIOPersistentVolumeSource represents a persistent ScaleIO volume",
					Properties: map[string]spec.Schema{
						"gateway": {
							SchemaProps: spec.SchemaProps{
								Description: "The host address of the ScaleIO API Gateway.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"system": {
							SchemaProps: spec.SchemaProps{
								Description: "The name of the storage system as configured in ScaleIO.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"secretRef": {
							SchemaProps: spec.SchemaProps{
								Description: "SecretRef references to the secret for ScaleIO user and other sensitive information. If this is not provided, Login operation will fail.",
								Ref:         ref("k8s.io/api/core/v1.SecretReference"),
							},
						},
						"sslEnabled": {
							SchemaProps: spec.SchemaProps{
								Description: "Flag to enable/disable SSL communication with Gateway, default false",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"protectionDomain": {
							SchemaProps: spec.SchemaProps{
								Description: "The name of the ScaleIO Protection Domain for the configured storage.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"storagePool": {
							SchemaProps: spec.SchemaProps{
								Description: "The ScaleIO Storage Pool associated with the protection domain.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"storageMode": {
							SchemaProps: spec.SchemaProps{
								Description: "Indicates whether the storage for a volume should be ThickProvisioned or ThinProvisioned.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"volumeName": {
							SchemaProps: spec.SchemaProps{
								Description: "The name of a volume already created in the ScaleIO system that is associated with this volume source.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"fsType": {
							SchemaProps: spec.SchemaProps{
								Description: "Filesystem type to mount. Must be a filesystem type supported by the host operating system. Ex. \"ext4\", \"xfs\", \"ntfs\". Implicitly inferred to be \"ext4\" if unspecified.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"readOnly": {
							SchemaProps: spec.SchemaProps{
								Description: "Defaults to false (read/write). ReadOnly here will force the ReadOnly setting in VolumeMounts.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
					},
					Required: []string{"gateway", "system", "secretRef"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.SecretReference"},
		},
		"k8s.io/api/core/v1.ScaleIOVolumeSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ScaleIOVolumeSource represents a persistent ScaleIO volume",
					Properties: map[string]spec.Schema{
						"gateway": {
							SchemaProps: spec.SchemaProps{
								Description: "The host address of the ScaleIO API Gateway.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"system": {
							SchemaProps: spec.SchemaProps{
								Description: "The name of the storage system as configured in ScaleIO.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"secretRef": {
							SchemaProps: spec.SchemaProps{
								Description: "SecretRef references to the secret for ScaleIO user and other sensitive information. If this is not provided, Login operation will fail.",
								Ref:         ref("k8s.io/api/core/v1.LocalObjectReference"),
							},
						},
						"sslEnabled": {
							SchemaProps: spec.SchemaProps{
								Description: "Flag to enable/disable SSL communication with Gateway, default false",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"protectionDomain": {
							SchemaProps: spec.SchemaProps{
								Description: "The name of the ScaleIO Protection Domain for the configured storage.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"storagePool": {
							SchemaProps: spec.SchemaProps{
								Description: "The ScaleIO Storage Pool associated with the protection domain.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"storageMode": {
							SchemaProps: spec.SchemaProps{
								Description: "Indicates whether the storage for a volume should be ThickProvisioned or ThinProvisioned.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"volumeName": {
							SchemaProps: spec.SchemaProps{
								Description: "The name of a volume already created in the ScaleIO system that is associated with this volume source.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"fsType": {
							SchemaProps: spec.SchemaProps{
								Description: "Filesystem type to mount. Must be a filesystem type supported by the host operating system. Ex. \"ext4\", \"xfs\", \"ntfs\". Implicitly inferred to be \"ext4\" if unspecified.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"readOnly": {
							SchemaProps: spec.SchemaProps{
								Description: "Defaults to false (read/write). ReadOnly here will force the ReadOnly setting in VolumeMounts.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
					},
					Required: []string{"gateway", "system", "secretRef"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.LocalObjectReference"},
		},
		"k8s.io/api/core/v1.Secret": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Secret holds secret data of a certain type. The total bytes of the values in the Data field must be less than MaxSecretSize bytes.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"data": {
							SchemaProps: spec.SchemaProps{
								Description: "Data contains the secret data. Each key must consist of alphanumeric characters, '-', '_' or '.'. The serialized form of the secret data is a base64 encoded string, representing the arbitrary (possibly non-string) data value here. Described in https://tools.ietf.org/html/rfc4648#section-4",
								Type:        []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "byte",
										},
									},
								},
							},
						},
						"stringData": {
							SchemaProps: spec.SchemaProps{
								Description: "stringData allows specifying non-binary secret data in string form. It is provided as a write-only convenience method. All keys and values are merged into the data field on write, overwriting any existing values. It is never output when reading from the API.",
								Type:        []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"type": {
							SchemaProps: spec.SchemaProps{
								Description: "Used to facilitate programmatic handling of secret data.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/core/v1.SecretEnvSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "SecretEnvSource selects a Secret to populate the environment variables with.\n\nThe contents of the target Secret's Data field will represent the key-value pairs as environment variables.",
					Properties: map[string]spec.Schema{
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "Name of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/names/#names",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"optional": {
							SchemaProps: spec.SchemaProps{
								Description: "Specify whether the Secret must be defined",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.SecretKeySelector": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "SecretKeySelector selects a key of a Secret.",
					Properties: map[string]spec.Schema{
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "Name of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/names/#names",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"key": {
							SchemaProps: spec.SchemaProps{
								Description: "The key of the secret to select from.  Must be a valid secret key.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"optional": {
							SchemaProps: spec.SchemaProps{
								Description: "Specify whether the Secret or it's key must be defined",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
					},
					Required: []string{"key"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.SecretList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "SecretList is a list of Secret.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "Items is a list of secret objects. More info: https://kubernetes.io/docs/concepts/configuration/secret",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.Secret"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.Secret", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/core/v1.SecretProjection": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Adapts a secret into a projected volume.\n\nThe contents of the target Secret's Data field will be presented in a projected volume as files using the keys in the Data field as the file names. Note that this is identical to a secret volume source without the default mode.",
					Properties: map[string]spec.Schema{
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "Name of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/names/#names",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "If unspecified, each key-value pair in the Data field of the referenced Secret will be projected into the volume as a file whose name is the key and content is the value. If specified, the listed keys will be projected into the specified paths, and unlisted keys will not be present. If a key is specified which is not present in the Secret, the volume setup will error unless it is marked optional. Paths must be relative and may not contain the '..' path or start with '..'.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.KeyToPath"),
										},
									},
								},
							},
						},
						"optional": {
							SchemaProps: spec.SchemaProps{
								Description: "Specify whether the Secret or its key must be defined",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.KeyToPath"},
		},
		"k8s.io/api/core/v1.SecretReference": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "SecretReference represents a Secret Reference. It has enough information to retrieve secret in any namespace",
					Properties: map[string]spec.Schema{
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "Name is unique within a namespace to reference a secret resource.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"namespace": {
							SchemaProps: spec.SchemaProps{
								Description: "Namespace defines the space within which the secret name must be unique.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.SecretVolumeSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Adapts a Secret into a volume.\n\nThe contents of the target Secret's Data field will be presented in a volume as files using the keys in the Data field as the file names. Secret volumes support ownership management and SELinux relabeling.",
					Properties: map[string]spec.Schema{
						"secretName": {
							SchemaProps: spec.SchemaProps{
								Description: "Name of the secret in the pod's namespace to use. More info: https://kubernetes.io/docs/concepts/storage/volumes#secret",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "If unspecified, each key-value pair in the Data field of the referenced Secret will be projected into the volume as a file whose name is the key and content is the value. If specified, the listed keys will be projected into the specified paths, and unlisted keys will not be present. If a key is specified which is not present in the Secret, the volume setup will error unless it is marked optional. Paths must be relative and may not contain the '..' path or start with '..'.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.KeyToPath"),
										},
									},
								},
							},
						},
						"defaultMode": {
							SchemaProps: spec.SchemaProps{
								Description: "Optional: mode bits to use on created files by default. Must be a value between 0 and 0777. Defaults to 0644. Directories within the path are not affected by this setting. This might be in conflict with other options that affect the file mode, like fsGroup, and the result can be other mode bits set.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"optional": {
							SchemaProps: spec.SchemaProps{
								Description: "Specify whether the Secret or it's keys must be defined",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.KeyToPath"},
		},
		"k8s.io/api/core/v1.SecurityContext": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "SecurityContext holds security configuration that will be applied to a container. Some fields are present in both SecurityContext and PodSecurityContext.  When both are set, the values in SecurityContext take precedence.",
					Properties: map[string]spec.Schema{
						"capabilities": {
							SchemaProps: spec.SchemaProps{
								Description: "The capabilities to add/drop when running containers. Defaults to the default set of capabilities granted by the container runtime.",
								Ref:         ref("k8s.io/api/core/v1.Capabilities"),
							},
						},
						"privileged": {
							SchemaProps: spec.SchemaProps{
								Description: "Run container in privileged mode. Processes in privileged containers are essentially equivalent to root on the host. Defaults to false.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"seLinuxOptions": {
							SchemaProps: spec.SchemaProps{
								Description: "The SELinux context to be applied to the container. If unspecified, the container runtime will allocate a random SELinux context for each container.  May also be set in PodSecurityContext.  If set in both SecurityContext and PodSecurityContext, the value specified in SecurityContext takes precedence.",
								Ref:         ref("k8s.io/api/core/v1.SELinuxOptions"),
							},
						},
						"runAsUser": {
							SchemaProps: spec.SchemaProps{
								Description: "The UID to run the entrypoint of the container process. Defaults to user specified in image metadata if unspecified. May also be set in PodSecurityContext.  If set in both SecurityContext and PodSecurityContext, the value specified in SecurityContext takes precedence.",
								Type:        []string{"integer"},
								Format:      "int64",
							},
						},
						"runAsNonRoot": {
							SchemaProps: spec.SchemaProps{
								Description: "Indicates that the container must run as a non-root user. If true, the Kubelet will validate the image at runtime to ensure that it does not run as UID 0 (root) and fail to start the container if it does. If unset or false, no such validation will be performed. May also be set in PodSecurityContext.  If set in both SecurityContext and PodSecurityContext, the value specified in SecurityContext takes precedence.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"readOnlyRootFilesystem": {
							SchemaProps: spec.SchemaProps{
								Description: "Whether this container has a read-only root filesystem. Default is false.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"allowPrivilegeEscalation": {
							SchemaProps: spec.SchemaProps{
								Description: "AllowPrivilegeEscalation controls whether a process can gain more privileges than its parent process. This bool directly controls if the no_new_privs flag will be set on the container process. AllowPrivilegeEscalation is true always when the container is: 1) run as Privileged 2) has CAP_SYS_ADMIN",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.Capabilities", "k8s.io/api/core/v1.SELinuxOptions"},
		},
		"k8s.io/api/core/v1.SerializedReference": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "SerializedReference is a reference to serialized object.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"reference": {
							SchemaProps: spec.SchemaProps{
								Description: "The reference to an object in the system.",
								Ref:         ref("k8s.io/api/core/v1.ObjectReference"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.ObjectReference"},
		},
		"k8s.io/api/core/v1.Service": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Service is a named abstraction of software service (for example, mysql) consisting of local port (for example 3306) that the proxy listens on, and the selector that determines which pods will answer requests sent through the proxy.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "Spec defines the behavior of a service. https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status",
								Ref:         ref("k8s.io/api/core/v1.ServiceSpec"),
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Most recently observed status of the service. Populated by the system. Read-only. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status",
								Ref:         ref("k8s.io/api/core/v1.ServiceStatus"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.ServiceSpec", "k8s.io/api/core/v1.ServiceStatus", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/core/v1.ServiceAccount": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ServiceAccount binds together: * a name, understood by users, and perhaps by peripheral systems, for an identity * a principal that can be authenticated and authorized * a set of secrets",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"secrets": {
							VendorExtensible: spec.VendorExtensible{
								Extensions: spec.Extensions{
									"x-kubernetes-patch-merge-key": "name",
									"x-kubernetes-patch-strategy":  "merge",
								},
							},
							SchemaProps: spec.SchemaProps{
								Description: "Secrets is the list of secrets allowed to be used by pods running using this ServiceAccount. More info: https://kubernetes.io/docs/concepts/configuration/secret",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.ObjectReference"),
										},
									},
								},
							},
						},
						"imagePullSecrets": {
							SchemaProps: spec.SchemaProps{
								Description: "ImagePullSecrets is a list of references to secrets in the same namespace to use for pulling any images in pods that reference this ServiceAccount. ImagePullSecrets are distinct from Secrets because Secrets can be mounted in the pod, but ImagePullSecrets are only accessed by the kubelet. More info: https://kubernetes.io/docs/concepts/containers/images/#specifying-imagepullsecrets-on-a-pod",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.LocalObjectReference"),
										},
									},
								},
							},
						},
						"automountServiceAccountToken": {
							SchemaProps: spec.SchemaProps{
								Description: "AutomountServiceAccountToken indicates whether pods running as this service account should have an API token automatically mounted. Can be overridden at the pod level.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.LocalObjectReference", "k8s.io/api/core/v1.ObjectReference", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/core/v1.ServiceAccountList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ServiceAccountList is a list of ServiceAccount objects",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "List of ServiceAccounts. More info: https://kubernetes.io/docs/tasks/configure-pod-container/configure-service-account/",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.ServiceAccount"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.ServiceAccount", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/core/v1.ServiceList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ServiceList holds a list of services.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "List of services",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.Service"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.Service", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/core/v1.ServicePort": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ServicePort contains information on service's port.",
					Properties: map[string]spec.Schema{
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "The name of this port within the service. This must be a DNS_LABEL. All ports within a ServiceSpec must have unique names. This maps to the 'Name' field in EndpointPort objects. Optional if only one ServicePort is defined on this service.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"protocol": {
							SchemaProps: spec.SchemaProps{
								Description: "The IP protocol for this port. Supports \"TCP\" and \"UDP\". Default is TCP.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"port": {
							SchemaProps: spec.SchemaProps{
								Description: "The port that will be exposed by this service.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"targetPort": {
							SchemaProps: spec.SchemaProps{
								Description: "Number or name of the port to access on the pods targeted by the service. Number must be in the range 1 to 65535. Name must be an IANA_SVC_NAME. If this is a string, it will be looked up as a named port in the target Pod's container ports. If this is not specified, the value of the 'port' field is used (an identity map). This field is ignored for services with clusterIP=None, and should be omitted or set equal to the 'port' field. More info: https://kubernetes.io/docs/concepts/services-networking/service/#defining-a-service",
								Ref:         ref("k8s.io/apimachinery/pkg/util/intstr.IntOrString"),
							},
						},
						"nodePort": {
							SchemaProps: spec.SchemaProps{
								Description: "The port on each node on which this service is exposed when type=NodePort or LoadBalancer. Usually assigned by the system. If specified, it will be allocated to the service if unused or else creation of the service will fail. Default is to auto-allocate a port if the ServiceType of this Service requires one. More info: https://kubernetes.io/docs/concepts/services-networking/service/#type-nodeport",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
					},
					Required: []string{"port"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/util/intstr.IntOrString"},
		},
		"k8s.io/api/core/v1.ServiceProxyOptions": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ServiceProxyOptions is the query options to a Service's proxy call.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"path": {
							SchemaProps: spec.SchemaProps{
								Description: "Path is the part of URLs that include service endpoints, suffixes, and parameters to use for the current proxy request to service. For example, the whole request URL is http://localhost/api/v1/namespaces/kube-system/services/elasticsearch-logging/_search?q=user:kimchy. Path is _search?q=user:kimchy.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.ServiceSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ServiceSpec describes the attributes that a user creates on a service.",
					Properties: map[string]spec.Schema{
						"ports": {
							VendorExtensible: spec.VendorExtensible{
								Extensions: spec.Extensions{
									"x-kubernetes-patch-merge-key": "port",
									"x-kubernetes-patch-strategy":  "merge",
								},
							},
							SchemaProps: spec.SchemaProps{
								Description: "The list of ports that are exposed by this service. More info: https://kubernetes.io/docs/concepts/services-networking/service/#virtual-ips-and-service-proxies",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.ServicePort"),
										},
									},
								},
							},
						},
						"selector": {
							SchemaProps: spec.SchemaProps{
								Description: "Route service traffic to pods with label keys and values matching this selector. If empty or not present, the service is assumed to have an external process managing its endpoints, which Kubernetes will not modify. Only applies to types ClusterIP, NodePort, and LoadBalancer. Ignored if type is ExternalName. More info: https://kubernetes.io/docs/concepts/services-networking/service/",
								Type:        []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"clusterIP": {
							SchemaProps: spec.SchemaProps{
								Description: "clusterIP is the IP address of the service and is usually assigned randomly by the master. If an address is specified manually and is not in use by others, it will be allocated to the service; otherwise, creation of the service will fail. This field can not be changed through updates. Valid values are \"None\", empty string (\"\"), or a valid IP address. \"None\" can be specified for headless services when proxying is not required. Only applies to types ClusterIP, NodePort, and LoadBalancer. Ignored if type is ExternalName. More info: https://kubernetes.io/docs/concepts/services-networking/service/#virtual-ips-and-service-proxies",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"type": {
							SchemaProps: spec.SchemaProps{
								Description: "type determines how the Service is exposed. Defaults to ClusterIP. Valid options are ExternalName, ClusterIP, NodePort, and LoadBalancer. \"ExternalName\" maps to the specified externalName. \"ClusterIP\" allocates a cluster-internal IP address for load-balancing to endpoints. Endpoints are determined by the selector or if that is not specified, by manual construction of an Endpoints object. If clusterIP is \"None\", no virtual IP is allocated and the endpoints are published as a set of endpoints rather than a stable IP. \"NodePort\" builds on ClusterIP and allocates a port on every node which routes to the clusterIP. \"LoadBalancer\" builds on NodePort and creates an external load-balancer (if supported in the current cloud) which routes to the clusterIP. More info: https://kubernetes.io/docs/concepts/services-networking/service/#publishing-services---service-types",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"externalIPs": {
							SchemaProps: spec.SchemaProps{
								Description: "externalIPs is a list of IP addresses for which nodes in the cluster will also accept traffic for this service.  These IPs are not managed by Kubernetes.  The user is responsible for ensuring that traffic arrives at a node with this IP.  A common example is external load-balancers that are not part of the Kubernetes system.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"sessionAffinity": {
							SchemaProps: spec.SchemaProps{
								Description: "Supports \"ClientIP\" and \"None\". Used to maintain session affinity. Enable client IP based session affinity. Must be ClientIP or None. Defaults to None. More info: https://kubernetes.io/docs/concepts/services-networking/service/#virtual-ips-and-service-proxies",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"loadBalancerIP": {
							SchemaProps: spec.SchemaProps{
								Description: "Only applies to Service Type: LoadBalancer LoadBalancer will get created with the IP specified in this field. This feature depends on whether the underlying cloud-provider supports specifying the loadBalancerIP when a load balancer is created. This field will be ignored if the cloud-provider does not support the feature.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"loadBalancerSourceRanges": {
							SchemaProps: spec.SchemaProps{
								Description: "If specified and supported by the platform, this will restrict traffic through the cloud-provider load-balancer will be restricted to the specified client IPs. This field will be ignored if the cloud-provider does not support the feature.\" More info: https://kubernetes.io/docs/tasks/access-application-cluster/configure-cloud-provider-firewall/",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"externalName": {
							SchemaProps: spec.SchemaProps{
								Description: "externalName is the external reference that kubedns or equivalent will return as a CNAME record for this service. No proxying will be involved. Must be a valid RFC-1123 hostname (https://tools.ietf.org/html/rfc1123) and requires Type to be ExternalName.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"externalTrafficPolicy": {
							SchemaProps: spec.SchemaProps{
								Description: "externalTrafficPolicy denotes if this Service desires to route external traffic to node-local or cluster-wide endpoints. \"Local\" preserves the client source IP and avoids a second hop for LoadBalancer and Nodeport type services, but risks potentially imbalanced traffic spreading. \"Cluster\" obscures the client source IP and may cause a second hop to another node, but should have good overall load-spreading.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"healthCheckNodePort": {
							SchemaProps: spec.SchemaProps{
								Description: "healthCheckNodePort specifies the healthcheck nodePort for the service. If not specified, HealthCheckNodePort is created by the service api backend with the allocated nodePort. Will use user-specified nodePort value if specified by the client. Only effects when Type is set to LoadBalancer and ExternalTrafficPolicy is set to Local.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"publishNotReadyAddresses": {
							SchemaProps: spec.SchemaProps{
								Description: "publishNotReadyAddresses, when set to true, indicates that DNS implementations must publish the notReadyAddresses of subsets for the Endpoints associated with the Service. The default value is false. The primary use case for setting this field is to use a StatefulSet's Headless Service to propagate SRV records for its Pods without respect to their readiness for purpose of peer discovery. This field will replace the service.alpha.kubernetes.io/tolerate-unready-endpoints when that annotation is deprecated and all clients have been converted to use this field.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"sessionAffinityConfig": {
							SchemaProps: spec.SchemaProps{
								Description: "sessionAffinityConfig contains the configurations of session affinity.",
								Ref:         ref("k8s.io/api/core/v1.SessionAffinityConfig"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.ServicePort", "k8s.io/api/core/v1.SessionAffinityConfig"},
		},
		"k8s.io/api/core/v1.ServiceStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ServiceStatus represents the current status of a service.",
					Properties: map[string]spec.Schema{
						"loadBalancer": {
							SchemaProps: spec.SchemaProps{
								Description: "LoadBalancer contains the current status of the load-balancer, if one is present.",
								Ref:         ref("k8s.io/api/core/v1.LoadBalancerStatus"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.LoadBalancerStatus"},
		},
		"k8s.io/api/core/v1.SessionAffinityConfig": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "SessionAffinityConfig represents the configurations of session affinity.",
					Properties: map[string]spec.Schema{
						"clientIP": {
							SchemaProps: spec.SchemaProps{
								Description: "clientIP contains the configurations of Client IP based session affinity.",
								Ref:         ref("k8s.io/api/core/v1.ClientIPConfig"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.ClientIPConfig"},
		},
		"k8s.io/api/core/v1.StorageOSPersistentVolumeSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Represents a StorageOS persistent volume resource.",
					Properties: map[string]spec.Schema{
						"volumeName": {
							SchemaProps: spec.SchemaProps{
								Description: "VolumeName is the human-readable name of the StorageOS volume.  Volume names are only unique within a namespace.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"volumeNamespace": {
							SchemaProps: spec.SchemaProps{
								Description: "VolumeNamespace specifies the scope of the volume within StorageOS.  If no namespace is specified then the Pod's namespace will be used.  This allows the Kubernetes name scoping to be mirrored within StorageOS for tighter integration. Set VolumeName to any name to override the default behaviour. Set to \"default\" if you are not using namespaces within StorageOS. Namespaces that do not pre-exist within StorageOS will be created.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"fsType": {
							SchemaProps: spec.SchemaProps{
								Description: "Filesystem type to mount. Must be a filesystem type supported by the host operating system. Ex. \"ext4\", \"xfs\", \"ntfs\". Implicitly inferred to be \"ext4\" if unspecified.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"readOnly": {
							SchemaProps: spec.SchemaProps{
								Description: "Defaults to false (read/write). ReadOnly here will force the ReadOnly setting in VolumeMounts.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"secretRef": {
							SchemaProps: spec.SchemaProps{
								Description: "SecretRef specifies the secret to use for obtaining the StorageOS API credentials.  If not specified, default values will be attempted.",
								Ref:         ref("k8s.io/api/core/v1.ObjectReference"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.ObjectReference"},
		},
		"k8s.io/api/core/v1.StorageOSVolumeSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Represents a StorageOS persistent volume resource.",
					Properties: map[string]spec.Schema{
						"volumeName": {
							SchemaProps: spec.SchemaProps{
								Description: "VolumeName is the human-readable name of the StorageOS volume.  Volume names are only unique within a namespace.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"volumeNamespace": {
							SchemaProps: spec.SchemaProps{
								Description: "VolumeNamespace specifies the scope of the volume within StorageOS.  If no namespace is specified then the Pod's namespace will be used.  This allows the Kubernetes name scoping to be mirrored within StorageOS for tighter integration. Set VolumeName to any name to override the default behaviour. Set to \"default\" if you are not using namespaces within StorageOS. Namespaces that do not pre-exist within StorageOS will be created.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"fsType": {
							SchemaProps: spec.SchemaProps{
								Description: "Filesystem type to mount. Must be a filesystem type supported by the host operating system. Ex. \"ext4\", \"xfs\", \"ntfs\". Implicitly inferred to be \"ext4\" if unspecified.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"readOnly": {
							SchemaProps: spec.SchemaProps{
								Description: "Defaults to false (read/write). ReadOnly here will force the ReadOnly setting in VolumeMounts.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"secretRef": {
							SchemaProps: spec.SchemaProps{
								Description: "SecretRef specifies the secret to use for obtaining the StorageOS API credentials.  If not specified, default values will be attempted.",
								Ref:         ref("k8s.io/api/core/v1.LocalObjectReference"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.LocalObjectReference"},
		},
		"k8s.io/api/core/v1.Sysctl": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Sysctl defines a kernel parameter to be set",
					Properties: map[string]spec.Schema{
						"Name": {
							SchemaProps: spec.SchemaProps{
								Description: "Name of a property to set",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"Value": {
							SchemaProps: spec.SchemaProps{
								Description: "Value of a property to set",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"Name", "Value"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.TCPSocketAction": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "TCPSocketAction describes an action based on opening a socket",
					Properties: map[string]spec.Schema{
						"port": {
							SchemaProps: spec.SchemaProps{
								Description: "Number or name of the port to access on the container. Number must be in the range 1 to 65535. Name must be an IANA_SVC_NAME.",
								Ref:         ref("k8s.io/apimachinery/pkg/util/intstr.IntOrString"),
							},
						},
						"host": {
							SchemaProps: spec.SchemaProps{
								Description: "Optional: Host name to connect to, defaults to the pod IP.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"port"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/util/intstr.IntOrString"},
		},
		"k8s.io/api/core/v1.Taint": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "The node this Taint is attached to has the \"effect\" on any pod that does not tolerate the Taint.",
					Properties: map[string]spec.Schema{
						"key": {
							SchemaProps: spec.SchemaProps{
								Description: "Required. The taint key to be applied to a node.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"value": {
							SchemaProps: spec.SchemaProps{
								Description: "Required. The taint value corresponding to the taint key.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"effect": {
							SchemaProps: spec.SchemaProps{
								Description: "Required. The effect of the taint on pods that do not tolerate the taint. Valid effects are NoSchedule, PreferNoSchedule and NoExecute.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"timeAdded": {
							SchemaProps: spec.SchemaProps{
								Description: "TimeAdded represents the time at which the taint was added. It is only written for NoExecute taints.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
					},
					Required: []string{"key", "effect"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.Time"},
		},
		"k8s.io/api/core/v1.Toleration": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "The pod this Toleration is attached to tolerates any taint that matches the triple <key,value,effect> using the matching operator <operator>.",
					Properties: map[string]spec.Schema{
						"key": {
							SchemaProps: spec.SchemaProps{
								Description: "Key is the taint key that the toleration applies to. Empty means match all taint keys. If the key is empty, operator must be Exists; this combination means to match all values and all keys.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"operator": {
							SchemaProps: spec.SchemaProps{
								Description: "Operator represents a key's relationship to the value. Valid operators are Exists and Equal. Defaults to Equal. Exists is equivalent to wildcard for value, so that a pod can tolerate all taints of a particular category.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"value": {
							SchemaProps: spec.SchemaProps{
								Description: "Value is the taint value the toleration matches to. If the operator is Exists, the value should be empty, otherwise just a regular string.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"effect": {
							SchemaProps: spec.SchemaProps{
								Description: "Effect indicates the taint effect to match. Empty means match all taint effects. When specified, allowed values are NoSchedule, PreferNoSchedule and NoExecute.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"tolerationSeconds": {
							SchemaProps: spec.SchemaProps{
								Description: "TolerationSeconds represents the period of time the toleration (which must be of effect NoExecute, otherwise this field is ignored) tolerates the taint. By default, it is not set, which means tolerate the taint forever (do not evict). Zero and negative values will be treated as 0 (evict immediately) by the system.",
								Type:        []string{"integer"},
								Format:      "int64",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.Volume": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Volume represents a named volume in a pod that may be accessed by any container in the pod.",
					Properties: map[string]spec.Schema{
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "Volume's name. Must be a DNS_LABEL and unique within the pod. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/names/#names",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"hostPath": {
							SchemaProps: spec.SchemaProps{
								Description: "HostPath represents a pre-existing file or directory on the host machine that is directly exposed to the container. This is generally used for system agents or other privileged things that are allowed to see the host machine. Most containers will NOT need this. More info: https://kubernetes.io/docs/concepts/storage/volumes#hostpath",
								Ref:         ref("k8s.io/api/core/v1.HostPathVolumeSource"),
							},
						},
						"emptyDir": {
							SchemaProps: spec.SchemaProps{
								Description: "EmptyDir represents a temporary directory that shares a pod's lifetime. More info: https://kubernetes.io/docs/concepts/storage/volumes#emptydir",
								Ref:         ref("k8s.io/api/core/v1.EmptyDirVolumeSource"),
							},
						},
						"gcePersistentDisk": {
							SchemaProps: spec.SchemaProps{
								Description: "GCEPersistentDisk represents a GCE Disk resource that is attached to a kubelet's host machine and then exposed to the pod. More info: https://kubernetes.io/docs/concepts/storage/volumes#gcepersistentdisk",
								Ref:         ref("k8s.io/api/core/v1.GCEPersistentDiskVolumeSource"),
							},
						},
						"awsElasticBlockStore": {
							SchemaProps: spec.SchemaProps{
								Description: "AWSElasticBlockStore represents an AWS Disk resource that is attached to a kubelet's host machine and then exposed to the pod. More info: https://kubernetes.io/docs/concepts/storage/volumes#awselasticblockstore",
								Ref:         ref("k8s.io/api/core/v1.AWSElasticBlockStoreVolumeSource"),
							},
						},
						"gitRepo": {
							SchemaProps: spec.SchemaProps{
								Description: "GitRepo represents a git repository at a particular revision.",
								Ref:         ref("k8s.io/api/core/v1.GitRepoVolumeSource"),
							},
						},
						"secret": {
							SchemaProps: spec.SchemaProps{
								Description: "Secret represents a secret that should populate this volume. More info: https://kubernetes.io/docs/concepts/storage/volumes#secret",
								Ref:         ref("k8s.io/api/core/v1.SecretVolumeSource"),
							},
						},
						"nfs": {
							SchemaProps: spec.SchemaProps{
								Description: "NFS represents an NFS mount on the host that shares a pod's lifetime More info: https://kubernetes.io/docs/concepts/storage/volumes#nfs",
								Ref:         ref("k8s.io/api/core/v1.NFSVolumeSource"),
							},
						},
						"iscsi": {
							SchemaProps: spec.SchemaProps{
								Description: "ISCSI represents an ISCSI Disk resource that is attached to a kubelet's host machine and then exposed to the pod. More info: https://releases.k8s.io/HEAD/examples/volumes/iscsi/README.md",
								Ref:         ref("k8s.io/api/core/v1.ISCSIVolumeSource"),
							},
						},
						"glusterfs": {
							SchemaProps: spec.SchemaProps{
								Description: "Glusterfs represents a Glusterfs mount on the host that shares a pod's lifetime. More info: https://releases.k8s.io/HEAD/examples/volumes/glusterfs/README.md",
								Ref:         ref("k8s.io/api/core/v1.GlusterfsVolumeSource"),
							},
						},
						"persistentVolumeClaim": {
							SchemaProps: spec.SchemaProps{
								Description: "PersistentVolumeClaimVolumeSource represents a reference to a PersistentVolumeClaim in the same namespace. More info: https://kubernetes.io/docs/concepts/storage/persistent-volumes#persistentvolumeclaims",
								Ref:         ref("k8s.io/api/core/v1.PersistentVolumeClaimVolumeSource"),
							},
						},
						"rbd": {
							SchemaProps: spec.SchemaProps{
								Description: "RBD represents a Rados Block Device mount on the host that shares a pod's lifetime. More info: https://releases.k8s.io/HEAD/examples/volumes/rbd/README.md",
								Ref:         ref("k8s.io/api/core/v1.RBDVolumeSource"),
							},
						},
						"flexVolume": {
							SchemaProps: spec.SchemaProps{
								Description: "FlexVolume represents a generic volume resource that is provisioned/attached using an exec based plugin.",
								Ref:         ref("k8s.io/api/core/v1.FlexVolumeSource"),
							},
						},
						"cinder": {
							SchemaProps: spec.SchemaProps{
								Description: "Cinder represents a cinder volume attached and mounted on kubelets host machine More info: https://releases.k8s.io/HEAD/examples/mysql-cinder-pd/README.md",
								Ref:         ref("k8s.io/api/core/v1.CinderVolumeSource"),
							},
						},
						"cephfs": {
							SchemaProps: spec.SchemaProps{
								Description: "CephFS represents a Ceph FS mount on the host that shares a pod's lifetime",
								Ref:         ref("k8s.io/api/core/v1.CephFSVolumeSource"),
							},
						},
						"flocker": {
							SchemaProps: spec.SchemaProps{
								Description: "Flocker represents a Flocker volume attached to a kubelet's host machine. This depends on the Flocker control service being running",
								Ref:         ref("k8s.io/api/core/v1.FlockerVolumeSource"),
							},
						},
						"downwardAPI": {
							SchemaProps: spec.SchemaProps{
								Description: "DownwardAPI represents downward API about the pod that should populate this volume",
								Ref:         ref("k8s.io/api/core/v1.DownwardAPIVolumeSource"),
							},
						},
						"fc": {
							SchemaProps: spec.SchemaProps{
								Description: "FC represents a Fibre Channel resource that is attached to a kubelet's host machine and then exposed to the pod.",
								Ref:         ref("k8s.io/api/core/v1.FCVolumeSource"),
							},
						},
						"azureFile": {
							SchemaProps: spec.SchemaProps{
								Description: "AzureFile represents an Azure File Service mount on the host and bind mount to the pod.",
								Ref:         ref("k8s.io/api/core/v1.AzureFileVolumeSource"),
							},
						},
						"configMap": {
							SchemaProps: spec.SchemaProps{
								Description: "ConfigMap represents a configMap that should populate this volume",
								Ref:         ref("k8s.io/api/core/v1.ConfigMapVolumeSource"),
							},
						},
						"vsphereVolume": {
							SchemaProps: spec.SchemaProps{
								Description: "VsphereVolume represents a vSphere volume attached and mounted on kubelets host machine",
								Ref:         ref("k8s.io/api/core/v1.VsphereVirtualDiskVolumeSource"),
							},
						},
						"quobyte": {
							SchemaProps: spec.SchemaProps{
								Description: "Quobyte represents a Quobyte mount on the host that shares a pod's lifetime",
								Ref:         ref("k8s.io/api/core/v1.QuobyteVolumeSource"),
							},
						},
						"azureDisk": {
							SchemaProps: spec.SchemaProps{
								Description: "AzureDisk represents an Azure Data Disk mount on the host and bind mount to the pod.",
								Ref:         ref("k8s.io/api/core/v1.AzureDiskVolumeSource"),
							},
						},
						"photonPersistentDisk": {
							SchemaProps: spec.SchemaProps{
								Description: "PhotonPersistentDisk represents a PhotonController persistent disk attached and mounted on kubelets host machine",
								Ref:         ref("k8s.io/api/core/v1.PhotonPersistentDiskVolumeSource"),
							},
						},
						"projected": {
							SchemaProps: spec.SchemaProps{
								Description: "Items for all in one resources secrets, configmaps, and downward API",
								Ref:         ref("k8s.io/api/core/v1.ProjectedVolumeSource"),
							},
						},
						"portworxVolume": {
							SchemaProps: spec.SchemaProps{
								Description: "PortworxVolume represents a portworx volume attached and mounted on kubelets host machine",
								Ref:         ref("k8s.io/api/core/v1.PortworxVolumeSource"),
							},
						},
						"scaleIO": {
							SchemaProps: spec.SchemaProps{
								Description: "ScaleIO represents a ScaleIO persistent volume attached and mounted on Kubernetes nodes.",
								Ref:         ref("k8s.io/api/core/v1.ScaleIOVolumeSource"),
							},
						},
						"storageos": {
							SchemaProps: spec.SchemaProps{
								Description: "StorageOS represents a StorageOS volume attached and mounted on Kubernetes nodes.",
								Ref:         ref("k8s.io/api/core/v1.StorageOSVolumeSource"),
							},
						},
					},
					Required: []string{"name"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.AWSElasticBlockStoreVolumeSource", "k8s.io/api/core/v1.AzureDiskVolumeSource", "k8s.io/api/core/v1.AzureFileVolumeSource", "k8s.io/api/core/v1.CephFSVolumeSource", "k8s.io/api/core/v1.CinderVolumeSource", "k8s.io/api/core/v1.ConfigMapVolumeSource", "k8s.io/api/core/v1.DownwardAPIVolumeSource", "k8s.io/api/core/v1.EmptyDirVolumeSource", "k8s.io/api/core/v1.FCVolumeSource", "k8s.io/api/core/v1.FlexVolumeSource", "k8s.io/api/core/v1.FlockerVolumeSource", "k8s.io/api/core/v1.GCEPersistentDiskVolumeSource", "k8s.io/api/core/v1.GitRepoVolumeSource", "k8s.io/api/core/v1.GlusterfsVolumeSource", "k8s.io/api/core/v1.HostPathVolumeSource", "k8s.io/api/core/v1.ISCSIVolumeSource", "k8s.io/api/core/v1.NFSVolumeSource", "k8s.io/api/core/v1.PersistentVolumeClaimVolumeSource", "k8s.io/api/core/v1.PhotonPersistentDiskVolumeSource", "k8s.io/api/core/v1.PortworxVolumeSource", "k8s.io/api/core/v1.ProjectedVolumeSource", "k8s.io/api/core/v1.QuobyteVolumeSource", "k8s.io/api/core/v1.RBDVolumeSource", "k8s.io/api/core/v1.ScaleIOVolumeSource", "k8s.io/api/core/v1.SecretVolumeSource", "k8s.io/api/core/v1.StorageOSVolumeSource", "k8s.io/api/core/v1.VsphereVirtualDiskVolumeSource"},
		},
		"k8s.io/api/core/v1.VolumeDevice": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "volumeDevice describes a mapping of a raw block device within a container.",
					Properties: map[string]spec.Schema{
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "name must match the name of a persistentVolumeClaim in the pod",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"devicePath": {
							SchemaProps: spec.SchemaProps{
								Description: "devicePath is the path inside of the container that the device will be mapped to.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"name", "devicePath"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.VolumeMount": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "VolumeMount describes a mounting of a Volume within a container.",
					Properties: map[string]spec.Schema{
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "This must match the Name of a Volume.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"readOnly": {
							SchemaProps: spec.SchemaProps{
								Description: "Mounted read-only if true, read-write otherwise (false or unspecified). Defaults to false.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"mountPath": {
							SchemaProps: spec.SchemaProps{
								Description: "Path within the container at which the volume should be mounted.  Must not contain ':'.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"subPath": {
							SchemaProps: spec.SchemaProps{
								Description: "Path within the volume from which the container's volume should be mounted. Defaults to \"\" (volume's root).",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"mountPropagation": {
							SchemaProps: spec.SchemaProps{
								Description: "mountPropagation determines how mounts are propagated from the host to container and the other way around. When not set, MountPropagationHostToContainer is used. This field is alpha in 1.8 and can be reworked or removed in a future release.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"name", "mountPath"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.VolumeProjection": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Projection that may be projected along with other supported volume types",
					Properties: map[string]spec.Schema{
						"secret": {
							SchemaProps: spec.SchemaProps{
								Description: "information about the secret data to project",
								Ref:         ref("k8s.io/api/core/v1.SecretProjection"),
							},
						},
						"downwardAPI": {
							SchemaProps: spec.SchemaProps{
								Description: "information about the downwardAPI data to project",
								Ref:         ref("k8s.io/api/core/v1.DownwardAPIProjection"),
							},
						},
						"configMap": {
							SchemaProps: spec.SchemaProps{
								Description: "information about the configMap data to project",
								Ref:         ref("k8s.io/api/core/v1.ConfigMapProjection"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.ConfigMapProjection", "k8s.io/api/core/v1.DownwardAPIProjection", "k8s.io/api/core/v1.SecretProjection"},
		},
		"k8s.io/api/core/v1.VolumeSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Represents the source of a volume to mount. Only one of its members may be specified.",
					Properties: map[string]spec.Schema{
						"hostPath": {
							SchemaProps: spec.SchemaProps{
								Description: "HostPath represents a pre-existing file or directory on the host machine that is directly exposed to the container. This is generally used for system agents or other privileged things that are allowed to see the host machine. Most containers will NOT need this. More info: https://kubernetes.io/docs/concepts/storage/volumes#hostpath",
								Ref:         ref("k8s.io/api/core/v1.HostPathVolumeSource"),
							},
						},
						"emptyDir": {
							SchemaProps: spec.SchemaProps{
								Description: "EmptyDir represents a temporary directory that shares a pod's lifetime. More info: https://kubernetes.io/docs/concepts/storage/volumes#emptydir",
								Ref:         ref("k8s.io/api/core/v1.EmptyDirVolumeSource"),
							},
						},
						"gcePersistentDisk": {
							SchemaProps: spec.SchemaProps{
								Description: "GCEPersistentDisk represents a GCE Disk resource that is attached to a kubelet's host machine and then exposed to the pod. More info: https://kubernetes.io/docs/concepts/storage/volumes#gcepersistentdisk",
								Ref:         ref("k8s.io/api/core/v1.GCEPersistentDiskVolumeSource"),
							},
						},
						"awsElasticBlockStore": {
							SchemaProps: spec.SchemaProps{
								Description: "AWSElasticBlockStore represents an AWS Disk resource that is attached to a kubelet's host machine and then exposed to the pod. More info: https://kubernetes.io/docs/concepts/storage/volumes#awselasticblockstore",
								Ref:         ref("k8s.io/api/core/v1.AWSElasticBlockStoreVolumeSource"),
							},
						},
						"gitRepo": {
							SchemaProps: spec.SchemaProps{
								Description: "GitRepo represents a git repository at a particular revision.",
								Ref:         ref("k8s.io/api/core/v1.GitRepoVolumeSource"),
							},
						},
						"secret": {
							SchemaProps: spec.SchemaProps{
								Description: "Secret represents a secret that should populate this volume. More info: https://kubernetes.io/docs/concepts/storage/volumes#secret",
								Ref:         ref("k8s.io/api/core/v1.SecretVolumeSource"),
							},
						},
						"nfs": {
							SchemaProps: spec.SchemaProps{
								Description: "NFS represents an NFS mount on the host that shares a pod's lifetime More info: https://kubernetes.io/docs/concepts/storage/volumes#nfs",
								Ref:         ref("k8s.io/api/core/v1.NFSVolumeSource"),
							},
						},
						"iscsi": {
							SchemaProps: spec.SchemaProps{
								Description: "ISCSI represents an ISCSI Disk resource that is attached to a kubelet's host machine and then exposed to the pod. More info: https://releases.k8s.io/HEAD/examples/volumes/iscsi/README.md",
								Ref:         ref("k8s.io/api/core/v1.ISCSIVolumeSource"),
							},
						},
						"glusterfs": {
							SchemaProps: spec.SchemaProps{
								Description: "Glusterfs represents a Glusterfs mount on the host that shares a pod's lifetime. More info: https://releases.k8s.io/HEAD/examples/volumes/glusterfs/README.md",
								Ref:         ref("k8s.io/api/core/v1.GlusterfsVolumeSource"),
							},
						},
						"persistentVolumeClaim": {
							SchemaProps: spec.SchemaProps{
								Description: "PersistentVolumeClaimVolumeSource represents a reference to a PersistentVolumeClaim in the same namespace. More info: https://kubernetes.io/docs/concepts/storage/persistent-volumes#persistentvolumeclaims",
								Ref:         ref("k8s.io/api/core/v1.PersistentVolumeClaimVolumeSource"),
							},
						},
						"rbd": {
							SchemaProps: spec.SchemaProps{
								Description: "RBD represents a Rados Block Device mount on the host that shares a pod's lifetime. More info: https://releases.k8s.io/HEAD/examples/volumes/rbd/README.md",
								Ref:         ref("k8s.io/api/core/v1.RBDVolumeSource"),
							},
						},
						"flexVolume": {
							SchemaProps: spec.SchemaProps{
								Description: "FlexVolume represents a generic volume resource that is provisioned/attached using an exec based plugin.",
								Ref:         ref("k8s.io/api/core/v1.FlexVolumeSource"),
							},
						},
						"cinder": {
							SchemaProps: spec.SchemaProps{
								Description: "Cinder represents a cinder volume attached and mounted on kubelets host machine More info: https://releases.k8s.io/HEAD/examples/mysql-cinder-pd/README.md",
								Ref:         ref("k8s.io/api/core/v1.CinderVolumeSource"),
							},
						},
						"cephfs": {
							SchemaProps: spec.SchemaProps{
								Description: "CephFS represents a Ceph FS mount on the host that shares a pod's lifetime",
								Ref:         ref("k8s.io/api/core/v1.CephFSVolumeSource"),
							},
						},
						"flocker": {
							SchemaProps: spec.SchemaProps{
								Description: "Flocker represents a Flocker volume attached to a kubelet's host machine. This depends on the Flocker control service being running",
								Ref:         ref("k8s.io/api/core/v1.FlockerVolumeSource"),
							},
						},
						"downwardAPI": {
							SchemaProps: spec.SchemaProps{
								Description: "DownwardAPI represents downward API about the pod that should populate this volume",
								Ref:         ref("k8s.io/api/core/v1.DownwardAPIVolumeSource"),
							},
						},
						"fc": {
							SchemaProps: spec.SchemaProps{
								Description: "FC represents a Fibre Channel resource that is attached to a kubelet's host machine and then exposed to the pod.",
								Ref:         ref("k8s.io/api/core/v1.FCVolumeSource"),
							},
						},
						"azureFile": {
							SchemaProps: spec.SchemaProps{
								Description: "AzureFile represents an Azure File Service mount on the host and bind mount to the pod.",
								Ref:         ref("k8s.io/api/core/v1.AzureFileVolumeSource"),
							},
						},
						"configMap": {
							SchemaProps: spec.SchemaProps{
								Description: "ConfigMap represents a configMap that should populate this volume",
								Ref:         ref("k8s.io/api/core/v1.ConfigMapVolumeSource"),
							},
						},
						"vsphereVolume": {
							SchemaProps: spec.SchemaProps{
								Description: "VsphereVolume represents a vSphere volume attached and mounted on kubelets host machine",
								Ref:         ref("k8s.io/api/core/v1.VsphereVirtualDiskVolumeSource"),
							},
						},
						"quobyte": {
							SchemaProps: spec.SchemaProps{
								Description: "Quobyte represents a Quobyte mount on the host that shares a pod's lifetime",
								Ref:         ref("k8s.io/api/core/v1.QuobyteVolumeSource"),
							},
						},
						"azureDisk": {
							SchemaProps: spec.SchemaProps{
								Description: "AzureDisk represents an Azure Data Disk mount on the host and bind mount to the pod.",
								Ref:         ref("k8s.io/api/core/v1.AzureDiskVolumeSource"),
							},
						},
						"photonPersistentDisk": {
							SchemaProps: spec.SchemaProps{
								Description: "PhotonPersistentDisk represents a PhotonController persistent disk attached and mounted on kubelets host machine",
								Ref:         ref("k8s.io/api/core/v1.PhotonPersistentDiskVolumeSource"),
							},
						},
						"projected": {
							SchemaProps: spec.SchemaProps{
								Description: "Items for all in one resources secrets, configmaps, and downward API",
								Ref:         ref("k8s.io/api/core/v1.ProjectedVolumeSource"),
							},
						},
						"portworxVolume": {
							SchemaProps: spec.SchemaProps{
								Description: "PortworxVolume represents a portworx volume attached and mounted on kubelets host machine",
								Ref:         ref("k8s.io/api/core/v1.PortworxVolumeSource"),
							},
						},
						"scaleIO": {
							SchemaProps: spec.SchemaProps{
								Description: "ScaleIO represents a ScaleIO persistent volume attached and mounted on Kubernetes nodes.",
								Ref:         ref("k8s.io/api/core/v1.ScaleIOVolumeSource"),
							},
						},
						"storageos": {
							SchemaProps: spec.SchemaProps{
								Description: "StorageOS represents a StorageOS volume attached and mounted on Kubernetes nodes.",
								Ref:         ref("k8s.io/api/core/v1.StorageOSVolumeSource"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.AWSElasticBlockStoreVolumeSource", "k8s.io/api/core/v1.AzureDiskVolumeSource", "k8s.io/api/core/v1.AzureFileVolumeSource", "k8s.io/api/core/v1.CephFSVolumeSource", "k8s.io/api/core/v1.CinderVolumeSource", "k8s.io/api/core/v1.ConfigMapVolumeSource", "k8s.io/api/core/v1.DownwardAPIVolumeSource", "k8s.io/api/core/v1.EmptyDirVolumeSource", "k8s.io/api/core/v1.FCVolumeSource", "k8s.io/api/core/v1.FlexVolumeSource", "k8s.io/api/core/v1.FlockerVolumeSource", "k8s.io/api/core/v1.GCEPersistentDiskVolumeSource", "k8s.io/api/core/v1.GitRepoVolumeSource", "k8s.io/api/core/v1.GlusterfsVolumeSource", "k8s.io/api/core/v1.HostPathVolumeSource", "k8s.io/api/core/v1.ISCSIVolumeSource", "k8s.io/api/core/v1.NFSVolumeSource", "k8s.io/api/core/v1.PersistentVolumeClaimVolumeSource", "k8s.io/api/core/v1.PhotonPersistentDiskVolumeSource", "k8s.io/api/core/v1.PortworxVolumeSource", "k8s.io/api/core/v1.ProjectedVolumeSource", "k8s.io/api/core/v1.QuobyteVolumeSource", "k8s.io/api/core/v1.RBDVolumeSource", "k8s.io/api/core/v1.ScaleIOVolumeSource", "k8s.io/api/core/v1.SecretVolumeSource", "k8s.io/api/core/v1.StorageOSVolumeSource", "k8s.io/api/core/v1.VsphereVirtualDiskVolumeSource"},
		},
		"k8s.io/api/core/v1.VsphereVirtualDiskVolumeSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Represents a vSphere volume resource.",
					Properties: map[string]spec.Schema{
						"volumePath": {
							SchemaProps: spec.SchemaProps{
								Description: "Path that identifies vSphere volume vmdk",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"fsType": {
							SchemaProps: spec.SchemaProps{
								Description: "Filesystem type to mount. Must be a filesystem type supported by the host operating system. Ex. \"ext4\", \"xfs\", \"ntfs\". Implicitly inferred to be \"ext4\" if unspecified.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"storagePolicyName": {
							SchemaProps: spec.SchemaProps{
								Description: "Storage Policy Based Management (SPBM) profile name.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"storagePolicyID": {
							SchemaProps: spec.SchemaProps{
								Description: "Storage Policy Based Management (SPBM) profile ID associated with the StoragePolicyName.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"volumePath"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/core/v1.WeightedPodAffinityTerm": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "The weights of all of the matched WeightedPodAffinityTerm fields are added per-node to find the most preferred node(s)",
					Properties: map[string]spec.Schema{
						"weight": {
							SchemaProps: spec.SchemaProps{
								Description: "weight associated with matching the corresponding podAffinityTerm, in the range 1-100.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"podAffinityTerm": {
							SchemaProps: spec.SchemaProps{
								Description: "Required. A pod affinity term, associated with the corresponding weight.",
								Ref:         ref("k8s.io/api/core/v1.PodAffinityTerm"),
							},
						},
					},
					Required: []string{"weight", "podAffinityTerm"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.PodAffinityTerm"},
		},
		"k8s.io/api/events/v1beta1.Event": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Event is a report of an event somewhere in the cluster. It generally denotes some state change in the system.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"eventTime": {
							SchemaProps: spec.SchemaProps{
								Description: "Required. Time when this Event was first observed.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.MicroTime"),
							},
						},
						"series": {
							SchemaProps: spec.SchemaProps{
								Description: "Data about the Event series this event represents or nil if it's a singleton Event.",
								Ref:         ref("k8s.io/api/events/v1beta1.EventSeries"),
							},
						},
						"reportingController": {
							SchemaProps: spec.SchemaProps{
								Description: "Name of the controller that emitted this Event, e.g. `kubernetes.io/kubelet`.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"reportingInstance": {
							SchemaProps: spec.SchemaProps{
								Description: "ID of the controller instance, e.g. `kubelet-xyzf`.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"action": {
							SchemaProps: spec.SchemaProps{
								Description: "What action was taken/failed regarding to the regarding object.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"reason": {
							SchemaProps: spec.SchemaProps{
								Description: "Why the action was taken.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"regarding": {
							SchemaProps: spec.SchemaProps{
								Description: "The object this Event is about. In most cases it's an Object reporting controller implements. E.g. ReplicaSetController implements ReplicaSets and this event is emitted because it acts on some changes in a ReplicaSet object.",
								Ref:         ref("k8s.io/api/core/v1.ObjectReference"),
							},
						},
						"related": {
							SchemaProps: spec.SchemaProps{
								Description: "Optional secondary object for more complex actions. E.g. when regarding object triggers a creation or deletion of related object.",
								Ref:         ref("k8s.io/api/core/v1.ObjectReference"),
							},
						},
						"note": {
							SchemaProps: spec.SchemaProps{
								Description: "Optional. A human-readable description of the status of this operation. Maximal length of the note is 1kB, but libraries should be prepared to handle values up to 64kB.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"type": {
							SchemaProps: spec.SchemaProps{
								Description: "Type of this event (Normal, Warning), new types could be added in the future.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"deprecatedSource": {
							SchemaProps: spec.SchemaProps{
								Description: "Deprecated field assuring backward compatibility with core.v1 Event type",
								Ref:         ref("k8s.io/api/core/v1.EventSource"),
							},
						},
						"deprecatedFirstTimestamp": {
							SchemaProps: spec.SchemaProps{
								Description: "Deprecated field assuring backward compatibility with core.v1 Event type",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"deprecatedLastTimestamp": {
							SchemaProps: spec.SchemaProps{
								Description: "Deprecated field assuring backward compatibility with core.v1 Event type",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"deprecatedCount": {
							SchemaProps: spec.SchemaProps{
								Description: "Deprecated field assuring backward compatibility with core.v1 Event type",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
					},
					Required: []string{"eventTime"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.EventSource", "k8s.io/api/core/v1.ObjectReference", "k8s.io/api/events/v1beta1.EventSeries", "k8s.io/apimachinery/pkg/apis/meta/v1.MicroTime", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta", "k8s.io/apimachinery/pkg/apis/meta/v1.Time"},
		},
		"k8s.io/api/events/v1beta1.EventList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "EventList is a list of Event objects.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "Items is a list of schema objects.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/events/v1beta1.Event"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/events/v1beta1.Event", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/events/v1beta1.EventSeries": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "EventSeries contain information on series of events, i.e. thing that was/is happening continously for some time.",
					Properties: map[string]spec.Schema{
						"count": {
							SchemaProps: spec.SchemaProps{
								Description: "Number of occurrences in this series up to the last heartbeat time",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"lastObservedTime": {
							SchemaProps: spec.SchemaProps{
								Description: "Time when last Event from the series was seen before last heartbeat.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.MicroTime"),
							},
						},
						"state": {
							SchemaProps: spec.SchemaProps{
								Description: "Information whether this series is ongoing or finished.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"count", "lastObservedTime", "state"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.MicroTime"},
		},
		"k8s.io/api/extensions/v1beta1.AllowedFlexVolume": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "AllowedFlexVolume represents a single Flexvolume that is allowed to be used.",
					Properties: map[string]spec.Schema{
						"driver": {
							SchemaProps: spec.SchemaProps{
								Description: "Driver is the name of the Flexvolume driver.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"driver"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/extensions/v1beta1.AllowedHostPath": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "defines the host volume conditions that will be enabled by a policy for pods to use. It requires the path prefix to be defined.",
					Properties: map[string]spec.Schema{
						"pathPrefix": {
							SchemaProps: spec.SchemaProps{
								Description: "is the path prefix that the host volume must match. It does not support `*`. Trailing slashes are trimmed when validating the path prefix with a host path.\n\nExamples: `/foo` would allow `/foo`, `/foo/` and `/foo/bar` `/foo` would not allow `/food` or `/etc/foo`",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/extensions/v1beta1.CustomMetricCurrentStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Properties: map[string]spec.Schema{
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "Custom Metric name.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"value": {
							SchemaProps: spec.SchemaProps{
								Description: "Custom Metric value (average).",
								Ref:         ref("k8s.io/apimachinery/pkg/api/resource.Quantity"),
							},
						},
					},
					Required: []string{"name", "value"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/api/resource.Quantity"},
		},
		"k8s.io/api/extensions/v1beta1.CustomMetricCurrentStatusList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Properties: map[string]spec.Schema{
						"items": {
							SchemaProps: spec.SchemaProps{
								Type: []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/extensions/v1beta1.CustomMetricCurrentStatus"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/extensions/v1beta1.CustomMetricCurrentStatus"},
		},
		"k8s.io/api/extensions/v1beta1.CustomMetricTarget": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Alpha-level support for Custom Metrics in HPA (as annotations).",
					Properties: map[string]spec.Schema{
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "Custom Metric name.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"value": {
							SchemaProps: spec.SchemaProps{
								Description: "Custom Metric value (average).",
								Ref:         ref("k8s.io/apimachinery/pkg/api/resource.Quantity"),
							},
						},
					},
					Required: []string{"name", "value"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/api/resource.Quantity"},
		},
		"k8s.io/api/extensions/v1beta1.CustomMetricTargetList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Properties: map[string]spec.Schema{
						"items": {
							SchemaProps: spec.SchemaProps{
								Type: []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/extensions/v1beta1.CustomMetricTarget"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/extensions/v1beta1.CustomMetricTarget"},
		},
		"k8s.io/api/extensions/v1beta1.DaemonSet": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DEPRECATED - This group version of DaemonSet is deprecated by apps/v1beta2/DaemonSet. See the release notes for more information. DaemonSet represents the configuration of a daemon set.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "The desired behavior of this daemon set. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status",
								Ref:         ref("k8s.io/api/extensions/v1beta1.DaemonSetSpec"),
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "The current status of this daemon set. This data may be out of date by some window of time. Populated by the system. Read-only. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status",
								Ref:         ref("k8s.io/api/extensions/v1beta1.DaemonSetStatus"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/extensions/v1beta1.DaemonSetSpec", "k8s.io/api/extensions/v1beta1.DaemonSetStatus", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/extensions/v1beta1.DaemonSetCondition": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DaemonSetCondition describes the state of a DaemonSet at a certain point.",
					Properties: map[string]spec.Schema{
						"type": {
							SchemaProps: spec.SchemaProps{
								Description: "Type of DaemonSet condition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Status of the condition, one of True, False, Unknown.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"lastTransitionTime": {
							SchemaProps: spec.SchemaProps{
								Description: "Last time the condition transitioned from one status to another.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"reason": {
							SchemaProps: spec.SchemaProps{
								Description: "The reason for the condition's last transition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"message": {
							SchemaProps: spec.SchemaProps{
								Description: "A human readable message indicating details about the transition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"type", "status"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.Time"},
		},
		"k8s.io/api/extensions/v1beta1.DaemonSetList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DaemonSetList is a collection of daemon sets.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "A list of daemon sets.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/extensions/v1beta1.DaemonSet"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/extensions/v1beta1.DaemonSet", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/extensions/v1beta1.DaemonSetSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DaemonSetSpec is the specification of a daemon set.",
					Properties: map[string]spec.Schema{
						"selector": {
							SchemaProps: spec.SchemaProps{
								Description: "A label query over pods that are managed by the daemon set. Must match in order to be controlled. If empty, defaulted to labels on Pod template. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/labels/#label-selectors",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"),
							},
						},
						"template": {
							SchemaProps: spec.SchemaProps{
								Description: "An object that describes the pod that will be created. The DaemonSet will create exactly one copy of this pod on every node that matches the template's node selector (or on every node if no node selector is specified). More info: https://kubernetes.io/docs/concepts/workloads/controllers/replicationcontroller#pod-template",
								Ref:         ref("k8s.io/api/core/v1.PodTemplateSpec"),
							},
						},
						"updateStrategy": {
							SchemaProps: spec.SchemaProps{
								Description: "An update strategy to replace existing DaemonSet pods with new pods.",
								Ref:         ref("k8s.io/api/extensions/v1beta1.DaemonSetUpdateStrategy"),
							},
						},
						"minReadySeconds": {
							SchemaProps: spec.SchemaProps{
								Description: "The minimum number of seconds for which a newly created DaemonSet pod should be ready without any of its container crashing, for it to be considered available. Defaults to 0 (pod will be considered available as soon as it is ready).",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"templateGeneration": {
							SchemaProps: spec.SchemaProps{
								Description: "DEPRECATED. A sequence number representing a specific generation of the template. Populated by the system. It can be set only during the creation.",
								Type:        []string{"integer"},
								Format:      "int64",
							},
						},
						"revisionHistoryLimit": {
							SchemaProps: spec.SchemaProps{
								Description: "The number of old history to retain to allow rollback. This is a pointer to distinguish between explicit zero and not specified. Defaults to 10.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
					},
					Required: []string{"template"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.PodTemplateSpec", "k8s.io/api/extensions/v1beta1.DaemonSetUpdateStrategy", "k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"},
		},
		"k8s.io/api/extensions/v1beta1.DaemonSetStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DaemonSetStatus represents the current status of a daemon set.",
					Properties: map[string]spec.Schema{
						"currentNumberScheduled": {
							SchemaProps: spec.SchemaProps{
								Description: "The number of nodes that are running at least 1 daemon pod and are supposed to run the daemon pod. More info: https://kubernetes.io/docs/concepts/workloads/controllers/daemonset/",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"numberMisscheduled": {
							SchemaProps: spec.SchemaProps{
								Description: "The number of nodes that are running the daemon pod, but are not supposed to run the daemon pod. More info: https://kubernetes.io/docs/concepts/workloads/controllers/daemonset/",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"desiredNumberScheduled": {
							SchemaProps: spec.SchemaProps{
								Description: "The total number of nodes that should be running the daemon pod (including nodes correctly running the daemon pod). More info: https://kubernetes.io/docs/concepts/workloads/controllers/daemonset/",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"numberReady": {
							SchemaProps: spec.SchemaProps{
								Description: "The number of nodes that should be running the daemon pod and have one or more of the daemon pod running and ready.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"observedGeneration": {
							SchemaProps: spec.SchemaProps{
								Description: "The most recent generation observed by the daemon set controller.",
								Type:        []string{"integer"},
								Format:      "int64",
							},
						},
						"updatedNumberScheduled": {
							SchemaProps: spec.SchemaProps{
								Description: "The total number of nodes that are running updated daemon pod",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"numberAvailable": {
							SchemaProps: spec.SchemaProps{
								Description: "The number of nodes that should be running the daemon pod and have one or more of the daemon pod running and available (ready for at least spec.minReadySeconds)",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"numberUnavailable": {
							SchemaProps: spec.SchemaProps{
								Description: "The number of nodes that should be running the daemon pod and have none of the daemon pod running and available (ready for at least spec.minReadySeconds)",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"collisionCount": {
							SchemaProps: spec.SchemaProps{
								Description: "Count of hash collisions for the DaemonSet. The DaemonSet controller uses this field as a collision avoidance mechanism when it needs to create the name for the newest ControllerRevision.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"conditions": {
							VendorExtensible: spec.VendorExtensible{
								Extensions: spec.Extensions{
									"x-kubernetes-patch-merge-key": "type",
									"x-kubernetes-patch-strategy":  "merge",
								},
							},
							SchemaProps: spec.SchemaProps{
								Description: "Represents the latest available observations of a DaemonSet's current state.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/extensions/v1beta1.DaemonSetCondition"),
										},
									},
								},
							},
						},
					},
					Required: []string{"currentNumberScheduled", "numberMisscheduled", "desiredNumberScheduled", "numberReady"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/extensions/v1beta1.DaemonSetCondition"},
		},
		"k8s.io/api/extensions/v1beta1.DaemonSetUpdateStrategy": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Properties: map[string]spec.Schema{
						"type": {
							SchemaProps: spec.SchemaProps{
								Description: "Type of daemon set update. Can be \"RollingUpdate\" or \"OnDelete\". Default is OnDelete.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"rollingUpdate": {
							SchemaProps: spec.SchemaProps{
								Description: "Rolling update config params. Present only if type = \"RollingUpdate\".",
								Ref:         ref("k8s.io/api/extensions/v1beta1.RollingUpdateDaemonSet"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/extensions/v1beta1.RollingUpdateDaemonSet"},
		},
		"k8s.io/api/extensions/v1beta1.Deployment": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DEPRECATED - This group version of Deployment is deprecated by apps/v1beta2/Deployment. See the release notes for more information. Deployment enables declarative updates for Pods and ReplicaSets.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object metadata.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "Specification of the desired behavior of the Deployment.",
								Ref:         ref("k8s.io/api/extensions/v1beta1.DeploymentSpec"),
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Most recently observed status of the Deployment.",
								Ref:         ref("k8s.io/api/extensions/v1beta1.DeploymentStatus"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/extensions/v1beta1.DeploymentSpec", "k8s.io/api/extensions/v1beta1.DeploymentStatus", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/extensions/v1beta1.DeploymentCondition": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DeploymentCondition describes the state of a deployment at a certain point.",
					Properties: map[string]spec.Schema{
						"type": {
							SchemaProps: spec.SchemaProps{
								Description: "Type of deployment condition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Status of the condition, one of True, False, Unknown.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"lastUpdateTime": {
							SchemaProps: spec.SchemaProps{
								Description: "The last time this condition was updated.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"lastTransitionTime": {
							SchemaProps: spec.SchemaProps{
								Description: "Last time the condition transitioned from one status to another.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"reason": {
							SchemaProps: spec.SchemaProps{
								Description: "The reason for the condition's last transition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"message": {
							SchemaProps: spec.SchemaProps{
								Description: "A human readable message indicating details about the transition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"type", "status"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.Time"},
		},
		"k8s.io/api/extensions/v1beta1.DeploymentList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DeploymentList is a list of Deployments.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard list metadata.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "Items is the list of Deployments.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/extensions/v1beta1.Deployment"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/extensions/v1beta1.Deployment", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/extensions/v1beta1.DeploymentRollback": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DEPRECATED. DeploymentRollback stores the information required to rollback a deployment.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "Required: This must match the Name of a deployment.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"updatedAnnotations": {
							SchemaProps: spec.SchemaProps{
								Description: "The annotations to be updated to a deployment",
								Type:        []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"rollbackTo": {
							SchemaProps: spec.SchemaProps{
								Description: "The config of this deployment rollback.",
								Ref:         ref("k8s.io/api/extensions/v1beta1.RollbackConfig"),
							},
						},
					},
					Required: []string{"name", "rollbackTo"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/extensions/v1beta1.RollbackConfig"},
		},
		"k8s.io/api/extensions/v1beta1.DeploymentSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DeploymentSpec is the specification of the desired behavior of the Deployment.",
					Properties: map[string]spec.Schema{
						"replicas": {
							SchemaProps: spec.SchemaProps{
								Description: "Number of desired pods. This is a pointer to distinguish between explicit zero and not specified. Defaults to 1.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"selector": {
							SchemaProps: spec.SchemaProps{
								Description: "Label selector for pods. Existing ReplicaSets whose pods are selected by this will be the ones affected by this deployment.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"),
							},
						},
						"template": {
							SchemaProps: spec.SchemaProps{
								Description: "Template describes the pods that will be created.",
								Ref:         ref("k8s.io/api/core/v1.PodTemplateSpec"),
							},
						},
						"strategy": {
							VendorExtensible: spec.VendorExtensible{
								Extensions: spec.Extensions{
									"x-kubernetes-patch-strategy": "retainKeys",
								},
							},
							SchemaProps: spec.SchemaProps{
								Description: "The deployment strategy to use to replace existing pods with new ones.",
								Ref:         ref("k8s.io/api/extensions/v1beta1.DeploymentStrategy"),
							},
						},
						"minReadySeconds": {
							SchemaProps: spec.SchemaProps{
								Description: "Minimum number of seconds for which a newly created pod should be ready without any of its container crashing, for it to be considered available. Defaults to 0 (pod will be considered available as soon as it is ready)",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"revisionHistoryLimit": {
							SchemaProps: spec.SchemaProps{
								Description: "The number of old ReplicaSets to retain to allow rollback. This is a pointer to distinguish between explicit zero and not specified.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"paused": {
							SchemaProps: spec.SchemaProps{
								Description: "Indicates that the deployment is paused and will not be processed by the deployment controller.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"rollbackTo": {
							SchemaProps: spec.SchemaProps{
								Description: "DEPRECATED. The config this deployment is rolling back to. Will be cleared after rollback is done.",
								Ref:         ref("k8s.io/api/extensions/v1beta1.RollbackConfig"),
							},
						},
						"progressDeadlineSeconds": {
							SchemaProps: spec.SchemaProps{
								Description: "The maximum time in seconds for a deployment to make progress before it is considered to be failed. The deployment controller will continue to process failed deployments and a condition with a ProgressDeadlineExceeded reason will be surfaced in the deployment status. Note that progress will not be estimated during the time a deployment is paused. This is not set by default.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
					},
					Required: []string{"template"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.PodTemplateSpec", "k8s.io/api/extensions/v1beta1.DeploymentStrategy", "k8s.io/api/extensions/v1beta1.RollbackConfig", "k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"},
		},
		"k8s.io/api/extensions/v1beta1.DeploymentStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DeploymentStatus is the most recently observed status of the Deployment.",
					Properties: map[string]spec.Schema{
						"observedGeneration": {
							SchemaProps: spec.SchemaProps{
								Description: "The generation observed by the deployment controller.",
								Type:        []string{"integer"},
								Format:      "int64",
							},
						},
						"replicas": {
							SchemaProps: spec.SchemaProps{
								Description: "Total number of non-terminated pods targeted by this deployment (their labels match the selector).",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"updatedReplicas": {
							SchemaProps: spec.SchemaProps{
								Description: "Total number of non-terminated pods targeted by this deployment that have the desired template spec.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"readyReplicas": {
							SchemaProps: spec.SchemaProps{
								Description: "Total number of ready pods targeted by this deployment.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"availableReplicas": {
							SchemaProps: spec.SchemaProps{
								Description: "Total number of available pods (ready for at least minReadySeconds) targeted by this deployment.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"unavailableReplicas": {
							SchemaProps: spec.SchemaProps{
								Description: "Total number of unavailable pods targeted by this deployment. This is the total number of pods that are still required for the deployment to have 100% available capacity. They may either be pods that are running but not yet available or pods that still have not been created.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"conditions": {
							VendorExtensible: spec.VendorExtensible{
								Extensions: spec.Extensions{
									"x-kubernetes-patch-merge-key": "type",
									"x-kubernetes-patch-strategy":  "merge",
								},
							},
							SchemaProps: spec.SchemaProps{
								Description: "Represents the latest available observations of a deployment's current state.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/extensions/v1beta1.DeploymentCondition"),
										},
									},
								},
							},
						},
						"collisionCount": {
							SchemaProps: spec.SchemaProps{
								Description: "Count of hash collisions for the Deployment. The Deployment controller uses this field as a collision avoidance mechanism when it needs to create the name for the newest ReplicaSet.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/extensions/v1beta1.DeploymentCondition"},
		},
		"k8s.io/api/extensions/v1beta1.DeploymentStrategy": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DeploymentStrategy describes how to replace existing pods with new ones.",
					Properties: map[string]spec.Schema{
						"type": {
							SchemaProps: spec.SchemaProps{
								Description: "Type of deployment. Can be \"Recreate\" or \"RollingUpdate\". Default is RollingUpdate.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"rollingUpdate": {
							SchemaProps: spec.SchemaProps{
								Description: "Rolling update config params. Present only if DeploymentStrategyType = RollingUpdate.",
								Ref:         ref("k8s.io/api/extensions/v1beta1.RollingUpdateDeployment"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/extensions/v1beta1.RollingUpdateDeployment"},
		},
		"k8s.io/api/extensions/v1beta1.FSGroupStrategyOptions": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "FSGroupStrategyOptions defines the strategy type and options used to create the strategy.",
					Properties: map[string]spec.Schema{
						"rule": {
							SchemaProps: spec.SchemaProps{
								Description: "Rule is the strategy that will dictate what FSGroup is used in the SecurityContext.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"ranges": {
							SchemaProps: spec.SchemaProps{
								Description: "Ranges are the allowed ranges of fs groups.  If you would like to force a single fs group then supply a single range with the same start and end.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/extensions/v1beta1.IDRange"),
										},
									},
								},
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/extensions/v1beta1.IDRange"},
		},
		"k8s.io/api/extensions/v1beta1.HTTPIngressPath": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "HTTPIngressPath associates a path regex with a backend. Incoming urls matching the path are forwarded to the backend.",
					Properties: map[string]spec.Schema{
						"path": {
							SchemaProps: spec.SchemaProps{
								Description: "Path is an extended POSIX regex as defined by IEEE Std 1003.1, (i.e this follows the egrep/unix syntax, not the perl syntax) matched against the path of an incoming request. Currently it can contain characters disallowed from the conventional \"path\" part of a URL as defined by RFC 3986. Paths must begin with a '/'. If unspecified, the path defaults to a catch all sending traffic to the backend.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"backend": {
							SchemaProps: spec.SchemaProps{
								Description: "Backend defines the referenced service endpoint to which the traffic will be forwarded to.",
								Ref:         ref("k8s.io/api/extensions/v1beta1.IngressBackend"),
							},
						},
					},
					Required: []string{"backend"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/extensions/v1beta1.IngressBackend"},
		},
		"k8s.io/api/extensions/v1beta1.HTTPIngressRuleValue": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "HTTPIngressRuleValue is a list of http selectors pointing to backends. In the example: http://<host>/<path>?<searchpart> -> backend where where parts of the url correspond to RFC 3986, this resource will be used to match against everything after the last '/' and before the first '?' or '#'.",
					Properties: map[string]spec.Schema{
						"paths": {
							SchemaProps: spec.SchemaProps{
								Description: "A collection of paths that map requests to backends.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/extensions/v1beta1.HTTPIngressPath"),
										},
									},
								},
							},
						},
					},
					Required: []string{"paths"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/extensions/v1beta1.HTTPIngressPath"},
		},
		"k8s.io/api/extensions/v1beta1.HostPortRange": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Host Port Range defines a range of host ports that will be enabled by a policy for pods to use.  It requires both the start and end to be defined.",
					Properties: map[string]spec.Schema{
						"min": {
							SchemaProps: spec.SchemaProps{
								Description: "min is the start of the range, inclusive.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"max": {
							SchemaProps: spec.SchemaProps{
								Description: "max is the end of the range, inclusive.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
					},
					Required: []string{"min", "max"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/extensions/v1beta1.IDRange": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ID Range provides a min/max of an allowed range of IDs.",
					Properties: map[string]spec.Schema{
						"min": {
							SchemaProps: spec.SchemaProps{
								Description: "Min is the start of the range, inclusive.",
								Type:        []string{"integer"},
								Format:      "int64",
							},
						},
						"max": {
							SchemaProps: spec.SchemaProps{
								Description: "Max is the end of the range, inclusive.",
								Type:        []string{"integer"},
								Format:      "int64",
							},
						},
					},
					Required: []string{"min", "max"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/extensions/v1beta1.IPBlock": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DEPRECATED 1.9 - This group version of IPBlock is deprecated by networking/v1/IPBlock. IPBlock describes a particular CIDR (Ex. \"192.168.1.1/24\") that is allowed to the pods matched by a NetworkPolicySpec's podSelector. The except entry describes CIDRs that should not be included within this rule.",
					Properties: map[string]spec.Schema{
						"cidr": {
							SchemaProps: spec.SchemaProps{
								Description: "CIDR is a string representing the IP Block Valid examples are \"192.168.1.1/24\"",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"except": {
							SchemaProps: spec.SchemaProps{
								Description: "Except is a slice of CIDRs that should not be included within an IP Block Valid examples are \"192.168.1.1/24\" Except values will be rejected if they are outside the CIDR range",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
					},
					Required: []string{"cidr"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/extensions/v1beta1.Ingress": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Ingress is a collection of rules that allow inbound connections to reach the endpoints defined by a backend. An Ingress can be configured to give services externally-reachable urls, load balance traffic, terminate SSL, offer name based virtual hosting etc.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "Spec is the desired state of the Ingress. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status",
								Ref:         ref("k8s.io/api/extensions/v1beta1.IngressSpec"),
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Status is the current state of the Ingress. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status",
								Ref:         ref("k8s.io/api/extensions/v1beta1.IngressStatus"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/extensions/v1beta1.IngressSpec", "k8s.io/api/extensions/v1beta1.IngressStatus", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/extensions/v1beta1.IngressBackend": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "IngressBackend describes all endpoints for a given service and port.",
					Properties: map[string]spec.Schema{
						"serviceName": {
							SchemaProps: spec.SchemaProps{
								Description: "Specifies the name of the referenced service.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"servicePort": {
							SchemaProps: spec.SchemaProps{
								Description: "Specifies the port of the referenced service.",
								Ref:         ref("k8s.io/apimachinery/pkg/util/intstr.IntOrString"),
							},
						},
					},
					Required: []string{"serviceName", "servicePort"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/util/intstr.IntOrString"},
		},
		"k8s.io/api/extensions/v1beta1.IngressList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "IngressList is a collection of Ingress.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "Items is the list of Ingress.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/extensions/v1beta1.Ingress"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/extensions/v1beta1.Ingress", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/extensions/v1beta1.IngressRule": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "IngressRule represents the rules mapping the paths under a specified host to the related backend services. Incoming requests are first evaluated for a host match, then routed to the backend associated with the matching IngressRuleValue.",
					Properties: map[string]spec.Schema{
						"host": {
							SchemaProps: spec.SchemaProps{
								Description: "Host is the fully qualified domain name of a network host, as defined by RFC 3986. Note the following deviations from the \"host\" part of the URI as defined in the RFC: 1. IPs are not allowed. Currently an IngressRuleValue can only apply to the\n\t  IP in the Spec of the parent Ingress.\n2. The `:` delimiter is not respected because ports are not allowed.\n\t  Currently the port of an Ingress is implicitly :80 for http and\n\t  :443 for https.\nBoth these may change in the future. Incoming requests are matched against the host before the IngressRuleValue. If the host is unspecified, the Ingress routes all traffic based on the specified IngressRuleValue.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"http": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/api/extensions/v1beta1.HTTPIngressRuleValue"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/extensions/v1beta1.HTTPIngressRuleValue"},
		},
		"k8s.io/api/extensions/v1beta1.IngressRuleValue": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "IngressRuleValue represents a rule to apply against incoming requests. If the rule is satisfied, the request is routed to the specified backend. Currently mixing different types of rules in a single Ingress is disallowed, so exactly one of the following must be set.",
					Properties: map[string]spec.Schema{
						"http": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/api/extensions/v1beta1.HTTPIngressRuleValue"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/extensions/v1beta1.HTTPIngressRuleValue"},
		},
		"k8s.io/api/extensions/v1beta1.IngressSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "IngressSpec describes the Ingress the user wishes to exist.",
					Properties: map[string]spec.Schema{
						"backend": {
							SchemaProps: spec.SchemaProps{
								Description: "A default backend capable of servicing requests that don't match any rule. At least one of 'backend' or 'rules' must be specified. This field is optional to allow the loadbalancer controller or defaulting logic to specify a global default.",
								Ref:         ref("k8s.io/api/extensions/v1beta1.IngressBackend"),
							},
						},
						"tls": {
							SchemaProps: spec.SchemaProps{
								Description: "TLS configuration. Currently the Ingress only supports a single TLS port, 443. If multiple members of this list specify different hosts, they will be multiplexed on the same port according to the hostname specified through the SNI TLS extension, if the ingress controller fulfilling the ingress supports SNI.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/extensions/v1beta1.IngressTLS"),
										},
									},
								},
							},
						},
						"rules": {
							SchemaProps: spec.SchemaProps{
								Description: "A list of host rules used to configure the Ingress. If unspecified, or no rule matches, all traffic is sent to the default backend.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/extensions/v1beta1.IngressRule"),
										},
									},
								},
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/extensions/v1beta1.IngressBackend", "k8s.io/api/extensions/v1beta1.IngressRule", "k8s.io/api/extensions/v1beta1.IngressTLS"},
		},
		"k8s.io/api/extensions/v1beta1.IngressStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "IngressStatus describe the current state of the Ingress.",
					Properties: map[string]spec.Schema{
						"loadBalancer": {
							SchemaProps: spec.SchemaProps{
								Description: "LoadBalancer contains the current status of the load-balancer.",
								Ref:         ref("k8s.io/api/core/v1.LoadBalancerStatus"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.LoadBalancerStatus"},
		},
		"k8s.io/api/extensions/v1beta1.IngressTLS": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "IngressTLS describes the transport layer security associated with an Ingress.",
					Properties: map[string]spec.Schema{
						"hosts": {
							SchemaProps: spec.SchemaProps{
								Description: "Hosts are a list of hosts included in the TLS certificate. The values in this list must match the name/s used in the tlsSecret. Defaults to the wildcard host setting for the loadbalancer controller fulfilling this Ingress, if left unspecified.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"secretName": {
							SchemaProps: spec.SchemaProps{
								Description: "SecretName is the name of the secret used to terminate SSL traffic on 443. Field is left optional to allow SSL routing based on SNI hostname alone. If the SNI host in a listener conflicts with the \"Host\" header field used by an IngressRule, the SNI host is used for termination and value of the Host header is used for routing.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/extensions/v1beta1.NetworkPolicy": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DEPRECATED 1.9 - This group version of NetworkPolicy is deprecated by networking/v1/NetworkPolicy. NetworkPolicy describes what network traffic is allowed for a set of Pods",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "Specification of the desired behavior for this NetworkPolicy.",
								Ref:         ref("k8s.io/api/extensions/v1beta1.NetworkPolicySpec"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/extensions/v1beta1.NetworkPolicySpec", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/extensions/v1beta1.NetworkPolicyEgressRule": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DEPRECATED 1.9 - This group version of NetworkPolicyEgressRule is deprecated by networking/v1/NetworkPolicyEgressRule. NetworkPolicyEgressRule describes a particular set of traffic that is allowed out of pods matched by a NetworkPolicySpec's podSelector. The traffic must match both ports and to. This type is beta-level in 1.8",
					Properties: map[string]spec.Schema{
						"ports": {
							SchemaProps: spec.SchemaProps{
								Description: "List of destination ports for outgoing traffic. Each item in this list is combined using a logical OR. If this field is empty or missing, this rule matches all ports (traffic not restricted by port). If this field is present and contains at least one item, then this rule allows traffic only if the traffic matches at least one port in the list.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/extensions/v1beta1.NetworkPolicyPort"),
										},
									},
								},
							},
						},
						"to": {
							SchemaProps: spec.SchemaProps{
								Description: "List of destinations for outgoing traffic of pods selected for this rule. Items in this list are combined using a logical OR operation. If this field is empty or missing, this rule matches all destinations (traffic not restricted by destination). If this field is present and contains at least one item, this rule allows traffic only if the traffic matches at least one item in the to list.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/extensions/v1beta1.NetworkPolicyPeer"),
										},
									},
								},
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/extensions/v1beta1.NetworkPolicyPeer", "k8s.io/api/extensions/v1beta1.NetworkPolicyPort"},
		},
		"k8s.io/api/extensions/v1beta1.NetworkPolicyIngressRule": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DEPRECATED 1.9 - This group version of NetworkPolicyIngressRule is deprecated by networking/v1/NetworkPolicyIngressRule. This NetworkPolicyIngressRule matches traffic if and only if the traffic matches both ports AND from.",
					Properties: map[string]spec.Schema{
						"ports": {
							SchemaProps: spec.SchemaProps{
								Description: "List of ports which should be made accessible on the pods selected for this rule. Each item in this list is combined using a logical OR. If this field is empty or missing, this rule matches all ports (traffic not restricted by port). If this field is present and contains at least one item, then this rule allows traffic only if the traffic matches at least one port in the list.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/extensions/v1beta1.NetworkPolicyPort"),
										},
									},
								},
							},
						},
						"from": {
							SchemaProps: spec.SchemaProps{
								Description: "List of sources which should be able to access the pods selected for this rule. Items in this list are combined using a logical OR operation. If this field is empty or missing, this rule matches all sources (traffic not restricted by source). If this field is present and contains at least on item, this rule allows traffic only if the traffic matches at least one item in the from list.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/extensions/v1beta1.NetworkPolicyPeer"),
										},
									},
								},
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/extensions/v1beta1.NetworkPolicyPeer", "k8s.io/api/extensions/v1beta1.NetworkPolicyPort"},
		},
		"k8s.io/api/extensions/v1beta1.NetworkPolicyList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DEPRECATED 1.9 - This group version of NetworkPolicyList is deprecated by networking/v1/NetworkPolicyList. Network Policy List is a list of NetworkPolicy objects.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "Items is a list of schema objects.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/extensions/v1beta1.NetworkPolicy"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/extensions/v1beta1.NetworkPolicy", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/extensions/v1beta1.NetworkPolicyPeer": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DEPRECATED 1.9 - This group version of NetworkPolicyPeer is deprecated by networking/v1/NetworkPolicyPeer.",
					Properties: map[string]spec.Schema{
						"podSelector": {
							SchemaProps: spec.SchemaProps{
								Description: "This is a label selector which selects Pods in this namespace. This field follows standard label selector semantics. If present but empty, this selector selects all pods in this namespace.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"),
							},
						},
						"namespaceSelector": {
							SchemaProps: spec.SchemaProps{
								Description: "Selects Namespaces using cluster scoped-labels.  This matches all pods in all namespaces selected by this label selector. This field follows standard label selector semantics. If present but empty, this selector selects all namespaces.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"),
							},
						},
						"ipBlock": {
							SchemaProps: spec.SchemaProps{
								Description: "IPBlock defines policy on a particular IPBlock",
								Ref:         ref("k8s.io/api/extensions/v1beta1.IPBlock"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/extensions/v1beta1.IPBlock", "k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"},
		},
		"k8s.io/api/extensions/v1beta1.NetworkPolicyPort": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DEPRECATED 1.9 - This group version of NetworkPolicyPort is deprecated by networking/v1/NetworkPolicyPort.",
					Properties: map[string]spec.Schema{
						"protocol": {
							SchemaProps: spec.SchemaProps{
								Description: "Optional.  The protocol (TCP or UDP) which traffic must match. If not specified, this field defaults to TCP.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"port": {
							SchemaProps: spec.SchemaProps{
								Description: "If specified, the port on the given protocol.  This can either be a numerical or named port on a pod.  If this field is not provided, this matches all port names and numbers. If present, only traffic on the specified protocol AND port will be matched.",
								Ref:         ref("k8s.io/apimachinery/pkg/util/intstr.IntOrString"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/util/intstr.IntOrString"},
		},
		"k8s.io/api/extensions/v1beta1.NetworkPolicySpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DEPRECATED 1.9 - This group version of NetworkPolicySpec is deprecated by networking/v1/NetworkPolicySpec.",
					Properties: map[string]spec.Schema{
						"podSelector": {
							SchemaProps: spec.SchemaProps{
								Description: "Selects the pods to which this NetworkPolicy object applies.  The array of ingress rules is applied to any pods selected by this field. Multiple network policies can select the same set of pods.  In this case, the ingress rules for each are combined additively. This field is NOT optional and follows standard label selector semantics. An empty podSelector matches all pods in this namespace.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"),
							},
						},
						"ingress": {
							SchemaProps: spec.SchemaProps{
								Description: "List of ingress rules to be applied to the selected pods. Traffic is allowed to a pod if there are no NetworkPolicies selecting the pod OR if the traffic source is the pod's local node, OR if the traffic matches at least one ingress rule across all of the NetworkPolicy objects whose podSelector matches the pod. If this field is empty then this NetworkPolicy does not allow any traffic (and serves solely to ensure that the pods it selects are isolated by default).",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/extensions/v1beta1.NetworkPolicyIngressRule"),
										},
									},
								},
							},
						},
						"egress": {
							SchemaProps: spec.SchemaProps{
								Description: "List of egress rules to be applied to the selected pods. Outgoing traffic is allowed if there are no NetworkPolicies selecting the pod (and cluster policy otherwise allows the traffic), OR if the traffic matches at least one egress rule across all of the NetworkPolicy objects whose podSelector matches the pod. If this field is empty then this NetworkPolicy limits all outgoing traffic (and serves solely to ensure that the pods it selects are isolated by default). This field is beta-level in 1.8",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/extensions/v1beta1.NetworkPolicyEgressRule"),
										},
									},
								},
							},
						},
						"policyTypes": {
							SchemaProps: spec.SchemaProps{
								Description: "List of rule types that the NetworkPolicy relates to. Valid options are Ingress, Egress, or Ingress,Egress. If this field is not specified, it will default based on the existence of Ingress or Egress rules; policies that contain an Egress section are assumed to affect Egress, and all policies (whether or not they contain an Ingress section) are assumed to affect Ingress. If you want to write an egress-only policy, you must explicitly specify policyTypes [ \"Egress\" ]. Likewise, if you want to write a policy that specifies that no egress is allowed, you must specify a policyTypes value that include \"Egress\" (since such a policy would not include an Egress section and would otherwise default to just [ \"Ingress\" ]). This field is beta-level in 1.8",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
					},
					Required: []string{"podSelector"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/extensions/v1beta1.NetworkPolicyEgressRule", "k8s.io/api/extensions/v1beta1.NetworkPolicyIngressRule", "k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"},
		},
		"k8s.io/api/extensions/v1beta1.PodSecurityPolicy": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Pod Security Policy governs the ability to make requests that affect the Security Context that will be applied to a pod and container.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "spec defines the policy enforced.",
								Ref:         ref("k8s.io/api/extensions/v1beta1.PodSecurityPolicySpec"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/extensions/v1beta1.PodSecurityPolicySpec", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/extensions/v1beta1.PodSecurityPolicyList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Pod Security Policy List is a list of PodSecurityPolicy objects.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "Items is a list of schema objects.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/extensions/v1beta1.PodSecurityPolicy"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/extensions/v1beta1.PodSecurityPolicy", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/extensions/v1beta1.PodSecurityPolicySpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Pod Security Policy Spec defines the policy enforced.",
					Properties: map[string]spec.Schema{
						"privileged": {
							SchemaProps: spec.SchemaProps{
								Description: "privileged determines if a pod can request to be run as privileged.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"defaultAddCapabilities": {
							SchemaProps: spec.SchemaProps{
								Description: "DefaultAddCapabilities is the default set of capabilities that will be added to the container unless the pod spec specifically drops the capability.  You may not list a capability in both DefaultAddCapabilities and RequiredDropCapabilities. Capabilities added here are implicitly allowed, and need not be included in the AllowedCapabilities list.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"requiredDropCapabilities": {
							SchemaProps: spec.SchemaProps{
								Description: "RequiredDropCapabilities are the capabilities that will be dropped from the container.  These are required to be dropped and cannot be added.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"allowedCapabilities": {
							SchemaProps: spec.SchemaProps{
								Description: "AllowedCapabilities is a list of capabilities that can be requested to add to the container. Capabilities in this field may be added at the pod author's discretion. You must not list a capability in both AllowedCapabilities and RequiredDropCapabilities.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"volumes": {
							SchemaProps: spec.SchemaProps{
								Description: "volumes is a white list of allowed volume plugins.  Empty indicates that all plugins may be used.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"hostNetwork": {
							SchemaProps: spec.SchemaProps{
								Description: "hostNetwork determines if the policy allows the use of HostNetwork in the pod spec.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"hostPorts": {
							SchemaProps: spec.SchemaProps{
								Description: "hostPorts determines which host port ranges are allowed to be exposed.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/extensions/v1beta1.HostPortRange"),
										},
									},
								},
							},
						},
						"hostPID": {
							SchemaProps: spec.SchemaProps{
								Description: "hostPID determines if the policy allows the use of HostPID in the pod spec.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"hostIPC": {
							SchemaProps: spec.SchemaProps{
								Description: "hostIPC determines if the policy allows the use of HostIPC in the pod spec.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"seLinux": {
							SchemaProps: spec.SchemaProps{
								Description: "seLinux is the strategy that will dictate the allowable labels that may be set.",
								Ref:         ref("k8s.io/api/extensions/v1beta1.SELinuxStrategyOptions"),
							},
						},
						"runAsUser": {
							SchemaProps: spec.SchemaProps{
								Description: "runAsUser is the strategy that will dictate the allowable RunAsUser values that may be set.",
								Ref:         ref("k8s.io/api/extensions/v1beta1.RunAsUserStrategyOptions"),
							},
						},
						"supplementalGroups": {
							SchemaProps: spec.SchemaProps{
								Description: "SupplementalGroups is the strategy that will dictate what supplemental groups are used by the SecurityContext.",
								Ref:         ref("k8s.io/api/extensions/v1beta1.SupplementalGroupsStrategyOptions"),
							},
						},
						"fsGroup": {
							SchemaProps: spec.SchemaProps{
								Description: "FSGroup is the strategy that will dictate what fs group is used by the SecurityContext.",
								Ref:         ref("k8s.io/api/extensions/v1beta1.FSGroupStrategyOptions"),
							},
						},
						"readOnlyRootFilesystem": {
							SchemaProps: spec.SchemaProps{
								Description: "ReadOnlyRootFilesystem when set to true will force containers to run with a read only root file system.  If the container specifically requests to run with a non-read only root file system the PSP should deny the pod. If set to false the container may run with a read only root file system if it wishes but it will not be forced to.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"defaultAllowPrivilegeEscalation": {
							SchemaProps: spec.SchemaProps{
								Description: "DefaultAllowPrivilegeEscalation controls the default setting for whether a process can gain more privileges than its parent process.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"allowPrivilegeEscalation": {
							SchemaProps: spec.SchemaProps{
								Description: "AllowPrivilegeEscalation determines if a pod can request to allow privilege escalation. If unspecified, defaults to true.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"allowedHostPaths": {
							SchemaProps: spec.SchemaProps{
								Description: "is a white list of allowed host paths. Empty indicates that all host paths may be used.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/extensions/v1beta1.AllowedHostPath"),
										},
									},
								},
							},
						},
						"allowedFlexVolumes": {
							SchemaProps: spec.SchemaProps{
								Description: "AllowedFlexVolumes is a whitelist of allowed Flexvolumes.  Empty or nil indicates that all Flexvolumes may be used.  This parameter is effective only when the usage of the Flexvolumes is allowed in the \"Volumes\" field.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/extensions/v1beta1.AllowedFlexVolume"),
										},
									},
								},
							},
						},
					},
					Required: []string{"seLinux", "runAsUser", "supplementalGroups", "fsGroup"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/extensions/v1beta1.AllowedFlexVolume", "k8s.io/api/extensions/v1beta1.AllowedHostPath", "k8s.io/api/extensions/v1beta1.FSGroupStrategyOptions", "k8s.io/api/extensions/v1beta1.HostPortRange", "k8s.io/api/extensions/v1beta1.RunAsUserStrategyOptions", "k8s.io/api/extensions/v1beta1.SELinuxStrategyOptions", "k8s.io/api/extensions/v1beta1.SupplementalGroupsStrategyOptions"},
		},
		"k8s.io/api/extensions/v1beta1.ReplicaSet": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DEPRECATED - This group version of ReplicaSet is deprecated by apps/v1beta2/ReplicaSet. See the release notes for more information. ReplicaSet ensures that a specified number of pod replicas are running at any given time.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "If the Labels of a ReplicaSet are empty, they are defaulted to be the same as the Pod(s) that the ReplicaSet manages. Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "Spec defines the specification of the desired behavior of the ReplicaSet. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status",
								Ref:         ref("k8s.io/api/extensions/v1beta1.ReplicaSetSpec"),
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Status is the most recently observed status of the ReplicaSet. This data may be out of date by some window of time. Populated by the system. Read-only. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status",
								Ref:         ref("k8s.io/api/extensions/v1beta1.ReplicaSetStatus"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/extensions/v1beta1.ReplicaSetSpec", "k8s.io/api/extensions/v1beta1.ReplicaSetStatus", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/extensions/v1beta1.ReplicaSetCondition": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ReplicaSetCondition describes the state of a replica set at a certain point.",
					Properties: map[string]spec.Schema{
						"type": {
							SchemaProps: spec.SchemaProps{
								Description: "Type of replica set condition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Status of the condition, one of True, False, Unknown.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"lastTransitionTime": {
							SchemaProps: spec.SchemaProps{
								Description: "The last time the condition transitioned from one status to another.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"reason": {
							SchemaProps: spec.SchemaProps{
								Description: "The reason for the condition's last transition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"message": {
							SchemaProps: spec.SchemaProps{
								Description: "A human readable message indicating details about the transition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"type", "status"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.Time"},
		},
		"k8s.io/api/extensions/v1beta1.ReplicaSetList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ReplicaSetList is a collection of ReplicaSets.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "List of ReplicaSets. More info: https://kubernetes.io/docs/concepts/workloads/controllers/replicationcontroller",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/extensions/v1beta1.ReplicaSet"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/extensions/v1beta1.ReplicaSet", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/extensions/v1beta1.ReplicaSetSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ReplicaSetSpec is the specification of a ReplicaSet.",
					Properties: map[string]spec.Schema{
						"replicas": {
							SchemaProps: spec.SchemaProps{
								Description: "Replicas is the number of desired replicas. This is a pointer to distinguish between explicit zero and unspecified. Defaults to 1. More info: https://kubernetes.io/docs/concepts/workloads/controllers/replicationcontroller/#what-is-a-replicationcontroller",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"minReadySeconds": {
							SchemaProps: spec.SchemaProps{
								Description: "Minimum number of seconds for which a newly created pod should be ready without any of its container crashing, for it to be considered available. Defaults to 0 (pod will be considered available as soon as it is ready)",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"selector": {
							SchemaProps: spec.SchemaProps{
								Description: "Selector is a label query over pods that should match the replica count. If the selector is empty, it is defaulted to the labels present on the pod template. Label keys and values that must match in order to be controlled by this replica set. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/labels/#label-selectors",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"),
							},
						},
						"template": {
							SchemaProps: spec.SchemaProps{
								Description: "Template is the object that describes the pod that will be created if insufficient replicas are detected. More info: https://kubernetes.io/docs/concepts/workloads/controllers/replicationcontroller#pod-template",
								Ref:         ref("k8s.io/api/core/v1.PodTemplateSpec"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.PodTemplateSpec", "k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"},
		},
		"k8s.io/api/extensions/v1beta1.ReplicaSetStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ReplicaSetStatus represents the current status of a ReplicaSet.",
					Properties: map[string]spec.Schema{
						"replicas": {
							SchemaProps: spec.SchemaProps{
								Description: "Replicas is the most recently oberved number of replicas. More info: https://kubernetes.io/docs/concepts/workloads/controllers/replicationcontroller/#what-is-a-replicationcontroller",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"fullyLabeledReplicas": {
							SchemaProps: spec.SchemaProps{
								Description: "The number of pods that have labels matching the labels of the pod template of the replicaset.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"readyReplicas": {
							SchemaProps: spec.SchemaProps{
								Description: "The number of ready replicas for this replica set.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"availableReplicas": {
							SchemaProps: spec.SchemaProps{
								Description: "The number of available replicas (ready for at least minReadySeconds) for this replica set.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"observedGeneration": {
							SchemaProps: spec.SchemaProps{
								Description: "ObservedGeneration reflects the generation of the most recently observed ReplicaSet.",
								Type:        []string{"integer"},
								Format:      "int64",
							},
						},
						"conditions": {
							VendorExtensible: spec.VendorExtensible{
								Extensions: spec.Extensions{
									"x-kubernetes-patch-merge-key": "type",
									"x-kubernetes-patch-strategy":  "merge",
								},
							},
							SchemaProps: spec.SchemaProps{
								Description: "Represents the latest available observations of a replica set's current state.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/extensions/v1beta1.ReplicaSetCondition"),
										},
									},
								},
							},
						},
					},
					Required: []string{"replicas"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/extensions/v1beta1.ReplicaSetCondition"},
		},
		"k8s.io/api/extensions/v1beta1.ReplicationControllerDummy": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Dummy definition",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/extensions/v1beta1.RollbackConfig": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DEPRECATED.",
					Properties: map[string]spec.Schema{
						"revision": {
							SchemaProps: spec.SchemaProps{
								Description: "The revision to rollback to. If set to 0, rollback to the last revision.",
								Type:        []string{"integer"},
								Format:      "int64",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/extensions/v1beta1.RollingUpdateDaemonSet": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Spec to control the desired behavior of daemon set rolling update.",
					Properties: map[string]spec.Schema{
						"maxUnavailable": {
							SchemaProps: spec.SchemaProps{
								Description: "The maximum number of DaemonSet pods that can be unavailable during the update. Value can be an absolute number (ex: 5) or a percentage of total number of DaemonSet pods at the start of the update (ex: 10%). Absolute number is calculated from percentage by rounding up. This cannot be 0. Default value is 1. Example: when this is set to 30%, at most 30% of the total number of nodes that should be running the daemon pod (i.e. status.desiredNumberScheduled) can have their pods stopped for an update at any given time. The update starts by stopping at most 30% of those DaemonSet pods and then brings up new DaemonSet pods in their place. Once the new pods are available, it then proceeds onto other DaemonSet pods, thus ensuring that at least 70% of original number of DaemonSet pods are available at all times during the update.",
								Ref:         ref("k8s.io/apimachinery/pkg/util/intstr.IntOrString"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/util/intstr.IntOrString"},
		},
		"k8s.io/api/extensions/v1beta1.RollingUpdateDeployment": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Spec to control the desired behavior of rolling update.",
					Properties: map[string]spec.Schema{
						"maxUnavailable": {
							SchemaProps: spec.SchemaProps{
								Description: "The maximum number of pods that can be unavailable during the update. Value can be an absolute number (ex: 5) or a percentage of desired pods (ex: 10%). Absolute number is calculated from percentage by rounding down. This can not be 0 if MaxSurge is 0. By default, a fixed value of 1 is used. Example: when this is set to 30%, the old RC can be scaled down to 70% of desired pods immediately when the rolling update starts. Once new pods are ready, old RC can be scaled down further, followed by scaling up the new RC, ensuring that the total number of pods available at all times during the update is at least 70% of desired pods.",
								Ref:         ref("k8s.io/apimachinery/pkg/util/intstr.IntOrString"),
							},
						},
						"maxSurge": {
							SchemaProps: spec.SchemaProps{
								Description: "The maximum number of pods that can be scheduled above the desired number of pods. Value can be an absolute number (ex: 5) or a percentage of desired pods (ex: 10%). This can not be 0 if MaxUnavailable is 0. Absolute number is calculated from percentage by rounding up. By default, a value of 1 is used. Example: when this is set to 30%, the new RC can be scaled up immediately when the rolling update starts, such that the total number of old and new pods do not exceed 130% of desired pods. Once old pods have been killed, new RC can be scaled up further, ensuring that total number of pods running at any time during the update is atmost 130% of desired pods.",
								Ref:         ref("k8s.io/apimachinery/pkg/util/intstr.IntOrString"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/util/intstr.IntOrString"},
		},
		"k8s.io/api/extensions/v1beta1.RunAsUserStrategyOptions": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Run A sUser Strategy Options defines the strategy type and any options used to create the strategy.",
					Properties: map[string]spec.Schema{
						"rule": {
							SchemaProps: spec.SchemaProps{
								Description: "Rule is the strategy that will dictate the allowable RunAsUser values that may be set.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"ranges": {
							SchemaProps: spec.SchemaProps{
								Description: "Ranges are the allowed ranges of uids that may be used.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/extensions/v1beta1.IDRange"),
										},
									},
								},
							},
						},
					},
					Required: []string{"rule"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/extensions/v1beta1.IDRange"},
		},
		"k8s.io/api/extensions/v1beta1.SELinuxStrategyOptions": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "SELinux  Strategy Options defines the strategy type and any options used to create the strategy.",
					Properties: map[string]spec.Schema{
						"rule": {
							SchemaProps: spec.SchemaProps{
								Description: "type is the strategy that will dictate the allowable labels that may be set.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"seLinuxOptions": {
							SchemaProps: spec.SchemaProps{
								Description: "seLinuxOptions required to run as; required for MustRunAs More info: https://kubernetes.io/docs/tasks/configure-pod-container/security-context/",
								Ref:         ref("k8s.io/api/core/v1.SELinuxOptions"),
							},
						},
					},
					Required: []string{"rule"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.SELinuxOptions"},
		},
		"k8s.io/api/extensions/v1beta1.Scale": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "represents a scaling request for a resource.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object metadata; More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "defines the behavior of the scale. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status.",
								Ref:         ref("k8s.io/api/extensions/v1beta1.ScaleSpec"),
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "current status of the scale. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status. Read-only.",
								Ref:         ref("k8s.io/api/extensions/v1beta1.ScaleStatus"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/extensions/v1beta1.ScaleSpec", "k8s.io/api/extensions/v1beta1.ScaleStatus", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/extensions/v1beta1.ScaleSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "describes the attributes of a scale subresource",
					Properties: map[string]spec.Schema{
						"replicas": {
							SchemaProps: spec.SchemaProps{
								Description: "desired number of instances for the scaled object.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/extensions/v1beta1.ScaleStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "represents the current status of a scale subresource.",
					Properties: map[string]spec.Schema{
						"replicas": {
							SchemaProps: spec.SchemaProps{
								Description: "actual number of observed instances of the scaled object.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"selector": {
							SchemaProps: spec.SchemaProps{
								Description: "label query over pods that should match the replicas count. More info: http://kubernetes.io/docs/user-guide/labels#label-selectors",
								Type:        []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"targetSelector": {
							SchemaProps: spec.SchemaProps{
								Description: "label selector for pods that should match the replicas count. This is a serializated version of both map-based and more expressive set-based selectors. This is done to avoid introspection in the clients. The string will be in the same format as the query-param syntax. If the target type only supports map-based selectors, both this field and map-based selector field are populated. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/labels/#label-selectors",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"replicas"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/extensions/v1beta1.SupplementalGroupsStrategyOptions": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "SupplementalGroupsStrategyOptions defines the strategy type and options used to create the strategy.",
					Properties: map[string]spec.Schema{
						"rule": {
							SchemaProps: spec.SchemaProps{
								Description: "Rule is the strategy that will dictate what supplemental groups is used in the SecurityContext.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"ranges": {
							SchemaProps: spec.SchemaProps{
								Description: "Ranges are the allowed ranges of supplemental groups.  If you would like to force a single supplemental group then supply a single range with the same start and end.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/extensions/v1beta1.IDRange"),
										},
									},
								},
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/extensions/v1beta1.IDRange"},
		},
		"k8s.io/api/imagepolicy/v1alpha1.ImageReview": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ImageReview checks if the set of images in a pod are allowed.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "Spec holds information about the pod being evaluated",
								Ref:         ref("k8s.io/api/imagepolicy/v1alpha1.ImageReviewSpec"),
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Status is filled in by the backend and indicates whether the pod should be allowed.",
								Ref:         ref("k8s.io/api/imagepolicy/v1alpha1.ImageReviewStatus"),
							},
						},
					},
					Required: []string{"spec"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/imagepolicy/v1alpha1.ImageReviewSpec", "k8s.io/api/imagepolicy/v1alpha1.ImageReviewStatus", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/imagepolicy/v1alpha1.ImageReviewContainerSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ImageReviewContainerSpec is a description of a container within the pod creation request.",
					Properties: map[string]spec.Schema{
						"image": {
							SchemaProps: spec.SchemaProps{
								Description: "This can be in the form image:tag or image@SHA:012345679abcdef.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/imagepolicy/v1alpha1.ImageReviewSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ImageReviewSpec is a description of the pod creation request.",
					Properties: map[string]spec.Schema{
						"containers": {
							SchemaProps: spec.SchemaProps{
								Description: "Containers is a list of a subset of the information in each container of the Pod being created.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/imagepolicy/v1alpha1.ImageReviewContainerSpec"),
										},
									},
								},
							},
						},
						"annotations": {
							SchemaProps: spec.SchemaProps{
								Description: "Annotations is a list of key-value pairs extracted from the Pod's annotations. It only includes keys which match the pattern `*.image-policy.k8s.io/*`. It is up to each webhook backend to determine how to interpret these annotations, if at all.",
								Type:        []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"namespace": {
							SchemaProps: spec.SchemaProps{
								Description: "Namespace is the namespace the pod is being created in.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/imagepolicy/v1alpha1.ImageReviewContainerSpec"},
		},
		"k8s.io/api/imagepolicy/v1alpha1.ImageReviewStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ImageReviewStatus is the result of the token authentication request.",
					Properties: map[string]spec.Schema{
						"allowed": {
							SchemaProps: spec.SchemaProps{
								Description: "Allowed indicates that all images were allowed to be run.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"reason": {
							SchemaProps: spec.SchemaProps{
								Description: "Reason should be empty unless Allowed is false in which case it may contain a short description of what is wrong.  Kubernetes may truncate excessively long errors when displaying to the user.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"allowed"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/networking/v1.IPBlock": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "IPBlock describes a particular CIDR (Ex. \"192.168.1.1/24\") that is allowed to the pods matched by a NetworkPolicySpec's podSelector. The except entry describes CIDRs that should not be included within this rule.",
					Properties: map[string]spec.Schema{
						"cidr": {
							SchemaProps: spec.SchemaProps{
								Description: "CIDR is a string representing the IP Block Valid examples are \"192.168.1.1/24\"",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"except": {
							SchemaProps: spec.SchemaProps{
								Description: "Except is a slice of CIDRs that should not be included within an IP Block Valid examples are \"192.168.1.1/24\" Except values will be rejected if they are outside the CIDR range",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
					},
					Required: []string{"cidr"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/networking/v1.NetworkPolicy": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "NetworkPolicy describes what network traffic is allowed for a set of Pods",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "Specification of the desired behavior for this NetworkPolicy.",
								Ref:         ref("k8s.io/api/networking/v1.NetworkPolicySpec"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/networking/v1.NetworkPolicySpec", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/networking/v1.NetworkPolicyEgressRule": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "NetworkPolicyEgressRule describes a particular set of traffic that is allowed out of pods matched by a NetworkPolicySpec's podSelector. The traffic must match both ports and to. This type is beta-level in 1.8",
					Properties: map[string]spec.Schema{
						"ports": {
							SchemaProps: spec.SchemaProps{
								Description: "List of destination ports for outgoing traffic. Each item in this list is combined using a logical OR. If this field is empty or missing, this rule matches all ports (traffic not restricted by port). If this field is present and contains at least one item, then this rule allows traffic only if the traffic matches at least one port in the list.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/networking/v1.NetworkPolicyPort"),
										},
									},
								},
							},
						},
						"to": {
							SchemaProps: spec.SchemaProps{
								Description: "List of destinations for outgoing traffic of pods selected for this rule. Items in this list are combined using a logical OR operation. If this field is empty or missing, this rule matches all destinations (traffic not restricted by destination). If this field is present and contains at least one item, this rule allows traffic only if the traffic matches at least one item in the to list.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/networking/v1.NetworkPolicyPeer"),
										},
									},
								},
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/networking/v1.NetworkPolicyPeer", "k8s.io/api/networking/v1.NetworkPolicyPort"},
		},
		"k8s.io/api/networking/v1.NetworkPolicyIngressRule": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "NetworkPolicyIngressRule describes a particular set of traffic that is allowed to the pods matched by a NetworkPolicySpec's podSelector. The traffic must match both ports and from.",
					Properties: map[string]spec.Schema{
						"ports": {
							SchemaProps: spec.SchemaProps{
								Description: "List of ports which should be made accessible on the pods selected for this rule. Each item in this list is combined using a logical OR. If this field is empty or missing, this rule matches all ports (traffic not restricted by port). If this field is present and contains at least one item, then this rule allows traffic only if the traffic matches at least one port in the list.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/networking/v1.NetworkPolicyPort"),
										},
									},
								},
							},
						},
						"from": {
							SchemaProps: spec.SchemaProps{
								Description: "List of sources which should be able to access the pods selected for this rule. Items in this list are combined using a logical OR operation. If this field is empty or missing, this rule matches all sources (traffic not restricted by source). If this field is present and contains at least on item, this rule allows traffic only if the traffic matches at least one item in the from list.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/networking/v1.NetworkPolicyPeer"),
										},
									},
								},
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/networking/v1.NetworkPolicyPeer", "k8s.io/api/networking/v1.NetworkPolicyPort"},
		},
		"k8s.io/api/networking/v1.NetworkPolicyList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "NetworkPolicyList is a list of NetworkPolicy objects.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "Items is a list of schema objects.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/networking/v1.NetworkPolicy"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/networking/v1.NetworkPolicy", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/networking/v1.NetworkPolicyPeer": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "NetworkPolicyPeer describes a peer to allow traffic from. Exactly one of its fields must be specified.",
					Properties: map[string]spec.Schema{
						"podSelector": {
							SchemaProps: spec.SchemaProps{
								Description: "This is a label selector which selects Pods in this namespace. This field follows standard label selector semantics. If present but empty, this selector selects all pods in this namespace.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"),
							},
						},
						"namespaceSelector": {
							SchemaProps: spec.SchemaProps{
								Description: "Selects Namespaces using cluster scoped-labels. This matches all pods in all namespaces selected by this label selector. This field follows standard label selector semantics. If present but empty, this selector selects all namespaces.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"),
							},
						},
						"ipBlock": {
							SchemaProps: spec.SchemaProps{
								Description: "IPBlock defines policy on a particular IPBlock",
								Ref:         ref("k8s.io/api/networking/v1.IPBlock"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/networking/v1.IPBlock", "k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"},
		},
		"k8s.io/api/networking/v1.NetworkPolicyPort": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "NetworkPolicyPort describes a port to allow traffic on",
					Properties: map[string]spec.Schema{
						"protocol": {
							SchemaProps: spec.SchemaProps{
								Description: "The protocol (TCP or UDP) which traffic must match. If not specified, this field defaults to TCP.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"port": {
							SchemaProps: spec.SchemaProps{
								Description: "The port on the given protocol. This can either be a numerical or named port on a pod. If this field is not provided, this matches all port names and numbers.",
								Ref:         ref("k8s.io/apimachinery/pkg/util/intstr.IntOrString"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/util/intstr.IntOrString"},
		},
		"k8s.io/api/networking/v1.NetworkPolicySpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "NetworkPolicySpec provides the specification of a NetworkPolicy",
					Properties: map[string]spec.Schema{
						"podSelector": {
							SchemaProps: spec.SchemaProps{
								Description: "Selects the pods to which this NetworkPolicy object applies. The array of ingress rules is applied to any pods selected by this field. Multiple network policies can select the same set of pods. In this case, the ingress rules for each are combined additively. This field is NOT optional and follows standard label selector semantics. An empty podSelector matches all pods in this namespace.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"),
							},
						},
						"ingress": {
							SchemaProps: spec.SchemaProps{
								Description: "List of ingress rules to be applied to the selected pods. Traffic is allowed to a pod if there are no NetworkPolicies selecting the pod (and cluster policy otherwise allows the traffic), OR if the traffic source is the pod's local node, OR if the traffic matches at least one ingress rule across all of the NetworkPolicy objects whose podSelector matches the pod. If this field is empty then this NetworkPolicy does not allow any traffic (and serves solely to ensure that the pods it selects are isolated by default)",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/networking/v1.NetworkPolicyIngressRule"),
										},
									},
								},
							},
						},
						"egress": {
							SchemaProps: spec.SchemaProps{
								Description: "List of egress rules to be applied to the selected pods. Outgoing traffic is allowed if there are no NetworkPolicies selecting the pod (and cluster policy otherwise allows the traffic), OR if the traffic matches at least one egress rule across all of the NetworkPolicy objects whose podSelector matches the pod. If this field is empty then this NetworkPolicy limits all outgoing traffic (and serves solely to ensure that the pods it selects are isolated by default). This field is beta-level in 1.8",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/networking/v1.NetworkPolicyEgressRule"),
										},
									},
								},
							},
						},
						"policyTypes": {
							SchemaProps: spec.SchemaProps{
								Description: "List of rule types that the NetworkPolicy relates to. Valid options are Ingress, Egress, or Ingress,Egress. If this field is not specified, it will default based on the existence of Ingress or Egress rules; policies that contain an Egress section are assumed to affect Egress, and all policies (whether or not they contain an Ingress section) are assumed to affect Ingress. If you want to write an egress-only policy, you must explicitly specify policyTypes [ \"Egress\" ]. Likewise, if you want to write a policy that specifies that no egress is allowed, you must specify a policyTypes value that include \"Egress\" (since such a policy would not include an Egress section and would otherwise default to just [ \"Ingress\" ]). This field is beta-level in 1.8",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
					},
					Required: []string{"podSelector"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/networking/v1.NetworkPolicyEgressRule", "k8s.io/api/networking/v1.NetworkPolicyIngressRule", "k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"},
		},
		"k8s.io/api/policy/v1beta1.Eviction": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Eviction evicts a pod from its node subject to certain policies and safety constraints. This is a subresource of Pod.  A request to cause such an eviction is created by POSTing to .../pods/<pod name>/evictions.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "ObjectMeta describes the pod that is being evicted.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"deleteOptions": {
							SchemaProps: spec.SchemaProps{
								Description: "DeleteOptions may be provided",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.DeleteOptions"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.DeleteOptions", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/policy/v1beta1.PodDisruptionBudget": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PodDisruptionBudget is an object to define the max disruption that can be caused to a collection of pods",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "Specification of the desired behavior of the PodDisruptionBudget.",
								Ref:         ref("k8s.io/api/policy/v1beta1.PodDisruptionBudgetSpec"),
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Most recently observed status of the PodDisruptionBudget.",
								Ref:         ref("k8s.io/api/policy/v1beta1.PodDisruptionBudgetStatus"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/policy/v1beta1.PodDisruptionBudgetSpec", "k8s.io/api/policy/v1beta1.PodDisruptionBudgetStatus", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/policy/v1beta1.PodDisruptionBudgetList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PodDisruptionBudgetList is a collection of PodDisruptionBudgets.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Type: []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/policy/v1beta1.PodDisruptionBudget"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/policy/v1beta1.PodDisruptionBudget", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/policy/v1beta1.PodDisruptionBudgetSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PodDisruptionBudgetSpec is a description of a PodDisruptionBudget.",
					Properties: map[string]spec.Schema{
						"minAvailable": {
							SchemaProps: spec.SchemaProps{
								Description: "An eviction is allowed if at least \"minAvailable\" pods selected by \"selector\" will still be available after the eviction, i.e. even in the absence of the evicted pod.  So for example you can prevent all voluntary evictions by specifying \"100%\".",
								Ref:         ref("k8s.io/apimachinery/pkg/util/intstr.IntOrString"),
							},
						},
						"selector": {
							SchemaProps: spec.SchemaProps{
								Description: "Label query over pods whose evictions are managed by the disruption budget.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"),
							},
						},
						"maxUnavailable": {
							SchemaProps: spec.SchemaProps{
								Description: "An eviction is allowed if at most \"maxUnavailable\" pods selected by \"selector\" are unavailable after the eviction, i.e. even in absence of the evicted pod. For example, one can prevent all voluntary evictions by specifying 0. This is a mutually exclusive setting with \"minAvailable\".",
								Ref:         ref("k8s.io/apimachinery/pkg/util/intstr.IntOrString"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector", "k8s.io/apimachinery/pkg/util/intstr.IntOrString"},
		},
		"k8s.io/api/policy/v1beta1.PodDisruptionBudgetStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PodDisruptionBudgetStatus represents information about the status of a PodDisruptionBudget. Status may trail the actual state of a system.",
					Properties: map[string]spec.Schema{
						"observedGeneration": {
							SchemaProps: spec.SchemaProps{
								Description: "Most recent generation observed when updating this PDB status. PodDisruptionsAllowed and other status informatio is valid only if observedGeneration equals to PDB's object generation.",
								Type:        []string{"integer"},
								Format:      "int64",
							},
						},
						"disruptedPods": {
							SchemaProps: spec.SchemaProps{
								Description: "DisruptedPods contains information about pods whose eviction was processed by the API server eviction subresource handler but has not yet been observed by the PodDisruptionBudget controller. A pod will be in this map from the time when the API server processed the eviction request to the time when the pod is seen by PDB controller as having been marked for deletion (or after a timeout). The key in the map is the name of the pod and the value is the time when the API server processed the eviction request. If the deletion didn't occur and a pod is still there it will be removed from the list automatically by PodDisruptionBudget controller after some time. If everything goes smooth this map should be empty for the most of the time. Large number of entries in the map may indicate problems with pod deletions.",
								Type:        []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
										},
									},
								},
							},
						},
						"disruptionsAllowed": {
							SchemaProps: spec.SchemaProps{
								Description: "Number of pod disruptions that are currently allowed.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"currentHealthy": {
							SchemaProps: spec.SchemaProps{
								Description: "current number of healthy pods",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"desiredHealthy": {
							SchemaProps: spec.SchemaProps{
								Description: "minimum desired number of healthy pods",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"expectedPods": {
							SchemaProps: spec.SchemaProps{
								Description: "total number of pods counted by this disruption budget",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
					},
					Required: []string{"disruptedPods", "disruptionsAllowed", "currentHealthy", "desiredHealthy", "expectedPods"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.Time"},
		},
		"k8s.io/api/rbac/v1.AggregationRule": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "AggregationRule describes how to locate ClusterRoles to aggregate into the ClusterRole",
					Properties: map[string]spec.Schema{
						"clusterRoleSelectors": {
							SchemaProps: spec.SchemaProps{
								Description: "ClusterRoleSelectors holds a list of selectors which will be used to find ClusterRoles and create the rules. If any of the selectors match, then the ClusterRole's permissions will be added",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"),
										},
									},
								},
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"},
		},
		"k8s.io/api/rbac/v1.ClusterRole": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ClusterRole is a cluster level, logical grouping of PolicyRules that can be referenced as a unit by a RoleBinding or ClusterRoleBinding.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"rules": {
							SchemaProps: spec.SchemaProps{
								Description: "Rules holds all the PolicyRules for this ClusterRole",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/rbac/v1.PolicyRule"),
										},
									},
								},
							},
						},
						"aggregationRule": {
							SchemaProps: spec.SchemaProps{
								Description: "AggregationRule is an optional field that describes how to build the Rules for this ClusterRole. If AggregationRule is set, then the Rules are controller managed and direct changes to Rules will be stomped by the controller.",
								Ref:         ref("k8s.io/api/rbac/v1.AggregationRule"),
							},
						},
					},
					Required: []string{"rules"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/rbac/v1.AggregationRule", "k8s.io/api/rbac/v1.PolicyRule", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/rbac/v1.ClusterRoleBinding": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ClusterRoleBinding references a ClusterRole, but not contain it.  It can reference a ClusterRole in the global namespace, and adds who information via Subject.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"subjects": {
							SchemaProps: spec.SchemaProps{
								Description: "Subjects holds references to the objects the role applies to.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/rbac/v1.Subject"),
										},
									},
								},
							},
						},
						"roleRef": {
							SchemaProps: spec.SchemaProps{
								Description: "RoleRef can only reference a ClusterRole in the global namespace. If the RoleRef cannot be resolved, the Authorizer must return an error.",
								Ref:         ref("k8s.io/api/rbac/v1.RoleRef"),
							},
						},
					},
					Required: []string{"subjects", "roleRef"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/rbac/v1.RoleRef", "k8s.io/api/rbac/v1.Subject", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/rbac/v1.ClusterRoleBindingList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ClusterRoleBindingList is a collection of ClusterRoleBindings",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "Items is a list of ClusterRoleBindings",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/rbac/v1.ClusterRoleBinding"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/rbac/v1.ClusterRoleBinding", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/rbac/v1.ClusterRoleList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ClusterRoleList is a collection of ClusterRoles",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "Items is a list of ClusterRoles",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/rbac/v1.ClusterRole"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/rbac/v1.ClusterRole", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/rbac/v1.PolicyRule": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PolicyRule holds information that describes a policy rule, but does not contain information about who the rule applies to or which namespace the rule applies to.",
					Properties: map[string]spec.Schema{
						"verbs": {
							SchemaProps: spec.SchemaProps{
								Description: "Verbs is a list of Verbs that apply to ALL the ResourceKinds and AttributeRestrictions contained in this rule.  VerbAll represents all kinds.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"apiGroups": {
							SchemaProps: spec.SchemaProps{
								Description: "APIGroups is the name of the APIGroup that contains the resources.  If multiple API groups are specified, any action requested against one of the enumerated resources in any API group will be allowed.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"resources": {
							SchemaProps: spec.SchemaProps{
								Description: "Resources is a list of resources this rule applies to.  ResourceAll represents all resources.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"resourceNames": {
							SchemaProps: spec.SchemaProps{
								Description: "ResourceNames is an optional white list of names that the rule applies to.  An empty set means that everything is allowed.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"nonResourceURLs": {
							SchemaProps: spec.SchemaProps{
								Description: "NonResourceURLs is a set of partial urls that a user should have access to.  *s are allowed, but only as the full, final step in the path Since non-resource URLs are not namespaced, this field is only applicable for ClusterRoles referenced from a ClusterRoleBinding. Rules can either apply to API resources (such as \"pods\" or \"secrets\") or non-resource URL paths (such as \"/api\"),  but not both.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
					},
					Required: []string{"verbs"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/rbac/v1.Role": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Role is a namespaced, logical grouping of PolicyRules that can be referenced as a unit by a RoleBinding.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"rules": {
							SchemaProps: spec.SchemaProps{
								Description: "Rules holds all the PolicyRules for this Role",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/rbac/v1.PolicyRule"),
										},
									},
								},
							},
						},
					},
					Required: []string{"rules"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/rbac/v1.PolicyRule", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/rbac/v1.RoleBinding": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "RoleBinding references a role, but does not contain it.  It can reference a Role in the same namespace or a ClusterRole in the global namespace. It adds who information via Subjects and namespace information by which namespace it exists in.  RoleBindings in a given namespace only have effect in that namespace.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"subjects": {
							SchemaProps: spec.SchemaProps{
								Description: "Subjects holds references to the objects the role applies to.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/rbac/v1.Subject"),
										},
									},
								},
							},
						},
						"roleRef": {
							SchemaProps: spec.SchemaProps{
								Description: "RoleRef can reference a Role in the current namespace or a ClusterRole in the global namespace. If the RoleRef cannot be resolved, the Authorizer must return an error.",
								Ref:         ref("k8s.io/api/rbac/v1.RoleRef"),
							},
						},
					},
					Required: []string{"subjects", "roleRef"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/rbac/v1.RoleRef", "k8s.io/api/rbac/v1.Subject", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/rbac/v1.RoleBindingList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "RoleBindingList is a collection of RoleBindings",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "Items is a list of RoleBindings",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/rbac/v1.RoleBinding"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/rbac/v1.RoleBinding", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/rbac/v1.RoleList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "RoleList is a collection of Roles",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "Items is a list of Roles",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/rbac/v1.Role"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/rbac/v1.Role", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/rbac/v1.RoleRef": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "RoleRef contains information that points to the role being used",
					Properties: map[string]spec.Schema{
						"apiGroup": {
							SchemaProps: spec.SchemaProps{
								Description: "APIGroup is the group for the resource being referenced",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is the type of resource being referenced",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "Name is the name of resource being referenced",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"apiGroup", "kind", "name"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/rbac/v1.Subject": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Subject contains a reference to the object or user identities a role binding applies to.  This can either hold a direct API object reference, or a value for non-objects such as user and group names.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind of object being referenced. Values defined by this API group are \"User\", \"Group\", and \"ServiceAccount\". If the Authorizer does not recognized the kind value, the Authorizer should report an error.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiGroup": {
							SchemaProps: spec.SchemaProps{
								Description: "APIGroup holds the API group of the referenced subject. Defaults to \"\" for ServiceAccount subjects. Defaults to \"rbac.authorization.k8s.io\" for User and Group subjects.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "Name of the object being referenced.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"namespace": {
							SchemaProps: spec.SchemaProps{
								Description: "Namespace of the referenced object.  If the object kind is non-namespace, such as \"User\" or \"Group\", and this value is not empty the Authorizer should report an error.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"kind", "name"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/rbac/v1alpha1.AggregationRule": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "AggregationRule describes how to locate ClusterRoles to aggregate into the ClusterRole",
					Properties: map[string]spec.Schema{
						"clusterRoleSelectors": {
							SchemaProps: spec.SchemaProps{
								Description: "ClusterRoleSelectors holds a list of selectors which will be used to find ClusterRoles and create the rules. If any of the selectors match, then the ClusterRole's permissions will be added",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"),
										},
									},
								},
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"},
		},
		"k8s.io/api/rbac/v1alpha1.ClusterRole": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ClusterRole is a cluster level, logical grouping of PolicyRules that can be referenced as a unit by a RoleBinding or ClusterRoleBinding.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"rules": {
							SchemaProps: spec.SchemaProps{
								Description: "Rules holds all the PolicyRules for this ClusterRole",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/rbac/v1alpha1.PolicyRule"),
										},
									},
								},
							},
						},
						"aggregationRule": {
							SchemaProps: spec.SchemaProps{
								Description: "AggregationRule is an optional field that describes how to build the Rules for this ClusterRole. If AggregationRule is set, then the Rules are controller managed and direct changes to Rules will be stomped by the controller.",
								Ref:         ref("k8s.io/api/rbac/v1alpha1.AggregationRule"),
							},
						},
					},
					Required: []string{"rules"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/rbac/v1alpha1.AggregationRule", "k8s.io/api/rbac/v1alpha1.PolicyRule", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/rbac/v1alpha1.ClusterRoleBinding": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ClusterRoleBinding references a ClusterRole, but not contain it.  It can reference a ClusterRole in the global namespace, and adds who information via Subject.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"subjects": {
							SchemaProps: spec.SchemaProps{
								Description: "Subjects holds references to the objects the role applies to.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/rbac/v1alpha1.Subject"),
										},
									},
								},
							},
						},
						"roleRef": {
							SchemaProps: spec.SchemaProps{
								Description: "RoleRef can only reference a ClusterRole in the global namespace. If the RoleRef cannot be resolved, the Authorizer must return an error.",
								Ref:         ref("k8s.io/api/rbac/v1alpha1.RoleRef"),
							},
						},
					},
					Required: []string{"subjects", "roleRef"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/rbac/v1alpha1.RoleRef", "k8s.io/api/rbac/v1alpha1.Subject", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/rbac/v1alpha1.ClusterRoleBindingList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ClusterRoleBindingList is a collection of ClusterRoleBindings",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "Items is a list of ClusterRoleBindings",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/rbac/v1alpha1.ClusterRoleBinding"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/rbac/v1alpha1.ClusterRoleBinding", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/rbac/v1alpha1.ClusterRoleList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ClusterRoleList is a collection of ClusterRoles",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "Items is a list of ClusterRoles",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/rbac/v1alpha1.ClusterRole"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/rbac/v1alpha1.ClusterRole", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/rbac/v1alpha1.PolicyRule": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PolicyRule holds information that describes a policy rule, but does not contain information about who the rule applies to or which namespace the rule applies to.",
					Properties: map[string]spec.Schema{
						"verbs": {
							SchemaProps: spec.SchemaProps{
								Description: "Verbs is a list of Verbs that apply to ALL the ResourceKinds and AttributeRestrictions contained in this rule.  VerbAll represents all kinds.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"apiGroups": {
							SchemaProps: spec.SchemaProps{
								Description: "APIGroups is the name of the APIGroup that contains the resources.  If multiple API groups are specified, any action requested against one of the enumerated resources in any API group will be allowed.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"resources": {
							SchemaProps: spec.SchemaProps{
								Description: "Resources is a list of resources this rule applies to.  ResourceAll represents all resources.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"resourceNames": {
							SchemaProps: spec.SchemaProps{
								Description: "ResourceNames is an optional white list of names that the rule applies to.  An empty set means that everything is allowed.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"nonResourceURLs": {
							SchemaProps: spec.SchemaProps{
								Description: "NonResourceURLs is a set of partial urls that a user should have access to.  *s are allowed, but only as the full, final step in the path This name is intentionally different than the internal type so that the DefaultConvert works nicely and because the ordering may be different. Since non-resource URLs are not namespaced, this field is only applicable for ClusterRoles referenced from a ClusterRoleBinding. Rules can either apply to API resources (such as \"pods\" or \"secrets\") or non-resource URL paths (such as \"/api\"),  but not both.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
					},
					Required: []string{"verbs"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/rbac/v1alpha1.Role": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Role is a namespaced, logical grouping of PolicyRules that can be referenced as a unit by a RoleBinding.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"rules": {
							SchemaProps: spec.SchemaProps{
								Description: "Rules holds all the PolicyRules for this Role",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/rbac/v1alpha1.PolicyRule"),
										},
									},
								},
							},
						},
					},
					Required: []string{"rules"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/rbac/v1alpha1.PolicyRule", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/rbac/v1alpha1.RoleBinding": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "RoleBinding references a role, but does not contain it.  It can reference a Role in the same namespace or a ClusterRole in the global namespace. It adds who information via Subjects and namespace information by which namespace it exists in.  RoleBindings in a given namespace only have effect in that namespace.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"subjects": {
							SchemaProps: spec.SchemaProps{
								Description: "Subjects holds references to the objects the role applies to.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/rbac/v1alpha1.Subject"),
										},
									},
								},
							},
						},
						"roleRef": {
							SchemaProps: spec.SchemaProps{
								Description: "RoleRef can reference a Role in the current namespace or a ClusterRole in the global namespace. If the RoleRef cannot be resolved, the Authorizer must return an error.",
								Ref:         ref("k8s.io/api/rbac/v1alpha1.RoleRef"),
							},
						},
					},
					Required: []string{"subjects", "roleRef"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/rbac/v1alpha1.RoleRef", "k8s.io/api/rbac/v1alpha1.Subject", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/rbac/v1alpha1.RoleBindingList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "RoleBindingList is a collection of RoleBindings",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "Items is a list of RoleBindings",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/rbac/v1alpha1.RoleBinding"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/rbac/v1alpha1.RoleBinding", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/rbac/v1alpha1.RoleList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "RoleList is a collection of Roles",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "Items is a list of Roles",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/rbac/v1alpha1.Role"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/rbac/v1alpha1.Role", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/rbac/v1alpha1.RoleRef": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "RoleRef contains information that points to the role being used",
					Properties: map[string]spec.Schema{
						"apiGroup": {
							SchemaProps: spec.SchemaProps{
								Description: "APIGroup is the group for the resource being referenced",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is the type of resource being referenced",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "Name is the name of resource being referenced",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"apiGroup", "kind", "name"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/rbac/v1alpha1.Subject": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Subject contains a reference to the object or user identities a role binding applies to.  This can either hold a direct API object reference, or a value for non-objects such as user and group names.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind of object being referenced. Values defined by this API group are \"User\", \"Group\", and \"ServiceAccount\". If the Authorizer does not recognized the kind value, the Authorizer should report an error.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion holds the API group and version of the referenced subject. Defaults to \"v1\" for ServiceAccount subjects. Defaults to \"rbac.authorization.k8s.io/v1alpha1\" for User and Group subjects.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "Name of the object being referenced.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"namespace": {
							SchemaProps: spec.SchemaProps{
								Description: "Namespace of the referenced object.  If the object kind is non-namespace, such as \"User\" or \"Group\", and this value is not empty the Authorizer should report an error.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"kind", "name"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/rbac/v1beta1.AggregationRule": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "AggregationRule describes how to locate ClusterRoles to aggregate into the ClusterRole",
					Properties: map[string]spec.Schema{
						"clusterRoleSelectors": {
							SchemaProps: spec.SchemaProps{
								Description: "ClusterRoleSelectors holds a list of selectors which will be used to find ClusterRoles and create the rules. If any of the selectors match, then the ClusterRole's permissions will be added",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"),
										},
									},
								},
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"},
		},
		"k8s.io/api/rbac/v1beta1.ClusterRole": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ClusterRole is a cluster level, logical grouping of PolicyRules that can be referenced as a unit by a RoleBinding or ClusterRoleBinding.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"rules": {
							SchemaProps: spec.SchemaProps{
								Description: "Rules holds all the PolicyRules for this ClusterRole",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/rbac/v1beta1.PolicyRule"),
										},
									},
								},
							},
						},
						"aggregationRule": {
							SchemaProps: spec.SchemaProps{
								Description: "AggregationRule is an optional field that describes how to build the Rules for this ClusterRole. If AggregationRule is set, then the Rules are controller managed and direct changes to Rules will be stomped by the controller.",
								Ref:         ref("k8s.io/api/rbac/v1beta1.AggregationRule"),
							},
						},
					},
					Required: []string{"rules"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/rbac/v1beta1.AggregationRule", "k8s.io/api/rbac/v1beta1.PolicyRule", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/rbac/v1beta1.ClusterRoleBinding": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ClusterRoleBinding references a ClusterRole, but not contain it.  It can reference a ClusterRole in the global namespace, and adds who information via Subject.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"subjects": {
							SchemaProps: spec.SchemaProps{
								Description: "Subjects holds references to the objects the role applies to.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/rbac/v1beta1.Subject"),
										},
									},
								},
							},
						},
						"roleRef": {
							SchemaProps: spec.SchemaProps{
								Description: "RoleRef can only reference a ClusterRole in the global namespace. If the RoleRef cannot be resolved, the Authorizer must return an error.",
								Ref:         ref("k8s.io/api/rbac/v1beta1.RoleRef"),
							},
						},
					},
					Required: []string{"subjects", "roleRef"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/rbac/v1beta1.RoleRef", "k8s.io/api/rbac/v1beta1.Subject", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/rbac/v1beta1.ClusterRoleBindingList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ClusterRoleBindingList is a collection of ClusterRoleBindings",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "Items is a list of ClusterRoleBindings",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/rbac/v1beta1.ClusterRoleBinding"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/rbac/v1beta1.ClusterRoleBinding", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/rbac/v1beta1.ClusterRoleList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ClusterRoleList is a collection of ClusterRoles",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "Items is a list of ClusterRoles",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/rbac/v1beta1.ClusterRole"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/rbac/v1beta1.ClusterRole", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/rbac/v1beta1.PolicyRule": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PolicyRule holds information that describes a policy rule, but does not contain information about who the rule applies to or which namespace the rule applies to.",
					Properties: map[string]spec.Schema{
						"verbs": {
							SchemaProps: spec.SchemaProps{
								Description: "Verbs is a list of Verbs that apply to ALL the ResourceKinds and AttributeRestrictions contained in this rule.  VerbAll represents all kinds.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"apiGroups": {
							SchemaProps: spec.SchemaProps{
								Description: "APIGroups is the name of the APIGroup that contains the resources.  If multiple API groups are specified, any action requested against one of the enumerated resources in any API group will be allowed.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"resources": {
							SchemaProps: spec.SchemaProps{
								Description: "Resources is a list of resources this rule applies to.  '*' represents all resources in the specified apiGroups. '*/foo' represents the subresource 'foo' for all resources in the specified apiGroups.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"resourceNames": {
							SchemaProps: spec.SchemaProps{
								Description: "ResourceNames is an optional white list of names that the rule applies to.  An empty set means that everything is allowed.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"nonResourceURLs": {
							SchemaProps: spec.SchemaProps{
								Description: "NonResourceURLs is a set of partial urls that a user should have access to.  *s are allowed, but only as the full, final step in the path Since non-resource URLs are not namespaced, this field is only applicable for ClusterRoles referenced from a ClusterRoleBinding. Rules can either apply to API resources (such as \"pods\" or \"secrets\") or non-resource URL paths (such as \"/api\"),  but not both.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
					},
					Required: []string{"verbs"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/rbac/v1beta1.Role": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Role is a namespaced, logical grouping of PolicyRules that can be referenced as a unit by a RoleBinding.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"rules": {
							SchemaProps: spec.SchemaProps{
								Description: "Rules holds all the PolicyRules for this Role",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/rbac/v1beta1.PolicyRule"),
										},
									},
								},
							},
						},
					},
					Required: []string{"rules"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/rbac/v1beta1.PolicyRule", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/rbac/v1beta1.RoleBinding": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "RoleBinding references a role, but does not contain it.  It can reference a Role in the same namespace or a ClusterRole in the global namespace. It adds who information via Subjects and namespace information by which namespace it exists in.  RoleBindings in a given namespace only have effect in that namespace.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"subjects": {
							SchemaProps: spec.SchemaProps{
								Description: "Subjects holds references to the objects the role applies to.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/rbac/v1beta1.Subject"),
										},
									},
								},
							},
						},
						"roleRef": {
							SchemaProps: spec.SchemaProps{
								Description: "RoleRef can reference a Role in the current namespace or a ClusterRole in the global namespace. If the RoleRef cannot be resolved, the Authorizer must return an error.",
								Ref:         ref("k8s.io/api/rbac/v1beta1.RoleRef"),
							},
						},
					},
					Required: []string{"subjects", "roleRef"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/rbac/v1beta1.RoleRef", "k8s.io/api/rbac/v1beta1.Subject", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/rbac/v1beta1.RoleBindingList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "RoleBindingList is a collection of RoleBindings",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "Items is a list of RoleBindings",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/rbac/v1beta1.RoleBinding"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/rbac/v1beta1.RoleBinding", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/rbac/v1beta1.RoleList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "RoleList is a collection of Roles",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "Items is a list of Roles",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/rbac/v1beta1.Role"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/rbac/v1beta1.Role", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/rbac/v1beta1.RoleRef": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "RoleRef contains information that points to the role being used",
					Properties: map[string]spec.Schema{
						"apiGroup": {
							SchemaProps: spec.SchemaProps{
								Description: "APIGroup is the group for the resource being referenced",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is the type of resource being referenced",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "Name is the name of resource being referenced",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"apiGroup", "kind", "name"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/rbac/v1beta1.Subject": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Subject contains a reference to the object or user identities a role binding applies to.  This can either hold a direct API object reference, or a value for non-objects such as user and group names.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind of object being referenced. Values defined by this API group are \"User\", \"Group\", and \"ServiceAccount\". If the Authorizer does not recognized the kind value, the Authorizer should report an error.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiGroup": {
							SchemaProps: spec.SchemaProps{
								Description: "APIGroup holds the API group of the referenced subject. Defaults to \"\" for ServiceAccount subjects. Defaults to \"rbac.authorization.k8s.io\" for User and Group subjects.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "Name of the object being referenced.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"namespace": {
							SchemaProps: spec.SchemaProps{
								Description: "Namespace of the referenced object.  If the object kind is non-namespace, such as \"User\" or \"Group\", and this value is not empty the Authorizer should report an error.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"kind", "name"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/scheduling/v1alpha1.PriorityClass": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PriorityClass defines mapping from a priority class name to the priority integer value. The value can be any valid integer.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata. More info: http://releases.k8s.io/HEAD/docs/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"value": {
							SchemaProps: spec.SchemaProps{
								Description: "The value of this priority class. This is the actual priority that pods receive when they have the name of this class in their pod spec.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"globalDefault": {
							SchemaProps: spec.SchemaProps{
								Description: "globalDefault specifies whether this PriorityClass should be considered as the default priority for pods that do not have any priority class.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"description": {
							SchemaProps: spec.SchemaProps{
								Description: "description is an arbitrary string that usually provides guidelines on when this priority class should be used.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"value"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/scheduling/v1alpha1.PriorityClassList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PriorityClassList is a collection of priority classes.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard list metadata More info: http://releases.k8s.io/HEAD/docs/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "items is the list of PriorityClasses",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/scheduling/v1alpha1.PriorityClass"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/scheduling/v1alpha1.PriorityClass", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/settings/v1alpha1.PodPreset": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PodPreset is a policy resource that defines additional runtime requirements for a Pod.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/api/settings/v1alpha1.PodPresetSpec"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/settings/v1alpha1.PodPresetSpec", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/settings/v1alpha1.PodPresetList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PodPresetList is a list of PodPreset objects.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "Items is a list of schema objects.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/settings/v1alpha1.PodPreset"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/settings/v1alpha1.PodPreset", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/settings/v1alpha1.PodPresetSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PodPresetSpec is a description of a pod preset.",
					Properties: map[string]spec.Schema{
						"selector": {
							SchemaProps: spec.SchemaProps{
								Description: "Selector is a label query over a set of resources, in this case pods. Required.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"),
							},
						},
						"env": {
							SchemaProps: spec.SchemaProps{
								Description: "Env defines the collection of EnvVar to inject into containers.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.EnvVar"),
										},
									},
								},
							},
						},
						"envFrom": {
							SchemaProps: spec.SchemaProps{
								Description: "EnvFrom defines the collection of EnvFromSource to inject into containers.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.EnvFromSource"),
										},
									},
								},
							},
						},
						"volumes": {
							SchemaProps: spec.SchemaProps{
								Description: "Volumes defines the collection of Volume to inject into the pod.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.Volume"),
										},
									},
								},
							},
						},
						"volumeMounts": {
							SchemaProps: spec.SchemaProps{
								Description: "VolumeMounts defines the collection of VolumeMount to inject into containers.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/core/v1.VolumeMount"),
										},
									},
								},
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.EnvFromSource", "k8s.io/api/core/v1.EnvVar", "k8s.io/api/core/v1.Volume", "k8s.io/api/core/v1.VolumeMount", "k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector"},
		},
		"k8s.io/api/storage/v1.StorageClass": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "StorageClass describes the parameters for a class of storage for which PersistentVolumes can be dynamically provisioned.\n\nStorageClasses are non-namespaced; the name of the storage class according to etcd is in ObjectMeta.Name.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"provisioner": {
							SchemaProps: spec.SchemaProps{
								Description: "Provisioner indicates the type of the provisioner.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"parameters": {
							SchemaProps: spec.SchemaProps{
								Description: "Parameters holds the parameters for the provisioner that should create volumes of this storage class.",
								Type:        []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"reclaimPolicy": {
							SchemaProps: spec.SchemaProps{
								Description: "Dynamically provisioned PersistentVolumes of this storage class are created with this reclaimPolicy. Defaults to Delete.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"mountOptions": {
							SchemaProps: spec.SchemaProps{
								Description: "Dynamically provisioned PersistentVolumes of this storage class are created with these mountOptions, e.g. [\"ro\", \"soft\"]. Not validated - mount of the PVs will simply fail if one is invalid.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"allowVolumeExpansion": {
							SchemaProps: spec.SchemaProps{
								Description: "AllowVolumeExpansion shows whether the storage class allow volume expand",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"volumeBindingMode": {
							SchemaProps: spec.SchemaProps{
								Description: "VolumeBindingMode indicates how PersistentVolumeClaims should be provisioned and bound.  When unset, VolumeBindingImmediate is used. This field is alpha-level and is only honored by servers that enable the VolumeScheduling feature.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"provisioner"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/storage/v1.StorageClassList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "StorageClassList is a collection of storage classes.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard list metadata More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "Items is the list of StorageClasses",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/storage/v1.StorageClass"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/storage/v1.StorageClass", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/storage/v1alpha1.VolumeAttachment": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "VolumeAttachment captures the intent to attach or detach the specified volume to/from the specified node.\n\nVolumeAttachment objects are non-namespaced.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "Specification of the desired attach/detach volume behavior. Populated by the Kubernetes system.",
								Ref:         ref("k8s.io/api/storage/v1alpha1.VolumeAttachmentSpec"),
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Status of the VolumeAttachment request. Populated by the entity completing the attach or detach operation, i.e. the external-attacher.",
								Ref:         ref("k8s.io/api/storage/v1alpha1.VolumeAttachmentStatus"),
							},
						},
					},
					Required: []string{"spec"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/storage/v1alpha1.VolumeAttachmentSpec", "k8s.io/api/storage/v1alpha1.VolumeAttachmentStatus", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/storage/v1alpha1.VolumeAttachmentList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "VolumeAttachmentList is a collection of VolumeAttachment objects.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard list metadata More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "Items is the list of VolumeAttachments",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/storage/v1alpha1.VolumeAttachment"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/storage/v1alpha1.VolumeAttachment", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/api/storage/v1alpha1.VolumeAttachmentSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "VolumeAttachmentSource represents a volume that should be attached. Right now only PersistenVolumes can be attached via external attacher, in future we may allow also inline volumes in pods. Exactly one member can be set.",
					Properties: map[string]spec.Schema{
						"persistentVolumeName": {
							SchemaProps: spec.SchemaProps{
								Description: "Name of the persistent volume to attach.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/api/storage/v1alpha1.VolumeAttachmentSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "VolumeAttachmentSpec is the specification of a VolumeAttachment request.",
					Properties: map[string]spec.Schema{
						"attacher": {
							SchemaProps: spec.SchemaProps{
								Description: "Attacher indicates the name of the volume driver that MUST handle this request. This is the name returned by GetPluginName().",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"source": {
							SchemaProps: spec.SchemaProps{
								Description: "Source represents the volume that should be attached.",
								Ref:         ref("k8s.io/api/storage/v1alpha1.VolumeAttachmentSource"),
							},
						},
						"nodeName": {
							SchemaProps: spec.SchemaProps{
								Description: "The node that the volume should be attached to.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"attacher", "source", "nodeName"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/storage/v1alpha1.VolumeAttachmentSource"},
		},
		"k8s.io/api/storage/v1alpha1.VolumeAttachmentStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "VolumeAttachmentStatus is the status of a VolumeAttachment request.",
					Properties: map[string]spec.Schema{
						"attached": {
							SchemaProps: spec.SchemaProps{
								Description: "Indicates the volume is successfully attached. This field must only be set by the entity completing the attach operation, i.e. the external-attacher.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"attachmentMetadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Upon successful attach, this field is populated with any information returned by the attach operation that must be passed into subsequent WaitForAttach or Mount calls. This field must only be set by the entity completing the attach operation, i.e. the external-attacher.",
								Type:        []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"attachError": {
							SchemaProps: spec.SchemaProps{
								Description: "The last error encountered during attach operation, if any. This field must only be set by the entity completing the attach operation, i.e. the external-attacher.",
								Ref:         ref("k8s.io/api/storage/v1alpha1.VolumeError"),
							},
						},
						"detachError": {
							SchemaProps: spec.SchemaProps{
								Description: "The last error encountered during detach operation, if any. This field must only be set by the entity completing the detach operation, i.e. the external-attacher.",
								Ref:         ref("k8s.io/api/storage/v1alpha1.VolumeError"),
							},
						},
					},
					Required: []string{"attached"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/storage/v1alpha1.VolumeError"},
		},
		"k8s.io/api/storage/v1alpha1.VolumeError": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "VolumeError captures an error encountered during a volume operation.",
					Properties: map[string]spec.Schema{
						"time": {
							SchemaProps: spec.SchemaProps{
								Description: "Time the error was encountered.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"message": {
							SchemaProps: spec.SchemaProps{
								Description: "String detailing the error encountered during Attach or Detach operation. This string maybe logged, so it should not contain sensitive information.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.Time"},
		},
		"k8s.io/api/storage/v1beta1.StorageClass": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "StorageClass describes the parameters for a class of storage for which PersistentVolumes can be dynamically provisioned.\n\nStorageClasses are non-namespaced; the name of the storage class according to etcd is in ObjectMeta.Name.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"provisioner": {
							SchemaProps: spec.SchemaProps{
								Description: "Provisioner indicates the type of the provisioner.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"parameters": {
							SchemaProps: spec.SchemaProps{
								Description: "Parameters holds the parameters for the provisioner that should create volumes of this storage class.",
								Type:        []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"reclaimPolicy": {
							SchemaProps: spec.SchemaProps{
								Description: "Dynamically provisioned PersistentVolumes of this storage class are created with this reclaimPolicy. Defaults to Delete.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"mountOptions": {
							SchemaProps: spec.SchemaProps{
								Description: "Dynamically provisioned PersistentVolumes of this storage class are created with these mountOptions, e.g. [\"ro\", \"soft\"]. Not validated - mount of the PVs will simply fail if one is invalid.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"allowVolumeExpansion": {
							SchemaProps: spec.SchemaProps{
								Description: "AllowVolumeExpansion shows whether the storage class allow volume expand",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"volumeBindingMode": {
							SchemaProps: spec.SchemaProps{
								Description: "VolumeBindingMode indicates how PersistentVolumeClaims should be provisioned and bound.  When unset, VolumeBindingImmediate is used. This field is alpha-level and is only honored by servers that enable the VolumeScheduling feature.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"provisioner"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/api/storage/v1beta1.StorageClassList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "StorageClassList is a collection of storage classes.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard list metadata More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "Items is the list of StorageClasses",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/api/storage/v1beta1.StorageClass"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/storage/v1beta1.StorageClass", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.CustomResourceDefinition": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "CustomResourceDefinition represents a resource that should be exposed on the API server.  Its name MUST be in the format <.spec.name>.<.spec.group>.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "Spec describes how the user wants the resources to appear",
								Ref:         ref("k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.CustomResourceDefinitionSpec"),
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Status indicates the actual state of the CustomResourceDefinition",
								Ref:         ref("k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.CustomResourceDefinitionStatus"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.CustomResourceDefinitionSpec", "k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.CustomResourceDefinitionStatus", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.CustomResourceDefinitionCondition": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "CustomResourceDefinitionCondition contains details for the current condition of this pod.",
					Properties: map[string]spec.Schema{
						"type": {
							SchemaProps: spec.SchemaProps{
								Description: "Type is the type of the condition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Status is the status of the condition. Can be True, False, Unknown.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"lastTransitionTime": {
							SchemaProps: spec.SchemaProps{
								Description: "Last time the condition transitioned from one status to another.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"reason": {
							SchemaProps: spec.SchemaProps{
								Description: "Unique, one-word, CamelCase reason for the condition's last transition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"message": {
							SchemaProps: spec.SchemaProps{
								Description: "Human-readable message indicating details about last transition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"type", "status"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.Time"},
		},
		"k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.CustomResourceDefinitionList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "CustomResourceDefinitionList is a list of CustomResourceDefinition objects.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "Items individual CustomResourceDefinitions",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.CustomResourceDefinition"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.CustomResourceDefinition", "k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"},
		},
		"k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.CustomResourceDefinitionNames": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "CustomResourceDefinitionNames indicates the names to serve this CustomResourceDefinition",
					Properties: map[string]spec.Schema{
						"plural": {
							SchemaProps: spec.SchemaProps{
								Description: "Plural is the plural name of the resource to serve.  It must match the name of the CustomResourceDefinition-registration too: plural.group and it must be all lowercase.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"singular": {
							SchemaProps: spec.SchemaProps{
								Description: "Singular is the singular name of the resource.  It must be all lowercase  Defaults to lowercased <kind>",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"shortNames": {
							SchemaProps: spec.SchemaProps{
								Description: "ShortNames are short names for the resource.  It must be all lowercase.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is the serialized kind of the resource.  It is normally CamelCase and singular.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"listKind": {
							SchemaProps: spec.SchemaProps{
								Description: "ListKind is the serialized kind of the list for this resource.  Defaults to <kind>List.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"plural", "kind"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.CustomResourceDefinitionSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "CustomResourceDefinitionSpec describes how a user wants their resource to appear",
					Properties: map[string]spec.Schema{
						"group": {
							SchemaProps: spec.SchemaProps{
								Description: "Group is the group this resource belongs in",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"version": {
							SchemaProps: spec.SchemaProps{
								Description: "Version is the version this resource belongs in",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"names": {
							SchemaProps: spec.SchemaProps{
								Description: "Names are the names used to describe this custom resource",
								Ref:         ref("k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.CustomResourceDefinitionNames"),
							},
						},
						"scope": {
							SchemaProps: spec.SchemaProps{
								Description: "Scope indicates whether this resource is cluster or namespace scoped.  Default is namespaced",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"validation": {
							SchemaProps: spec.SchemaProps{
								Description: "Validation describes the validation methods for CustomResources",
								Ref:         ref("k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.CustomResourceValidation"),
							},
						},
					},
					Required: []string{"group", "version", "names", "scope"},
				},
			},
			Dependencies: []string{
				"k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.CustomResourceDefinitionNames", "k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.CustomResourceValidation"},
		},
		"k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.CustomResourceDefinitionStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "CustomResourceDefinitionStatus indicates the state of the CustomResourceDefinition",
					Properties: map[string]spec.Schema{
						"conditions": {
							SchemaProps: spec.SchemaProps{
								Description: "Conditions indicate state for particular aspects of a CustomResourceDefinition",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.CustomResourceDefinitionCondition"),
										},
									},
								},
							},
						},
						"acceptedNames": {
							SchemaProps: spec.SchemaProps{
								Description: "AcceptedNames are the names that are actually being used to serve discovery They may be different than the names in spec.",
								Ref:         ref("k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.CustomResourceDefinitionNames"),
							},
						},
					},
					Required: []string{"conditions", "acceptedNames"},
				},
			},
			Dependencies: []string{
				"k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.CustomResourceDefinitionCondition", "k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.CustomResourceDefinitionNames"},
		},
		"k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.CustomResourceValidation": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "CustomResourceValidation is a list of validation methods for CustomResources.",
					Properties: map[string]spec.Schema{
						"openAPIV3Schema": {
							SchemaProps: spec.SchemaProps{
								Description: "OpenAPIV3Schema is the OpenAPI v3 schema to be validated against.",
								Ref:         ref("k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.JSONSchemaProps"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.JSONSchemaProps"},
		},
		"k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.ExternalDocumentation": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ExternalDocumentation allows referencing an external resource for extended documentation.",
					Properties: map[string]spec.Schema{
						"description": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
						"url": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.JSON": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "JSON represents any valid JSON value. These types are supported: bool, int64, float64, string, []interface{}, map[string]interface{} and nil.",
					Properties: map[string]spec.Schema{
						"Raw": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "byte",
							},
						},
					},
					Required: []string{"Raw"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.JSONSchemaProps": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "JSONSchemaProps is a JSON-Schema following Specification Draft 4 (http://json-schema.org/).",
					Properties: map[string]spec.Schema{
						"id": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
						"$schema": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
						"$ref": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
						"description": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
						"type": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
						"format": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
						"title": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
						"default": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.JSON"),
							},
						},
						"maximum": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"number"},
								Format: "double",
							},
						},
						"exclusiveMaximum": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"boolean"},
								Format: "",
							},
						},
						"minimum": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"number"},
								Format: "double",
							},
						},
						"exclusiveMinimum": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"boolean"},
								Format: "",
							},
						},
						"maxLength": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"integer"},
								Format: "int64",
							},
						},
						"minLength": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"integer"},
								Format: "int64",
							},
						},
						"pattern": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
						"maxItems": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"integer"},
								Format: "int64",
							},
						},
						"minItems": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"integer"},
								Format: "int64",
							},
						},
						"uniqueItems": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"boolean"},
								Format: "",
							},
						},
						"multipleOf": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"number"},
								Format: "double",
							},
						},
						"enum": {
							SchemaProps: spec.SchemaProps{
								Type: []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.JSON"),
										},
									},
								},
							},
						},
						"maxProperties": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"integer"},
								Format: "int64",
							},
						},
						"minProperties": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"integer"},
								Format: "int64",
							},
						},
						"required": {
							SchemaProps: spec.SchemaProps{
								Type: []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.JSONSchemaPropsOrArray"),
							},
						},
						"allOf": {
							SchemaProps: spec.SchemaProps{
								Type: []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.JSONSchemaProps"),
										},
									},
								},
							},
						},
						"oneOf": {
							SchemaProps: spec.SchemaProps{
								Type: []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.JSONSchemaProps"),
										},
									},
								},
							},
						},
						"anyOf": {
							SchemaProps: spec.SchemaProps{
								Type: []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.JSONSchemaProps"),
										},
									},
								},
							},
						},
						"not": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.JSONSchemaProps"),
							},
						},
						"properties": {
							SchemaProps: spec.SchemaProps{
								Type: []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.JSONSchemaProps"),
										},
									},
								},
							},
						},
						"additionalProperties": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.JSONSchemaPropsOrBool"),
							},
						},
						"patternProperties": {
							SchemaProps: spec.SchemaProps{
								Type: []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.JSONSchemaProps"),
										},
									},
								},
							},
						},
						"dependencies": {
							SchemaProps: spec.SchemaProps{
								Type: []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.JSONSchemaPropsOrStringArray"),
										},
									},
								},
							},
						},
						"additionalItems": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.JSONSchemaPropsOrBool"),
							},
						},
						"definitions": {
							SchemaProps: spec.SchemaProps{
								Type: []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.JSONSchemaProps"),
										},
									},
								},
							},
						},
						"externalDocs": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.ExternalDocumentation"),
							},
						},
						"example": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.JSON"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.ExternalDocumentation", "k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.JSON", "k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.JSONSchemaProps", "k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.JSONSchemaPropsOrArray", "k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.JSONSchemaPropsOrBool", "k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.JSONSchemaPropsOrStringArray"},
		},
		"k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.JSONSchemaPropsOrArray": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "JSONSchemaPropsOrArray represents a value that can either be a JSONSchemaProps or an array of JSONSchemaProps. Mainly here for serialization purposes.",
					Properties: map[string]spec.Schema{
						"Schema": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.JSONSchemaProps"),
							},
						},
						"JSONSchemas": {
							SchemaProps: spec.SchemaProps{
								Type: []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.JSONSchemaProps"),
										},
									},
								},
							},
						},
					},
					Required: []string{"Schema", "JSONSchemas"},
				},
			},
			Dependencies: []string{
				"k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.JSONSchemaProps"},
		},
		"k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.JSONSchemaPropsOrBool": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "JSONSchemaPropsOrBool represents JSONSchemaProps or a boolean value. Defaults to true for the boolean property.",
					Properties: map[string]spec.Schema{
						"Allows": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"boolean"},
								Format: "",
							},
						},
						"Schema": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.JSONSchemaProps"),
							},
						},
					},
					Required: []string{"Allows", "Schema"},
				},
			},
			Dependencies: []string{
				"k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.JSONSchemaProps"},
		},
		"k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.JSONSchemaPropsOrStringArray": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "JSONSchemaPropsOrStringArray represents a JSONSchemaProps or a string array.",
					Properties: map[string]spec.Schema{
						"Schema": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.JSONSchemaProps"),
							},
						},
						"Property": {
							SchemaProps: spec.SchemaProps{
								Type: []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
					},
					Required: []string{"Schema", "Property"},
				},
			},
			Dependencies: []string{
				"k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1.JSONSchemaProps"},
		},
		"k8s.io/apimachinery/pkg/api/resource.Quantity": resource.Quantity{}.OpenAPIDefinition(),
		"k8s.io/apimachinery/pkg/api/resource.int64Amount": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "int64Amount represents a fixed precision numerator and arbitrary scale exponent. It is faster than operations on inf.Dec for values that can be represented as int64.",
					Properties: map[string]spec.Schema{
						"value": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"integer"},
								Format: "int64",
							},
						},
						"scale": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"integer"},
								Format: "int32",
							},
						},
					},
					Required: []string{"value", "scale"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/apimachinery/pkg/apis/meta/v1.APIGroup": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "APIGroup contains the name, the supported versions, and the preferred version of a group.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "name is the name of the group.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"versions": {
							SchemaProps: spec.SchemaProps{
								Description: "versions are the versions supported in this group.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.GroupVersionForDiscovery"),
										},
									},
								},
							},
						},
						"preferredVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "preferredVersion is the version preferred by the API server, which probably is the storage version.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.GroupVersionForDiscovery"),
							},
						},
						"serverAddressByClientCIDRs": {
							SchemaProps: spec.SchemaProps{
								Description: "a map of client CIDR to server address that is serving this group. This is to help clients reach servers in the most network-efficient way possible. Clients can use the appropriate server address as per the CIDR that they match. In case of multiple matches, clients should use the longest matching CIDR. The server returns only those CIDRs that it thinks that the client can match. For example: the master will return an internal IP CIDR only, if the client reaches the server using an internal IP. Server looks at X-Forwarded-For header or X-Real-Ip header or request.RemoteAddr (in that order) to get the client IP.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.ServerAddressByClientCIDR"),
										},
									},
								},
							},
						},
					},
					Required: []string{"name", "versions", "serverAddressByClientCIDRs"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.GroupVersionForDiscovery", "k8s.io/apimachinery/pkg/apis/meta/v1.ServerAddressByClientCIDR"},
		},
		"k8s.io/apimachinery/pkg/apis/meta/v1.APIGroupList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "APIGroupList is a list of APIGroup, to allow clients to discover the API at /apis.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"groups": {
							SchemaProps: spec.SchemaProps{
								Description: "groups is a list of APIGroup.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.APIGroup"),
										},
									},
								},
							},
						},
					},
					Required: []string{"groups"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.APIGroup"},
		},
		"k8s.io/apimachinery/pkg/apis/meta/v1.APIResource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "APIResource specifies the name of a resource and whether it is namespaced.",
					Properties: map[string]spec.Schema{
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "name is the plural name of the resource.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"singularName": {
							SchemaProps: spec.SchemaProps{
								Description: "singularName is the singular name of the resource.  This allows clients to handle plural and singular opaquely. The singularName is more correct for reporting status on a single item and both singular and plural are allowed from the kubectl CLI interface.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"namespaced": {
							SchemaProps: spec.SchemaProps{
								Description: "namespaced indicates if a resource is namespaced or not.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"group": {
							SchemaProps: spec.SchemaProps{
								Description: "group is the preferred group of the resource.  Empty implies the group of the containing resource list. For subresources, this may have a different value, for example: Scale\".",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"version": {
							SchemaProps: spec.SchemaProps{
								Description: "version is the preferred version of the resource.  Empty implies the version of the containing resource list For subresources, this may have a different value, for example: v1 (while inside a v1beta1 version of the core resource's group)\".",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "kind is the kind for the resource (e.g. 'Foo' is the kind for a resource 'foo')",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"verbs": {
							SchemaProps: spec.SchemaProps{
								Description: "verbs is a list of supported kube verbs (this includes get, list, watch, create, update, patch, delete, deletecollection, and proxy)",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"shortNames": {
							SchemaProps: spec.SchemaProps{
								Description: "shortNames is a list of suggested short names of the resource.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"categories": {
							SchemaProps: spec.SchemaProps{
								Description: "categories is a list of the grouped resources this resource belongs to (e.g. 'all')",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
					},
					Required: []string{"name", "singularName", "namespaced", "kind", "verbs"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/apimachinery/pkg/apis/meta/v1.APIResourceList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "APIResourceList is a list of APIResource, it is used to expose the name of the resources supported in a specific group and version, and if the resource is namespaced.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"groupVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "groupVersion is the group and version this APIResourceList is for.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"resources": {
							SchemaProps: spec.SchemaProps{
								Description: "resources contains the name of the resources and if they are namespaced.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.APIResource"),
										},
									},
								},
							},
						},
					},
					Required: []string{"groupVersion", "resources"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.APIResource"},
		},
		"k8s.io/apimachinery/pkg/apis/meta/v1.APIVersions": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "APIVersions lists the versions that are available, to allow clients to discover the API at /api, which is the root path of the legacy v1 API.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"versions": {
							SchemaProps: spec.SchemaProps{
								Description: "versions are the api versions that are available.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"serverAddressByClientCIDRs": {
							SchemaProps: spec.SchemaProps{
								Description: "a map of client CIDR to server address that is serving this group. This is to help clients reach servers in the most network-efficient way possible. Clients can use the appropriate server address as per the CIDR that they match. In case of multiple matches, clients should use the longest matching CIDR. The server returns only those CIDRs that it thinks that the client can match. For example: the master will return an internal IP CIDR only, if the client reaches the server using an internal IP. Server looks at X-Forwarded-For header or X-Real-Ip header or request.RemoteAddr (in that order) to get the client IP.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.ServerAddressByClientCIDR"),
										},
									},
								},
							},
						},
					},
					Required: []string{"versions", "serverAddressByClientCIDRs"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.ServerAddressByClientCIDR"},
		},
		"k8s.io/apimachinery/pkg/apis/meta/v1.DeleteOptions": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "DeleteOptions may be provided when deleting an API object.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"gracePeriodSeconds": {
							SchemaProps: spec.SchemaProps{
								Description: "The duration in seconds before the object should be deleted. Value must be non-negative integer. The value zero indicates delete immediately. If this value is nil, the default grace period for the specified type will be used. Defaults to a per object value if not specified. zero means delete immediately.",
								Type:        []string{"integer"},
								Format:      "int64",
							},
						},
						"preconditions": {
							SchemaProps: spec.SchemaProps{
								Description: "Must be fulfilled before a deletion is carried out. If not possible, a 409 Conflict status will be returned.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Preconditions"),
							},
						},
						"orphanDependents": {
							SchemaProps: spec.SchemaProps{
								Description: "Deprecated: please use the PropagationPolicy, this field will be deprecated in 1.7. Should the dependent objects be orphaned. If true/false, the \"orphan\" finalizer will be added to/removed from the object's finalizers list. Either this field or PropagationPolicy may be set, but not both.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"propagationPolicy": {
							SchemaProps: spec.SchemaProps{
								Description: "Whether and how garbage collection will be performed. Either this field or OrphanDependents may be set, but not both. The default policy is decided by the existing finalizer set in the metadata.finalizers and the resource-specific default policy. Acceptable values are: 'Orphan' - orphan the dependents; 'Background' - allow the garbage collector to delete the dependents in the background; 'Foreground' - a cascading policy that deletes all dependents in the foreground.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.Preconditions"},
		},
		"k8s.io/apimachinery/pkg/apis/meta/v1.Duration": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Duration is a wrapper around time.Duration which supports correct marshaling to YAML and JSON. In particular, it marshals into strings, which can be used as map keys in json.",
					Properties: map[string]spec.Schema{
						"Duration": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"integer"},
								Format: "int64",
							},
						},
					},
					Required: []string{"Duration"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/apimachinery/pkg/apis/meta/v1.ExportOptions": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ExportOptions is the query options to the standard REST get call.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"export": {
							SchemaProps: spec.SchemaProps{
								Description: "Should this value be exported.  Export strips fields that a user can not specify.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"exact": {
							SchemaProps: spec.SchemaProps{
								Description: "Should the export be exact.  Exact export maintains cluster-specific fields like 'Namespace'.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
					},
					Required: []string{"export", "exact"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/apimachinery/pkg/apis/meta/v1.GetOptions": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "GetOptions is the standard query options to the standard REST get call.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"resourceVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "When specified: - if unset, then the result is returned from remote storage based on quorum-read flag; - if it's 0, then we simply return what we currently have in cache, no guarantee; - if set to non zero, then the result is at least as fresh as given rv.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"includeUninitialized": {
							SchemaProps: spec.SchemaProps{
								Description: "If true, partially initialized resources are included in the response.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/apimachinery/pkg/apis/meta/v1.GroupKind": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "GroupKind specifies a Group and a Kind, but does not force a version.  This is useful for identifying concepts during lookup stages without having partially valid types",
					Properties: map[string]spec.Schema{
						"group": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
						"kind": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
					},
					Required: []string{"group", "kind"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/apimachinery/pkg/apis/meta/v1.GroupResource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "GroupResource specifies a Group and a Resource, but does not force a version.  This is useful for identifying concepts during lookup stages without having partially valid types",
					Properties: map[string]spec.Schema{
						"group": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
						"resource": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
					},
					Required: []string{"group", "resource"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/apimachinery/pkg/apis/meta/v1.GroupVersion": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "GroupVersion contains the \"group\" and the \"version\", which uniquely identifies the API.",
					Properties: map[string]spec.Schema{
						"group": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
						"version": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
					},
					Required: []string{"group", "version"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/apimachinery/pkg/apis/meta/v1.GroupVersionForDiscovery": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "GroupVersion contains the \"group/version\" and \"version\" string of a version. It is made a struct to keep extensibility.",
					Properties: map[string]spec.Schema{
						"groupVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "groupVersion specifies the API group and version in the form \"group/version\"",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"version": {
							SchemaProps: spec.SchemaProps{
								Description: "version specifies the version in the form of \"version\". This is to save the clients the trouble of splitting the GroupVersion.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"groupVersion", "version"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/apimachinery/pkg/apis/meta/v1.GroupVersionKind": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "GroupVersionKind unambiguously identifies a kind.  It doesn't anonymously include GroupVersion to avoid automatic coersion.  It doesn't use a GroupVersion to avoid custom marshalling",
					Properties: map[string]spec.Schema{
						"group": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
						"version": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
						"kind": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
					},
					Required: []string{"group", "version", "kind"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/apimachinery/pkg/apis/meta/v1.GroupVersionResource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "GroupVersionResource unambiguously identifies a resource.  It doesn't anonymously include GroupVersion to avoid automatic coersion.  It doesn't use a GroupVersion to avoid custom marshalling",
					Properties: map[string]spec.Schema{
						"group": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
						"version": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
						"resource": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
					},
					Required: []string{"group", "version", "resource"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/apimachinery/pkg/apis/meta/v1.Initializer": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Initializer is information about an initializer that has not yet completed.",
					Properties: map[string]spec.Schema{
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "name of the process that is responsible for initializing this object.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"name"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/apimachinery/pkg/apis/meta/v1.Initializers": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Initializers tracks the progress of initialization.",
					Properties: map[string]spec.Schema{
						"pending": {
							VendorExtensible: spec.VendorExtensible{
								Extensions: spec.Extensions{
									"x-kubernetes-patch-merge-key": "name",
									"x-kubernetes-patch-strategy":  "merge",
								},
							},
							SchemaProps: spec.SchemaProps{
								Description: "Pending is a list of initializers that must execute in order before this object is visible. When the last pending initializer is removed, and no failing result is set, the initializers struct will be set to nil and the object is considered as initialized and visible to all clients.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.Initializer"),
										},
									},
								},
							},
						},
						"result": {
							SchemaProps: spec.SchemaProps{
								Description: "If result is set with the Failure field, the object will be persisted to storage and then deleted, ensuring that other clients can observe the deletion.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Status"),
							},
						},
					},
					Required: []string{"pending"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.Initializer", "k8s.io/apimachinery/pkg/apis/meta/v1.Status"},
		},
		"k8s.io/apimachinery/pkg/apis/meta/v1.InternalEvent": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "InternalEvent makes watch.Event versioned",
					Properties: map[string]spec.Schema{
						"Type": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
						"Object": {
							SchemaProps: spec.SchemaProps{
								Description: "Object is:\n * If Type is Added or Modified: the new state of the object.\n * If Type is Deleted: the state of the object immediately before deletion.\n * If Type is Error: *api.Status is recommended; other types may make sense\n   depending on context.",
								Ref:         ref("k8s.io/apimachinery/pkg/runtime.Object"),
							},
						},
					},
					Required: []string{"Type", "Object"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/runtime.Object"},
		},
		"k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelector": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "A label selector is a label query over a set of resources. The result of matchLabels and matchExpressions are ANDed. An empty label selector matches all objects. A null label selector matches no objects.",
					Properties: map[string]spec.Schema{
						"matchLabels": {
							SchemaProps: spec.SchemaProps{
								Description: "matchLabels is a map of {key,value} pairs. A single {key,value} in the matchLabels map is equivalent to an element of matchExpressions, whose key field is \"key\", the operator is \"In\", and the values array contains only \"value\". The requirements are ANDed.",
								Type:        []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"matchExpressions": {
							SchemaProps: spec.SchemaProps{
								Description: "matchExpressions is a list of label selector requirements. The requirements are ANDed.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelectorRequirement"),
										},
									},
								},
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelectorRequirement"},
		},
		"k8s.io/apimachinery/pkg/apis/meta/v1.LabelSelectorRequirement": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "A label selector requirement is a selector that contains values, a key, and an operator that relates the key and values.",
					Properties: map[string]spec.Schema{
						"key": {
							VendorExtensible: spec.VendorExtensible{
								Extensions: spec.Extensions{
									"x-kubernetes-patch-merge-key": "key",
									"x-kubernetes-patch-strategy":  "merge",
								},
							},
							SchemaProps: spec.SchemaProps{
								Description: "key is the label key that the selector applies to.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"operator": {
							SchemaProps: spec.SchemaProps{
								Description: "operator represents a key's relationship to a set of values. Valid operators are In, NotIn, Exists and DoesNotExist.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"values": {
							SchemaProps: spec.SchemaProps{
								Description: "values is an array of string values. If the operator is In or NotIn, the values array must be non-empty. If the operator is Exists or DoesNotExist, the values array must be empty. This array is replaced during a strategic merge patch.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
					},
					Required: []string{"key", "operator"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/apimachinery/pkg/apis/meta/v1.List": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "List holds a list of objects, which may not be known by the server.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "List of objects",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apimachinery/pkg/runtime.RawExtension"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta", "k8s.io/apimachinery/pkg/runtime.RawExtension"},
		},
		"k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ListMeta describes metadata that synthetic resources must have, including lists and various status objects. A resource may have only one of {ObjectMeta, ListMeta}.",
					Properties: map[string]spec.Schema{
						"selfLink": {
							SchemaProps: spec.SchemaProps{
								Description: "selfLink is a URL representing this object. Populated by the system. Read-only.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"resourceVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "String that identifies the server's internal version of this object that can be used by clients to determine when objects have changed. Value must be treated as opaque by clients and passed unmodified back to the server. Populated by the system. Read-only. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#concurrency-control-and-consistency",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"continue": {
							SchemaProps: spec.SchemaProps{
								Description: "continue may be set if the user set a limit on the number of items returned, and indicates that the server has more data available. The value is opaque and may be used to issue another request to the endpoint that served this list to retrieve the next set of available objects. Continuing a list may not be possible if the server configuration has changed or more than a few minutes have passed. The resourceVersion field returned when using this continue value will be identical to the value in the first response.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/apimachinery/pkg/apis/meta/v1.ListOptions": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ListOptions is the query options to a standard REST list call.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"labelSelector": {
							SchemaProps: spec.SchemaProps{
								Description: "A selector to restrict the list of returned objects by their labels. Defaults to everything.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"fieldSelector": {
							SchemaProps: spec.SchemaProps{
								Description: "A selector to restrict the list of returned objects by their fields. Defaults to everything.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"includeUninitialized": {
							SchemaProps: spec.SchemaProps{
								Description: "If true, partially initialized resources are included in the response.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"watch": {
							SchemaProps: spec.SchemaProps{
								Description: "Watch for changes to the described resources and return them as a stream of add, update, and remove notifications. Specify resourceVersion.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"resourceVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "When specified with a watch call, shows changes that occur after that particular version of a resource. Defaults to changes from the beginning of history. When specified for list: - if unset, then the result is returned from remote storage based on quorum-read flag; - if it's 0, then we simply return what we currently have in cache, no guarantee; - if set to non zero, then the result is at least as fresh as given rv.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"timeoutSeconds": {
							SchemaProps: spec.SchemaProps{
								Description: "Timeout for the list/watch call.",
								Type:        []string{"integer"},
								Format:      "int64",
							},
						},
						"limit": {
							SchemaProps: spec.SchemaProps{
								Description: "limit is a maximum number of responses to return for a list call. If more items exist, the server will set the `continue` field on the list metadata to a value that can be used with the same initial query to retrieve the next set of results. Setting a limit may return fewer than the requested amount of items (up to zero items) in the event all requested objects are filtered out and clients should only use the presence of the continue field to determine whether more results are available. Servers may choose not to support the limit argument and will return all of the available results. If limit is specified and the continue field is empty, clients may assume that no more results are available. This field is not supported if watch is true.\n\nThe server guarantees that the objects returned when using continue will be identical to issuing a single list call without a limit - that is, no objects created, modified, or deleted after the first request is issued will be included in any subsequent continued requests. This is sometimes referred to as a consistent snapshot, and ensures that a client that is using limit to receive smaller chunks of a very large result can ensure they see all possible objects. If objects are updated during a chunked list the version of the object that was present at the time the first list result was calculated is returned.",
								Type:        []string{"integer"},
								Format:      "int64",
							},
						},
						"continue": {
							SchemaProps: spec.SchemaProps{
								Description: "The continue option should be set when retrieving more results from the server. Since this value is server defined, clients may only use the continue value from a previous query result with identical query parameters (except for the value of continue) and the server may reject a continue value it does not recognize. If the specified continue value is no longer valid whether due to expiration (generally five to fifteen minutes) or a configuration change on the server the server will respond with a 410 ResourceExpired error indicating the client must restart their list without the continue field. This field is not supported when watch is true. Clients may start a watch from the last resourceVersion value returned by the server and not miss any modifications.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/apimachinery/pkg/apis/meta/v1.MicroTime": v1.MicroTime{}.OpenAPIDefinition(),
		"k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ObjectMeta is metadata that all persisted resources must have, which includes all objects users must create.",
					Properties: map[string]spec.Schema{
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "Name must be unique within a namespace. Is required when creating resources, although some resources may allow a client to request the generation of an appropriate name automatically. Name is primarily intended for creation idempotence and configuration definition. Cannot be updated. More info: http://kubernetes.io/docs/user-guide/identifiers#names",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"generateName": {
							SchemaProps: spec.SchemaProps{
								Description: "GenerateName is an optional prefix, used by the server, to generate a unique name ONLY IF the Name field has not been provided. If this field is used, the name returned to the client will be different than the name passed. This value will also be combined with a unique suffix. The provided value has the same validation rules as the Name field, and may be truncated by the length of the suffix required to make the value unique on the server.\n\nIf this field is specified and the generated name exists, the server will NOT return a 409 - instead, it will either return 201 Created or 500 with Reason ServerTimeout indicating a unique name could not be found in the time allotted, and the client should retry (optionally after the time indicated in the Retry-After header).\n\nApplied only if Name is not specified. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#idempotency",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"namespace": {
							SchemaProps: spec.SchemaProps{
								Description: "Namespace defines the space within each name must be unique. An empty namespace is equivalent to the \"default\" namespace, but \"default\" is the canonical representation. Not all objects are required to be scoped to a namespace - the value of this field for those objects will be empty.\n\nMust be a DNS_LABEL. Cannot be updated. More info: http://kubernetes.io/docs/user-guide/namespaces",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"selfLink": {
							SchemaProps: spec.SchemaProps{
								Description: "SelfLink is a URL representing this object. Populated by the system. Read-only.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"uid": {
							SchemaProps: spec.SchemaProps{
								Description: "UID is the unique in time and space value for this object. It is typically generated by the server on successful creation of a resource and is not allowed to change on PUT operations.\n\nPopulated by the system. Read-only. More info: http://kubernetes.io/docs/user-guide/identifiers#uids",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"resourceVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "An opaque value that represents the internal version of this object that can be used by clients to determine when objects have changed. May be used for optimistic concurrency, change detection, and the watch operation on a resource or set of resources. Clients must treat these values as opaque and passed unmodified back to the server. They may only be valid for a particular resource or set of resources.\n\nPopulated by the system. Read-only. Value must be treated as opaque by clients and . More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#concurrency-control-and-consistency",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"generation": {
							SchemaProps: spec.SchemaProps{
								Description: "A sequence number representing a specific generation of the desired state. Populated by the system. Read-only.",
								Type:        []string{"integer"},
								Format:      "int64",
							},
						},
						"creationTimestamp": {
							SchemaProps: spec.SchemaProps{
								Description: "CreationTimestamp is a timestamp representing the server time when this object was created. It is not guaranteed to be set in happens-before order across separate operations. Clients may not set this value. It is represented in RFC3339 form and is in UTC.\n\nPopulated by the system. Read-only. Null for lists. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"deletionTimestamp": {
							SchemaProps: spec.SchemaProps{
								Description: "DeletionTimestamp is RFC 3339 date and time at which this resource will be deleted. This field is set by the server when a graceful deletion is requested by the user, and is not directly settable by a client. The resource is expected to be deleted (no longer visible from resource lists, and not reachable by name) after the time in this field, once the finalizers list is empty. As long as the finalizers list contains items, deletion is blocked. Once the deletionTimestamp is set, this value may not be unset or be set further into the future, although it may be shortened or the resource may be deleted prior to this time. For example, a user may request that a pod is deleted in 30 seconds. The Kubelet will react by sending a graceful termination signal to the containers in the pod. After that 30 seconds, the Kubelet will send a hard termination signal (SIGKILL) to the container and after cleanup, remove the pod from the API. In the presence of network partitions, this object may still exist after this timestamp, until an administrator or automated process can determine the resource is fully terminated. If not set, graceful deletion of the object has not been requested.\n\nPopulated by the system when a graceful deletion is requested. Read-only. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"deletionGracePeriodSeconds": {
							SchemaProps: spec.SchemaProps{
								Description: "Number of seconds allowed for this object to gracefully terminate before it will be removed from the system. Only set when deletionTimestamp is also set. May only be shortened. Read-only.",
								Type:        []string{"integer"},
								Format:      "int64",
							},
						},
						"labels": {
							SchemaProps: spec.SchemaProps{
								Description: "Map of string keys and values that can be used to organize and categorize (scope and select) objects. May match selectors of replication controllers and services. More info: http://kubernetes.io/docs/user-guide/labels",
								Type:        []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"annotations": {
							SchemaProps: spec.SchemaProps{
								Description: "Annotations is an unstructured key value map stored with a resource that may be set by external tools to store and retrieve arbitrary metadata. They are not queryable and should be preserved when modifying objects. More info: http://kubernetes.io/docs/user-guide/annotations",
								Type:        []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"ownerReferences": {
							VendorExtensible: spec.VendorExtensible{
								Extensions: spec.Extensions{
									"x-kubernetes-patch-merge-key": "uid",
									"x-kubernetes-patch-strategy":  "merge",
								},
							},
							SchemaProps: spec.SchemaProps{
								Description: "List of objects depended by this object. If ALL objects in the list have been deleted, this object will be garbage collected. If this object is managed by a controller, then an entry in this list will point to this controller, with the controller field set to true. There cannot be more than one managing controller.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.OwnerReference"),
										},
									},
								},
							},
						},
						"initializers": {
							SchemaProps: spec.SchemaProps{
								Description: "An initializer is a controller which enforces some system invariant at object creation time. This field is a list of initializers that have not yet acted on this object. If nil or empty, this object has been completely initialized. Otherwise, the object is considered uninitialized and is hidden (in list/watch and get calls) from clients that haven't explicitly asked to observe uninitialized objects.\n\nWhen an object is created, the system will populate this list with the current set of initializers. Only privileged users may set or modify this list. Once it is empty, it may not be modified further by any user.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Initializers"),
							},
						},
						"finalizers": {
							VendorExtensible: spec.VendorExtensible{
								Extensions: spec.Extensions{
									"x-kubernetes-patch-strategy": "merge",
								},
							},
							SchemaProps: spec.SchemaProps{
								Description: "Must be empty before the object is deleted from the registry. Each entry is an identifier for the responsible component that will remove the entry from the list. If the deletionTimestamp of the object is non-nil, entries in this list can only be removed.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"clusterName": {
							SchemaProps: spec.SchemaProps{
								Description: "The name of the cluster which the object belongs to. This is used to distinguish resources with same name and namespace in different clusters. This field is not set anywhere right now and apiserver is going to ignore it if set in create or update request.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.Initializers", "k8s.io/apimachinery/pkg/apis/meta/v1.OwnerReference", "k8s.io/apimachinery/pkg/apis/meta/v1.Time"},
		},
		"k8s.io/apimachinery/pkg/apis/meta/v1.OwnerReference": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "OwnerReference contains enough information to let you identify an owning object. Currently, an owning object must be in the same namespace, so there is no namespace field.",
					Properties: map[string]spec.Schema{
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "API version of the referent.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind of the referent. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "Name of the referent. More info: http://kubernetes.io/docs/user-guide/identifiers#names",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"uid": {
							SchemaProps: spec.SchemaProps{
								Description: "UID of the referent. More info: http://kubernetes.io/docs/user-guide/identifiers#uids",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"controller": {
							SchemaProps: spec.SchemaProps{
								Description: "If true, this reference points to the managing controller.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"blockOwnerDeletion": {
							SchemaProps: spec.SchemaProps{
								Description: "If true, AND if the owner has the \"foregroundDeletion\" finalizer, then the owner cannot be deleted from the key-value store until this reference is removed. Defaults to false. To set this field, a user needs \"delete\" permission of the owner, otherwise 422 (Unprocessable Entity) will be returned.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
					},
					Required: []string{"apiVersion", "kind", "name", "uid"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/apimachinery/pkg/apis/meta/v1.Patch": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Patch is provided to give a concrete name and type to the Kubernetes PATCH request body.",
					Properties:  map[string]spec.Schema{},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/apimachinery/pkg/apis/meta/v1.Preconditions": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Preconditions must be fulfilled before an operation (update, delete, etc.) is carried out.",
					Properties: map[string]spec.Schema{
						"uid": {
							SchemaProps: spec.SchemaProps{
								Description: "Specifies the target UID.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/apimachinery/pkg/apis/meta/v1.RootPaths": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "RootPaths lists the paths available at root. For example: \"/healthz\", \"/apis\".",
					Properties: map[string]spec.Schema{
						"paths": {
							SchemaProps: spec.SchemaProps{
								Description: "paths are the paths available at root.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
					},
					Required: []string{"paths"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/apimachinery/pkg/apis/meta/v1.ServerAddressByClientCIDR": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ServerAddressByClientCIDR helps the client to determine the server address that they should use, depending on the clientCIDR that they match.",
					Properties: map[string]spec.Schema{
						"clientCIDR": {
							SchemaProps: spec.SchemaProps{
								Description: "The CIDR with which clients can match their IP to figure out the server address that they should use.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"serverAddress": {
							SchemaProps: spec.SchemaProps{
								Description: "Address of this server, suitable for a client that matches the above CIDR. This can be a hostname, hostname:port, IP or IP:port.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"clientCIDR", "serverAddress"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/apimachinery/pkg/apis/meta/v1.Status": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Status is a return value for calls that don't return other objects.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Status of the operation. One of: \"Success\" or \"Failure\". More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"message": {
							SchemaProps: spec.SchemaProps{
								Description: "A human-readable description of the status of this operation.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"reason": {
							SchemaProps: spec.SchemaProps{
								Description: "A machine-readable description of why this operation is in the \"Failure\" status. If this value is empty there is no information available. A Reason clarifies an HTTP status code but does not override it.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"details": {
							SchemaProps: spec.SchemaProps{
								Description: "Extended data associated with the reason.  Each reason may define its own extended details. This field is optional and the data returned is not guaranteed to conform to any schema except that defined by the reason type.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.StatusDetails"),
							},
						},
						"code": {
							SchemaProps: spec.SchemaProps{
								Description: "Suggested HTTP return code for this status, 0 if not set.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta", "k8s.io/apimachinery/pkg/apis/meta/v1.StatusDetails"},
		},
		"k8s.io/apimachinery/pkg/apis/meta/v1.StatusCause": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "StatusCause provides more information about an api.Status failure, including cases when multiple errors are encountered.",
					Properties: map[string]spec.Schema{
						"reason": {
							SchemaProps: spec.SchemaProps{
								Description: "A machine-readable description of the cause of the error. If this value is empty there is no information available.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"message": {
							SchemaProps: spec.SchemaProps{
								Description: "A human-readable description of the cause of the error.  This field may be presented as-is to a reader.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"field": {
							SchemaProps: spec.SchemaProps{
								Description: "The field of the resource that has caused this error, as named by its JSON serialization. May include dot and postfix notation for nested attributes. Arrays are zero-indexed.  Fields may appear more than once in an array of causes due to fields having multiple errors. Optional.\n\nExamples:\n  \"name\" - the field \"name\" on the current resource\n  \"items[0].name\" - the field \"name\" on the first array entry in \"items\"",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/apimachinery/pkg/apis/meta/v1.StatusDetails": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "StatusDetails is a set of additional properties that MAY be set by the server to provide additional information about a response. The Reason field of a Status object defines what attributes will be set. Clients must ignore fields that do not match the defined type of each attribute, and should assume that any attribute may be empty, invalid, or under defined.",
					Properties: map[string]spec.Schema{
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "The name attribute of the resource associated with the status StatusReason (when there is a single name which can be described).",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"group": {
							SchemaProps: spec.SchemaProps{
								Description: "The group attribute of the resource associated with the status StatusReason.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "The kind attribute of the resource associated with the status StatusReason. On some operations may differ from the requested resource Kind. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"uid": {
							SchemaProps: spec.SchemaProps{
								Description: "UID of the resource. (when there is a single resource which can be described). More info: http://kubernetes.io/docs/user-guide/identifiers#uids",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"causes": {
							SchemaProps: spec.SchemaProps{
								Description: "The Causes array includes more details associated with the StatusReason failure. Not all StatusReasons may provide detailed causes.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.StatusCause"),
										},
									},
								},
							},
						},
						"retryAfterSeconds": {
							SchemaProps: spec.SchemaProps{
								Description: "If specified, the time in seconds before the operation should be retried. Some errors may indicate the client must take an alternate action - for those errors this field may indicate how long to wait before taking the alternate action.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.StatusCause"},
		},
		"k8s.io/apimachinery/pkg/apis/meta/v1.Time": v1.Time{}.OpenAPIDefinition(),
		"k8s.io/apimachinery/pkg/apis/meta/v1.Timestamp": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Timestamp is a struct that is equivalent to Time, but intended for protobuf marshalling/unmarshalling. It is generated into a serialization that matches Time. Do not use in Go structs.",
					Properties: map[string]spec.Schema{
						"seconds": {
							SchemaProps: spec.SchemaProps{
								Description: "Represents seconds of UTC time since Unix epoch 1970-01-01T00:00:00Z. Must be from 0001-01-01T00:00:00Z to 9999-12-31T23:59:59Z inclusive.",
								Type:        []string{"integer"},
								Format:      "int64",
							},
						},
						"nanos": {
							SchemaProps: spec.SchemaProps{
								Description: "Non-negative fractions of a second at nanosecond resolution. Negative second values with fractions must still have non-negative nanos values that count forward in time. Must be from 0 to 999,999,999 inclusive. This field may be limited in precision depending on context.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
					},
					Required: []string{"seconds", "nanos"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/apimachinery/pkg/apis/meta/v1.TypeMeta": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "TypeMeta describes an individual object in an API response or request with strings representing the type of the object and its API schema version. Structures that are versioned or persisted should inline TypeMeta.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/apimachinery/pkg/apis/meta/v1.WatchEvent": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Event represents a single event to a watched resource.",
					Properties: map[string]spec.Schema{
						"type": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
						"object": {
							SchemaProps: spec.SchemaProps{
								Description: "Object is:\n * If Type is Added or Modified: the new state of the object.\n * If Type is Deleted: the state of the object immediately before deletion.\n * If Type is Error: *Status is recommended; other types may make sense\n   depending on context.",
								Ref:         ref("k8s.io/apimachinery/pkg/runtime.RawExtension"),
							},
						},
					},
					Required: []string{"type", "object"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/runtime.RawExtension"},
		},
		"k8s.io/apimachinery/pkg/apis/meta/v1alpha1.PartialObjectMetadata": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PartialObjectMetadata is a generic representation of any object with ObjectMeta. It allows clients to get access to a particular ObjectMeta schema without knowing the details of the version.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"},
		},
		"k8s.io/apimachinery/pkg/apis/meta/v1alpha1.PartialObjectMetadataList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PartialObjectMetadataList contains a list of objects containing only their metadata",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "items contains each of the included items.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1alpha1.PartialObjectMetadata"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1alpha1.PartialObjectMetadata"},
		},
		"k8s.io/apimachinery/pkg/apis/meta/v1alpha1.Table": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Table is a tabular representation of a set of API resources. The server transforms the object into a set of preferred columns for quickly reviewing the objects.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"columnDefinitions": {
							SchemaProps: spec.SchemaProps{
								Description: "columnDefinitions describes each column in the returned items array. The number of cells per row will always match the number of column definitions.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1alpha1.TableColumnDefinition"),
										},
									},
								},
							},
						},
						"rows": {
							SchemaProps: spec.SchemaProps{
								Description: "rows is the list of items in the table.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1alpha1.TableRow"),
										},
									},
								},
							},
						},
					},
					Required: []string{"columnDefinitions", "rows"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta", "k8s.io/apimachinery/pkg/apis/meta/v1alpha1.TableColumnDefinition", "k8s.io/apimachinery/pkg/apis/meta/v1alpha1.TableRow"},
		},
		"k8s.io/apimachinery/pkg/apis/meta/v1alpha1.TableColumnDefinition": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "TableColumnDefinition contains information about a column returned in the Table.",
					Properties: map[string]spec.Schema{
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "name is a human readable name for the column.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"type": {
							SchemaProps: spec.SchemaProps{
								Description: "type is an OpenAPI type definition for this column. See https://github.com/OAI/OpenAPI-Specification/blob/master/versions/2.0.md#data-types for more.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"format": {
							SchemaProps: spec.SchemaProps{
								Description: "format is an optional OpenAPI type definition for this column. The 'name' format is applied to the primary identifier column to assist in clients identifying column is the resource name. See https://github.com/OAI/OpenAPI-Specification/blob/master/versions/2.0.md#data-types for more.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"description": {
							SchemaProps: spec.SchemaProps{
								Description: "description is a human readable description of this column.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"priority": {
							SchemaProps: spec.SchemaProps{
								Description: "priority is an integer defining the relative importance of this column compared to others. Lower numbers are considered higher priority. Columns that may be omitted in limited space scenarios should be given a higher priority.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
					},
					Required: []string{"name", "type", "format", "description", "priority"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/apimachinery/pkg/apis/meta/v1alpha1.TableOptions": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "TableOptions are used when a Table is requested by the caller.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"includeObject": {
							SchemaProps: spec.SchemaProps{
								Description: "includeObject decides whether to include each object along with its columnar information. Specifying \"None\" will return no object, specifying \"Object\" will return the full object contents, and specifying \"Metadata\" (the default) will return the object's metadata in the PartialObjectMetadata kind in version v1alpha1 of the meta.k8s.io API group.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/apimachinery/pkg/apis/meta/v1alpha1.TableRow": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "TableRow is an individual row in a table.",
					Properties: map[string]spec.Schema{
						"cells": {
							SchemaProps: spec.SchemaProps{
								Description: "cells will be as wide as headers and may contain strings, numbers, booleans, simple maps, or lists, or null. See the type field of the column definition for a more detailed description.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"object"},
											Format: "",
										},
									},
								},
							},
						},
						"conditions": {
							SchemaProps: spec.SchemaProps{
								Description: "conditions describe additional status of a row that are relevant for a human user.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1alpha1.TableRowCondition"),
										},
									},
								},
							},
						},
						"object": {
							SchemaProps: spec.SchemaProps{
								Description: "This field contains the requested additional information about each object based on the includeObject policy when requesting the Table. If \"None\", this field is empty, if \"Object\" this will be the default serialization of the object for the current API version, and if \"Metadata\" (the default) will contain the object metadata. Check the returned kind and apiVersion of the object before parsing.",
								Ref:         ref("k8s.io/apimachinery/pkg/runtime.RawExtension"),
							},
						},
					},
					Required: []string{"cells"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1alpha1.TableRowCondition", "k8s.io/apimachinery/pkg/runtime.RawExtension"},
		},
		"k8s.io/apimachinery/pkg/apis/meta/v1alpha1.TableRowCondition": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "TableRowCondition allows a row to be marked with additional information.",
					Properties: map[string]spec.Schema{
						"type": {
							SchemaProps: spec.SchemaProps{
								Description: "Type of row condition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Status of the condition, one of True, False, Unknown.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"reason": {
							SchemaProps: spec.SchemaProps{
								Description: "(brief) machine readable reason for the condition's last transition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"message": {
							SchemaProps: spec.SchemaProps{
								Description: "Human readable message indicating details about last transition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"type", "status"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/apimachinery/pkg/runtime.RawExtension": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "RawExtension is used to hold extensions in external versions.\n\nTo use this, make a field which has RawExtension as its type in your external, versioned struct, and Object in your internal struct. You also need to register your various plugin types.\n\n// Internal package: type MyAPIObject struct {\n\truntime.TypeMeta `json:\",inline\"`\n\tMyPlugin runtime.Object `json:\"myPlugin\"`\n} type PluginA struct {\n\tAOption string `json:\"aOption\"`\n}\n\n// External package: type MyAPIObject struct {\n\truntime.TypeMeta `json:\",inline\"`\n\tMyPlugin runtime.RawExtension `json:\"myPlugin\"`\n} type PluginA struct {\n\tAOption string `json:\"aOption\"`\n}\n\n// On the wire, the JSON will look something like this: {\n\t\"kind\":\"MyAPIObject\",\n\t\"apiVersion\":\"v1\",\n\t\"myPlugin\": {\n\t\t\"kind\":\"PluginA\",\n\t\t\"aOption\":\"foo\",\n\t},\n}\n\nSo what happens? Decode first uses json or yaml to unmarshal the serialized data into your external MyAPIObject. That causes the raw JSON to be stored, but not unpacked. The next step is to copy (using pkg/conversion) into the internal struct. The runtime package's DefaultScheme has conversion functions installed which will unpack the JSON stored in RawExtension, turning it into the correct object type, and storing it in the Object. (TODO: In the case where the object is of an unknown type, a runtime.Unknown object will be created and stored.)",
					Properties: map[string]spec.Schema{
						"Raw": {
							SchemaProps: spec.SchemaProps{
								Description: "Raw is the underlying serialization of this object.",
								Type:        []string{"string"},
								Format:      "byte",
							},
						},
					},
					Required: []string{"Raw"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/apimachinery/pkg/runtime.TypeMeta": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "TypeMeta is shared by all top level objects. The proper way to use it is to inline it in your type, like this: type MyAwesomeAPIObject struct {\n     runtime.TypeMeta    `json:\",inline\"`\n     ... // other fields\n} func (obj *MyAwesomeAPIObject) SetGroupVersionKind(gvk *metav1.GroupVersionKind) { metav1.UpdateTypeMeta(obj,gvk) }; GroupVersionKind() *GroupVersionKind\n\nTypeMeta is provided here for convenience. You may use it directly from this package or define your own with the same fields.",
					Properties: map[string]spec.Schema{
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
						"kind": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/apimachinery/pkg/runtime.Unknown": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Unknown allows api objects with unknown types to be passed-through. This can be used to deal with the API objects from a plug-in. Unknown objects still have functioning TypeMeta features-- kind, version, etc. metadata and field mutatation.",
					Properties: map[string]spec.Schema{
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
						"kind": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
						"Raw": {
							SchemaProps: spec.SchemaProps{
								Description: "Raw will hold the complete serialized object which couldn't be matched with a registered type. Most likely, nothing should be done with this except for passing it through the system.",
								Type:        []string{"string"},
								Format:      "byte",
							},
						},
						"ContentEncoding": {
							SchemaProps: spec.SchemaProps{
								Description: "ContentEncoding is encoding used to encode 'Raw' data. Unspecified means no encoding.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"ContentType": {
							SchemaProps: spec.SchemaProps{
								Description: "ContentType  is serialization method used to serialize 'Raw'. Unspecified means ContentTypeJSON.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"Raw", "ContentEncoding", "ContentType"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/apimachinery/pkg/util/intstr.IntOrString": intstr.IntOrString{}.OpenAPIDefinition(),
		"k8s.io/apimachinery/pkg/version.Info": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Info contains versioning information. how we'll want to distribute that information.",
					Properties: map[string]spec.Schema{
						"major": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
						"minor": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
						"gitVersion": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
						"gitCommit": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
						"gitTreeState": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
						"buildDate": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
						"goVersion": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
						"compiler": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
						"platform": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
					},
					Required: []string{"major", "minor", "gitVersion", "gitCommit", "gitTreeState", "buildDate", "goVersion", "compiler", "platform"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/apiserver/pkg/apis/audit/v1alpha1.Event": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Event captures all the information that can be included in an API audit log.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "ObjectMeta is included for interoperability with API infrastructure.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"level": {
							SchemaProps: spec.SchemaProps{
								Description: "AuditLevel at which event was generated",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"timestamp": {
							SchemaProps: spec.SchemaProps{
								Description: "Time the request reached the apiserver.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"auditID": {
							SchemaProps: spec.SchemaProps{
								Description: "Unique audit ID, generated for each request.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"stage": {
							SchemaProps: spec.SchemaProps{
								Description: "Stage of the request handling when this event instance was generated.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"requestURI": {
							SchemaProps: spec.SchemaProps{
								Description: "RequestURI is the request URI as sent by the client to a server.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"verb": {
							SchemaProps: spec.SchemaProps{
								Description: "Verb is the kubernetes verb associated with the request. For non-resource requests, this is the lower-cased HTTP method.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"user": {
							SchemaProps: spec.SchemaProps{
								Description: "Authenticated user information.",
								Ref:         ref("k8s.io/api/authentication/v1.UserInfo"),
							},
						},
						"impersonatedUser": {
							SchemaProps: spec.SchemaProps{
								Description: "Impersonated user information.",
								Ref:         ref("k8s.io/api/authentication/v1.UserInfo"),
							},
						},
						"sourceIPs": {
							SchemaProps: spec.SchemaProps{
								Description: "Source IPs, from where the request originated and intermediate proxies.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"objectRef": {
							SchemaProps: spec.SchemaProps{
								Description: "Object reference this request is targeted at. Does not apply for List-type requests, or non-resource requests.",
								Ref:         ref("k8s.io/apiserver/pkg/apis/audit/v1alpha1.ObjectReference"),
							},
						},
						"responseStatus": {
							SchemaProps: spec.SchemaProps{
								Description: "The response status, populated even when the ResponseObject is not a Status type. For successful responses, this will only include the Code and StatusSuccess. For non-status type error responses, this will be auto-populated with the error Message.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Status"),
							},
						},
						"requestObject": {
							SchemaProps: spec.SchemaProps{
								Description: "API object from the request, in JSON format. The RequestObject is recorded as-is in the request (possibly re-encoded as JSON), prior to version conversion, defaulting, admission or merging. It is an external versioned object type, and may not be a valid object on its own. Omitted for non-resource requests.  Only logged at Request Level and higher.",
								Ref:         ref("k8s.io/apimachinery/pkg/runtime.Unknown"),
							},
						},
						"responseObject": {
							SchemaProps: spec.SchemaProps{
								Description: "API object returned in the response, in JSON. The ResponseObject is recorded after conversion to the external type, and serialized as JSON.  Omitted for non-resource requests.  Only logged at Response Level.",
								Ref:         ref("k8s.io/apimachinery/pkg/runtime.Unknown"),
							},
						},
						"requestReceivedTimestamp": {
							SchemaProps: spec.SchemaProps{
								Description: "Time the request reached the apiserver.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.MicroTime"),
							},
						},
						"stageTimestamp": {
							SchemaProps: spec.SchemaProps{
								Description: "Time the request reached current audit stage.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.MicroTime"),
							},
						},
					},
					Required: []string{"level", "timestamp", "auditID", "stage", "requestURI", "verb", "user"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/authentication/v1.UserInfo", "k8s.io/apimachinery/pkg/apis/meta/v1.MicroTime", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta", "k8s.io/apimachinery/pkg/apis/meta/v1.Status", "k8s.io/apimachinery/pkg/apis/meta/v1.Time", "k8s.io/apimachinery/pkg/runtime.Unknown", "k8s.io/apiserver/pkg/apis/audit/v1alpha1.ObjectReference"},
		},
		"k8s.io/apiserver/pkg/apis/audit/v1alpha1.EventList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "EventList is a list of audit Events.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Type: []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apiserver/pkg/apis/audit/v1alpha1.Event"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta", "k8s.io/apiserver/pkg/apis/audit/v1alpha1.Event"},
		},
		"k8s.io/apiserver/pkg/apis/audit/v1alpha1.GroupResources": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "GroupResources represents resource kinds in an API group.",
					Properties: map[string]spec.Schema{
						"group": {
							SchemaProps: spec.SchemaProps{
								Description: "Group is the name of the API group that contains the resources. The empty string represents the core API group.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"resources": {
							SchemaProps: spec.SchemaProps{
								Description: "Resources is a list of resources within the API group. Subresources are matched using a \"/\" to indicate the subresource. For example, \"pods/logs\" would match request to the logs subresource of pods. The top level resource does not match subresources, \"pods\" doesn't match \"pods/logs\".",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"resourceNames": {
							SchemaProps: spec.SchemaProps{
								Description: "ResourceNames is a list of resource instance names that the policy matches. Using this field requires Resources to be specified. An empty list implies that every instance of the resource is matched.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/apiserver/pkg/apis/audit/v1alpha1.ObjectReference": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ObjectReference contains enough information to let you inspect or modify the referred object.",
					Properties: map[string]spec.Schema{
						"resource": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
						"namespace": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
						"name": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
						"uid": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
						"resourceVersion": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
						"subresource": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/apiserver/pkg/apis/audit/v1alpha1.Policy": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Policy defines the configuration of audit logging, and the rules for how different request categories are logged.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "ObjectMeta is included for interoperability with API infrastructure.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"rules": {
							SchemaProps: spec.SchemaProps{
								Description: "Rules specify the audit Level a request should be recorded at. A request may match multiple rules, in which case the FIRST matching rule is used. The default audit level is None, but can be overridden by a catch-all rule at the end of the list. PolicyRules are strictly ordered.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apiserver/pkg/apis/audit/v1alpha1.PolicyRule"),
										},
									},
								},
							},
						},
						"omitStages": {
							SchemaProps: spec.SchemaProps{
								Description: "OmitStages is a list of stages for which no events are created. Note that this can also be specified per rule in which case the union of both are omitted.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
					},
					Required: []string{"rules"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta", "k8s.io/apiserver/pkg/apis/audit/v1alpha1.PolicyRule"},
		},
		"k8s.io/apiserver/pkg/apis/audit/v1alpha1.PolicyList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PolicyList is a list of audit Policies.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Type: []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apiserver/pkg/apis/audit/v1alpha1.Policy"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta", "k8s.io/apiserver/pkg/apis/audit/v1alpha1.Policy"},
		},
		"k8s.io/apiserver/pkg/apis/audit/v1alpha1.PolicyRule": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PolicyRule maps requests based off metadata to an audit Level. Requests must match the rules of every field (an intersection of rules).",
					Properties: map[string]spec.Schema{
						"level": {
							SchemaProps: spec.SchemaProps{
								Description: "The Level that requests matching this rule are recorded at.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"users": {
							SchemaProps: spec.SchemaProps{
								Description: "The users (by authenticated user name) this rule applies to. An empty list implies every user.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"userGroups": {
							SchemaProps: spec.SchemaProps{
								Description: "The user groups this rule applies to. A user is considered matching if it is a member of any of the UserGroups. An empty list implies every user group.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"verbs": {
							SchemaProps: spec.SchemaProps{
								Description: "The verbs that match this rule. An empty list implies every verb.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"resources": {
							SchemaProps: spec.SchemaProps{
								Description: "Resources that this rule matches. An empty list implies all kinds in all API groups.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apiserver/pkg/apis/audit/v1alpha1.GroupResources"),
										},
									},
								},
							},
						},
						"namespaces": {
							SchemaProps: spec.SchemaProps{
								Description: "Namespaces that this rule matches. The empty string \"\" matches non-namespaced resources. An empty list implies every namespace.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"nonResourceURLs": {
							SchemaProps: spec.SchemaProps{
								Description: "NonResourceURLs is a set of URL paths that should be audited. *s are allowed, but only as the full, final step in the path. Examples:\n \"/metrics\" - Log requests for apiserver metrics\n \"/healthz*\" - Log all health checks",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"omitStages": {
							SchemaProps: spec.SchemaProps{
								Description: "OmitStages is a list of stages for which no events are created. Note that this can also be specified policy wide in which case the union of both are omitted. An empty list means no restrictions will apply.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
					},
					Required: []string{"level"},
				},
			},
			Dependencies: []string{
				"k8s.io/apiserver/pkg/apis/audit/v1alpha1.GroupResources"},
		},
		"k8s.io/apiserver/pkg/apis/audit/v1beta1.Event": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Event captures all the information that can be included in an API audit log.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "ObjectMeta is included for interoperability with API infrastructure. DEPRECATED: Use StageTimestamp which supports micro second instead of ObjectMeta.CreateTimestamp and the rest of the object is not used",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"level": {
							SchemaProps: spec.SchemaProps{
								Description: "AuditLevel at which event was generated",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"timestamp": {
							SchemaProps: spec.SchemaProps{
								Description: "Time the request reached the apiserver. DEPRECATED: Use RequestReceivedTimestamp which supports micro second instead.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"auditID": {
							SchemaProps: spec.SchemaProps{
								Description: "Unique audit ID, generated for each request.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"stage": {
							SchemaProps: spec.SchemaProps{
								Description: "Stage of the request handling when this event instance was generated.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"requestURI": {
							SchemaProps: spec.SchemaProps{
								Description: "RequestURI is the request URI as sent by the client to a server.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"verb": {
							SchemaProps: spec.SchemaProps{
								Description: "Verb is the kubernetes verb associated with the request. For non-resource requests, this is the lower-cased HTTP method.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"user": {
							SchemaProps: spec.SchemaProps{
								Description: "Authenticated user information.",
								Ref:         ref("k8s.io/api/authentication/v1.UserInfo"),
							},
						},
						"impersonatedUser": {
							SchemaProps: spec.SchemaProps{
								Description: "Impersonated user information.",
								Ref:         ref("k8s.io/api/authentication/v1.UserInfo"),
							},
						},
						"sourceIPs": {
							SchemaProps: spec.SchemaProps{
								Description: "Source IPs, from where the request originated and intermediate proxies.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"objectRef": {
							SchemaProps: spec.SchemaProps{
								Description: "Object reference this request is targeted at. Does not apply for List-type requests, or non-resource requests.",
								Ref:         ref("k8s.io/apiserver/pkg/apis/audit/v1beta1.ObjectReference"),
							},
						},
						"responseStatus": {
							SchemaProps: spec.SchemaProps{
								Description: "The response status, populated even when the ResponseObject is not a Status type. For successful responses, this will only include the Code and StatusSuccess. For non-status type error responses, this will be auto-populated with the error Message.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Status"),
							},
						},
						"requestObject": {
							SchemaProps: spec.SchemaProps{
								Description: "API object from the request, in JSON format. The RequestObject is recorded as-is in the request (possibly re-encoded as JSON), prior to version conversion, defaulting, admission or merging. It is an external versioned object type, and may not be a valid object on its own. Omitted for non-resource requests.  Only logged at Request Level and higher.",
								Ref:         ref("k8s.io/apimachinery/pkg/runtime.Unknown"),
							},
						},
						"responseObject": {
							SchemaProps: spec.SchemaProps{
								Description: "API object returned in the response, in JSON. The ResponseObject is recorded after conversion to the external type, and serialized as JSON.  Omitted for non-resource requests.  Only logged at Response Level.",
								Ref:         ref("k8s.io/apimachinery/pkg/runtime.Unknown"),
							},
						},
						"requestReceivedTimestamp": {
							SchemaProps: spec.SchemaProps{
								Description: "Time the request reached the apiserver.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.MicroTime"),
							},
						},
						"stageTimestamp": {
							SchemaProps: spec.SchemaProps{
								Description: "Time the request reached current audit stage.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.MicroTime"),
							},
						},
					},
					Required: []string{"level", "timestamp", "auditID", "stage", "requestURI", "verb", "user"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/authentication/v1.UserInfo", "k8s.io/apimachinery/pkg/apis/meta/v1.MicroTime", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta", "k8s.io/apimachinery/pkg/apis/meta/v1.Status", "k8s.io/apimachinery/pkg/apis/meta/v1.Time", "k8s.io/apimachinery/pkg/runtime.Unknown", "k8s.io/apiserver/pkg/apis/audit/v1beta1.ObjectReference"},
		},
		"k8s.io/apiserver/pkg/apis/audit/v1beta1.EventList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "EventList is a list of audit Events.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Type: []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apiserver/pkg/apis/audit/v1beta1.Event"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta", "k8s.io/apiserver/pkg/apis/audit/v1beta1.Event"},
		},
		"k8s.io/apiserver/pkg/apis/audit/v1beta1.GroupResources": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "GroupResources represents resource kinds in an API group.",
					Properties: map[string]spec.Schema{
						"group": {
							SchemaProps: spec.SchemaProps{
								Description: "Group is the name of the API group that contains the resources. The empty string represents the core API group.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"resources": {
							SchemaProps: spec.SchemaProps{
								Description: "Resources is a list of resources within the API group. Subresources are matched using a \"/\" to indicate the subresource. For example, \"pods/log\" would match request to the log subresource of pods. The top level resource does not match subresources, \"pods\" doesn't match \"pods/log\".",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"resourceNames": {
							SchemaProps: spec.SchemaProps{
								Description: "ResourceNames is a list of resource instance names that the policy matches. Using this field requires Resources to be specified. An empty list implies that every instance of the resource is matched.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/apiserver/pkg/apis/audit/v1beta1.ObjectReference": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ObjectReference contains enough information to let you inspect or modify the referred object.",
					Properties: map[string]spec.Schema{
						"resource": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
						"namespace": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
						"name": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
						"uid": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
						"apiGroup": {
							SchemaProps: spec.SchemaProps{
								Description: "APIGroup is the name of the API group that contains the referred object. The empty string represents the core API group.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion is the version of the API group that contains the referred object.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"resourceVersion": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
						"subresource": {
							SchemaProps: spec.SchemaProps{
								Type:   []string{"string"},
								Format: "",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/apiserver/pkg/apis/audit/v1beta1.Policy": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Policy defines the configuration of audit logging, and the rules for how different request categories are logged.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "ObjectMeta is included for interoperability with API infrastructure.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"rules": {
							SchemaProps: spec.SchemaProps{
								Description: "Rules specify the audit Level a request should be recorded at. A request may match multiple rules, in which case the FIRST matching rule is used. The default audit level is None, but can be overridden by a catch-all rule at the end of the list. PolicyRules are strictly ordered.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apiserver/pkg/apis/audit/v1beta1.PolicyRule"),
										},
									},
								},
							},
						},
						"omitStages": {
							SchemaProps: spec.SchemaProps{
								Description: "OmitStages is a list of stages for which no events are created. Note that this can also be specified per rule in which case the union of both are omitted.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
					},
					Required: []string{"rules"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta", "k8s.io/apiserver/pkg/apis/audit/v1beta1.PolicyRule"},
		},
		"k8s.io/apiserver/pkg/apis/audit/v1beta1.PolicyList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PolicyList is a list of audit Policies.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Type: []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apiserver/pkg/apis/audit/v1beta1.Policy"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta", "k8s.io/apiserver/pkg/apis/audit/v1beta1.Policy"},
		},
		"k8s.io/apiserver/pkg/apis/audit/v1beta1.PolicyRule": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PolicyRule maps requests based off metadata to an audit Level. Requests must match the rules of every field (an intersection of rules).",
					Properties: map[string]spec.Schema{
						"level": {
							SchemaProps: spec.SchemaProps{
								Description: "The Level that requests matching this rule are recorded at.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"users": {
							SchemaProps: spec.SchemaProps{
								Description: "The users (by authenticated user name) this rule applies to. An empty list implies every user.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"userGroups": {
							SchemaProps: spec.SchemaProps{
								Description: "The user groups this rule applies to. A user is considered matching if it is a member of any of the UserGroups. An empty list implies every user group.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"verbs": {
							SchemaProps: spec.SchemaProps{
								Description: "The verbs that match this rule. An empty list implies every verb.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"resources": {
							SchemaProps: spec.SchemaProps{
								Description: "Resources that this rule matches. An empty list implies all kinds in all API groups.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apiserver/pkg/apis/audit/v1beta1.GroupResources"),
										},
									},
								},
							},
						},
						"namespaces": {
							SchemaProps: spec.SchemaProps{
								Description: "Namespaces that this rule matches. The empty string \"\" matches non-namespaced resources. An empty list implies every namespace.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"nonResourceURLs": {
							SchemaProps: spec.SchemaProps{
								Description: "NonResourceURLs is a set of URL paths that should be audited. *s are allowed, but only as the full, final step in the path. Examples:\n \"/metrics\" - Log requests for apiserver metrics\n \"/healthz*\" - Log all health checks",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"omitStages": {
							SchemaProps: spec.SchemaProps{
								Description: "OmitStages is a list of stages for which no events are created. Note that this can also be specified policy wide in which case the union of both are omitted. An empty list means no restrictions will apply.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
					},
					Required: []string{"level"},
				},
			},
			Dependencies: []string{
				"k8s.io/apiserver/pkg/apis/audit/v1beta1.GroupResources"},
		},
		"k8s.io/kube-aggregator/pkg/apis/apiregistration/v1beta1.APIService": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "APIService represents a server for a particular GroupVersion. Name must be \"version.group\".",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "Spec contains information for locating and communicating with a server",
								Ref:         ref("k8s.io/kube-aggregator/pkg/apis/apiregistration/v1beta1.APIServiceSpec"),
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Status contains derived information about an API server",
								Ref:         ref("k8s.io/kube-aggregator/pkg/apis/apiregistration/v1beta1.APIServiceStatus"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta", "k8s.io/kube-aggregator/pkg/apis/apiregistration/v1beta1.APIServiceSpec", "k8s.io/kube-aggregator/pkg/apis/apiregistration/v1beta1.APIServiceStatus"},
		},
		"k8s.io/kube-aggregator/pkg/apis/apiregistration/v1beta1.APIServiceCondition": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Properties: map[string]spec.Schema{
						"type": {
							SchemaProps: spec.SchemaProps{
								Description: "Type is the type of the condition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"status": {
							SchemaProps: spec.SchemaProps{
								Description: "Status is the status of the condition. Can be True, False, Unknown.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"lastTransitionTime": {
							SchemaProps: spec.SchemaProps{
								Description: "Last time the condition transitioned from one status to another.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"reason": {
							SchemaProps: spec.SchemaProps{
								Description: "Unique, one-word, CamelCase reason for the condition's last transition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"message": {
							SchemaProps: spec.SchemaProps{
								Description: "Human-readable message indicating details about last transition.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"type", "status"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.Time"},
		},
		"k8s.io/kube-aggregator/pkg/apis/apiregistration/v1beta1.APIServiceList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "APIServiceList is a list of APIService objects.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Type: []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/kube-aggregator/pkg/apis/apiregistration/v1beta1.APIService"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta", "k8s.io/kube-aggregator/pkg/apis/apiregistration/v1beta1.APIService"},
		},
		"k8s.io/kube-aggregator/pkg/apis/apiregistration/v1beta1.APIServiceSpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "APIServiceSpec contains information for locating and communicating with a server. Only https is supported, though you are able to disable certificate verification.",
					Properties: map[string]spec.Schema{
						"service": {
							SchemaProps: spec.SchemaProps{
								Description: "Service is a reference to the service for this API server.  It must communicate on port 443 If the Service is nil, that means the handling for the API groupversion is handled locally on this server. The call will simply delegate to the normal handler chain to be fulfilled.",
								Ref:         ref("k8s.io/kube-aggregator/pkg/apis/apiregistration/v1beta1.ServiceReference"),
							},
						},
						"group": {
							SchemaProps: spec.SchemaProps{
								Description: "Group is the API group name this server hosts",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"version": {
							SchemaProps: spec.SchemaProps{
								Description: "Version is the API version this server hosts.  For example, \"v1\"",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"insecureSkipTLSVerify": {
							SchemaProps: spec.SchemaProps{
								Description: "InsecureSkipTLSVerify disables TLS certificate verification when communicating with this server. This is strongly discouraged.  You should use the CABundle instead.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"caBundle": {
							SchemaProps: spec.SchemaProps{
								Description: "CABundle is a PEM encoded CA bundle which will be used to validate an API server's serving certificate.",
								Type:        []string{"string"},
								Format:      "byte",
							},
						},
						"groupPriorityMinimum": {
							SchemaProps: spec.SchemaProps{
								Description: "GroupPriorityMininum is the priority this group should have at least. Higher priority means that the group is preferred by clients over lower priority ones. Note that other versions of this group might specify even higher GroupPriorityMininum values such that the whole group gets a higher priority. The primary sort is based on GroupPriorityMinimum, ordered highest number to lowest (20 before 10). The secondary sort is based on the alphabetical comparison of the name of the object.  (v1.bar before v1.foo) We'd recommend something like: *.k8s.io (except extensions) at 18000 and PaaSes (OpenShift, Deis) are recommended to be in the 2000s",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"versionPriority": {
							SchemaProps: spec.SchemaProps{
								Description: "VersionPriority controls the ordering of this API version inside of its group.  Must be greater than zero. The primary sort is based on VersionPriority, ordered highest to lowest (20 before 10). The secondary sort is based on the alphabetical comparison of the name of the object.  (v1.bar before v1.foo) Since it's inside of a group, the number can be small, probably in the 10s.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
					},
					Required: []string{"service", "caBundle", "groupPriorityMinimum", "versionPriority"},
				},
			},
			Dependencies: []string{
				"k8s.io/kube-aggregator/pkg/apis/apiregistration/v1beta1.ServiceReference"},
		},
		"k8s.io/kube-aggregator/pkg/apis/apiregistration/v1beta1.APIServiceStatus": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "APIServiceStatus contains derived information about an API server",
					Properties: map[string]spec.Schema{
						"conditions": {
							VendorExtensible: spec.VendorExtensible{
								Extensions: spec.Extensions{
									"x-kubernetes-patch-merge-key": "type",
									"x-kubernetes-patch-strategy":  "merge",
								},
							},
							SchemaProps: spec.SchemaProps{
								Description: "Current service state of apiService.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/kube-aggregator/pkg/apis/apiregistration/v1beta1.APIServiceCondition"),
										},
									},
								},
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/kube-aggregator/pkg/apis/apiregistration/v1beta1.APIServiceCondition"},
		},
		"k8s.io/kube-aggregator/pkg/apis/apiregistration/v1beta1.ServiceReference": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ServiceReference holds a reference to Service.legacy.k8s.io",
					Properties: map[string]spec.Schema{
						"namespace": {
							SchemaProps: spec.SchemaProps{
								Description: "Namespace is the namespace of the service",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "Name is the name of the service",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/kubernetes/pkg/apis/abac/v1beta1.Policy": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "Policy contains a single ABAC policy rule",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"spec": {
							SchemaProps: spec.SchemaProps{
								Description: "Spec describes the policy rule",
								Ref:         ref("k8s.io/kubernetes/pkg/apis/abac/v1beta1.PolicySpec"),
							},
						},
					},
					Required: []string{"spec"},
				},
			},
			Dependencies: []string{
				"k8s.io/kubernetes/pkg/apis/abac/v1beta1.PolicySpec"},
		},
		"k8s.io/kubernetes/pkg/apis/abac/v1beta1.PolicySpec": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PolicySpec contains the attributes for a policy rule",
					Properties: map[string]spec.Schema{
						"user": {
							SchemaProps: spec.SchemaProps{
								Description: "User is the username this rule applies to. Either user or group is required to match the request. \"*\" matches all users.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"group": {
							SchemaProps: spec.SchemaProps{
								Description: "Group is the group this rule applies to. Either user or group is required to match the request. \"*\" matches all groups.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"readonly": {
							SchemaProps: spec.SchemaProps{
								Description: "Readonly matches readonly requests when true, and all requests when false",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"apiGroup": {
							SchemaProps: spec.SchemaProps{
								Description: "APIGroup is the name of an API group. APIGroup, Resource, and Namespace are required to match resource requests. \"*\" matches all API groups",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"resource": {
							SchemaProps: spec.SchemaProps{
								Description: "Resource is the name of a resource. APIGroup, Resource, and Namespace are required to match resource requests. \"*\" matches all resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"namespace": {
							SchemaProps: spec.SchemaProps{
								Description: "Namespace is the name of a namespace. APIGroup, Resource, and Namespace are required to match resource requests. \"*\" matches all namespaces (including unnamespaced requests)",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"nonResourcePath": {
							SchemaProps: spec.SchemaProps{
								Description: "NonResourcePath matches non-resource request paths. \"*\" matches all paths \"/foo/*\" matches all subpaths of foo",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/kubernetes/pkg/apis/componentconfig/v1alpha1.ClientConnectionConfiguration": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ClientConnectionConfiguration contains details for constructing a client.",
					Properties: map[string]spec.Schema{
						"kubeconfig": {
							SchemaProps: spec.SchemaProps{
								Description: "kubeConfigFile is the path to a kubeconfig file.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"acceptContentTypes": {
							SchemaProps: spec.SchemaProps{
								Description: "acceptContentTypes defines the Accept header sent by clients when connecting to a server, overriding the default value of 'application/json'. This field will control all connections to the server used by a particular client.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"contentType": {
							SchemaProps: spec.SchemaProps{
								Description: "contentType is the content type used when sending data to the server from this client.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"qps": {
							SchemaProps: spec.SchemaProps{
								Description: "cps controls the number of queries per second allowed for this connection.",
								Type:        []string{"number"},
								Format:      "float",
							},
						},
						"burst": {
							SchemaProps: spec.SchemaProps{
								Description: "burst allows extra queries to accumulate when a client is exceeding its rate.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
					},
					Required: []string{"kubeconfig", "acceptContentTypes", "contentType", "qps", "burst"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/kubernetes/pkg/apis/componentconfig/v1alpha1.KubeSchedulerConfiguration": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"schedulerName": {
							SchemaProps: spec.SchemaProps{
								Description: "SchedulerName is name of the scheduler, used to select which pods will be processed by this scheduler, based on pod's \"spec.SchedulerName\".",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"algorithmSource": {
							SchemaProps: spec.SchemaProps{
								Description: "AlgorithmSource specifies the scheduler algorithm source.",
								Ref:         ref("k8s.io/kubernetes/pkg/apis/componentconfig/v1alpha1.SchedulerAlgorithmSource"),
							},
						},
						"hardPodAffinitySymmetricWeight": {
							SchemaProps: spec.SchemaProps{
								Description: "RequiredDuringScheduling affinity is not symmetric, but there is an implicit PreferredDuringScheduling affinity rule corresponding to every RequiredDuringScheduling affinity rule. HardPodAffinitySymmetricWeight represents the weight of implicit PreferredDuringScheduling affinity rule, in the range 0-100.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"leaderElection": {
							SchemaProps: spec.SchemaProps{
								Description: "LeaderElection defines the configuration of leader election client.",
								Ref:         ref("k8s.io/kubernetes/pkg/apis/componentconfig/v1alpha1.KubeSchedulerLeaderElectionConfiguration"),
							},
						},
						"clientConnection": {
							SchemaProps: spec.SchemaProps{
								Description: "ClientConnection specifies the kubeconfig file and client connection settings for the proxy server to use when communicating with the apiserver.",
								Ref:         ref("k8s.io/kubernetes/pkg/apis/componentconfig/v1alpha1.ClientConnectionConfiguration"),
							},
						},
						"healthzBindAddress": {
							SchemaProps: spec.SchemaProps{
								Description: "HealthzBindAddress is the IP address and port for the health check server to serve on, defaulting to 0.0.0.0:10251",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metricsBindAddress": {
							SchemaProps: spec.SchemaProps{
								Description: "MetricsBindAddress is the IP address and port for the metrics server to serve on, defaulting to 0.0.0.0:10251.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"enableProfiling": {
							SchemaProps: spec.SchemaProps{
								Description: "EnableProfiling enables profiling via web interface on /debug/pprof handler. Profiling handlers will be handled by metrics server.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"enableContentionProfiling": {
							SchemaProps: spec.SchemaProps{
								Description: "EnableContentionProfiling enables lock contention profiling, if EnableProfiling is true.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"failureDomains": {
							SchemaProps: spec.SchemaProps{
								Description: "Indicate the \"all topologies\" set for empty topologyKey when it's used for PreferredDuringScheduling pod anti-affinity.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"schedulerName", "algorithmSource", "hardPodAffinitySymmetricWeight", "leaderElection", "clientConnection", "healthzBindAddress", "metricsBindAddress", "enableProfiling", "enableContentionProfiling", "failureDomains"},
				},
			},
			Dependencies: []string{
				"k8s.io/kubernetes/pkg/apis/componentconfig/v1alpha1.ClientConnectionConfiguration", "k8s.io/kubernetes/pkg/apis/componentconfig/v1alpha1.KubeSchedulerLeaderElectionConfiguration", "k8s.io/kubernetes/pkg/apis/componentconfig/v1alpha1.SchedulerAlgorithmSource"},
		},
		"k8s.io/kubernetes/pkg/apis/componentconfig/v1alpha1.KubeSchedulerLeaderElectionConfiguration": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "KubeSchedulerLeaderElectionConfiguration expands LeaderElectionConfiguration to include scheduler specific configuration.",
					Properties: map[string]spec.Schema{
						"leaderElect": {
							SchemaProps: spec.SchemaProps{
								Description: "leaderElect enables a leader election client to gain leadership before executing the main loop. Enable this when running replicated components for high availability.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"leaseDuration": {
							SchemaProps: spec.SchemaProps{
								Description: "leaseDuration is the duration that non-leader candidates will wait after observing a leadership renewal until attempting to acquire leadership of a led but unrenewed leader slot. This is effectively the maximum duration that a leader can be stopped before it is replaced by another candidate. This is only applicable if leader election is enabled.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Duration"),
							},
						},
						"renewDeadline": {
							SchemaProps: spec.SchemaProps{
								Description: "renewDeadline is the interval between attempts by the acting master to renew a leadership slot before it stops leading. This must be less than or equal to the lease duration. This is only applicable if leader election is enabled.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Duration"),
							},
						},
						"retryPeriod": {
							SchemaProps: spec.SchemaProps{
								Description: "retryPeriod is the duration the clients should wait between attempting acquisition and renewal of a leadership. This is only applicable if leader election is enabled.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Duration"),
							},
						},
						"resourceLock": {
							SchemaProps: spec.SchemaProps{
								Description: "resourceLock indicates the resource object type that will be used to lock during leader election cycles.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"lockObjectNamespace": {
							SchemaProps: spec.SchemaProps{
								Description: "LockObjectNamespace defines the namespace of the lock object",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"lockObjectName": {
							SchemaProps: spec.SchemaProps{
								Description: "LockObjectName defines the lock object name",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"leaderElect", "leaseDuration", "renewDeadline", "retryPeriod", "resourceLock", "lockObjectNamespace", "lockObjectName"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.Duration"},
		},
		"k8s.io/kubernetes/pkg/apis/componentconfig/v1alpha1.LeaderElectionConfiguration": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "LeaderElectionConfiguration defines the configuration of leader election clients for components that can run with leader election enabled.",
					Properties: map[string]spec.Schema{
						"leaderElect": {
							SchemaProps: spec.SchemaProps{
								Description: "leaderElect enables a leader election client to gain leadership before executing the main loop. Enable this when running replicated components for high availability.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"leaseDuration": {
							SchemaProps: spec.SchemaProps{
								Description: "leaseDuration is the duration that non-leader candidates will wait after observing a leadership renewal until attempting to acquire leadership of a led but unrenewed leader slot. This is effectively the maximum duration that a leader can be stopped before it is replaced by another candidate. This is only applicable if leader election is enabled.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Duration"),
							},
						},
						"renewDeadline": {
							SchemaProps: spec.SchemaProps{
								Description: "renewDeadline is the interval between attempts by the acting master to renew a leadership slot before it stops leading. This must be less than or equal to the lease duration. This is only applicable if leader election is enabled.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Duration"),
							},
						},
						"retryPeriod": {
							SchemaProps: spec.SchemaProps{
								Description: "retryPeriod is the duration the clients should wait between attempting acquisition and renewal of a leadership. This is only applicable if leader election is enabled.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Duration"),
							},
						},
						"resourceLock": {
							SchemaProps: spec.SchemaProps{
								Description: "resourceLock indicates the resource object type that will be used to lock during leader election cycles.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"leaderElect", "leaseDuration", "renewDeadline", "retryPeriod", "resourceLock"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.Duration"},
		},
		"k8s.io/kubernetes/pkg/apis/componentconfig/v1alpha1.SchedulerAlgorithmSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "SchedulerAlgorithmSource is the source of a scheduler algorithm. One source field must be specified, and source fields are mutually exclusive.",
					Properties: map[string]spec.Schema{
						"policy": {
							SchemaProps: spec.SchemaProps{
								Description: "Policy is a policy based algorithm source.",
								Ref:         ref("k8s.io/kubernetes/pkg/apis/componentconfig/v1alpha1.SchedulerPolicySource"),
							},
						},
						"provider": {
							SchemaProps: spec.SchemaProps{
								Description: "Provider is the name of a scheduling algorithm provider to use.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/kubernetes/pkg/apis/componentconfig/v1alpha1.SchedulerPolicySource"},
		},
		"k8s.io/kubernetes/pkg/apis/componentconfig/v1alpha1.SchedulerPolicyConfigMapSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "SchedulerPolicyConfigMapSource is a policy serialized into a config map value under the SchedulerPolicyConfigMapKey key.",
					Properties: map[string]spec.Schema{
						"namespace": {
							SchemaProps: spec.SchemaProps{
								Description: "Namespace is the namespace of the policy config map.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "Name is the name of hte policy config map.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"namespace", "name"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/kubernetes/pkg/apis/componentconfig/v1alpha1.SchedulerPolicyFileSource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "SchedulerPolicyFileSource is a policy serialized to disk and accessed via path.",
					Properties: map[string]spec.Schema{
						"path": {
							SchemaProps: spec.SchemaProps{
								Description: "Path is the location of a serialized policy.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"path"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/kubernetes/pkg/apis/componentconfig/v1alpha1.SchedulerPolicySource": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "SchedulerPolicySource configures a means to obtain a scheduler Policy. One source field must be specified, and source fields are mutually exclusive.",
					Properties: map[string]spec.Schema{
						"file": {
							SchemaProps: spec.SchemaProps{
								Description: "File is a file policy source.",
								Ref:         ref("k8s.io/kubernetes/pkg/apis/componentconfig/v1alpha1.SchedulerPolicyFileSource"),
							},
						},
						"configMap": {
							SchemaProps: spec.SchemaProps{
								Description: "ConfigMap is a config map policy source.",
								Ref:         ref("k8s.io/kubernetes/pkg/apis/componentconfig/v1alpha1.SchedulerPolicyConfigMapSource"),
							},
						},
					},
				},
			},
			Dependencies: []string{
				"k8s.io/kubernetes/pkg/apis/componentconfig/v1alpha1.SchedulerPolicyConfigMapSource", "k8s.io/kubernetes/pkg/apis/componentconfig/v1alpha1.SchedulerPolicyFileSource"},
		},
		"k8s.io/kubernetes/pkg/kubelet/apis/kubeletconfig/v1alpha1.KubeletAnonymousAuthentication": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Properties: map[string]spec.Schema{
						"enabled": {
							SchemaProps: spec.SchemaProps{
								Description: "enabled allows anonymous requests to the kubelet server. Requests that are not rejected by another authentication method are treated as anonymous requests. Anonymous requests have a username of system:anonymous, and a group name of system:unauthenticated.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
					},
					Required: []string{"enabled"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/kubernetes/pkg/kubelet/apis/kubeletconfig/v1alpha1.KubeletAuthentication": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Properties: map[string]spec.Schema{
						"x509": {
							SchemaProps: spec.SchemaProps{
								Description: "x509 contains settings related to x509 client certificate authentication",
								Ref:         ref("k8s.io/kubernetes/pkg/kubelet/apis/kubeletconfig/v1alpha1.KubeletX509Authentication"),
							},
						},
						"webhook": {
							SchemaProps: spec.SchemaProps{
								Description: "webhook contains settings related to webhook bearer token authentication",
								Ref:         ref("k8s.io/kubernetes/pkg/kubelet/apis/kubeletconfig/v1alpha1.KubeletWebhookAuthentication"),
							},
						},
						"anonymous": {
							SchemaProps: spec.SchemaProps{
								Description: "anonymous contains settings related to anonymous authentication",
								Ref:         ref("k8s.io/kubernetes/pkg/kubelet/apis/kubeletconfig/v1alpha1.KubeletAnonymousAuthentication"),
							},
						},
					},
					Required: []string{"x509", "webhook", "anonymous"},
				},
			},
			Dependencies: []string{
				"k8s.io/kubernetes/pkg/kubelet/apis/kubeletconfig/v1alpha1.KubeletAnonymousAuthentication", "k8s.io/kubernetes/pkg/kubelet/apis/kubeletconfig/v1alpha1.KubeletWebhookAuthentication", "k8s.io/kubernetes/pkg/kubelet/apis/kubeletconfig/v1alpha1.KubeletX509Authentication"},
		},
		"k8s.io/kubernetes/pkg/kubelet/apis/kubeletconfig/v1alpha1.KubeletAuthorization": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Properties: map[string]spec.Schema{
						"mode": {
							SchemaProps: spec.SchemaProps{
								Description: "mode is the authorization mode to apply to requests to the kubelet server. Valid values are AlwaysAllow and Webhook. Webhook mode uses the SubjectAccessReview API to determine authorization.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"webhook": {
							SchemaProps: spec.SchemaProps{
								Description: "webhook contains settings related to Webhook authorization.",
								Ref:         ref("k8s.io/kubernetes/pkg/kubelet/apis/kubeletconfig/v1alpha1.KubeletWebhookAuthorization"),
							},
						},
					},
					Required: []string{"mode", "webhook"},
				},
			},
			Dependencies: []string{
				"k8s.io/kubernetes/pkg/kubelet/apis/kubeletconfig/v1alpha1.KubeletWebhookAuthorization"},
		},
		"k8s.io/kubernetes/pkg/kubelet/apis/kubeletconfig/v1alpha1.KubeletConfiguration": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "A configuration field should go in KubeletFlags instead of KubeletConfiguration if its value cannot be safely shared between nodes at the same time (e.g. a hostname) In general, please try to avoid adding flags or configuration fields, we already have a confusingly large amount of them.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"configTrialDuration": {
							SchemaProps: spec.SchemaProps{
								Description: "Only used for dynamic configuration. The length of the trial period for this configuration. This configuration will become the last-known-good after this duration.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Duration"),
							},
						},
						"podManifestPath": {
							SchemaProps: spec.SchemaProps{
								Description: "podManifestPath is the path to the directory containing pod manifests to run, or the path to a single manifest file",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"syncFrequency": {
							SchemaProps: spec.SchemaProps{
								Description: "syncFrequency is the max period between synchronizing running containers and config",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Duration"),
							},
						},
						"fileCheckFrequency": {
							SchemaProps: spec.SchemaProps{
								Description: "fileCheckFrequency is the duration between checking config files for new data",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Duration"),
							},
						},
						"httpCheckFrequency": {
							SchemaProps: spec.SchemaProps{
								Description: "httpCheckFrequency is the duration between checking http for new data",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Duration"),
							},
						},
						"manifestURL": {
							SchemaProps: spec.SchemaProps{
								Description: "manifestURL is the URL for accessing the container manifest",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"manifestURLHeader": {
							SchemaProps: spec.SchemaProps{
								Description: "manifestURLHeader is the HTTP header to use when accessing the manifest URL, with the key separated from the value with a ':', as in 'key:value'",
								Type:        []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type: []string{"array"},
											Items: &spec.SchemaOrArray{
												Schema: &spec.Schema{
													SchemaProps: spec.SchemaProps{
														Type:   []string{"string"},
														Format: "",
													},
												},
											},
										},
									},
								},
							},
						},
						"enableServer": {
							SchemaProps: spec.SchemaProps{
								Description: "enableServer enables the Kubelet's server",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"address": {
							SchemaProps: spec.SchemaProps{
								Description: "address is the IP address for the Kubelet to serve on (set to 0.0.0.0 for all interfaces)",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"port": {
							SchemaProps: spec.SchemaProps{
								Description: "port is the port for the Kubelet to serve on.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"readOnlyPort": {
							SchemaProps: spec.SchemaProps{
								Description: "readOnlyPort is the read-only port for the Kubelet to serve on with no authentication/authorization (set to 0 to disable)",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"tlsCertFile": {
							SchemaProps: spec.SchemaProps{
								Description: "tlsCertFile is the file containing x509 Certificate for HTTPS.  (CA cert, if any, concatenated after server cert). If tlsCertFile and tlsPrivateKeyFile are not provided, a self-signed certificate and key are generated for the public address and saved to the directory passed to certDir.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"tlsPrivateKeyFile": {
							SchemaProps: spec.SchemaProps{
								Description: "tlsPrivateKeyFile is the ile containing x509 private key matching tlsCertFile.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"authentication": {
							SchemaProps: spec.SchemaProps{
								Description: "authentication specifies how requests to the Kubelet's server are authenticated",
								Ref:         ref("k8s.io/kubernetes/pkg/kubelet/apis/kubeletconfig/v1alpha1.KubeletAuthentication"),
							},
						},
						"authorization": {
							SchemaProps: spec.SchemaProps{
								Description: "authorization specifies how requests to the Kubelet's server are authorized",
								Ref:         ref("k8s.io/kubernetes/pkg/kubelet/apis/kubeletconfig/v1alpha1.KubeletAuthorization"),
							},
						},
						"allowPrivileged": {
							SchemaProps: spec.SchemaProps{
								Description: "allowPrivileged enables containers to request privileged mode. Defaults to false.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"hostNetworkSources": {
							SchemaProps: spec.SchemaProps{
								Description: "hostNetworkSources is a comma-separated list of sources from which the Kubelet allows pods to use of host network. Defaults to \"*\". Valid options are \"file\", \"http\", \"api\", and \"*\" (all sources).",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"hostPIDSources": {
							SchemaProps: spec.SchemaProps{
								Description: "hostPIDSources is a comma-separated list of sources from which the Kubelet allows pods to use the host pid namespace. Defaults to \"*\".",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"hostIPCSources": {
							SchemaProps: spec.SchemaProps{
								Description: "hostIPCSources is a comma-separated list of sources from which the Kubelet allows pods to use the host ipc namespace. Defaults to \"*\".",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"registryPullQPS": {
							SchemaProps: spec.SchemaProps{
								Description: "registryPullQPS is the limit of registry pulls per second. If 0, unlimited. Set to 0 for no limit. Defaults to 5.0.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"registryBurst": {
							SchemaProps: spec.SchemaProps{
								Description: "registryBurst is the maximum size of a bursty pulls, temporarily allows pulls to burst to this number, while still not exceeding registryQps. Only used if registryQPS > 0.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"eventRecordQPS": {
							SchemaProps: spec.SchemaProps{
								Description: "eventRecordQPS is the maximum event creations per second. If 0, there is no limit enforced.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"eventBurst": {
							SchemaProps: spec.SchemaProps{
								Description: "eventBurst is the maximum size of a bursty event records, temporarily allows event records to burst to this number, while still not exceeding event-qps. Only used if eventQps > 0",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"enableDebuggingHandlers": {
							SchemaProps: spec.SchemaProps{
								Description: "enableDebuggingHandlers enables server endpoints for log collection and local running of containers and commands",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"enableContentionProfiling": {
							SchemaProps: spec.SchemaProps{
								Description: "enableContentionProfiling enables lock contention profiling, if enableDebuggingHandlers is true.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"cAdvisorPort": {
							SchemaProps: spec.SchemaProps{
								Description: "cAdvisorPort is the port of the localhost cAdvisor endpoint (set to 0 to disable)",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"healthzPort": {
							SchemaProps: spec.SchemaProps{
								Description: "healthzPort is the port of the localhost healthz endpoint (set to 0 to disable)",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"healthzBindAddress": {
							SchemaProps: spec.SchemaProps{
								Description: "healthzBindAddress is the IP address for the healthz server to serve on.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"oomScoreAdj": {
							SchemaProps: spec.SchemaProps{
								Description: "oomScoreAdj is The oom-score-adj value for kubelet process. Values must be within the range [-1000, 1000].",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"clusterDomain": {
							SchemaProps: spec.SchemaProps{
								Description: "clusterDomain is the DNS domain for this cluster. If set, kubelet will configure all containers to search this domain in addition to the host's search domains.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"clusterDNS": {
							SchemaProps: spec.SchemaProps{
								Description: "clusterDNS is a list of IP address for the cluster DNS server.  If set, kubelet will configure all containers to use this for DNS resolution instead of the host's DNS servers",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"streamingConnectionIdleTimeout": {
							SchemaProps: spec.SchemaProps{
								Description: "streamingConnectionIdleTimeout is the maximum time a streaming connection can be idle before the connection is automatically closed.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Duration"),
							},
						},
						"nodeStatusUpdateFrequency": {
							SchemaProps: spec.SchemaProps{
								Description: "nodeStatusUpdateFrequency is the frequency that kubelet posts node status to master. Note: be cautious when changing the constant, it must work with nodeMonitorGracePeriod in nodecontroller.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Duration"),
							},
						},
						"imageMinimumGCAge": {
							SchemaProps: spec.SchemaProps{
								Description: "imageMinimumGCAge is the minimum age for an unused image before it is garbage collected.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Duration"),
							},
						},
						"imageGCHighThresholdPercent": {
							SchemaProps: spec.SchemaProps{
								Description: "imageGCHighThresholdPercent is the percent of disk usage after which image garbage collection is always run. The percent is calculated as this field value out of 100.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"imageGCLowThresholdPercent": {
							SchemaProps: spec.SchemaProps{
								Description: "imageGCLowThresholdPercent is the percent of disk usage before which image garbage collection is never run. Lowest disk usage to garbage collect to. The percent is calculated as this field value out of 100.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"volumeStatsAggPeriod": {
							SchemaProps: spec.SchemaProps{
								Description: "How frequently to calculate and cache volume disk usage for all pods",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Duration"),
							},
						},
						"kubeletCgroups": {
							SchemaProps: spec.SchemaProps{
								Description: "kubeletCgroups is the absolute name of cgroups to isolate the kubelet in.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"systemCgroups": {
							SchemaProps: spec.SchemaProps{
								Description: "systemCgroups is absolute name of cgroups in which to place all non-kernel processes that are not already in a container. Empty for no container. Rolling back the flag requires a reboot.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"cgroupRoot": {
							SchemaProps: spec.SchemaProps{
								Description: "cgroupRoot is the root cgroup to use for pods. This is handled by the container runtime on a best effort basis.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"cgroupsPerQOS": {
							SchemaProps: spec.SchemaProps{
								Description: "Enable QoS based Cgroup hierarchy: top level cgroups for QoS Classes And all Burstable and BestEffort pods are brought up under their specific top level QoS cgroup.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"cgroupDriver": {
							SchemaProps: spec.SchemaProps{
								Description: "driver that the kubelet uses to manipulate cgroups on the host (cgroupfs or systemd)",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"cpuManagerPolicy": {
							SchemaProps: spec.SchemaProps{
								Description: "CPUManagerPolicy is the name of the policy to use.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"cpuManagerReconcilePeriod": {
							SchemaProps: spec.SchemaProps{
								Description: "CPU Manager reconciliation period.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Duration"),
							},
						},
						"runtimeRequestTimeout": {
							SchemaProps: spec.SchemaProps{
								Description: "runtimeRequestTimeout is the timeout for all runtime requests except long running requests - pull, logs, exec and attach.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Duration"),
							},
						},
						"hairpinMode": {
							SchemaProps: spec.SchemaProps{
								Description: "How should the kubelet configure the container bridge for hairpin packets. Setting this flag allows endpoints in a Service to loadbalance back to themselves if they should try to access their own Service. Values:\n  \"promiscuous-bridge\": make the container bridge promiscuous.\n  \"hairpin-veth\":       set the hairpin flag on container veth interfaces.\n  \"none\":               do nothing.\nGenerally, one must set --hairpin-mode=veth-flag to achieve hairpin NAT, because promiscous-bridge assumes the existence of a container bridge named cbr0.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"maxPods": {
							SchemaProps: spec.SchemaProps{
								Description: "maxPods is the number of pods that can run on this Kubelet.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"podCIDR": {
							SchemaProps: spec.SchemaProps{
								Description: "The CIDR to use for pod IP addresses, only used in standalone mode. In cluster mode, this is obtained from the master.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"resolvConf": {
							SchemaProps: spec.SchemaProps{
								Description: "ResolverConfig is the resolver configuration file used as the basis for the container DNS resolution configuration.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"cpuCFSQuota": {
							SchemaProps: spec.SchemaProps{
								Description: "cpuCFSQuota is Enable CPU CFS quota enforcement for containers that specify CPU limits",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"maxOpenFiles": {
							SchemaProps: spec.SchemaProps{
								Description: "maxOpenFiles is Number of files that can be opened by Kubelet process.",
								Type:        []string{"integer"},
								Format:      "int64",
							},
						},
						"contentType": {
							SchemaProps: spec.SchemaProps{
								Description: "contentType is contentType of requests sent to apiserver.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"kubeAPIQPS": {
							SchemaProps: spec.SchemaProps{
								Description: "kubeAPIQPS is the QPS to use while talking with kubernetes apiserver",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"kubeAPIBurst": {
							SchemaProps: spec.SchemaProps{
								Description: "kubeAPIBurst is the burst to allow while talking with kubernetes apiserver",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"serializeImagePulls": {
							SchemaProps: spec.SchemaProps{
								Description: "serializeImagePulls when enabled, tells the Kubelet to pull images one at a time. We recommend *not* changing the default value on nodes that run docker daemon with version  < 1.9 or an Aufs storage backend. Issue #10959 has more details.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"evictionHard": {
							SchemaProps: spec.SchemaProps{
								Description: "Map of signal names to quantities that defines hard eviction thresholds. For example: {\"memory.available\": \"300Mi\"}.",
								Type:        []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"evictionSoft": {
							SchemaProps: spec.SchemaProps{
								Description: "Map of signal names to quantities that defines soft eviction thresholds.  For example: {\"memory.available\": \"300Mi\"}.",
								Type:        []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"evictionSoftGracePeriod": {
							SchemaProps: spec.SchemaProps{
								Description: "Map of signal names to quantities that defines grace periods for each soft eviction signal. For example: {\"memory.available\": \"30s\"}.",
								Type:        []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"evictionPressureTransitionPeriod": {
							SchemaProps: spec.SchemaProps{
								Description: "Duration for which the kubelet has to wait before transitioning out of an eviction pressure condition.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Duration"),
							},
						},
						"evictionMaxPodGracePeriod": {
							SchemaProps: spec.SchemaProps{
								Description: "Maximum allowed grace period (in seconds) to use when terminating pods in response to a soft eviction threshold being met.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"evictionMinimumReclaim": {
							SchemaProps: spec.SchemaProps{
								Description: "Map of signal names to quantities that defines minimum reclaims, which describe the minimum amount of a given resource the kubelet will reclaim when performing a pod eviction while that resource is under pressure. For example: {\"imagefs.available\": \"2Gi\"}",
								Type:        []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"podsPerCore": {
							SchemaProps: spec.SchemaProps{
								Description: "Maximum number of pods per core. Cannot exceed MaxPods",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"enableControllerAttachDetach": {
							SchemaProps: spec.SchemaProps{
								Description: "enableControllerAttachDetach enables the Attach/Detach controller to manage attachment/detachment of volumes scheduled to this node, and disables kubelet from executing any attach/detach operations",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"protectKernelDefaults": {
							SchemaProps: spec.SchemaProps{
								Description: "Default behaviour for kernel tuning",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"makeIPTablesUtilChains": {
							SchemaProps: spec.SchemaProps{
								Description: "If true, Kubelet ensures a set of iptables rules are present on host. These rules will serve as utility rules for various components, e.g. KubeProxy. The rules will be created based on IPTablesMasqueradeBit and IPTablesDropBit.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"iptablesMasqueradeBit": {
							SchemaProps: spec.SchemaProps{
								Description: "iptablesMasqueradeBit is the bit of the iptables fwmark space to mark for SNAT Values must be within the range [0, 31]. Must be different from other mark bits. Warning: Please match the value of corresponding parameter in kube-proxy",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"iptablesDropBit": {
							SchemaProps: spec.SchemaProps{
								Description: "iptablesDropBit is the bit of the iptables fwmark space to mark for dropping packets. Values must be within the range [0, 31]. Must be different from other mark bits.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"featureGates": {
							SchemaProps: spec.SchemaProps{
								Description: "featureGates is a map of feature names to bools that enable or disable alpha/experimental features.",
								Type:        []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"boolean"},
											Format: "",
										},
									},
								},
							},
						},
						"failSwapOn": {
							SchemaProps: spec.SchemaProps{
								Description: "Tells the Kubelet to fail to start if swap is enabled on the node.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"systemReserved": {
							SchemaProps: spec.SchemaProps{
								Description: "A set of ResourceName=ResourceQuantity (e.g. cpu=200m,memory=150G) pairs that describe resources reserved for non-kubernetes components. Currently only cpu and memory are supported. [default=none] See http://kubernetes.io/docs/user-guide/compute-resources for more detail.",
								Type:        []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"kubeReserved": {
							SchemaProps: spec.SchemaProps{
								Description: "A set of ResourceName=ResourceQuantity (e.g. cpu=200m,memory=150G) pairs that describe resources reserved for kubernetes system components. Currently cpu, memory and local storage for root file system are supported. [default=none] See http://kubernetes.io/docs/user-guide/compute-resources for more detail.",
								Type:        []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
						"systemReservedCgroup": {
							SchemaProps: spec.SchemaProps{
								Description: "This flag helps kubelet identify absolute name of top level cgroup used to enforce `SystemReserved` compute resource reservation for OS system daemons. Refer to [Node Allocatable](https://git.k8s.io/community/contributors/design-proposals/node/node-allocatable.md) doc for more information.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"kubeReservedCgroup": {
							SchemaProps: spec.SchemaProps{
								Description: "This flag helps kubelet identify absolute name of top level cgroup used to enforce `KubeReserved` compute resource reservation for Kubernetes node system daemons. Refer to [Node Allocatable](https://git.k8s.io/community/contributors/design-proposals/node/node-allocatable.md) doc for more information.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"enforceNodeAllocatable": {
							SchemaProps: spec.SchemaProps{
								Description: "This flag specifies the various Node Allocatable enforcements that Kubelet needs to perform. This flag accepts a list of options. Acceptible options are `pods`, `system-reserved` & `kube-reserved`. Refer to [Node Allocatable](https://git.k8s.io/community/contributors/design-proposals/node/node-allocatable.md) doc for more information.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Type:   []string{"string"},
											Format: "",
										},
									},
								},
							},
						},
					},
					Required: []string{"configTrialDuration", "podManifestPath", "syncFrequency", "fileCheckFrequency", "httpCheckFrequency", "manifestURL", "manifestURLHeader", "enableServer", "address", "port", "readOnlyPort", "tlsCertFile", "tlsPrivateKeyFile", "authentication", "authorization", "allowPrivileged", "hostNetworkSources", "hostPIDSources", "hostIPCSources", "registryPullQPS", "registryBurst", "eventRecordQPS", "eventBurst", "enableDebuggingHandlers", "enableContentionProfiling", "cAdvisorPort", "healthzPort", "healthzBindAddress", "oomScoreAdj", "clusterDomain", "clusterDNS", "streamingConnectionIdleTimeout", "nodeStatusUpdateFrequency", "imageMinimumGCAge", "imageGCHighThresholdPercent", "imageGCLowThresholdPercent", "volumeStatsAggPeriod", "kubeletCgroups", "systemCgroups", "cgroupRoot", "cpuManagerPolicy", "cpuManagerReconcilePeriod", "runtimeRequestTimeout", "hairpinMode", "maxPods", "podCIDR", "resolvConf", "cpuCFSQuota", "maxOpenFiles", "contentType", "kubeAPIQPS", "kubeAPIBurst", "serializeImagePulls", "evictionPressureTransitionPeriod", "evictionMaxPodGracePeriod", "podsPerCore", "enableControllerAttachDetach", "protectKernelDefaults", "makeIPTablesUtilChains", "iptablesMasqueradeBit", "iptablesDropBit", "systemReserved", "kubeReserved", "enforceNodeAllocatable"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.Duration", "k8s.io/kubernetes/pkg/kubelet/apis/kubeletconfig/v1alpha1.KubeletAuthentication", "k8s.io/kubernetes/pkg/kubelet/apis/kubeletconfig/v1alpha1.KubeletAuthorization"},
		},
		"k8s.io/kubernetes/pkg/kubelet/apis/kubeletconfig/v1alpha1.KubeletWebhookAuthentication": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Properties: map[string]spec.Schema{
						"enabled": {
							SchemaProps: spec.SchemaProps{
								Description: "enabled allows bearer token authentication backed by the tokenreviews.authentication.k8s.io API",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"cacheTTL": {
							SchemaProps: spec.SchemaProps{
								Description: "cacheTTL enables caching of authentication results",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Duration"),
							},
						},
					},
					Required: []string{"enabled", "cacheTTL"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.Duration"},
		},
		"k8s.io/kubernetes/pkg/kubelet/apis/kubeletconfig/v1alpha1.KubeletWebhookAuthorization": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Properties: map[string]spec.Schema{
						"cacheAuthorizedTTL": {
							SchemaProps: spec.SchemaProps{
								Description: "cacheAuthorizedTTL is the duration to cache 'authorized' responses from the webhook authorizer.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Duration"),
							},
						},
						"cacheUnauthorizedTTL": {
							SchemaProps: spec.SchemaProps{
								Description: "cacheUnauthorizedTTL is the duration to cache 'unauthorized' responses from the webhook authorizer.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Duration"),
							},
						},
					},
					Required: []string{"cacheAuthorizedTTL", "cacheUnauthorizedTTL"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.Duration"},
		},
		"k8s.io/kubernetes/pkg/kubelet/apis/kubeletconfig/v1alpha1.KubeletX509Authentication": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Properties: map[string]spec.Schema{
						"clientCAFile": {
							SchemaProps: spec.SchemaProps{
								Description: "clientCAFile is the path to a PEM-encoded certificate bundle. If set, any request presenting a client certificate signed by one of the authorities in the bundle is authenticated with a username corresponding to the CommonName, and groups corresponding to the Organization in the client certificate.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"clientCAFile"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/kubernetes/pkg/proxy/apis/kubeproxyconfig/v1alpha1.ClientConnectionConfiguration": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "ClientConnectionConfiguration contains details for constructing a client.",
					Properties: map[string]spec.Schema{
						"kubeconfig": {
							SchemaProps: spec.SchemaProps{
								Description: "kubeConfigFile is the path to a kubeconfig file.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"acceptContentTypes": {
							SchemaProps: spec.SchemaProps{
								Description: "acceptContentTypes defines the Accept header sent by clients when connecting to a server, overriding the default value of 'application/json'. This field will control all connections to the server used by a particular client.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"contentType": {
							SchemaProps: spec.SchemaProps{
								Description: "contentType is the content type used when sending data to the server from this client.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"qps": {
							SchemaProps: spec.SchemaProps{
								Description: "cps controls the number of queries per second allowed for this connection.",
								Type:        []string{"number"},
								Format:      "float",
							},
						},
						"burst": {
							SchemaProps: spec.SchemaProps{
								Description: "burst allows extra queries to accumulate when a client is exceeding its rate.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
					},
					Required: []string{"kubeconfig", "acceptContentTypes", "contentType", "qps", "burst"},
				},
			},
			Dependencies: []string{},
		},
		"k8s.io/kubernetes/pkg/proxy/apis/kubeproxyconfig/v1alpha1.KubeProxyConfiguration": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "KubeProxyConfiguration contains everything necessary to configure the Kubernetes proxy server.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"featureGates": {
							SchemaProps: spec.SchemaProps{
								Description: "featureGates is a comma-separated list of key=value pairs that control which alpha/beta features are enabled.\n\ncomponents to use config files because local-up-cluster.sh only supports the --feature-gates flag right now, which is comma-separated key=value pairs.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"bindAddress": {
							SchemaProps: spec.SchemaProps{
								Description: "bindAddress is the IP address for the proxy server to serve on (set to 0.0.0.0 for all interfaces)",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"healthzBindAddress": {
							SchemaProps: spec.SchemaProps{
								Description: "healthzBindAddress is the IP address and port for the health check server to serve on, defaulting to 0.0.0.0:10256",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metricsBindAddress": {
							SchemaProps: spec.SchemaProps{
								Description: "metricsBindAddress is the IP address and port for the metrics server to serve on, defaulting to 127.0.0.1:10249 (set to 0.0.0.0 for all interfaces)",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"enableProfiling": {
							SchemaProps: spec.SchemaProps{
								Description: "enableProfiling enables profiling via web interface on /debug/pprof handler. Profiling handlers will be handled by metrics server.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"clusterCIDR": {
							SchemaProps: spec.SchemaProps{
								Description: "clusterCIDR is the CIDR range of the pods in the cluster. It is used to bridge traffic coming from outside of the cluster. If not provided, no off-cluster bridging will be performed.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"hostnameOverride": {
							SchemaProps: spec.SchemaProps{
								Description: "hostnameOverride, if non-empty, will be used as the identity instead of the actual hostname.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"clientConnection": {
							SchemaProps: spec.SchemaProps{
								Description: "clientConnection specifies the kubeconfig file and client connection settings for the proxy server to use when communicating with the apiserver.",
								Ref:         ref("k8s.io/kubernetes/pkg/proxy/apis/kubeproxyconfig/v1alpha1.ClientConnectionConfiguration"),
							},
						},
						"iptables": {
							SchemaProps: spec.SchemaProps{
								Description: "iptables contains iptables-related configuration options.",
								Ref:         ref("k8s.io/kubernetes/pkg/proxy/apis/kubeproxyconfig/v1alpha1.KubeProxyIPTablesConfiguration"),
							},
						},
						"ipvs": {
							SchemaProps: spec.SchemaProps{
								Description: "ipvs contains ipvs-related configuration options.",
								Ref:         ref("k8s.io/kubernetes/pkg/proxy/apis/kubeproxyconfig/v1alpha1.KubeProxyIPVSConfiguration"),
							},
						},
						"oomScoreAdj": {
							SchemaProps: spec.SchemaProps{
								Description: "oomScoreAdj is the oom-score-adj value for kube-proxy process. Values must be within the range [-1000, 1000]",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"mode": {
							SchemaProps: spec.SchemaProps{
								Description: "mode specifies which proxy mode to use.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"portRange": {
							SchemaProps: spec.SchemaProps{
								Description: "portRange is the range of host ports (beginPort-endPort, inclusive) that may be consumed in order to proxy service traffic. If unspecified (0-0) then ports will be randomly chosen.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"resourceContainer": {
							SchemaProps: spec.SchemaProps{
								Description: "resourceContainer is the bsolute name of the resource-only container to create and run the Kube-proxy in (Default: /kube-proxy).",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"udpTimeoutMilliseconds": {
							SchemaProps: spec.SchemaProps{
								Description: "udpIdleTimeout is how long an idle UDP connection will be kept open (e.g. '250ms', '2s'). Must be greater than 0. Only applicable for proxyMode=userspace.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Duration"),
							},
						},
						"conntrack": {
							SchemaProps: spec.SchemaProps{
								Description: "conntrack contains conntrack-related configuration options.",
								Ref:         ref("k8s.io/kubernetes/pkg/proxy/apis/kubeproxyconfig/v1alpha1.KubeProxyConntrackConfiguration"),
							},
						},
						"configSyncPeriod": {
							SchemaProps: spec.SchemaProps{
								Description: "configSyncPeriod is how often configuration from the apiserver is refreshed. Must be greater than 0.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Duration"),
							},
						},
					},
					Required: []string{"featureGates", "bindAddress", "healthzBindAddress", "metricsBindAddress", "enableProfiling", "clusterCIDR", "hostnameOverride", "clientConnection", "iptables", "ipvs", "oomScoreAdj", "mode", "portRange", "resourceContainer", "udpTimeoutMilliseconds", "conntrack", "configSyncPeriod"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.Duration", "k8s.io/kubernetes/pkg/proxy/apis/kubeproxyconfig/v1alpha1.ClientConnectionConfiguration", "k8s.io/kubernetes/pkg/proxy/apis/kubeproxyconfig/v1alpha1.KubeProxyConntrackConfiguration", "k8s.io/kubernetes/pkg/proxy/apis/kubeproxyconfig/v1alpha1.KubeProxyIPTablesConfiguration", "k8s.io/kubernetes/pkg/proxy/apis/kubeproxyconfig/v1alpha1.KubeProxyIPVSConfiguration"},
		},
		"k8s.io/kubernetes/pkg/proxy/apis/kubeproxyconfig/v1alpha1.KubeProxyConntrackConfiguration": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "KubeProxyConntrackConfiguration contains conntrack settings for the Kubernetes proxy server.",
					Properties: map[string]spec.Schema{
						"max": {
							SchemaProps: spec.SchemaProps{
								Description: "max is the maximum number of NAT connections to track (0 to leave as-is).  This takes precedence over maxPerCore and min.",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"maxPerCore": {
							SchemaProps: spec.SchemaProps{
								Description: "maxPerCore is the maximum number of NAT connections to track per CPU core (0 to leave the limit as-is and ignore min).",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"min": {
							SchemaProps: spec.SchemaProps{
								Description: "min is the minimum value of connect-tracking records to allocate, regardless of conntrackMaxPerCore (set maxPerCore=0 to leave the limit as-is).",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"tcpEstablishedTimeout": {
							SchemaProps: spec.SchemaProps{
								Description: "tcpEstablishedTimeout is how long an idle TCP connection will be kept open (e.g. '2s').  Must be greater than 0 to set.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Duration"),
							},
						},
						"tcpCloseWaitTimeout": {
							SchemaProps: spec.SchemaProps{
								Description: "tcpCloseWaitTimeout is how long an idle conntrack entry in CLOSE_WAIT state will remain in the conntrack table. (e.g. '60s'). Must be greater than 0 to set.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Duration"),
							},
						},
					},
					Required: []string{"max", "maxPerCore", "min", "tcpEstablishedTimeout", "tcpCloseWaitTimeout"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.Duration"},
		},
		"k8s.io/kubernetes/pkg/proxy/apis/kubeproxyconfig/v1alpha1.KubeProxyIPTablesConfiguration": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "KubeProxyIPTablesConfiguration contains iptables-related configuration details for the Kubernetes proxy server.",
					Properties: map[string]spec.Schema{
						"masqueradeBit": {
							SchemaProps: spec.SchemaProps{
								Description: "masqueradeBit is the bit of the iptables fwmark space to use for SNAT if using the pure iptables proxy mode. Values must be within the range [0, 31].",
								Type:        []string{"integer"},
								Format:      "int32",
							},
						},
						"masqueradeAll": {
							SchemaProps: spec.SchemaProps{
								Description: "masqueradeAll tells kube-proxy to SNAT everything if using the pure iptables proxy mode.",
								Type:        []string{"boolean"},
								Format:      "",
							},
						},
						"syncPeriod": {
							SchemaProps: spec.SchemaProps{
								Description: "syncPeriod is the period that iptables rules are refreshed (e.g. '5s', '1m', '2h22m').  Must be greater than 0.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Duration"),
							},
						},
						"minSyncPeriod": {
							SchemaProps: spec.SchemaProps{
								Description: "minSyncPeriod is the minimum period that iptables rules are refreshed (e.g. '5s', '1m', '2h22m').",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Duration"),
							},
						},
					},
					Required: []string{"masqueradeBit", "masqueradeAll", "syncPeriod", "minSyncPeriod"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.Duration"},
		},
		"k8s.io/kubernetes/pkg/proxy/apis/kubeproxyconfig/v1alpha1.KubeProxyIPVSConfiguration": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "KubeProxyIPVSConfiguration contains ipvs-related configuration details for the Kubernetes proxy server.",
					Properties: map[string]spec.Schema{
						"syncPeriod": {
							SchemaProps: spec.SchemaProps{
								Description: "syncPeriod is the period that ipvs rules are refreshed (e.g. '5s', '1m', '2h22m').  Must be greater than 0.",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Duration"),
							},
						},
						"minSyncPeriod": {
							SchemaProps: spec.SchemaProps{
								Description: "minSyncPeriod is the minimum period that ipvs rules are refreshed (e.g. '5s', '1m', '2h22m').",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Duration"),
							},
						},
						"scheduler": {
							SchemaProps: spec.SchemaProps{
								Description: "ipvs scheduler",
								Type:        []string{"string"},
								Format:      "",
							},
						},
					},
					Required: []string{"syncPeriod", "minSyncPeriod", "scheduler"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.Duration"},
		},
		"k8s.io/metrics/pkg/apis/custom_metrics/v1beta1.MetricValue": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "a metric value for some object",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"describedObject": {
							SchemaProps: spec.SchemaProps{
								Description: "a reference to the described object",
								Ref:         ref("k8s.io/api/core/v1.ObjectReference"),
							},
						},
						"metricName": {
							SchemaProps: spec.SchemaProps{
								Description: "the name of the metric",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"timestamp": {
							SchemaProps: spec.SchemaProps{
								Description: "indicates the time at which the metrics were produced",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"window": {
							SchemaProps: spec.SchemaProps{
								Description: "indicates the window ([Timestamp-Window, Timestamp]) from which these metrics were calculated, when returning rate metrics calculated from cumulative metrics (or zero for non-calculated instantaneous metrics).",
								Type:        []string{"integer"},
								Format:      "int64",
							},
						},
						"value": {
							SchemaProps: spec.SchemaProps{
								Description: "the value of the metric for this",
								Ref:         ref("k8s.io/apimachinery/pkg/api/resource.Quantity"),
							},
						},
					},
					Required: []string{"describedObject", "metricName", "timestamp", "value"},
				},
			},
			Dependencies: []string{
				"k8s.io/api/core/v1.ObjectReference", "k8s.io/apimachinery/pkg/api/resource.Quantity", "k8s.io/apimachinery/pkg/apis/meta/v1.Time"},
		},
		"k8s.io/metrics/pkg/apis/custom_metrics/v1beta1.MetricValueList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "a list of values for a given metric for some set of objects",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "the value of the metric across the described objects",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/metrics/pkg/apis/custom_metrics/v1beta1.MetricValue"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta", "k8s.io/metrics/pkg/apis/custom_metrics/v1beta1.MetricValue"},
		},
		"k8s.io/metrics/pkg/apis/metrics/v1alpha1.ContainerMetrics": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "resource usage metrics of a container.",
					Properties: map[string]spec.Schema{
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "Container name corresponding to the one from pod.spec.containers.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"usage": {
							SchemaProps: spec.SchemaProps{
								Description: "The memory usage is the memory working set.",
								Type:        []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apimachinery/pkg/api/resource.Quantity"),
										},
									},
								},
							},
						},
					},
					Required: []string{"name", "usage"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/api/resource.Quantity"},
		},
		"k8s.io/metrics/pkg/apis/metrics/v1alpha1.NodeMetrics": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "resource usage metrics of a node.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"timestamp": {
							SchemaProps: spec.SchemaProps{
								Description: "The following fields define time interval from which metrics were collected from the interval [Timestamp-Window, Timestamp].",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"window": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.Duration"),
							},
						},
						"usage": {
							SchemaProps: spec.SchemaProps{
								Description: "The memory usage is the memory working set.",
								Type:        []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apimachinery/pkg/api/resource.Quantity"),
										},
									},
								},
							},
						},
					},
					Required: []string{"timestamp", "window", "usage"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/api/resource.Quantity", "k8s.io/apimachinery/pkg/apis/meta/v1.Duration", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta", "k8s.io/apimachinery/pkg/apis/meta/v1.Time"},
		},
		"k8s.io/metrics/pkg/apis/metrics/v1alpha1.NodeMetricsList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "NodeMetricsList is a list of NodeMetrics.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "List of node metrics.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/metrics/pkg/apis/metrics/v1alpha1.NodeMetrics"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta", "k8s.io/metrics/pkg/apis/metrics/v1alpha1.NodeMetrics"},
		},
		"k8s.io/metrics/pkg/apis/metrics/v1alpha1.PodMetrics": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "resource usage metrics of a pod.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"timestamp": {
							SchemaProps: spec.SchemaProps{
								Description: "The following fields define time interval from which metrics were collected from the interval [Timestamp-Window, Timestamp].",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"window": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.Duration"),
							},
						},
						"containers": {
							SchemaProps: spec.SchemaProps{
								Description: "Metrics for all containers are collected within the same time window.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/metrics/pkg/apis/metrics/v1alpha1.ContainerMetrics"),
										},
									},
								},
							},
						},
					},
					Required: []string{"timestamp", "window", "containers"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.Duration", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta", "k8s.io/apimachinery/pkg/apis/meta/v1.Time", "k8s.io/metrics/pkg/apis/metrics/v1alpha1.ContainerMetrics"},
		},
		"k8s.io/metrics/pkg/apis/metrics/v1alpha1.PodMetricsList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PodMetricsList is a list of PodMetrics.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "List of pod metrics.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/metrics/pkg/apis/metrics/v1alpha1.PodMetrics"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta", "k8s.io/metrics/pkg/apis/metrics/v1alpha1.PodMetrics"},
		},
		"k8s.io/metrics/pkg/apis/metrics/v1beta1.ContainerMetrics": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "resource usage metrics of a container.",
					Properties: map[string]spec.Schema{
						"name": {
							SchemaProps: spec.SchemaProps{
								Description: "Container name corresponding to the one from pod.spec.containers.",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"usage": {
							SchemaProps: spec.SchemaProps{
								Description: "The memory usage is the memory working set.",
								Type:        []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apimachinery/pkg/api/resource.Quantity"),
										},
									},
								},
							},
						},
					},
					Required: []string{"name", "usage"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/api/resource.Quantity"},
		},
		"k8s.io/metrics/pkg/apis/metrics/v1beta1.NodeMetrics": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "resource usage metrics of a node.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"timestamp": {
							SchemaProps: spec.SchemaProps{
								Description: "The following fields define time interval from which metrics were collected from the interval [Timestamp-Window, Timestamp].",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"window": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.Duration"),
							},
						},
						"usage": {
							SchemaProps: spec.SchemaProps{
								Description: "The memory usage is the memory working set.",
								Type:        []string{"object"},
								AdditionalProperties: &spec.SchemaOrBool{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/apimachinery/pkg/api/resource.Quantity"),
										},
									},
								},
							},
						},
					},
					Required: []string{"timestamp", "window", "usage"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/api/resource.Quantity", "k8s.io/apimachinery/pkg/apis/meta/v1.Duration", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta", "k8s.io/apimachinery/pkg/apis/meta/v1.Time"},
		},
		"k8s.io/metrics/pkg/apis/metrics/v1beta1.NodeMetricsList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "NodeMetricsList is a list of NodeMetrics.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "List of node metrics.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/metrics/pkg/apis/metrics/v1beta1.NodeMetrics"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta", "k8s.io/metrics/pkg/apis/metrics/v1beta1.NodeMetrics"},
		},
		"k8s.io/metrics/pkg/apis/metrics/v1beta1.PodMetrics": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "resource usage metrics of a pod.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta"),
							},
						},
						"timestamp": {
							SchemaProps: spec.SchemaProps{
								Description: "The following fields define time interval from which metrics were collected from the interval [Timestamp-Window, Timestamp].",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.Time"),
							},
						},
						"window": {
							SchemaProps: spec.SchemaProps{
								Ref: ref("k8s.io/apimachinery/pkg/apis/meta/v1.Duration"),
							},
						},
						"containers": {
							SchemaProps: spec.SchemaProps{
								Description: "Metrics for all containers are collected within the same time window.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/metrics/pkg/apis/metrics/v1beta1.ContainerMetrics"),
										},
									},
								},
							},
						},
					},
					Required: []string{"timestamp", "window", "containers"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.Duration", "k8s.io/apimachinery/pkg/apis/meta/v1.ObjectMeta", "k8s.io/apimachinery/pkg/apis/meta/v1.Time", "k8s.io/metrics/pkg/apis/metrics/v1beta1.ContainerMetrics"},
		},
		"k8s.io/metrics/pkg/apis/metrics/v1beta1.PodMetricsList": {
			Schema: spec.Schema{
				SchemaProps: spec.SchemaProps{
					Description: "PodMetricsList is a list of PodMetrics.",
					Properties: map[string]spec.Schema{
						"kind": {
							SchemaProps: spec.SchemaProps{
								Description: "Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"apiVersion": {
							SchemaProps: spec.SchemaProps{
								Description: "APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources",
								Type:        []string{"string"},
								Format:      "",
							},
						},
						"metadata": {
							SchemaProps: spec.SchemaProps{
								Description: "Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds",
								Ref:         ref("k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta"),
							},
						},
						"items": {
							SchemaProps: spec.SchemaProps{
								Description: "List of pod metrics.",
								Type:        []string{"array"},
								Items: &spec.SchemaOrArray{
									Schema: &spec.Schema{
										SchemaProps: spec.SchemaProps{
											Ref: ref("k8s.io/metrics/pkg/apis/metrics/v1beta1.PodMetrics"),
										},
									},
								},
							},
						},
					},
					Required: []string{"items"},
				},
			},
			Dependencies: []string{
				"k8s.io/apimachinery/pkg/apis/meta/v1.ListMeta", "k8s.io/metrics/pkg/apis/metrics/v1beta1.PodMetrics"},
		},
	}
}
